// Datei Abbildungen01.java
/* ------------------------------------------------------------------------
Eine Abbildung (engl. map)   besteht aus Eintraegen.
Ein  Eintrag   (engl. entry) besteht aus zwei Objekten, einem
Schluessel (-Objekt) und einem Wert (-Objekt). Diese beiden Objekte
duerfen zu beliebigen (auch verschiedenen) Referenztypen gehoeren.
---------------------------------------------------------------------------
Eine Abbildung (ein Map-Objekt) wird erzeugt, fuenf Eintraege werden
eingefuegt, verschiedene Methoden werden auf die Abbildung angewendet
und die Ergebnisse werden ausgegeben.

In Abbildungen der Klasse HashMap darf man anstelle eines Objekts
auch den Wert null einfuegen (einmal als Schluessel, beliebig oft
als Wert). Damit handelt man sich aber Probleme ein und man sollte
einen null-Schluessel und null-Werte moeglichst vermeiden (ausser
in Beispielprogrammen wie diesem hier :-).
------------------------------------------------------------------------ */
import java.util.Map;       // Eine Schnittstelle
import java.util.HashMap;   // Eine Map-Klasse, erlaubt null
import java.util.Set;       // Eine Schnittstelle
import java.util.Iterator;  // Eine Schnittstelle

class Abbildungen01 {
   static public void main(String [] sonja) {

      // Vier Schluessel-Objekte des Typs String:
      String  key01 = new String("Hoehe");
      String  key02 = new String("Breite");
      String  key03 = new String("Anzahl");
      String  key04 = new String("Anhang");

      // Zwei Wert-Objekte verschiedener Typen:
      Double  val01 = new Double (27.5);
      Integer val02 = new Integer(12);

      // Ein Abbildungs-Objekt namen ab01 wird vereinbart
      // (Number ist eine Obertyp von Double und von Integer):
      Map<String, Number>  ab01  = new HashMap<String, Number>();

      // In die Abbildung ab01 werden 5 Eintraege eingetragen:
      ab01.put(key01, val01);
      ab01.put(key02, val01);
      ab01.put(key03, val02);
      ab01.put(key04, null );
      ab01.put(null,  val02);

      pln("Abbildungen01: Jetzt geht es los!");
      pln("-------------------------------");
      pln("A key01: " + key01 + ",   val01: " + val01);
      pln("B key02: " + key02 +  ",  val01: " + val01);
      pln("C key03: " + key03 +  ",  val02: " + val02);
      pln("-------------------------------");
      pln("D ab01                       : "   + ab01);
      pln("E ab01.keySet()              : "   + ab01.keySet());
      pln("F ab01.values()              : "   + ab01.values());
      pln("G ab01.get          (key01)  : "   + ab01.get          (key01));
      pln("H ab01.get          (\"Hoehe\"): " + ab01.get          ("Hoehe"));
      pln("I ab01.get          (\"Tiefe\"): " + ab01.get          ("Tiefe"));
      pln("J ab01.get          (key03)  : "   + ab01.get          (key03));
      pln("K ab01.containsKey  (key02)  : "   + ab01.containsKey  (key02));
      pln("L ab01.containsKey  (val01)  : "   + ab01.containsKey  (val01));
      pln("M ab01.containsValue(key02)  : "   + ab01.containsValue(key02));
      pln("N ab01.containsValue(val01)  : "   + ab01.containsValue(val01));

      //  Die Menge der Eintraege in der Abbildung ab01
      // (The set   of  entries   of the map       ab01)
      // wird jetzt geholt und die einzelnen Eintraege werden
      // zweimal ausgegeben ("altmodisch" und "modern").
      // Jeder *Eintrag* der Abbildung ab01 gehoert zum
      // parametrisierten Typ Map.Entry<String, Number>.

      Set<Map.Entry<String, Number>> eintraege = ab01.entrySet();

      pln("-------------------------------");
      pln("O Die Eintraege der Abbildung ab01:");

      // Altmodische for-Schleife mit Iterator:
      for (Iterator it = eintraege.iterator(); it.hasNext();) {
         pln("  " + it.next());
      }

      pln("-------------------------------");
      pln("P Die Eintraege der Abbildung ab01:");

      // Moderne for-Schleife (einfach und sicher):
      for (Map.Entry<String,Number> eintrag: eintraege) {
         pln("  " + eintrag);
      }

      pln("-------------------------------");
      pln("Q ab01                       : " + ab01);
      ab01.remove("Breite");
      pln("R ab01 nach remvove(\"Breite\"): " + ab01);
      ab01.remove("Hoehe");
      pln("S ab01 nach remvove(\"Hoehe\") : " + ab01);
      ab01.remove(null);
      pln("T ab01 nach remvove( null )  : " + ab01);
      pln("-------------------------------");
      pln("U ab01.isEmpty()             : " + ab01.isEmpty());
      ab01.clear();
      pln("V ab01 nach clear();         : " + ab01);
      pln("W ab01.isEmpty()             : " + ab01.isEmpty());
      pln("-------------------------------");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Abbildungen01
/* ------------------------------------------------------------------------
Ausgabe des Programms Abbildungen01:

Abbildungen01: Jetzt geht es los!
-------------------------------
A key01: Hoehe,   val01: 27.5
B key02: Breite,  val01: 27.5
C key03: Anzahl,  val02: 12
-------------------------------
D ab01                       : {null=12, Breite=27.5, Anzahl=12, Anhang=null, Hoehe=27.5}
E ab01.keySet()              : [null, Breite, Anzahl, Anhang, Hoehe]
F ab01.values()              : [12, 27.5, 12, null, 27.5]
G ab01.get          (key01)  : 27.5
H ab01.get          ("Hoehe"): 27.5
I ab01.get          ("Tiefe"): null
J ab01.get          (key03)  : 12
K ab01.containsKey  (key02)  : true
L ab01.containsKey  (val01)  : false
M ab01.containsValue(key02)  : false
N ab01.containsValue(val01)  : true
-------------------------------
O Die Eintraege der Abbildung ab01:
  null=12
  Breite=27.5
  Anzahl=12
  Anhang=null
  Hoehe=27.5
-------------------------------
P Die Eintraege der Abbildung ab01:
  null=12
  Breite=27.5
  Anzahl=12
  Anhang=null
  Hoehe=27.5
-------------------------------
Q ab01                       : {null=12, Breite=27.5, Anzahl=12, Anhang=null, Hoehe=27.5}
R ab01 nach remvove("Breite"): {null=12, Anzahl=12, Anhang=null, Hoehe=27.5}
S ab01 nach remvove("Hoehe") : {null=12, Anzahl=12, Anhang=null}
T ab01 nach remvove( null )  : {Anzahl=12, Anhang=null}
-------------------------------
U ab01.isEmpty()             : false
V ab01 nach clear();         : {}
W ab01.isEmpty()             : true
-------------------------------
------------------------------------------------------------------------ */