// Datei Abbildungen02.java
/* ------------------------------------------------------------------------
Seien s1 und s2 die Schluessel von zwei (unterschiedlichen) Eintraegen
einer Abbildung (d.h. eines Map-Objekts). Normalerweise muss dann der
Ausdruck s1.equals(s2) den Wert false haben (d.h. Schluessel muessen
eindeutig sein, oder: Unterschiedliche Eintraege einer Abbildung
muessen unterschiedliche Schluessel haben).

Abbildungen der Klasse IdentityHashMap sind von dieser Regel
ausgenommen. Fuer sie muss nur gelten, dass der Ausdruck s1 == s2
den Wert false hat (der Wert des Ausdrucks s1.equals(s2) ist bei
solchen Abbildungen unwichtig).

In Abbildungen der Klasse IdentityHashMap ist null als Schluessel
und als Wert erlaubt (man sollte einen null-Schluessel und null-
Werte aber moeglichst vermeiden).
------------------------------------------------------------------------ */
import java.util.Map;              // Eine Schnittstelle
import java.util.IdentityHashMap;  // Eine Map-Klasse

class Abbildungen02 {
   static public void main(String[] sonja) {

      // Vier Schluessel (davon einer doppelt und einer gleich null):
      String key01 = new String("Hoehe");
      String key02 = new String("Hoehe");
      String key03 = new String("Anzahl");
      String key04 = null;
      // (key01.equals(key02)) ist gleich true, aber
      // (key01   ==   key02 ) ist gleich false! Das reicht!

      // Vier Werte (davon zwei gleich null):
      Double val01 = new Double(12.3);
      Double val02 = new Double(45.6);
      Double val03 = null;
      Double val04 = null;

      // Ein Abbildungs-Objekt:
      Map<String, Double> ihm01  = new IdentityHashMap<String, Double>();

      // In die Abbildung ihm01 werden 4 Eintraege eingetragen:
      ihm01.put(key01, val01);
      ihm01.put(key02, val02);
      ihm01.put(key03, val03);
      ihm01.put(key04, val04);

      printf("Abbildungen02: Jetzt geht es los!\n");
      printf("-------------------------\n");
      printf("A key01: %-6s, val01 : %s\n", key01, val01+"");
      printf("B key02: %-6s, val02 : %s\n", key02, val02+"");
      printf("C key03: %-6s, val03 : %s\n", key03, val03+"");
      printf("D key04: %-6s, val04 : %s\n", key04, val04+"");
      printf("-------------------------\n");
      printf("E key01 == key02       : %s\n", key01 == key02);
      printf("F key01.equals(key02)  : %s\n", key01.equals(key02));
      printf("-------------------------\n");
      printf("G ihm01                : %s\n", ihm01);
      ihm01.put(null, 33.3);
      printf("H nach put(null, 33.3) : %s\n", ihm01);
      ihm01.remove(null);
      printf("I nach remove(null)    : %s\n", ihm01);
      ihm01.remove(key01);
      printf("J nach remove(key01)   : %s\n", ihm01);
      ihm01.clear();
      printf("K nach clear()         : %s\n", ihm01);
      printf("-------------------------\n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Abbildungen02
/* ------------------------------------------------------------------------
Ausgabe des Programms Abbildungen02:

Abbildungen02: Jetzt geht es los!
-------------------------
A key01: Hoehe,  val01 : 12.3
B key02: Hoehe,  val02 : 45.6
C key03: Anzahl, val03 : null
D key04: null,   val04 : null
-------------------------
E key01 == key02       : false
F key01.equals(key02)  : true
-------------------------
G ihm01                : {Hoehe=12.3, null=null, Hoehe=45.6, Anzahl=null}
H nach put(null, 33.3) : {Hoehe=12.3, null=33.3, Hoehe=45.6, Anzahl=null}
I nach remove(null)    : {Hoehe=12.3, Hoehe=45.6, Anzahl=null}
J nach remove(key01)   : {Hoehe=45.6, Anzahl=null}
K nach clear()         : {}
-------------------------
------------------------------------------------------------------------ */