// Datei Anmerk01.java

import java.lang.annotation.Annotation;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Ein Anmerkungstyp (engl. annotation type) namens AutorArno wird vereinbart
und benutzt.

Mit Anmerkungen des Typs AutorArno kann man alle Arten von Programm-
teilen versehen (Klassen, Methoden, Attribute, ...).
Anmerkungen dieses Typs werden vom Compiler aus Quelldateien in
Bytecode-Dateien (.class-Dateien) uebernommen, mann kann aber waehrend
der Ausfuehrung eines Programms *nicht* darauf zugreifen (siehe Ausgabe
des Programms Anmerk01).

Anmerkung: Der Anmerkungstyp AutorArno wird hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit er zusammen
mit der Klasse Anmerk01 in *einer* Datei vereinbart werden kann.
------------------------------------------------------------------------ */
// Ein Anmerkungstyp namens AutorArno wird vereinbart:
@interface AutorArno {}

// Die Klasse Anmerk01 und einige ihrer Elemente werden mit Anmerkungen
// des Typs AutorArno versehen:

@AutorArno
class Anmerk01 {

   @AutorArno
   static void druckeA() {printf("Methode druckeA%n");}

   static void druckeB() {printf("Methode druckeB%n");}

   @AutorArno
   static String text1 = "Attribut text1";

   static String text2 = "Attribut text2";

   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk01: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk01.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk01", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk01.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk01
/* ------------------------------------------------------------------------
Ausgabe des Programms Anmerk01:

Anmerk01: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk01 (0 Stueck):
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (0 Stueck):
Anmerkungen der Methode druckeB (0 Stueck):
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (0 Stueck):
Anmerkungen des Attributs text2 (0 Stueck):
------------------------------------------
------------------------------------------------------------------------ */
