// Datei Anmerk02.java
// Hinweis: Vor dieser Datei sollte man die Datei Anmerk01 compilieren!
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;       // Ein   Anmerkungstyp
import java.lang.annotation.RetentionPolicy; // Ein Aufzaehlungstyp
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Ein Anmerkungstyp namens AutorBert wird vereinbart und benutzt.

Mit Anmerkungen des Typs AutorBert kann man alle Arten von Programm-
teilen versehen (Klassen, Methoden, Attribute, ...).
Anmerkungen dieses Typs werden vom Compiler aus Quelldateien in
Bytecode-Dateien (.class-Dateien) uebernommen und mann kann waehrend
der Ausfuehrung eines Programms darauf zugreifen (RetentionPolicy.RUNTIME):

Anmerkung: Der Anmerkungstyp AutorBert wird hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit er zusammen
mit der Klasse Anmerk02 in *einer* Datei vereinbart werden kann.
------------------------------------------------------------------------ */
// Ein Anmerkungstyp namens AutorBert wird vereinbart:

@Retention(RetentionPolicy.RUNTIME)
@interface AutorBert {}

// Die Klasse Anmerk02 und einige ihrer Elemente werden mit Anmerkungen
// der folgenden Typen versehen:
// Typ AutorArno  (vereinbart in Datei Anmerk01)
// Typ AutorBert  (vereinbart in dieser Datei)

@AutorArno
@AutorBert
class Anmerk02 {

   @AutorArno
   static void druckeA() {printf("Methode druckeA%n");}

   @AutorBert
   static void druckeB() {printf("Methode druckeB%n");}

   @AutorBert
   @AutorArno
   static String text1 = "Attribut text1";

   static String text2 = "Attribut text2";
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk02: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk02.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk02", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk02.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk02
/* ------------------------------------------------------------------------
Ausgabe des Programms Anmerk02:

Anmerk02: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk02 (1 Stueck):
  @AutorBert()
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (0 Stueck):
Anmerkungen der Methode druckeB (1 Stueck):
  @AutorBert()
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (1 Stueck):
  @AutorBert()
Anmerkungen des Attributs text2 (0 Stueck):
------------------------------------------
------------------------------------------------------------------------ */
