// Datei Anmerk03.java
// Hinweis: Vor dieser Datei sollte man die Dateien Anmerk01 und
// Anmerk02 compilieren!
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;       // Ein   Anmerkungstyp
import java.lang.annotation.RetentionPolicy; // Ein Aufzaehlungstyp
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Ein Anmerkungstyp namens Autor wird vereinbart und benutzt.

Mit einer Anmerkung des Typs Autor koennen alle Arten von Programmteilen
versehen werden und auf Anmerkungen dieses Typs kann man waehrend der
Ausfuehrung eines Programms zugreifen (RetentionPolicy.RUNTIME).
Eine Anmerkung dieses Typs besteht aus einem String-Objekt value.
Als Standardbelegung wird das String-Objekt "Noch unklar"
festgelegt.

Anmerkung: Der Anmerkungstyp Autor wird hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit er zusammen
mit der Klasse Anmerk03 in *einer* Datei vereinbart werden kann.
------------------------------------------------------------------------ */
// Ein Anmerkungstyp namens Autor wird vereinbart:

@Retention(RetentionPolicy.RUNTIME)
@interface Autor {
   String value() default "Noch unklar";
} // @interface Autor


// Die Klasse Anmerk03 und einige ihrer Elemente werden mit Anmerkungen
// der folgenden Typen versehen:
// Typ AutorArno  (vereinbart in Datei Anmerk01)
// Typ AutorBert  (vereinbart in Datei Anmerk02)
// Typ Autor      (vereinbart in dieser Datei)

@AutorArno
@AutorBert
@Autor("Carl")
class Anmerk03 {

   @Autor               // Standardbelegung "Noch unklar"
   @AutorArno
   static void druckeA() {printf("Methode druckeA%n");}

   @AutorBert
   static void druckeB() {printf("Methode druckeB%n");}

   @AutorBert
   @Autor(value="Dora") // Alternative Syntax
   @AutorArno
   static String text1 = "Attribut text1";

   static String text2 = "Attribut text2";

   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk03: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk03.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk03", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk03.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk03
/* ------------------------------------------------------------------------
Ausgabe des Programms Anmerk03:

Anmerk03: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk03 (2 Stueck):
  @AutorBert()
  @Autor(value=Carl)
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (1 Stueck):
  @Autor(value=Noch unklar)
Anmerkungen der Methode druckeB (1 Stueck):
  @AutorBert()
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (2 Stueck):
  @AutorBert()
  @Autor(value=Dora)
Anmerkungen des Attributs text2 (0 Stueck):
------------------------------------------
------------------------------------------------------------------------ */
