// Datei Anmerk04.java
// Hinweis: Vor dieser Datei sollte man die Dateien Anmerk01, Anmerk02
// und Anmerk03 compilieren!
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;       // Ein   Anmerkungstyp
import java.lang.annotation.RetentionPolicy; // Ein Aufzaehlungstyp
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Ein Anmerkungstyp namens Hersteller wird vereinbart und benutzt.

Mit einer Anmerkung des Typs Hersteller koennen alle Arten von Programm-
teilen versehen werden und auf Anmerkungen dieses Typs kann man
waehrend der Ausfuehrung eines Programms zugreifen.
Eine Anmerkung dieses Typs besteht aus einem String-Objekt und einem
int-Wert. Fuer beide Komponenten werden Standardbelegungen festgelegt:

Anmerkung: Der Anmerkungstyp Hersteller wird hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit er zusammen
mit der Klasse Anmerk04 in *einer* Datei vereinbart werden kann.
------------------------------------------------------------------------ */
// Ein Anmerkungstyp namens Hersteller wird vereinbart

@Retention(RetentionPolicy.RUNTIME)
@interface Hersteller {
   String firma() default "ABC GmbH";
   int    jahr()  default 2003;
} // @interface Hersteller

// Zum Aufzaehlungstyp RetentionPolicy gehoeren insgesamt 3 Werte:
// SOURCE, CLASS (Standardwert) und RUNTIME.

/* ------------------------------------------------------------------------
Die Klasse Anmerk04 und einige ihrer Elemente werden mit Anmerkungen
der folgenden Typen versehen:
Typ AutorArno  (vereinbart in Datei Anmerk01)
Typ AutorBert  (vereinbart in Datei Anmerk02)
Typ Autor      (vereinbart in Datei Anmerk03)
Typ Hersteller (vereinbart in dieser Datei)
------------------------------------------------------------------------ */
@Hersteller(jahr=2004, firma="Meyer&Sohn")
@AutorArno
@AutorBert
@Autor("Carl")
class Anmerk04 {

   @Autor                         // Standardbelegung "Noch unklar"
   @AutorArno
   static void druckeA() {printf("Methode druckeA%n");}

   @AutorBert
   static void druckeB() {printf("Methode druckeB%n");}

   @AutorBert
   @Autor(value="Dora")           // Alternative Syntax
   @AutorArno
   @Hersteller(firma="Cox Lmtd.") // jahr hat Standardbelegung 2003
   static String text1 = "Attribut text1";

   @Hersteller(jahr=2002)         // firma hat Standardbelegung "ABC GmbH"
   static String text2 = "Attribut text2";

   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk04: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk04.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk04", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk04.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk04
/* ------------------------------------------------------------------------
Ausgabe des Programms Anmerk04:

Anmerk04: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk04 (3 Stueck):
  @Autor(value=Carl)
  @Hersteller(jahr=2004, firma=Meyer&Sohn)
  @AutorBert()
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (1 Stueck):
  @Autor(value=Noch unklar)
Anmerkungen der Methode druckeB (1 Stueck):
  @AutorBert()
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (3 Stueck):
  @AutorBert()
  @Autor(value=Dora)
  @Hersteller(jahr=2003, firma=Cox Lmtd.)
Anmerkungen des Attributs text2 (1 Stueck):
  @Hersteller(jahr=2002, firma=ABC GmbH)
------------------------------------------
------------------------------------------------------------------------ */
