// Datei Anmerk05.java
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;       // Ein   Anmerkungstyp
import java.lang.annotation.RetentionPolicy; // Ein Aufzaehlungstyp
import java.lang.annotation.Target;          // Ein   Anmerkungstyp
import java.lang.annotation.ElementType;     // Ein Aufzaehlungstyp
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Zwei Anmerkungstypen werden vereinbart.
Mit Anmerkungen des Typs AutorT darf man nur *Typen* versehen (d.h.
    Klassen, Schnittstellen, Aufzaehlungstypen und Anmerkungstypen)
Mit Anmerkungen des Typs AutorMA darf man nur *Methoden* und
    *Attribute* versehen (aber keine Typen, Pakete, Konstruktoren, ...).

Anmerkung: Die Anmerkungstypen AutorT und AutorMA werden hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit mehrere solche
Typen und die Klasse Anmerk05 in *einer* Datei vereinbart werden koennen.
------------------------------------------------------------------------ */
// Zwei Anmerkungstypen namens AutorT und AutorMA werden vereinbart:

@Retention(RetentionPolicy.RUNTIME)
@Target   (ElementType.TYPE)                        // Nur *ein* Ziel
@interface AutorT {
   String value();
} // @interface AutorT

@Retention(RetentionPolicy.RUNTIME)
@Target   ({ElementType.METHOD, ElementType.FIELD}) // Mehrere Ziele!
@interface AutorMA {
   String value();
} // @interface AutorMA

// Zum Aufzaehlungstyp RetentionPolicy gehoeren insgesamt 3 Werte:
// SOURCE, CLASS (Standardwert) und RUNTIME.

// Zum Aufzaehlungstyp ElementType gehoeren insgesamt 8 Werte:
// ANNOTATION_TYPE, CONSTRUCTOR, FIELD, LOCAL_VARIABLE, METHOD,
// PACKAGE, PARAMETER und TYPE.
// ------------------------------------------------------------------------
// Die Klasse Anmerk05 und einige ihrer Elemente werden mit Anmerkungen
// der oben vereinbarten Typen AutorT und AutorMA versehen.

  @AutorT ("Arno")   // Erlaubt
//@AutorMA("Bert")   // Nicht erlaubt
class Anmerk05 {

// @AutorT ("Arno")  // Nicht erlaubt
   @AutorMA("Bert")  // Erlaubt
   static void druckeA() {printf("Methode druckeA%n");}

   static void druckeB() {printf("Methode druckeB%n");}

// @AutorT ("Arno")  // Nicht erlaubt
   @AutorMA("Carl")  // Erlaubt
   static String text1 = "Attribut text1";

   static String text2 = "Attribut text2";

   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk05: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk05.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk05", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk05.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk05
/* ------------------------------------------------------------------------
Fehlermeldungen des Compilers java von Sun, wenn die entsprechenden Zeilen
keine Ausnahmen sind:

D:\Java\Anmerkungen\Anmerk05.java:39: annotation type not applicable to
this kind of declaration
@AutorMA("Bert")   // Nicht erlaubt
         ^
D:\Java\Anmerkungen\Anmerk05.java:42: annotation type not applicable to
this kind of declaration
 @AutorT ("Arno")  // Nicht erlaubt
          ^
D:\Java\Anmerkungen\Anmerk05.java:48: annotation type not applicable to
this kind of declaration
 @AutorT ("Arno")  // Nicht erlaubt
          ^
---------------------------------------------------------------------------
Ausgabe des Programms Anmerk05:

Anmerk05: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk05 (1 Stueck):
  @AutorT(value=Arno)
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (1 Stueck):
  @AutorMA(value=Bert)
Anmerkungen der Methode druckeB (0 Stueck):
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (1 Stueck):
  @AutorMA(value=Carl)
Anmerkungen des Attributs text2 (0 Stueck):
------------------------------------------
------------------------------------------------------------------------ */
