// Datei Anmerk06.java
import java.lang.annotation.Annotation;
import java.lang.annotation.Retention;       // Ein   Anmerkungstyp
import java.lang.annotation.RetentionPolicy; // Ein Aufzaehlungstyp
import java.lang.annotation.Target;          // Ein   Anmerkungstyp
import java.lang.reflect.Method;
import java.lang.reflect.Field;
/* ------------------------------------------------------------------------
Drei Anmerkungstypen (engl. annotation types) mit Elementen von
Reihungstypen werden vereinbart und benutzt.

Mit einer Anmerkung des Typs AutorR (wie "Reihung") kann man beliebig
viele (0, 1, 2, ...) Namen von Autoren als Anmerkung angeben.

Die Anmerkungstypen ProTag und ProMitarbeiter gehoeren eng zusammen:

Mit einer Anmerkung des Typs ProTag kann man angeben, wieviele
Stunden ein Mitarbeiter am i-ten Projekttag gearbeitet hat (dabei
laeuft i von 0 bis zu einer beliebigen Obergrenze).
Mit Anmerkungen dieses Typs darf man keinen Programmteil direkt
versehen (siehe @Target-Anmerkung mit leerer Reihunge).

Mit einer Anmerkung des Typs pro Mitarbeiter kann man fuer beliebig
viele Mitarbeiter je eine Anmerkung des Typs ProTag angeben.

Anmerkung: Der Anmerkungstyp AutorR wird hier nicht public
(sondern mit paketweiter Erreichbarkeit) vereinbart, damit er zusammen
mit der Klasse Anmerk06 in *einer* Datei vereinbart werden kann.
------------------------------------------------------------------------ */
// Drei Anmerkungstypen namens AutorR, ProTag und ProMitarbeiter
// werden vereinbart:

@Retention(RetentionPolicy.RUNTIME)
@interface AutorR {
   String[] value() default {};
} // @interface AutorR

@Retention(RetentionPolicy.RUNTIME)
@Target   ({})
@interface ProTag {
   // Jede Komponente value()[i] enthaelt die Anzahl der Stunden,
   // die ein Mitarbeiter am i-ten Projekttag gearbeitet hat:
   int[] value() default {};
} // @interface AutorR

@Retention(RetentionPolicy.RUNTIME)
@interface ProMitarbeiter {
   // Jede Komponente value()[j] beschreibt die Arbeitszeiten des
   // j-ten Mitarbeiters
   ProTag[] value() default {};
} // @interface AutorR
/* ------------------------------------------------------------------------
Die Klasse Anmerk06 und einige ihrer Elemente werden mit Anmerkungen
der Typen AutorR und ProMitarbeiter versehen.

Die Klasse Anmekr06 wurde von Arno, Bert, und Carl programmiert.
Arno hat an den ersten drei Projekttagen 4, 8 und 6 Stunden daran gearbeitet.
Bert kam erst am 2.Tag dazu und hat dann 3 mal 8 Stunden gearbeitet.
Carl war nur die ersten beiden Tage dabei mit 8 und 4 Stunden.

Diese Tatsachen werden durch zwei Anmerkungen (der Typen @AutorR und
@ProMitarbeiter) beschrieben
------------------------------------------------------------------------ */
// Die Klasse Anmerk06 und einige ihrer Elemente werden mit Anmerkungen
// der Typen AutorR, ProTag und ProMitarbeiter versehen:

@AutorR({"Arno", "Bert", "Carl"})
@ProMitarbeiter({@ProTag({4, 8, 6}),
                 @ProTag({0, 8, 8, 8}),
                 @ProTag({8, 4})
                })
class Anmerk06 {

   @AutorR("Bert")
   static void druckeA() {printf("Methode druckeA%n");}

   @AutorR({"Carl", "Arno"})
   static void druckeB() {printf("Methode druckeB%n");}

   @AutorR({"Anna", "Bert"})
   static String text1 = "Attribut text1";

   @AutorR
   static String text2 = "Attribut text2";
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Anmerk06: Jetzt geht es los!%n");
      printf("------------------------------------------%n");
      druckeA();
      druckeB();
      printf("%s%n", text1);
      printf("%s%n", text2);
      printf("------------------------------------------%n");
      druckeC(Anmerk06.class);
      printf("------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   static void druckeC(Class cob) {
      // Gibt alle Anmerkungen der Klasse cob, alle Anmerkungen ihrer
      // Methoden und alle Anmerkungen ihrer Attribute aus:
      Annotation[] ar = cob.getAnnotations();
      druckeD("der Klasse Anmerk06", ar);
      printf("------------------------------------------%n");
      Method[] mr = cob.getDeclaredMethods();
      for (Method m: mr) {
         ar = m.getAnnotations();
         druckeD("der Methode " + m.getName(), ar);
      }
      printf("------------------------------------------%n");
      Field[]  fr = Anmerk06.class.getDeclaredFields();
      for (Field f: fr) {
         ar = f.getAnnotations();
         druckeD("des Attributs " + f.getName(), ar);
      }
   } // druckeC
   // ---------------------------------------------------------------------
   static void druckeD(String progTeil, Annotation[] ar) {
      // Gibt ar (und ein paar Texte) lesbar aus
      printf("Anmerkungen %-19s (%d Stueck):%n", progTeil, ar.length);
      for (Annotation a: ar) printf("  %s%n", a);
   } // druckeD
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Anmerk06
/* ------------------------------------------------------------------------
Ausgabe des Programms Anmerk06 (eine ziemlich lange Zeile wurde
nachtraeglich von Hand lesbar umgebrochen):

Anmerk06: Jetzt geht es los!
------------------------------------------
Methode druckeA
Methode druckeB
Attribut text1
Attribut text2
------------------------------------------
Anmerkungen der Klasse Anmerk06 (2 Stueck):
  @ProMitarbeiter(value=[@ProTag(value=[4, 8, 6]),
                         @ProTag(value=[0, 8, 8, 8]),
                         @ProTag(value=[8, 4])
                        ])
  @AutorR(value=[Arno, Bert, Carl])
------------------------------------------
Anmerkungen der Methode main    (0 Stueck):
Anmerkungen der Methode printf  (0 Stueck):
Anmerkungen der Methode druckeA (1 Stueck):
  @AutorR(value=[Bert])
Anmerkungen der Methode druckeB (1 Stueck):
  @AutorR(value=[Carl, Arno])
Anmerkungen der Methode druckeC (0 Stueck):
Anmerkungen der Methode druckeD (0 Stueck):
------------------------------------------
Anmerkungen des Attributs text1 (1 Stueck):
  @AutorR(value=[Anna, Bert])
Anmerkungen des Attributs text2 (1 Stueck):
  @AutorR(value=[])
------------------------------------------
------------------------------------------------------------------------ */
