// Datei ArrayList06.java
/* ------------------------------------------------------------------------
Eine *Reihung von Objekten* kann man mit der Methode Arrays.asList (etwas
vereinfacht gesagt) in ein entsprechendes Sammlungsobjekt (collection
object) umwandeln. In der Java-Standardbibliothek fehlen aber Methoden,
mit denen man eine *Reihung mit primitiven Komponenten* in eine Sammlung
von entsprechenden Huellobjekten (z.B. eine Reihung von int-Werten in eine
Sammlung von Integer-Objekten) umwandeln kann oder umgekehrt eine Sammlung
von Huellobjekten in eine entsprechende Reihung mit primitiven Komponenten
(z.B. eine Sammlung von Integer-Objekten in eine Reihung von int-Werten)
umwandeln kann.

Diese Klasse ist ein reiner Modul. Er enthaelt fuer jeden der 8 primitiven
Typen (byte, char, short, int, long, float, double und boolean) 3 Methoden.
Die 3 Methoden fuer den Typ byte werden hier (als Beispiel) etwas naeher
beschrieben. Die Methoden fuer die anderen Typen leisten ganz Entsprechen-
des:

Die 3 Methoden fuer den Typ byte:

asList:    Wandelt eine Reihung vom Typ byte[] in eine Sammlung vom Typ
           ArrayList<Byte> um.
byteArray: Wandelt eine Sammlung vom Typ ArrayList<Byte> in eine Reihung
           vom Typ byte[] um.
toString:  Wandelt eine Reihung vom Typ byte[] in einen entsprechenden
           String um, der wie ein Reihungsinitialisierer aussieht (z.B.
           so: "{17, 12, 25}".

Diese 3x8 gleich 24 Funktionen werden in der main-Methode einem kleinen
Test unterzogen.
------------------------------------------------------------------------ */
import java.util.ArrayList;
import java.util.Arrays;

class ArrayList06 {
   // ---------------------------------------------------------------------
   static public ArrayList<Byte> asList(byte[] br) {
      // Wandelt br in eine entsprechende Sammlung um.
      ArrayList<Byte>  erg = new ArrayList<Byte>(br.length);
      for (byte b: br) erg.add(b);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Character> asList(char[] cr) {
      // Wandelt cr in eine entsprechende Sammlung um.
      ArrayList<Character> erg = new ArrayList<Character>(cr.length);
      for (char c: cr) erg.add(c);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Short> asList(short[] sr) {
      // Wandelt sr in eine entsprechende Sammlung um.
      ArrayList<Short> erg = new ArrayList<Short>(sr.length);
      for (short s: sr) erg.add(s);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Integer> asList(int[] ir) {
      // Wandelt ir in eine entsprechende Sammlung um.
      ArrayList<Integer> erg = new ArrayList<Integer>(ir.length);
      for (int i: ir) erg.add(i);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Long> asList(long[] lr) {
      // Wandelt lr in eine entsprechende Sammlung um.
      ArrayList<Long> erg = new ArrayList<Long>(lr.length);
      for (long l: lr) erg.add(l);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Float> asList(float[] fr) {
      // Wandelt fr in eine entsprechende Sammlung um.
      ArrayList<Float> erg = new ArrayList<Float>(fr.length);
      for (float f: fr) erg.add(f);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Double> asList(double[] dr) {
      // Wandelt dr in eine entsprechende Sammlung um.
      ArrayList<Double> erg = new ArrayList<Double>(dr.length);
      for (double d: dr) erg.add(d);
      return erg;
   } // asList
   // ---------------------------------------------------------------------
   static public ArrayList<Boolean> asList(boolean[] wr) {
      // Wandelt wr in eine entsprechende Sammlung um.
      ArrayList<Boolean> erg = new ArrayList<Boolean>(wr.length);
      for (boolean w: wr) erg.add(w);
      return erg;
   } // asList
   // =====================================================================
   static public byte[] byteArray(ArrayList<Byte> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs byte[] um.
      byte[] erg = new byte[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).byteValue();
      } // for
      return erg;
   } // byteArray
   // ---------------------------------------------------------------------
   static public char[] charArray(ArrayList<Character> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs char[] um.
      char[] erg = new char[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).charValue();
      } // for
      return erg;
   } // charArray
   // ---------------------------------------------------------------------
   static public short[] shortArray(ArrayList<Short> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs short[] um.
      short[] erg = new short[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).shortValue();
      } // for
      return erg;
   } // shortArray
   // ---------------------------------------------------------------------
   static public int[] intArray(ArrayList<Integer> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs int[] um.
      int[] erg = new int[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).intValue();
      } // for
      return erg;
   } // intArray
   // ---------------------------------------------------------------------
   static public long[] longArray(ArrayList<Long> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs long[] um.
      long[] erg = new long[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).longValue();
      } // for
      return erg;
   } // longArray
   // ---------------------------------------------------------------------
   static public float[] floatArray(ArrayList<Float> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs float[] um.
      float[] erg = new float[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).floatValue();
      } // for
      return erg;
   } // floatArray
   // ---------------------------------------------------------------------
   static public double[] doubleArray(ArrayList<Double> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs double[] um.
      double[] erg = new double[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).doubleValue();
      } // for
      return erg;
   } // doubleArray
   // ---------------------------------------------------------------------
   static public boolean[] booleanArray(ArrayList<Boolean> sam) {
      // Wandelt die Sammlung sam in eine Reihung des typs boolean[] um.
      boolean[] erg = new boolean[sam.size()];
      for (int i=0; i<sam.size(); i++) {
         erg[i] = sam.get(i).booleanValue();
      } // for
      return erg;
   } // booleanArray
   // =====================================================================
   // Ein paar String-Konstanten fuer die Methoden toString:
   static final String ANFANG  = "{";
   static final String ENDE    = "}";
   static final String TRENNER = ", ";
   // ---------------------------------------------------------------------
   static public String toString(byte[] br) {
      // Wandelt die Reihung br in einen entsprechenden String um.
      if (br.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (byte b: br)  erg.append(b + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(char[] cr) {
      // Wandelt die Reihung cr in einen entsprechenden String um.
      if (cr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (char c: cr)  erg.append(c + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(short[] sr) {
      // Wandelt die Reihung sr in einen entsprechenden String um.
      if (sr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (short s: sr)  erg.append(s + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(int[] ir) {
      // Wandelt die Reihung ir in einen entsprechenden String um.
      if (ir.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (int i: ir)  erg.append(i + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(long[] lr) {
      // Wandelt die Reihung lr in einen entsprechenden String um.
      if (lr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (long l: lr)  erg.append(l + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(float[] fr) {
      // Wandelt die Reihung fr in einen entsprechenden String um.
      if (fr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (float f: fr)  erg.append(f + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(double[] dr) {
      // Wandelt die Reihung dr in einen entsprechenden String um.
      if (dr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (double d: dr)  erg.append(d + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   static public String toString(boolean[] wr) {
      // Wandelt die Reihung wr in einen entsprechenden String um.
      if (wr.length == 0) return "{}";
      StringBuffer erg = new StringBuffer(ANFANG);
      for (boolean w: wr)  erg.append(w + TRENNER);
      erg.delete(erg.length()-2, erg.length());
      erg.append(ENDE);
      return erg.toString();
   } // toString
   // =====================================================================
   static public void main(String[] sonja) {
      // Jede Methode dieses Moduls wird einem minimalen Test unterworfen
      // (um sicher zu stellen, dass sie mindestens in einem Fall richtig
      // funktioniert :-)
      //---------------------------------------
      // Fuer jeden primitiven Typ pt werden zwei Reihungen vom Typ
      // pt[] vereinbart, eine nicht-leere und eine leere:
         byte[]  br01 = {   17,   12,    25}, br04 = {};
         char[]  cr01 = {   65,   66,    67}, cr04 = {};
        short[]  sr01 = { 1700, 1200,  2500}, sr04 = {};
          int[]  ir01 = { 1700, 1200,  2500}, ir04 = {};
         long[]  lr01 = { 1700, 1200,  2500}, lr04 = {};
        float[]  fr01 = {   17,   12,    25}, fr04 = {};
       double[]  dr01 = {   17,   12,    25}, dr04 = {};
      boolean[]  wr01 = {false, true, false}, wr04 = {};
      //---------------------------------------
      // Jede Reihung wird in eine Sammlung umgewandelt (mit asList):
      ArrayList<Byte>      samB02 = asList(br01), samB05 = asList(br04);
      ArrayList<Character> samC02 = asList(cr01), samC05 = asList(cr04);
      ArrayList<Short>     samS02 = asList(sr01), samS05 = asList(sr04);
      ArrayList<Integer>   samI02 = asList(ir01), samI05 = asList(ir04);
      ArrayList<Long>      samL02 = asList(lr01), samL05 = asList(lr04);
      ArrayList<Float>     samF02 = asList(fr01), samF05 = asList(fr04);
      ArrayList<Double>    samD02 = asList(dr01), samD05 = asList(dr04);
      ArrayList<Boolean>   samW02 = asList(wr01), samW05 = asList(wr04);
      //---------------------------------------
      // Jede Sammlung wird zurueck in eine Reihung umgewandelt:
         byte[]  br03 =    byteArray(samB02), br06 =    byteArray(samB05);
         char[]  cr03 =    charArray(samC02), cr06 =    charArray(samC05);
        short[]  sr03 =   shortArray(samS02), sr06 =   shortArray(samS05);
          int[]  ir03 =     intArray(samI02), ir06 =     intArray(samI05);
         long[]  lr03 =    longArray(samL02), lr06 =    longArray(samL05);
        float[]  fr03 =   floatArray(samF02), fr06 =   floatArray(samF05);
       double[]  dr03 =  doubleArray(samD02), dr06 =  doubleArray(samD05);
      boolean[]  wr03 = booleanArray(samW02), wr06 = booleanArray(samW05);
      pln("---------------------------------");
      // Alle Reihungen und Sammlungen werden ausgegeben:
      pln("  br01: " + toString(br01));
      pln("  cr01: " + toString(cr01));
      pln("  sr01: " + toString(sr01));
      pln("  ir01: " + toString(ir01));
      pln("  lr01: " + toString(lr01));
      pln("  fr01: " + toString(fr01));
      pln("  dr01: " + toString(dr01));
      pln("  wr01: " + toString(wr01));
      pln("---------------------------------");
      pln("samB02: " + samB02);
      pln("samC02: " + samC02);
      pln("samS02: " + samS02);
      pln("samI02: " + samI02);
      pln("samL02: " + samL02);
      pln("samF02: " + samF02);
      pln("samD02: " + samD02);
      pln("samW02: " + samW02);
      pln("---------------------------------");
      pln("  br03: " + toString(br03));
      pln("  cr03: " + toString(cr03));
      pln("  sr03: " + toString(sr03));
      pln("  ir03: " + toString(ir03));
      pln("  lr03: " + toString(lr03));
      pln("  fr03: " + toString(fr03));
      pln("  dr03: " + toString(dr03));
      pln("  wr03: " + toString(wr03));
      pln("---------------------------------");
      pln("  br04: " + toString(br04));
      pln("  cr04: " + toString(cr04));
      pln("  sr04: " + toString(sr04));
      pln("  ir04: " + toString(ir04));
      pln("  lr04: " + toString(lr04));
      pln("  fr04: " + toString(fr04));
      pln("  dr04: " + toString(dr04));
      pln("  wr04: " + toString(wr04));
      pln("---------------------------------");
      pln("samB05: " + samB05);
      pln("samC05: " + samC05);
      pln("samS05: " + samS05);
      pln("samI05: " + samI05);
      pln("samL05: " + samL05);
      pln("samF05: " + samF05);
      pln("samD05: " + samD05);
      pln("samW05: " + samW05);
      pln("---------------------------------");
      pln("  br06: " + toString(br06));
      pln("  cr06: " + toString(cr06));
      pln("  sr06: " + toString(sr06));
      pln("  ir06: " + toString(ir06));
      pln("  lr06: " + toString(lr06));
      pln("  fr06: " + toString(fr06));
      pln("  dr06: " + toString(dr06));
      pln("  wr06: " + toString(wr06));
      pln("---------------------------------");
      pln("  ArrayList06: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Ein Unterprogramm mit kurzem Namen:
   static void pln(String s) {System.out.println(s);}
   // ---------------------------------------------------------------------
} // class ArrayList06
/* ------------------------------------------------------------------------
Ausgabe des Programms ArrayList06:

---------------------------------
  br01: {17, 12, 25}
  cr01: {A, B, C}
  sr01: {1700, 1200, 2500}
  ir01: {1700, 1200, 2500}
  lr01: {1700, 1200, 2500}
  fr01: {17.0, 12.0, 25.0}
  dr01: {17.0, 12.0, 25.0}
  wr01: {false, true, false}
---------------------------------
samB02: [17, 12, 25]
samC02: [A, B, C]
samS02: [1700, 1200, 2500]
samI02: [1700, 1200, 2500]
samL02: [1700, 1200, 2500]
samF02: [17.0, 12.0, 25.0]
samD02: [17.0, 12.0, 25.0]
samW02: [false, true, false]
---------------------------------
  br03: {17, 12, 25}
  cr03: {A, B, C}
  sr03: {1700, 1200, 2500}
  ir03: {1700, 1200, 2500}
  lr03: {1700, 1200, 2500}
  fr03: {17.0, 12.0, 25.0}
  dr03: {17.0, 12.0, 25.0}
  wr03: {false, true, false}
---------------------------------
  br04: {}
  cr04: {}
  sr04: {}
  ir04: {}
  lr04: {}
  fr04: {}
  dr04: {}
  wr04: {}
---------------------------------
samB05: []
samC05: []
samS05: []
samI05: []
samL05: []
samF05: []
samD05: []
samW05: []
---------------------------------
  br06: {}
  cr06: {}
  sr06: {}
  ir06: {}
  lr06: {}
  fr06: {}
  dr06: {}
  wr06: {}
---------------------------------
  ArrayList06: Das war's erstmal!
------------------------------------------------------------------------ */