// Datei ArraysP.java
/* ------------------------------------------------------------------------
Seien die Variablen or und os wie folgt vereinbart:

Object[]     or = ... ;               // Objekt-Reihung
List<String> os = Arrays.asList(or);  // Objekt-Sammlung

Das Objekt os bietet eine "List-Sicht auf die Reihung or". Jede
Veraenderung von or wirkt sich auf os aus und umgekehrt, jede
Veraenderung von os wirkt sich auf or aus. Methoden, die die
Laenge der Sammlung os veraemdern wuerden, funktionieren nicht richtig
sondern werfen eine Ausnahme des Typs UnsupportedOperationException.

Die Methode Arrays.asList kann aber nur auf Reihungen von Objekten ange-
wendet werden, nicht auf Reihungen von primitiven Werten.
---------------------------------------------------------------------------
Diese Klasse ArraysP ("P" wie "primitiv") stellt ihren Benutzern
8 Erweiterungen der Schnittstelle List
(namens ByteList, CharList, ..., BooleanList) und
8 Methoden namens asList mit folgenden Profilen zur Verfuegung:

   ByteList asList(   byte[]);
   CharList asList(   char[]);
  ShortList asList(  short[]);
    IntList asList(    int[]);
   LongList asList(   long[]);
  FloatList asList(  float[]);
 DoubleList asList( double[]);
BooleanList asList(boolean[]);
---------------------------------------------------------------------------
Die folgenden Erlaeuterungen beziehen sich auf den primitiven Typ byte.
Fuer die anderen 7 primitiven Typen gilt ganz Entsprechendes.

Seien br uns bs wie folgt vereinbart:

byte[]   br = ... ;               // byte-Reihung
ByteList bs = ArraysP.asList(br); // byte-Sammlung

Das Objekt bs bietet eine "List-Sicht auf die Reihung br". Jede
Veraenderung von br wirkt sich auf bs aus und umgekehrt, jede
Veraenderung von bs wirkt sich auf br aus. Methoden, die die
Laenge der Sammlung bs veraendern wuerden, funktionieren nicht richtig
sondern werfen eine Ausnahme des Typs UnsupportedOperationException.

Die Schnittstelle ByteList erweitert die Schnittstelle List und enthaelt
unter anderem folgende Methoden (die "richtig funktionieren" und nicht
die Ausnahme UnsopportedOperationException werfen):

boolean containsAll (Collection c)                    // aus List
boolean containsAllP(byte[] suchReihung)              // neu in BytList
boolean    contains (Byte   objekt);                  // aus List
boolean    containsP(byte   wert);                    // neu in ByteList
Byte            get (int    index);                   // aus List
byte            getP(int    index);                   // neu in ByteList
int         indexOf (Byte   objekt);                  // aus List
int         indexOfP(byte   wert);                    // neu in ByteList
Byte            set (int    index, Byte neuesObjekt); // aus List
byte            setP(int    index, byte neuerWert);   // neu in ByteList
Object[]    toArray ();                               // aus List
byte[]      toArrayP();                               // neu in ByteList
---------------------------------------------------------------------------
Die Klasse ArraysP enthaelt fuer jeden primtiven Typ PT
eine Methode       PTList asList(PT[] r)
eine Schnittstelle PTList und
eine Klasse        ListOfPT
insgesamt also 8 Methoden, 8 Schnittstellen und 8 Klassen.
------------------------------------------------------------------------ */
import java.util.AbstractList; // Eine abstrakte Klase
import java.util.List;         // Eine Schnittstelle
import java.util.RandomAccess; // Eine Schnittstelle
import java.util.Iterator;     // Eine Schnittstelle
import java.util.Collection;   // Eine Schnittstelle
import java.io  .Serializable; // Eine Schnittstelle

class ArraysP {
   // =====================================================================
   public static ByteList asList(byte[] r) {
      return new ListOfByte(r);
   } // asList fuer byte[]
   // ---------------------------------------------------------------------
   public static interface ByteList extends List<Byte> {
      boolean containsAllP(byte[] suchReihung);
      boolean    containsP(byte   wert);
      byte            getP(int    index);
      int         indexOfP(byte   wert);
      byte            setP(int    index, byte neuerWert);
      byte[]      toArrayP();
   } // interface ByteList
   // ---------------------------------------------------------------------
   private static class ListOfByte extends AbstractList<Byte>
      implements ByteList, RandomAccess, Serializable
   {
      private byte[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfByte(byte[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfByte
      // ------------------------------------------------------------------
      public boolean  containsAllP(byte[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Byte o)    {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(byte w)    {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Byte     get      (int index) {return new Byte(r[index]);}
      // ------------------------------------------------------------------
      public byte     getP     (int index) {return          r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Byte o) {
         byte such =  o.byteValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(byte w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Byte   set (int index, Byte neuesObjekt) {
          Byte altesObjekt = new Byte(r[index]);
          r[index] = neuesObjekt.byteValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public byte      setP(int index, byte neuerWert) {
         byte alterWert = r[index];
         r[index]       = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Byte[] toArray () {
         Byte[] obr = new Byte[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Byte(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public byte[]   toArrayP() {return (byte[]) r.clone();}
   } // private class ListOfByte
   // =====================================================================
   public static CharList asList(char[] r) {
      return new ListOfChar(r);
   } // asList fuer char[]
   // ---------------------------------------------------------------------
   public static interface CharList extends List<Character> {
      boolean containsAllP(char[] suchReihung);
      boolean    containsP(char   wert);
      char            getP(int    index);
      int         indexOfP(char   wert);
      char            setP(int    index, char neuerWert);
      char[]      toArrayP();
   } // interface CharList
   // ---------------------------------------------------------------------
   private static class ListOfChar extends AbstractList<Character>
      implements CharList, RandomAccess, Serializable
   {
      private char[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfChar(char[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfChar
      // ------------------------------------------------------------------
      public boolean  containsAllP(char[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean   contains (Character o){return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean   containsP(char   w)   {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Character get      (int index)  {return new Character(r[index]);}
      // ------------------------------------------------------------------
      public char      getP     (int index)  {return               r[index] ;}
      // ------------------------------------------------------------------
      public int       size     ()           {return r.length;}
      // ------------------------------------------------------------------
      public int       indexOf (Character o) {
         char such = ((Character) o).charValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(char w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Character set (int index, Character neuesObjekt) {
          Character altesObjekt = new Character(r[index]);
          r[index] = neuesObjekt.charValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public char      setP(int index, char neuerWert) {
         char alterWert = r[index];
         r[index]       = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Character[] toArray () {
         Character[] obr = new Character[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Character(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public char[]   toArrayP() {return (char[]) r.clone();}
   } // private class ListOfChar
   // =====================================================================
   public static ShortList asList(short[] r) {
      return new ListOfShort(r);
   } // asList fuer short[]
   // ---------------------------------------------------------------------
   public static interface ShortList extends List<Short> {
      boolean containsAllP(short[] suchReihung);
      boolean    containsP(short   wert);
      short           getP(int     index);
      int         indexOfP(short   wert);
      short           setP(int     index, short neuerWert);
      short[]     toArrayP();
   } // interface ShortList
   // ---------------------------------------------------------------------
   private static class ListOfShort extends AbstractList<Short>
      implements ShortList, RandomAccess, Serializable
   {
      private short[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfShort(short[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfShort
      // ------------------------------------------------------------------
      public boolean  containsAllP(short[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Short o)   {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(short  w)  {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Short    get      (int index) {return new Short(r[index]);}
      // ------------------------------------------------------------------
      public short    getP     (int index) {return           r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Short o) {
         short such = o.shortValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(short w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Short   set (int index, Short neuesObjekt) {
          Short altesObjekt = new Short(r[index]);
          r[index] = neuesObjekt.shortValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public short      setP(int index, short neuerWert) {
         short alterWert = r[index];
         r[index]        = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Short[] toArray () {
         Short[] obr = new Short[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Short(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public short[]    toArrayP() {return (short[]) r.clone();}
   } // private class ListOfShort
   // =====================================================================
   public static IntList asList(int[] a) {
      return new ListOfInteger(a);
   } // asList fuer int[]
   // ---------------------------------------------------------------------
   public static interface IntList extends List<Integer> {
      boolean containsAllP(int[] suchReihung);
      boolean    containsP(int   wert);
      int             getP(int   index);
      int         indexOfP(int   wert);
      int             setP(int   index, int neuerWert);
      int[]       toArrayP();
   } // interface IntList
   // ---------------------------------------------------------------------
   private static class ListOfInteger extends AbstractList<Integer>
      implements IntList, RandomAccess, Serializable
   {
      private int[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfInteger(int[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfInteger
      // ------------------------------------------------------------------
      public boolean  containsAllP(int[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Integer o) {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(int    w)  {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Integer  get      (int index) {return new Integer(r[index]);}
      // ------------------------------------------------------------------
      public int      getP     (int index) {return             r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Integer o) {
         int such = o.intValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(int w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Integer   set (int index, Integer neuesObjekt) {
          Integer altesObjekt = new Integer(r[index]);
          r[index] = neuesObjekt.intValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public int      setP(int index, int neuerWert) {
         int alterWert = r[index];
         r[index]      = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Integer[] toArray() {
         Integer[] obr = new Integer[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Integer(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public int[]    toArrayP   () {return (int[]) r.clone();}
      // ------------------------------------------------------------------
   } // private class ListOfInteger
   // =====================================================================
   public static LongList asList(long[] r) {
      return new ListOfLong(r);
   } // asList fuer long[]
   // ---------------------------------------------------------------------
   public static interface LongList extends List<Long> {
      boolean containsAllP(long[] suchReihung);
      boolean    containsP(long   wert);
      long            getP(int    index);
      int         indexOfP(long   wert);
      long            setP(int    index, long neuerWert);
      long[]      toArrayP();
   } // interface LongList
   // ---------------------------------------------------------------------
   private static class ListOfLong extends AbstractList<Long>
      implements LongList, RandomAccess, Serializable
   {
      private long[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfLong(long[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfLong
      // ------------------------------------------------------------------
      public boolean  containsAllP(long[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Long o)  {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(long   w)  {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Long     get      (int index) {return new Long(r[index]);}
      // ------------------------------------------------------------------
      public long     getP     (int index) {return          r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Long o) {
         long such = o.longValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(long w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Long   set (int index, Long neuesObjekt) {
          Long altesObjekt = new Long(r[index]);
          r[index] = neuesObjekt.longValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public long      setP(int index, long neuerWert) {
         long alterWert = r[index];
         r[index]       = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Long[] toArray() {
         Long[] obr = new Long[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Long(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public long[]   toArrayP() {return (long[]) r.clone();}
   } // private class ListOfLong
   // =====================================================================
   public static FloatList asList(float[] r) {
      return new ListOfFloat(r);
   } // asList fuer float[]
   // ---------------------------------------------------------------------
   public static interface FloatList extends List<Float> {
      boolean containsAllP(float[] suchReihung);
      boolean    containsP(float   wert);
      float           getP(int     index);
      int         indexOfP(float   wert);
      float           setP(int     index, float neuerWert);
      float[]     toArrayP();
   } // interface FloatList
   // ---------------------------------------------------------------------
   private static class ListOfFloat extends AbstractList<Float>
      implements FloatList, RandomAccess, Serializable
   {
      private float[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfFloat(float[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfFloat
      // ------------------------------------------------------------------
      public boolean  containsAllP(float[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Float o)  {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(float  w)  {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Float    get      (int index) {return new Float(r[index]);}
      // ------------------------------------------------------------------
      public float    getP     (int index) {return          r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Float o) {
         float such = o.floatValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(float w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Float   set (int index, Float neuesObjekt) {
          Float altesObjekt = new Float(r[index]);
          r[index] = neuesObjekt.floatValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public float      setP   (int index, float neuerWert) {
         float alterWert = r[index];
         r[index]        = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Float[] toArray() {
         Float[] obr = new Float[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Float(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public float[]  toArrayP() {return (float[]) r.clone();}
   } // private class ListOfFloat
   // =====================================================================
   public static DoubleList asList(double[] r) {
      return new ListOfDouble(r);
   } // asList fuer double[]
   // ---------------------------------------------------------------------
   public static interface DoubleList extends List<Double> {
      boolean containsAllP(double[] suchReihung);
      boolean    containsP(double   wert);
      double          getP(int      index);
      int         indexOfP(double   wert);
      double          setP(int      index, double neuerWert);
      double[]    toArrayP();
   } // interface DoubleList
   // ---------------------------------------------------------------------
   private static class ListOfDouble extends AbstractList<Double>
      implements DoubleList, RandomAccess, Serializable
   {
      private double[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfDouble(double[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfDouble
      // ------------------------------------------------------------------
      public boolean  containsAllP(double[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Double o)  {return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(double w)  {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Double   get      (int index) {return new Double(r[index]);}
      // ------------------------------------------------------------------
      public double   getP     (int index) {return            r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Double o) {
         double such = o.doubleValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(double w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Double   set (int index, Double neuesObjekt) {
          Double altesObjekt = new Double(r[index]);
          r[index] = neuesObjekt.doubleValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public double   setP(int index, double neuerWert) {
         double alterWert = r[index];
         r[index]         = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Double[] toArray() {
         Double[] obr = new Double[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Double(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public double[] toArrayP() {return (double[]) r.clone();}
   } // private class ListOfDouble
   // =====================================================================
   public static BooleanList asList(boolean[] r) {
      return new ListOfBoolean(r);
   } // asList fuer boolean[]
   // ---------------------------------------------------------------------
   public static interface BooleanList extends List<Boolean> {
      boolean containsAllP(boolean[] suchReihung);
      boolean    containsP(boolean   wert);
      boolean         getP(int       index);
      int         indexOfP(boolean   wert);
      boolean         setP(int       index, boolean neuerWert);
      boolean[]   toArrayP();
   } // interface BooleanList
   // ---------------------------------------------------------------------
   private static class ListOfBoolean extends AbstractList<Boolean>
      implements BooleanList, RandomAccess, Serializable
   {
      private boolean[] r; // Wichtigstes (und einziges) Attribut!
      // ------------------------------------------------------------------
      ListOfBoolean(boolean[] r) {
         if (r==null) throw new NullPointerException();
         this.r = r;
      } // Konstruktor ListOfBoolean
      // ------------------------------------------------------------------
      public boolean  containsAllP(boolean[] suchReihung) {
         for (int i=0; i<suchReihung.length; i++) {
            if (! containsP(suchReihung[i])) return false;
         }
         return true;
      } // containsAll
      // ------------------------------------------------------------------
      public boolean  contains (Boolean  o){return indexOf (o) != -1;}
      // ------------------------------------------------------------------
      public boolean  containsP(boolean w) {return indexOfP(w) != -1;}
      // ------------------------------------------------------------------
      public Boolean  get      (int index) {return new Boolean(r[index]);}
      // ------------------------------------------------------------------
      public boolean  getP     (int index) {return             r[index] ;}
      // ------------------------------------------------------------------
      public int      size     ()          {return r.length;}
      // ------------------------------------------------------------------
      public int      indexOf (Boolean o) {
         boolean such = o.booleanValue();
         return indexOfP(such);
      } // indexOf
      // ------------------------------------------------------------------
      public int      indexOfP(boolean w) {
         for (int i=0; i<r.length; i++) {
            if (w == r[i]) return i;
         }
         return -1;
      } // indexOfP
      // ------------------------------------------------------------------
      public Boolean   set (int index, Boolean neuesObjekt) {
          Boolean altesObjekt = new Boolean(r[index]);
          r[index] = neuesObjekt.booleanValue();
          return altesObjekt;
      } // set
      // ------------------------------------------------------------------
      public boolean  setP(int index, boolean neuerWert) {
         boolean alterWert = r[index];
         r[index]          = neuerWert;
         return alterWert;
      } // setP
      // ------------------------------------------------------------------
      public Boolean[] toArray () {
         Boolean[] obr = new Boolean[r.length];
         for (int i=0; i<r.length; i++) {
            obr[i] = new Boolean(r[i]);
         }
         return obr;
      } // toArray
      // ------------------------------------------------------------------
      public boolean[]toArrayP() {return (boolean[]) r.clone();}
   } // private class ListOfBoolean
   // =====================================================================
   // Zum Schluss die main-Methode der Klasse ArraysP:

   static public void main(String[] sonja) {
      pln("--------------------------------------------------------------");
      pln("Den Modul ArraysP kann man mit dem Programm ArraysPTst testen!");
      pln("--------------------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
   // Damit niemand aus Versehen (leere) Objekte dieser Klasse erzeugt:
   private ArraysP() {} // Ein privater Konstruktor
   // ---------------------------------------------------------------------
 } // class ArraysP