// Datei Aufzaehlungen02.java
/* ------------------------------------------------------------------------
Ein besonders schlichter Aufzaehlungstyp namens FarbeA wird vereinbart,
alle Objekte dieses Typs werden ausgegeben, Strings mit FarbeA-Namen
werden in FarbeA-Objekte umgerechnet und FarbeA-Objekte werden mit
einer switch-Anweisung bearbeitet.
---------------------------------------------------------------------------
Wenn der Ausfuehrer die enum-Klasse FarbeA erzeugt, erzeugt er auch
4 FarbeA-Objekte (namens rot, gruen, blau und farblos). Der Ausfuehrer
garantiert, dass er danach keine weiteren FarbeA-Objekte erzeugen wird
(auch wenn der Programmierer es im befehlen sollte).

Die Klassenmethode values (Aufruf: FarbeA.values()) liefert eine Reihung,
die alle 4 FarbeA-Objekte enthaelt.

Die Objektmethode ordinal (Aufruf: f.ordinal(), mit f vom Typ FarbeA)
liefert die Ordinalzahl des Objekts f. Das ist der Index, an der dieses
Objekt in der Reihung FarbeA.values() steht.
------------------------------------------------------------------------ */
import java.util.Arrays;
// Die Vereinbarung des enum-Typs FarbeA ist nur eine Zeile lang:
enum FarbeA {rot, gruen, blau, farblos;} // Es gibt genau 4 FarbeA-Objekte
// ------------------------------------------------------------------------
// Die Klasse Aufzaehlungen02 enthaelt nur eine main-Methode. Sie dient
// zum Testen der Klasse FarbeA:
class Aufzaehlungen02 {
   static public void main(String [] sonja) {
      printf("Aufzaehlungen02: Jetzt geht es los!%n");
      printf("----------------------------------%n");
      printf("Alle Farben und ihre Daten:%n%n");
      printf("Name   Ordinalzahl%n");
      for (FarbeA f: FarbeA.values()) {
         printf("%-8s",  f);           // Breite 8 Zeichen, linksbuendig
         printf("%5d%n", f.ordinal()); // Breite 5 Zeichen, rechtsbuendig
      }

      // Alle FarbeA-Namen als Strings:
      String[] sr = {"rot", "gruen", "blau", "farblos"};

      printf("----------------------------------%n");
      printf("Strings in Farb-Objekte umrechnen:%n%n");
      for (String s: sr) {
         printf("FarbeA.valueOf(\"%-7s\"): ", s);
         printf("%s%n", FarbeA.valueOf(s));
      }

      // Eine Reihung mit einer "wilden Folge von Farben":
      FarbeA[] fr = {FarbeA.blau, FarbeA.rot, FarbeA.rot, FarbeA.farblos};
      // Hinweis: Hier muss man FarbeA.blau (und darf nicht nur blau)
      // schreiben. In einer switsch-Anweisung, nach case, muss man
      // blau (und darf nicht FarbeA.blau) schreiben.

      printf("----------------------------------%n");
      printf("Eine switch-Anweisung mit Farben:%n%n");
      for (FarbeA f: fr) {
         printf("%-9s", f); // Die FarbeA f in der Breite 9 ausgeben
         switch(f) {
            case rot:   printf("Ziemlich grell!%n");     break;
            case gruen: printf("Sehr natuerlich!%n");    break;
            case blau:  printf("Ein Zustand?%n");        break;
            default:    printf("Ein bisschen blass!%n"); break;
         } // switch
      } // for
      printf("----------------------------------%n");
      printf("Aufzaehlungen02: Das war's erstmal!%n");
      FarbeA[] frA = FarbeA.values();
      frA[0] = frA[1];
      printf("%s%n", Arrays.asList(frA));
      printf("%s%n", Arrays.asList(FarbeA.values()));
      FarbeA frB[] = (FarbeA[]) frA.clone();

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // Aufzaehlungen02
/* ------------------------------------------------------------------------
Ausgabe des Programms Aufzaehlungen02:

Aufzaehlungen02: Jetzt geht es los!
----------------------------------
Alle Farben und ihre Daten:

Name   Ordinalzahl
rot         0
gruen       1
blau        2
farblos     3
----------------------------------
Strings in Farb-Objekte umrechnen:

FarbeA.valueOf("rot"):     rot
FarbeA.valueOf("gruen"):   gruen
FarbeA.valueOf("blau"):    blau
FarbeA.valueOf("farblos"): farblos
----------------------------------
Eine switch-Anweisung mit Farben:

blau     Ein Zustand?
rot      Ziemlich grell!
rot      Ziemlich grell!
farblos  Ein bisschen blass!
----------------------------------
Aufzaehlungen02: Das war's erstmal!
------------------------------------------------------------------------ */
