// Datei Aufzaehlungen04.java
/* ------------------------------------------------------------------------
Ein Aufzaehlungstyp namens FarbeC wird vereinbart. Die Klasse FarbeC
hat einen Konstruktor und jedes FarbeC-Objekt hat ein Attribut namens
uebersetzung vom Typ String.

Die Objekte der Klasse FarbeC sind natuerlich geordnet (ueber die
Methode compareTo der Schnittstelle Comparable). Diese Methode
compareTo ist final, kann also nicht ueberschrieben werden.
Man kann aber weitere Ordnungen ueber die Schnittstelle Comparator
und ihre Methode compare vereinbaren, wie in diesem Beispiel.
------------------------------------------------------------------------ */
import java.util.List;       // Eine Sammlungsschnittstelle
import java.util.TreeSet;    // Eine Klasse *sortierter* Sammlungen
import java.util.Arrays;     // Fuer asList (Reihung in Liste umwandeln)
import java.util.Comparator; // Zusaetzliche Ordnung

enum FarbeC implements Comparable<FarbeC>, Comparator<FarbeC> {
   // Die Angabe "implements Comparable<FarbeC>" ist hier nur
   // eine Art Kommentar und koennte auch weggelassen werden.
   rot ("red"), gruen  ("green"),
   blau("blue"),farblos("colourless");

   public final String uebersetzung;
   FarbeC (String uebersetzung) {
      this.uebersetzung = uebersetzung;
   } // Konstruktor FarbeC

   // Eine compare-Methode, mit der man FarbeC-Objekte nach ihrer
   // uebersetzung aufsteigend sortieren lassen kann:
   public int compare(FarbeC f1, FarbeC f2) {
      return f1.uebersetzung.compareTo(f2.uebersetzung);
   } // compare ("zusaetzliche Ordnungsmethode")

} // enum FarbeC
// ------------------------------------------------------------------------
// Die Klasse Aufzaehlungen04 dient zum Testen der Klasse FarbeC:
class Aufzaehlungen04 {

   // Eine compare-Methode, mit der man FarbeC-Objekte nach ihrer
   // uebersetzung aufsteigend sortieren lassen kann:
   static class FC_UebersetzungAuf implements Comparator<FarbeC> {
      public int compare(FarbeC f1, FarbeC f2) {
         return f1.uebersetzung.compareTo(f2.uebersetzung);
      } // compare ("zusaetzliche Ordnungsmethode")
   } // class FC_UebersetzungAuf

   // Eine compare-Methode, mit der man FarbeC-Objekte nach ihrer
   // uebersetzung absteigend sortieren lassen kann:
   static class FC_UebersetzungAb implements Comparator<FarbeC> {
      public int compare(FarbeC f1, FarbeC f2) {
         return -(f1.uebersetzung.compareTo(f2.uebersetzung));
      } // compare ("zusaetzliche Ordnungsmethode")
   } // class FC_UebersetzungAb

   static public void main(String [] sonja) {
      printf("Aufzaehlungen04: Jetzt geht es los!%n");
      printf("---------------------------------%n");
      printf("Alle Farben und ihre Daten:%n%n");
      printf("Name   Ordinalzahl   Uebersetzung%n");
      for (FarbeC f: FarbeC.values()) {
         printf("%-8s",   f);
         printf("%5d",    f.ordinal());
         printf("%20s%n", f.uebersetzung);
      }
      printf("---------------------------------%n");
      // Ein Compara
      FC_UebersetzungAuf aufsteigendNachUeb = new FC_UebersetzungAuf();
      FC_UebersetzungAb   absteigendNachUeb = new FC_UebersetzungAb ();

      // Aus der Reihung FarbeC.values() aller FarbeC-Objekte zwei
      // sortierte Sammlungen sosA und sosB erzeugen:
      List   <FarbeC> list = Arrays.asList(FarbeC.values());

      TreeSet<FarbeC> sosA = new TreeSet<FarbeC>(); // Natuerlich geordnet
      TreeSet<FarbeC> sosB = new TreeSet<FarbeC>(list.get(0));
      TreeSet<FarbeC> sosC = new TreeSet<FarbeC>(aufsteigendNachUeb);
      TreeSet<FarbeC> sosD = new TreeSet<FarbeC>( absteigendNachUeb);

      sosA.addAll(list);
      sosB.addAll(list);
      sosC.addAll(list);
      sosD.addAll(list);

      printf("sosA: %s%n", sosA); // Natuerlich geordnet (nach Ordinalzahl)
      printf("sosB: %s%n", sosB); // Aufsteigend nach uebersetzung
      printf("sosC: %s%n", sosC); // Aufsteigend nach uebersetzung
      printf("sosD: %s%n", sosD); //  Absteigend nach uebersetzung
      printf("---------------------------------%n");
      printf("Aufzaehlungen04: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // Aufzaehlungen04
/* ------------------------------------------------------------------------
Ausgabe des Programms Aufzaehlungen04:

Aufzaehlungen04: Jetzt geht es los!
---------------------------------
Alle Farben und ihre Daten:

Name   Ordinalzahl   Uebersetzung
rot         0                 red
gruen       1               green
blau        2                blue
farblos     3          colourless
---------------------------------
sosA: [rot, gruen, blau, farblos]
sosB: [blau, farblos, gruen, rot]
sosC: [blau, farblos, gruen, rot]
sosD: [rot, gruen, farblos, blau]
---------------------------------
Aufzaehlungen04: Das war's erstmal!
------------------------------------------------------------------------ */
