// Datei Aufzaehlungen06.java
/* ------------------------------------------------------------------------
Wenn der Programmierer einen Aufzaehlungtyp vereinbart, z.B. so:

enum FarbeA {rot, gruen, blau, farblos;}

erzeugt der Ausfuehrer eine entsprechende Aufzaehlungsklasse namens
FarbeA als Erweiterung der Klasse Enum. Der Programmierer kann solche
Aufzaehlungsklassen nicht selbst vereinbaren, hier wird aber trotzdem
der Versuch gemacht. Die diversen Fehlermeldungen, mit denen der Aus-
fuehrer diesen Versuch ablehnt, sind unten als Kommentar wiedergegeben.
------------------------------------------------------------------------ */
class FarbeA extends Enum<FarbeA> implements Comparable<FarbeA> {
   // Private Objektattribute und oeffentliche getter-Methoden:
   private String objektName;
   private int    ordinalzahl;
   public  String name   () {return objektName;}
   public  int    ordinal() {return ordinalzahl;}

   // Implementierung von Comparable<FarbeA>:
   public int compareTo(FarbeA that) {
      return this.ordinalzahl - that.ordinalzahl;
   } // compareTo ("natuerliche Ordnungsmethode")

   // Ein einziger privater Konstruktor:
   private FarbeA(String name, int ordinalzahl) {
      this.name        = name;
      this.ordinalzahl = ordinalzahl;
   } // Konstruktor FarbeA

   // ffentliche Klassenattribute:
   static public  FarbeA rot     = new FarbeA("rot",      0);
   static public  FarbeA gruen   = new FarbeA("gruen",    1);
   static public  FarbeA blau    = new FarbeA("blau",     2);
   static public  FarbeA farblos = new FarbeA("farblos",  3);

   // Ein priv. Klassenatt. r mit oeffentlicher getter-Methode:
   static private FarbeA[] r     = {rot, gruen, blau, farblos};
   static public  FarbeA[] values() {return (FarbeA[])r.clone();}

   static public  FarbeA valueOf(String name) {
      for (FarbeA f: r) {
         if (name == f.name) return f;
      }
      throw new IllegalArgumentException(name + "ist kein FarbeA-Objekt");
   }
} // class FarbeA
/* ------------------------------------------------------------------------
Fehlermeldungen des Compilers:

D:\Java\Aufzaehlungen06.java:17: name() in FarbeA cannot override
name() in java.lang.Enum; overridden method is final
   public  String name   () {return objektName;}
                  ^
D:\Java\Aufzaehlungen06.java:18: ordinal() in FarbeA cannot override
ordinal() in java.lang.Enum; overridden method is final
   public  int    ordinal() {return ordinalzahl;}
                  ^
D:\Java\Aufzaehlungen06.java:27: name has private access in java.lang.Enum
      this.name        = name;
      ^
D:\Java\Aufzaehlungen06.java:32: enum types may not be instantiated
   static public  FarbeA rot     = new FarbeA("rot",      0);
                                   ^
D:\Java\Aufzaehlungen06.java:33: enum types may not be instantiated
   static public  FarbeA gruen   = new FarbeA("gruen",    1);
                                   ^
D:\Java\Aufzaehlungen06.java:34: enum types may not be instantiated
   static public  FarbeA blau    = new FarbeA("blau",     2);
                                   ^
D:\Java\Aufzaehlungen06.java:35: enum types may not be instantiated
   static public  FarbeA farblos = new FarbeA("farblos",  3);
                                   ^
D:\Java\Aufzaehlungen06.java:43: name has private access in java.lang.Enum
         if (name == f.name) return f;
                      ^
------------------------------------------------------------------------ */
