// Datei Ausnahmen04.java
/* ------------------------------------------------------------------------
Jede Ausnahme kann eine andere Ausnahme als Ursache (cause) haben und
diese Ursache kann eine weitere Ausnahme als Ursache haben etc.
Somit koennen Ausnahmen Ketten beliebiger Laenge bilden. Die Ursache
einer Ausnahme kann man mit einem geeigneten Konstruktor oder mit der
Methode initCause festlegen, etwa so:

String              text1     = "Betrag ist negativ!");
String              text2     = "Die Rechnung ist falsch!");
Throwable           ursache   = new Throwable(text1);
Throwable           ausnahme1 = new Throwable(text2, ursache);

ArithmeticException ausnahme2 = new ArithmeticException(text);
ausnahme2.initCause(ursache);

Die Funktion getCause liefert die Ursache einer Ausnahme
(z.B. liefert der Aufruf ausnahme1.getCause() die Ausnahme ursache).

Siehe auch Ausnahmen03.
------------------------------------------------------------------------ */
class Ausnahmen04 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Ruft die gefaehrliche Methode metA auf. Falls dadurch eine
      // Ausnahme geworfen wird, wird sie hier gefangen und behandelt.

      pln("Ausnahmen04: Jetzt geht es los!\n");

      try {
         metA(12, 3);
         metA(17, 0);
         metA(14, 2);
      } catch (Error ex) {
         bearbeiteAusnahmeobjekt(ex);
      } // try-catch-Befehl

   } // main
   // ---------------------------------------------------------------------
   static void metA(int a, int b) {
      // Ruft die gefaehrliche Methode metB auf. Faengt die dadurch
      // geworfene Ausnahme alt und wirft eine Ausnahme neu
      // (mit alt als Ursache von neu)
      try {
         metB(a, b);
      } catch (Error alt) {
         String text = "Fehler in metA!";
         Error neu = new Error(text);
         neu.initCause(alt);
         throw neu;
      } // try-catch
   } // metA
   // ---------------------------------------------------------------------
   static void metB(int a, int b) {
      // Ruft die gefaehrliche Methode metC auf. Faengt die dadurch
      // geworfene Ausnahme alt und wirft eine Ausnahme neu
      // (mit alt als Ursache von neu)
      try {
         metC(a, b);
      } catch (ArithmeticException alt) {
         String text = "Fehler in metB!";
         Error neu = new Error(text);
         neu.initCause(alt);
         throw neu;
      } // try-catch
   } // metB
   // ---------------------------------------------------------------------
   static void metC(int a, int b) {
      // Gibt auf jeden Fall eine kleine Meldung ("A ... ist") aus.
      // Wirft ein Ausnahme, falls b gleich 0 ist.
      p("A " + a + " / " + b + " ist ");
      if (b == 0) {
         throw new ArithmeticException("Divisor 0!");
      }
      pln("gleich " + a/b);
   } // metC
   // ---------------------------------------------------------------------
   static void bearbeiteAusnahmeobjekt(Throwable ausnahme) {
      // Gibt von der ausnahme folgende Informationen aus:
      // B: Den (unvollstaendigen) StackTrace (mit printStackTrace)
      // C: Die Kette aller Ursachen
      // D: Den (vollstaendigen) StackTrace (selbst programmiert)
      pln();
      pln("---------------------------------------------------------");
      pln("B Der (unvollstaendige) StackTrace (mit printStackTrace):\n");
      ausnahme.printStackTrace();

      pln("---------------------------------------------------------");
      pln("C Die Kette der Ursachen dieser Ausnahme:\n");
      Throwable aus = ausnahme;  // Referenz ausnahme kopieren.
      while (aus != null) {
         pln("  " + aus);
         aus = aus.getCause();   // Kopie wird veraendert!
      } // while

      pln("---------------------------------------------------------");
      pln("D Der vollstaendige StackTrace ('selbst programmiert'):\n");
      aus = ausnahme;            // Referenz ausnahme kopieren.

      gibAusnahmeAus("  Ausnahme: ", aus);

      while (true) {
         aus = aus.getCause();   // Kopie wird veraendert!
         if (aus == null) break;
         gibAusnahmeAus("  Ursache:  ", aus);
      } // while

      pln("---------------------------------------------------------");
   } // bearbeiteAusnahmeobjekt
   // ---------------------------------------------------------------------
   static void gibAusnahmeAus(String text, Throwable ausOb) {
      // Gibt die Ausnahme ausOb und ihren StackTrace aus (verziert
      // mit dem text).
      pln(text + ausOb);
      StackTraceElement[] ster = ausOb.getStackTrace();
      for (int i=0; i<ster.length; i++) {
         pln("      in " + ster[i]);
      }
   } // gibAus
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Ausnahmen04
/* ------------------------------------------------------------------------
Ausgabe des Programms Ausnahmen04:

Ausnahmen04: Jetzt geht es los!

A 12 / 3 ist gleich 4
A 17 / 0 ist
---------------------------------------------------------
B Der (unvollstaendige) StackTrace (mit printStackTrace):

java.lang.Error: Fehler in metA!
    at Ausnahmen04.metA(Ausnahmen04.java:46)
    at Ausnahmen04.main(Ausnahmen04.java:30)
Caused by: java.lang.Error: Fehler in metB!
    at Ausnahmen04.metB(Ausnahmen04.java:60)
    at Ausnahmen04.metA(Ausnahmen04.java:43)
    ... 1 more
Caused by: java.lang.ArithmeticException: Divisor 0!
    at Ausnahmen04.metC(Ausnahmen04.java:71)
    at Ausnahmen04.metB(Ausnahmen04.java:57)
    ... 2 more
---------------------------------------------------------
C Die Kette der Ursachen dieser Ausnahme:

  java.lang.Error: Fehler in metA!
  java.lang.Error: Fehler in metB!
  java.lang.ArithmeticException: Divisor 0!
---------------------------------------------------------
D Der vollstaendige StackTrace ('selbst programmiert'):

  Ausnahme: java.lang.Error: Fehler in metA!
      in Ausnahmen04.metA(Ausnahmen04.java:46)
      in Ausnahmen04.main(Ausnahmen04.java:30)
  Ursache:  java.lang.Error: Fehler in metB!
      in Ausnahmen04.metB(Ausnahmen04.java:60)
      in Ausnahmen04.metA(Ausnahmen04.java:43)
      in Ausnahmen04.main(Ausnahmen04.java:30)
  Ursache:  java.lang.ArithmeticException: Divisor 0!
      in Ausnahmen04.metC(Ausnahmen04.java:71)
      in Ausnahmen04.metB(Ausnahmen04.java:57)
      in Ausnahmen04.metA(Ausnahmen04.java:43)
      in Ausnahmen04.main(Ausnahmen04.java:30)
---------------------------------------------------------
------------------------------------------------------------------------ */