// Datei Ausnahmen06.java
/* ------------------------------------------------------------------------
Die Methode met1 soll deutlich machen:

Finally-Regel: Wenn der Ausfuehrer einen try-Block betritt, dann betritt
er garantiert auch den zugehoerigen finally-Block (falls vorhanden).

Die Finally-Regel gilt auch in den folgenden Faellen:
1. Der try-Block befindet sich im Rumpf einer Schleife und wird
   mit einem break-Befehl beendet.
2. Der try-Block befindet sich im Rumpf einer Methode und wird
   mit einem return-Befehl beendet.
3. Der try-Block wird auf Grund einer Ausnahme abgebrochen, fuer
   die es keinen zugehoerigen catch-Block gibt.
4. Der try-Block wird auf Grund einer Ausnahme abgebrochen und der
   zugehoerige catch-Block wird mit einem break-Befehl beendet.
5. Der try-Block wird auf Grund einer Ausnahme abgebrochen und der
   zugehoerige catch-Block wird mit einem return-Befehl beendet.
6. Der try-Block wird auf Grund einer Ausnahme abgebrochen und der
   zugehoerige catch-Block wird ebenfalls auf Grund einer Ausnahme
   abgebrochen.

Die Finally-Regel gilt nicht in den folgenden beiden Faellen:
7. Der try-Block wird auf Grund einer Ausnahme abgebrochen und der
   zugehoerige catch-Block wird durch einen System.exit-Befehl beendet.
8. Der try-Block           wird durch einen System.exit-Befehl beendet
   (dieser Fall tritt hier nie ein, weil das Programm schon vorher
   durch 6. beendet wird).
------------------------------------------------------------------------ */
class Ausnahmen06 {
   // ---------------------------------------------------------------------
   static void met1(int n) {
      // Diese Methode enthaelt eine Schleife und die enthaelt einen
      // try-catch-finally-Befehl. In Abhaengigkeit vom Parameter n
      // wird der try-Block bzw. der catch-Bloch "auf alle moegliche
      // Weisen" beendet: mit break, return, throw bzw. exit.
      // Der finally-Block wird immer ausgefuehrt (ausser wenn das
      // Programm vorher mit System.exit beendet wird).
      while (true) {
         try {
            pln("----------------------------------------------");
            pln("A try-Block wurde betreten!     n ist gleich " + n);
            if (n==1) {pln("B break  wird ausgefuehrt!"); break;            }
            if (n==2) {pln("C return wird ausgefuehrt!"); return;           }
            if (n==3) {pln("D throw  wird ausgefuehrt!"); throw new Error();}
            if (n==8) {pln("E exit   wird ausgefuehrt!"); System.exit(1);   }
            throw new ArithmeticException();
         } catch (ArithmeticException ex01) {
            if (n==4) {pln("F break  wird ausgefuehrt!"); break;            }
            if (n==5) {pln("G return wird ausgefuehrt!"); return;           }
            if (n==6) {pln("H throw  wird ausgefuehrt!"); throw new Error();}
            if (n==7) {pln("I exit   wird ausgefuehrt!"); System.exit(1);   }
         } finally {
            // Dieser finally-Block wird (fast) immer ausgefuehrt:
            pln("J finally-Block wurde betreten, n ist gleich " + n);
         } // try-catch-finally-Befehl
      } // while
   } // met1
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Ruft die Methode met1 mit verschiedenen Parametern auf.

      pln("Ausnahmen06: Jetzt geht es los!\n");

      for (int n=1; n<=8; n++) {
         // Der Aufruf met1(5) wirft eine Ausnahme. Die wird hier
         // gefangen und mit einem leeren catch-Block behandelt
         // (damit das Programm nicht abgebrochen wird)
         try {
            met1(n);
         } catch (Throwable t) {
            // leerer catch-Block!
         } // try-catch-Befehl
      } // for
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Ausnahmen06
/* ------------------------------------------------------------------------
Ausgabe des Programms Ausnahmen06:

Ausnahmen06: Jetzt geht es los!

----------------------------------------------
A try-Block wurde betreten!     n ist gleich 1
B break  wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 1
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 2
C return wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 2
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 3
D throw  wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 3
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 4
F break  wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 4
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 5
G return wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 5
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 6
H throw  wird ausgefuehrt!
J finally-Block wurde betreten, n ist gleich 6
----------------------------------------------
A try-Block wurde betreten!     n ist gleich 7
I exit   wird ausgefuehrt!
------------------------------------------------------------------------ */