// Datei Ausnahmen10.java
/* ------------------------------------------------------------------------
In dieser Datei werden vier Ausnahmeklassen und die Hauptklasse des
Programms Ausnahmen10 vereinbart.

Die vier Ausnahmeklassen bilden (zusammen mit der Klasse Exception)
folgenden Vererbungsbaum:

Exception
   |
   +--- ZahlIstNichtGutB
              |
              +--- ZahlIstNegativB
              |
              +--- ZahlIstNullB
              |
              +--- ZahlIstGeradeB

Alle vier Ausnahmeklassen sind geprueft, weil ihre (direkte bzw.
indirekte) Oberklasse Exception geprueft ist.

Siehe auch Ausnahmen10.
------------------------------------------------------------------------ */
class ZahlIstNichtGutB extends Exception{
   private int zahl; // Die nicht-gute Zahl

   ZahlIstNichtGutB(int zahl, String meldung) {
      super(meldung);
      this.zahl = zahl;
   } // Konstruktor ZahlIstNichtGutB

   int getZahl() {return zahl;}
} // ZahlIstNichtGutB
// ------------------------------------------------------------------------
class ZahlIstNegativB  extends ZahlIstNichtGutB{
   ZahlIstNegativB(int z) {super(z, z + " ist negativ!");}
} // ZahlIstNegativB
// ------------------------------------------------------------------------
class ZahlIstNullB     extends ZahlIstNichtGutB{
   ZahlIstNullB() {super(0, "Die Zahl ist gleich 0!");}
} // ZahlIstNullB
// ------------------------------------------------------------------------
class ZahlIstGeradeB   extends ZahlIstNichtGutB{
   ZahlIstGeradeB(int z) {super(z, z + " ist gerade!");}
} // ZahlIstGeradeB
// ------------------------------------------------------------------------
class Ausnahmen10 {
   // ---------------------------------------------------------------------
   static int liesGuteZahl() throws ZahlIstNichtGutB {
      // Liest einen String ein, versucht, ihn in einen int-Wert
      // umzuwandeln (evtl. NumberFormatException), wirft eine
      // Ausnahme, falls der int-Wert negativ, gleich 0 oder gerade
      // ist und liefert den eingelesenen int-Wert sonst als Eregebnis.
      String einS = EM.liesString();
      int    einI = Integer.decode(einS).intValue();
      if (einI <  0)     throw new ZahlIstNegativB(einI);
      if (einI == 0)     throw new ZahlIstNullB();
      if (einI % 2 == 0) throw new ZahlIstGeradeB(einI);
      return einI;
   } // liesGuteZahl
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Es wird wiederholt eine Ganzzahl eingelesen und kommentiert
      // wieder ausgegeben, bis der Benutzer eine 0 eingibt.
      while (true) {
         try {
            pln("----------------------------------- ");
            p  ("Eine gute Ganzzahl (0 zum Beenden)? ");
            int n = liesGuteZahl();
            pln(n + " ist tatsaechlich gut!");
         } catch (ZahlIstNullB aus) {
            break;
         } catch (ZahlIstNichtGutB aus) {
            pln("Die Zahl " + aus.getZahl() + " ist nicht gut!");
            pln("Genauer: " + aus.getMessage());
         } // try-catch
      }
      pln("Ausnahmen10 beendet sich ordnungsgemaess!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // Ausnahmen10
/* ------------------------------------------------------------------------
Ein Dialog mit dem Programm Ausnahmen10:

-----------------------------------
Eine gute Ganzzahl (0 zum Beenden)? 13
13 ist tatsaechlich gut!
-----------------------------------
Eine gute Ganzzahl (0 zum Beenden)? 12
Die Zahl 12 ist nicht gut!
Genauer: 12 ist gerade!
-----------------------------------
Eine gute Ganzzahl (0 zum Beenden)? -5
Die Zahl -5 ist nicht gut!
Genauer: -5 ist negativ!
-----------------------------------
Eine gute Ganzzahl (0 zum Beenden)? 0
Ausnahmen10 beendet sich ordnungsgemaess!
------------------------------------------------------------------------ */
