// Datei BigDecimal03.java
/* ------------------------------------------------------------------------
Einem Binaerbruch mit n Stellen nach dem Punkt entspricht ein
Dezimalbruch, der (erstaunlicherweise?) n Dezimalstellen nach dem
Punkt hat.

Dieses Programm versucht, diese Tatsache zu veranschaulichen,
indem es die Stellenwerte der Binaerziffern nach dem Punkt (d.h.
die Zahlen 1/2, 1/4, 1/8 etc.) als Tabelle ausgibt. Der Wert
eines Binaerbruchs ist eine Summe solcher Stellenwerte
(naemlich der Stellenwerte seiner 1-er-Bits).
------------------------------------------------------------------------ */
import java.math.BigDecimal;
import java.math.RoundingMode;

class BigDecimal03 {
   static public void main(String [] sonja) {
      printf("BigDecimal03: Jetzt geht es los!%n");
      printf("----------------------------------------------%n");
      final
      BigDecimal ZWEI         = new BigDecimal("2"); // Konstante 2
      BigDecimal zweierPotenz = new BigDecimal("2"); // Anfangs 2 hoch 1
      BigDecimal erg;

      for (int i=1; i <=20; i++) {
         erg = BigDecimal.ONE.divide(zweierPotenz, 30,
            RoundingMode.UNNECESSARY);
         printf("%2d. 1 / %-10s = %25.23f%n", i, zweierPotenz, erg);
         zweierPotenz = zweierPotenz.multiply(ZWEI);
      } // for i

      printf("----------------------------------------------%n");
      printf("BigDecimal03: Das war's erstmal!%n");
   } // main
   // ------------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ------------------------------------------------------------------------
} // class BigDecimal03
/* ------------------------------------------------------------------------
Ausgabe des Programms BigDecimal03:

BigDecimal03: Jetzt geht es los!
----------------------------------------------
 1. 1 / 2          = 0,50000000000000000000000
 2. 1 / 4          = 0,25000000000000000000000
 3. 1 / 8          = 0,12500000000000000000000
 4. 1 / 16         = 0,06250000000000000000000
 5. 1 / 32         = 0,03125000000000000000000
 6. 1 / 64         = 0,01562500000000000000000
 7. 1 / 128        = 0,00781250000000000000000
 8. 1 / 256        = 0,00390625000000000000000
 9. 1 / 512        = 0,00195312500000000000000
10. 1 / 1024       = 0,00097656250000000000000
11. 1 / 2048       = 0,00048828125000000000000
12. 1 / 4096       = 0,00024414062500000000000
13. 1 / 8192       = 0,00012207031250000000000
14. 1 / 16384      = 0,00006103515625000000000
15. 1 / 32768      = 0,00003051757812500000000
16. 1 / 65536      = 0,00001525878906250000000
17. 1 / 131072     = 0,00000762939453125000000
18. 1 / 262144     = 0,00000381469726562500000
19. 1 / 524288     = 0,00000190734863281250000
20. 1 / 1048576    = 0,00000095367431640625000
----------------------------------------------
BigDecimal03: Das war's erstmal!
------------------------------------------------------------------------ */