// Datei BigDecimal05.java
/* ------------------------------------------------------------------------
Demonstriert die sieben Moeglichkeiten, BigDecimal-Zahlen zu runden
(CEILING, UP, FLOOR,  DOWN, HALF_DOWN, HALF_EVEN und HALF_UP).
------------------------------------------------------------------------ */
import java.math.BigDecimal;
import java.math.RoundingMode;          // Ein Aufzaehlungstyp
import static java.math.RoundingMode.*; // Seine "Literale" sichtbar machen

class BigDecimal05 {
   // ---------------------------------------------------------------------
   public static void main(String[] S) {
      // Die Reihung br enthaelt BigDecimal-Zahlen, die mehrmals auf
      // verschiedene Weise gerundet und ausgegeben und dann um delta
      // erhoeht werden:
      BigDecimal br[] = {
         new BigDecimal("12.345"),
         new BigDecimal("-2.345"),
      };
      BigDecimal delta = new BigDecimal( "0.001");

      // Die gerundeten Versionen der jeweils aktuellen BigDecimal-Zahlen:
      BigDecimal dRound_Ceiling;
      BigDecimal dRound_Down;
      BigDecimal dRound_Floor;
      BigDecimal dRound_Half_Down;
      BigDecimal dRound_Half_Even;
      BigDecimal dRound_Half_Up;
      BigDecimal dRound_Up;

      // Die Ueberschriftzeilen ausgeben:
      printf("BigDecimal05: Die Zahl wird auf jeweils 8 verschiedene\n");
      printf("Weisen und immer auf 2 Nachpunktstellen gerundet:\n");
      printMinuszeile();
      printf(
         "Zahl       CEILING   UP        FLOOR     DOWN      " +
                    "HALF_DOWN HALF_EVEN HALF_UP\n"
      );
      printMinuszeile();
      // ------------------------------------------------------------------
      // Die BigDecimal-Zahlen in der Reihung br mehrmals auf verschiedene
      // Weise runden, ausgeben und dann um delta erhoehen:

      // 2 mal 20 Zeilen ausgeben:
      for (int j=0; j<br.length; j++) {
         BigDecimal bd = br[j];

         // 20 Zeilen ausgeben:
         for (int i=1; i<=20; i++) { // 20 Mal um delta erhoehen
            // CEILING ist hier ein Abkuerzung fuer
            // java.math.RoundingMode.CEILING. Fuer die anderen
            // Rundungsarten entsprechend.
            dRound_Ceiling   = bd.setScale(2, CEILING);
            dRound_Up        = bd.setScale(2, UP);
            dRound_Floor     = bd.setScale(2, FLOOR);
            dRound_Down      = bd.setScale(2, DOWN);
            dRound_Half_Down = bd.setScale(2, HALF_DOWN);
            dRound_Half_Even = bd.setScale(2, HALF_EVEN);
            dRound_Half_Up   = bd.setScale(2, HALF_UP);

            // Eine Zeile ausgeben:
            final String BLANKS = "     ";
            printf("%s\n",
               bd                 + BLANKS +
               dRound_Ceiling     + BLANKS +
               dRound_Up          + BLANKS +
               dRound_Floor       + BLANKS +
               dRound_Down        + BLANKS +
               dRound_Half_Down   + BLANKS +
               dRound_Half_Even   + BLANKS +
               dRound_Half_Up
            );
            bd = bd.add(delta); // Einen Schritt weiter
         } // for
         printMinuszeile();
      } // for
      // ------------------------------------------------------------------
      printf("BigDecimal05: Das war's erstmal!\n");
   } // main
   // ---------------------------------------------------------------------
   static void printMinuszeile() {
      printf("---------------------------------------" +
             "---------------------------------------\n");
   } // printMinuszeile
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class BigDecimal05

/* ------------------------------------------------------------------------
Die Ausgabe des Programms BigDecimal05:

BigDecimal05: Die Zahl wird auf jeweils 8 verschiedene
Weisen und immer auf 2 Nachpunktstellen gerundet:
------------------------------------------------------------------------------
Zahl       CEILING   UP        FLOOR     DOWN      HALF_DOWN HALF_EVEN HALF_UP
------------------------------------------------------------------------------
12.345     12.35     12.35     12.34     12.34     12.34     12.34     12.35
12.346     12.35     12.35     12.34     12.34     12.35     12.35     12.35
12.347     12.35     12.35     12.34     12.34     12.35     12.35     12.35
12.348     12.35     12.35     12.34     12.34     12.35     12.35     12.35
12.349     12.35     12.35     12.34     12.34     12.35     12.35     12.35
12.350     12.35     12.35     12.35     12.35     12.35     12.35     12.35
12.351     12.36     12.36     12.35     12.35     12.35     12.35     12.35
12.352     12.36     12.36     12.35     12.35     12.35     12.35     12.35
12.353     12.36     12.36     12.35     12.35     12.35     12.35     12.35
12.354     12.36     12.36     12.35     12.35     12.35     12.35     12.35
12.355     12.36     12.36     12.35     12.35     12.35     12.36     12.36
12.356     12.36     12.36     12.35     12.35     12.36     12.36     12.36
12.357     12.36     12.36     12.35     12.35     12.36     12.36     12.36
12.358     12.36     12.36     12.35     12.35     12.36     12.36     12.36
12.359     12.36     12.36     12.35     12.35     12.36     12.36     12.36
12.360     12.36     12.36     12.36     12.36     12.36     12.36     12.36
12.361     12.37     12.37     12.36     12.36     12.36     12.36     12.36
12.362     12.37     12.37     12.36     12.36     12.36     12.36     12.36
12.363     12.37     12.37     12.36     12.36     12.36     12.36     12.36
12.364     12.37     12.37     12.36     12.36     12.36     12.36     12.36
------------------------------------------------------------------------------
-2.345     -2.34     -2.35     -2.35     -2.34     -2.34     -2.34     -2.35
-2.344     -2.34     -2.35     -2.35     -2.34     -2.34     -2.34     -2.34
-2.343     -2.34     -2.35     -2.35     -2.34     -2.34     -2.34     -2.34
-2.342     -2.34     -2.35     -2.35     -2.34     -2.34     -2.34     -2.34
-2.341     -2.34     -2.35     -2.35     -2.34     -2.34     -2.34     -2.34
-2.340     -2.34     -2.34     -2.34     -2.34     -2.34     -2.34     -2.34
-2.339     -2.33     -2.34     -2.34     -2.33     -2.34     -2.34     -2.34
-2.338     -2.33     -2.34     -2.34     -2.33     -2.34     -2.34     -2.34
-2.337     -2.33     -2.34     -2.34     -2.33     -2.34     -2.34     -2.34
-2.336     -2.33     -2.34     -2.34     -2.33     -2.34     -2.34     -2.34
-2.335     -2.33     -2.34     -2.34     -2.33     -2.33     -2.34     -2.34
-2.334     -2.33     -2.34     -2.34     -2.33     -2.33     -2.33     -2.33
-2.333     -2.33     -2.34     -2.34     -2.33     -2.33     -2.33     -2.33
-2.332     -2.33     -2.34     -2.34     -2.33     -2.33     -2.33     -2.33
-2.331     -2.33     -2.34     -2.34     -2.33     -2.33     -2.33     -2.33
-2.330     -2.33     -2.33     -2.33     -2.33     -2.33     -2.33     -2.33
-2.329     -2.32     -2.33     -2.33     -2.32     -2.33     -2.33     -2.33
-2.328     -2.32     -2.33     -2.33     -2.32     -2.33     -2.33     -2.33
-2.327     -2.32     -2.33     -2.33     -2.32     -2.33     -2.33     -2.33
-2.326     -2.32     -2.33     -2.33     -2.32     -2.33     -2.33     -2.33
------------------------------------------------------------------------------
BigDecimal05: Das war's erstmal!
------------------------------------------------------------------------ */