// Datei BigInteger03.java
/* ------------------------------------------------------------------------
Die Klasse BigInteger03 ermoeglicht es, mit rationalen Zahlen von praktisch
beliebiger Groesse und Genauigkeit zu rechnen.
------------------------------------------------------------------------ */
import java.math.BigInteger;

class BigInteger03 {
   // ---------------------------------------------------------------------
   // Jedes Objekt dieser Klasse repraesentiert eine rationale Zahl, die
   // aus zwei Ganzzahlen (Zaehler und Nenner) besteht und einen Namen hat.
   // Wenn der Programmierer fuer eine Zahl keinen Namen festlegt, bekommt
   // sie automatisch einen der Namen rat1, rat2, rat3 ... .
   // ---------------------------------------------------------------------
   // Nummer fuer die automatisch erzeugten Namen:
   static private int nr = 1;
   // ---------------------------------------------------------------------
   private BigInteger zaehler;
   private BigInteger nenner;
   private String     name;
   // ---------------------------------------------------------------------
   // Konstruktoren:
   public BigInteger03() {
      name    = new String    ("rat" + nr++);
      zaehler = new BigInteger("0");
      nenner  = new BigInteger("1");
   }

   public BigInteger03(String nam, String zaehl, String nenn) {
      name    = nam;
      zaehler = new BigInteger(zaehl);
      nenner  = new BigInteger(nenn);
      kuerze();
   }
   // ---------------------------------------------------------------------
   private void kuerze() {
      // Kuerzt this (d.h. "diese rationale Zahl"):
      BigInteger teiler = zaehler.gcd(nenner); // greatest common divisor
      zaehler = zaehler.divide(teiler);
      nenner  = nenner .divide(teiler);
   } // kuerze
   // ---------------------------------------------------------------------
   public String toString() {
      // Wandelt this in einen ("zweizeiligen") String um:
      return new String(
         name + ", Zaehler: " + zaehler.toString() + "\n" +
         name + ", Nenner:  " + nenner .toString() + "\n"
      );
   } // toString

   public BigInteger03 add(BigInteger03 that) {
      // Liefert die Summe this + that:
      BigInteger03 erg  = new BigInteger03();
      BigInteger   big1 = this.zaehler.multiply(that.nenner);
      BigInteger   big2 = that.zaehler.multiply(this.nenner);
      erg.zaehler       = big1.add(big2);
      erg.nenner        = this.nenner.multiply(that.nenner);
      erg.kuerze();
      return erg;
   } // add

   public BigInteger03 sub(BigInteger03 that) {
      // Liefert die Differenz this - that:
      BigInteger03 erg  = new BigInteger03();
      BigInteger   big1 = this.zaehler.multiply(that.nenner);
      BigInteger   big2 = that.zaehler.multiply(this.nenner);
      erg.zaehler       = big1.subtract(big2);
      erg.nenner        = this.nenner.multiply(that.nenner);
      erg.kuerze();
      return erg;
   } // sub
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      // Kleiner Test der Klasse BigInteger03:
      printf("BigInteger03: Jetzt geht es los!\n");

      BigInteger03 otto = new BigInteger03();
      BigInteger03 emil = new BigInteger03("emil", "2", "3");
      BigInteger03 carl = new BigInteger03("carl", "99999999999999999999",
                                                  "100000000000000000000");

      printf("%s\n", otto);
      printf("%s\n", emil);
      printf("%s\n", carl);

      printf("rat1 + emil: \n%s\n", otto.add(emil));
      printf("carl - emil: \n%s\n", carl.sub(emil));
      printf("carl - carl: \n%s\n", carl.sub(carl));

      printf("BigInteger03: Das war's erstmal!\n");
   // ---------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class BigInteger03
/* ------------------------------------------------------------------------
Ausgabe des Programms BigInteger03:

BigInteger03: Jetzt geht es los!
rat1, Zaehler: 0
rat1, Nenner:  1

emil, Zaehler: 2
emil, Nenner:  3

carl, Zaehler: 99999999999999999999
carl, Nenner:  100000000000000000000

rat1 + emil:
rat2, Zaehler: 2
rat2, Nenner:  3

carl - emil:
rat3, Zaehler: 99999999999999999997
rat3, Nenner:  300000000000000000000

carl - carl:
rat4, Zaehler: 0
rat4, Nenner:  1

BigInteger03: Das war's erstmal!
------------------------------------------------------------------------ */
