// Datei E01Anwendung.java
/* ------------------------------------------------------------------------
Zeigt wie man verschiedene Objekte (Rechteck-, Quadrat- und Ellipsen-
Objekte) mit demselben Unterprogramm (urAbstaendeAusgeben, reihungBear-
beiten, reihungAusgeben) bearbeiten kann, ohne genau zu unterscheiden,
um was fuer Objekte es sich handelt.
------------------------------------------------------------------------ */
class E01Anwendung {
   // ---------------------------------------------------------------------
   static private E01Punkt[] produziereReihung() {
      // Erzeugt und liefert eine Reihung, die eine "wilde Mischung" von
      // E01Quadrat-, E01Rechteck- und E01Ellipse-Objekten enthaelt:
      E01Punkt[] pr = new E01Punkt[6];
      pr[0] = new E01Ellipse (1.0, -2.0, 3.0, 4.0);
      pr[1] = new E01Quadrat (2.0, -3.0, 4.0);
      pr[2] = new E01Ellipse (3.0, -4.0, 5.0, 6.0);
      pr[3] = new E01Rechteck(4.0, -5.0, 6.0, 7.0);
      pr[4] = new E01Punkt   (5.0, -6);
      pr[5] = new E01Rechteck(5.0, -6.0, 7.0, 8.0);
//    pr[5] = new E01Kreis   (6.0, -7.0, 8.0);

      return pr;
   } // produziereReihung
   // ---------------------------------------------------------------------
   static void urAbstaendeAusgeben(E01Punkt[] pr, String name) {
      // Gibt die Urabstaende aller Komponenten von pr aus:
      printf("Die Urabstaende der %s-Komponenten:\n", name);
      for (int i=0; i<pr.length; i++) {
         printf("%s[%d].urAbstand:", name, i);
         // Der double-Wert pr[i].urAbstand() wird mit 2 Stellen
         // vor und 3 Stellen nach dem Dezimalpunkt ausgegeben:
         printf("%6.3f\n", pr[i].urAbstand());
      }
   } // urAbstaendeAusgeben
   // ---------------------------------------------------------------------
   static void reihungBearbeiten(E01Punkt[] pr) {
      // Spiegelt alle Komponenten von pr am Ursprung:
      for (int i=0; i<pr.length; i++) {
         pr[i].urSpiegeln();
      }
   } // reihungBearbeiten
   // ---------------------------------------------------------------------
   static void reihungAusgeben(E01Punkt[] pr, String name) {
      // Gibt alle Komponenten von pr aus:
      printf("Die Komponenten der Reihung %s:\n", name);
      for (int i=0; i<pr.length; i++) {
         printf("%s[%d]: ", name, i);
         printf(pr[i].toString()); // Oder kuerzer: printf(pr[i]);
         printf("\n");
      }
   } // reihungAusgeben
   // ---------------------------------------------------------------------
   static void flaechenSummeAusgeben(E01Punkt[] pr) {
      // Berechnet die Flaechen aller Reihungskomponenten pr[i]
      // und gibt die Summe dieser Flaechen aus.

      // Diese Methode enthaelt eine aenderungsunfreundliche
      // Fallunterscheidung. Die laesst sich aber nur vermeiden,
      // indem man in der Klasse E01Punkt Methoden namens
      // getFlaeche (und getUmfang) vereinbart oder "die ganze
      // Klasse E01Punkt durch eine bessere Klasse ersetzt".

      double summe = 0;

      for (int i=0; i<pr.length; i++) {
         if (pr[i] instanceof E01Rechteck) {
            summe += ((E01Rechteck) pr[i]).getFlaeche();
         }
         if (pr[i] instanceof E01Ellipse) {
            summe += ((E01Ellipse) pr[i]).getFlaeche();
         }
      } // for i

      // Die summe wird mit 3 Stellen vor und 2 Stellen nach dem
      // Dezimalpunkt ausgegeben:
      printf("Die Summe aller Flaechen: %6.2f\n", summe);
   } // flaechenSummeAusgeben
   // ---------------------------------------------------------------------
   static public void main(String[] susi) {
      // Eine Reihung von E01Punkt-Objekten wird erzeugt, ausgegeben,
      // mit der Methode reihungBearbeiten bearbeitet und erneut
      // ausgegeben.

      E01Punkt[] tab = produziereReihung();
      printf("E01Anwendung: Jetzt geht es los!\n");
      printf("-----------------------------------------------------------\n");
      printf("Vor der Bearbeitung der Reihung:\n");
      printf("-----------------------------------------------------------\n");
      reihungAusgeben       (tab, "tab");
      flaechenSummeAusgeben (tab);
      printf("-----------------------------------------------------------\n");
      urAbstaendeAusgeben   (tab, "tab");
      reihungBearbeiten     (tab);
      printf("-----------------------------------------------------------\n");
      printf("Nach der Bearbeitung der Reihung:\n");
      printf("-----------------------------------------------------------\n");
      reihungAusgeben       (tab, "tab");
      printf("-----------------------------------------------------------\n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class E01Anwendung
/* ------------------------------------------------------------------------
Ausgabe des Programms E01Anwendung:

E01Anwendung: Jetzt geht es los!
-----------------------------------------------------------
Vor der Bearbeitung der Reihung:
-----------------------------------------------------------
Die Komponenten der Reihung tab:
tab[0]: Ellipse,  Mittelpunkt: (1.0, -2.0), Radien: 3.0|4.0
tab[1]: Quadrat,  Mittelpunkt: (2.0, -3.0), Seite:  4.0
tab[2]: Ellipse,  Mittelpunkt: (3.0, -4.0), Radien: 5.0|6.0
tab[3]: Rechteck, Mittelpunkt: (4.0, -5.0), Seiten: 6.0|7.0
tab[4]: Punkt: (5.0, -6.0)
tab[5]: Rechteck, Mittelpunkt: (5.0, -6.0), Seiten: 7.0|8.0
Die Summe aller Flaechen: 245,95
-----------------------------------------------------------
Die Urabstaende der tab-Komponenten:
tab[0].urAbstand: 2,236
tab[1].urAbstand: 3,606
tab[2].urAbstand: 5,000
tab[3].urAbstand: 6,403
tab[4].urAbstand: 7,810
tab[5].urAbstand: 7,810
-----------------------------------------------------------
Nach der Bearbeitung der Reihung:
-----------------------------------------------------------
Die Komponenten der Reihung tab:
tab[0]: Ellipse,  Mittelpunkt: (-1.0, 2.0), Radien: 3.0|4.0
tab[1]: Quadrat,  Mittelpunkt: (-2.0, 3.0), Seite:  4.0
tab[2]: Ellipse,  Mittelpunkt: (-3.0, 4.0), Radien: 5.0|6.0
tab[3]: Rechteck, Mittelpunkt: (-4.0, 5.0), Seiten: 6.0|7.0
tab[4]: Punkt: (-5.0, 6.0)
tab[5]: Rechteck, Mittelpunkt: (-5.0, 6.0), Seiten: 7.0|8.0
-----------------------------------------------------------
------------------------------------------------------------------------ */