// Datei E01Anwendung03.java
/* ------------------------------------------------------------------------
Demonstriert den Unterschied zwischen dem Typ und dem Zieltyp
einer Referenzvariablen. Der Typ wird bei der Vereinbarung der
Variablen festgelegt und ist unveraenderbar. Der Zieltyp ist der
Typ des Objekts, auf das die Variable zeigt. Der Zieltyp kann
(z.B. mit einer Zuweisung an die Variable) veraendert werden.
Eine Referenzvariable, die noch nicht initialisiert ist oder den
Wert null hat, hat den Zieltyp void.

Ein Cast-Befehl mit Quelltyp QT und Zieltyp ZT ist nur dann erlaubt, wenn
QT und ZT beides primitive, numerische Typen sind oder wenn
QT und ZT beides Referenztypen sind und "auf einem Ast eines Vererbungs-
baumes liegen", d.h. wenn QT eine Unterklasse von ZT oder ZT eine Unter-
klasse von QT ist.

Vererbungsbaum:

Object
 |
 +----E01Punkt
        |
        +---E01Rechteck
        |     |
        |     +---E01Quadrat
        |
        +---E01Ellipse
------------------------------------------------------------------------ */
class E01Anwendung03 {
   // ---------------------------------------------------------------------
   static public void main(String[] susi) {

      // Vier Variablen, bei denen (zumindest anfaenglich) der Typ
      // gleich dem Zieltyp ist:
      E01Punkt    p01 = new E01Punkt   (1.0, 2.0);
      E01Rechteck r01 = new E01Rechteck(2.0, 3.0, 4.0, 5.0);
      E01Quadrat  q01 = new E01Quadrat (3.0, 4.0, 5.0);
      E01Ellipse  e01 = new E01Ellipse (4.0, 5.0, 6.0, 7.0);

      // Vier Variablen mit unterschiedlichen Typen. Alle
      // haben (zumindest anfaenglich) den Zieltyp void.
      E01Punkt    p02;
      E01Rechteck r02;
      E01Quadrat  q02;
      E01Ellipse  e02;

      pln("E01Anwendung03: Jetzt geht es los!");
      pln("-------------------------------------------------------");
      // Jetzt wird der Zieltyp der Variablen p02 mehrmals veraendert
      // und ausgegeben:
                                   // Typ von p02 | Zieltyp von p02
                                   // ------------+--------------
      p02 = null; pln("A " + p02); // E01Punkt    | void
      p02 = e01;  pln("B " + p02); // E01Punkt    | E01Ellipse
      p02 = q01;  pln("C " + p02); // E01Punkt    | E01Quadrat
      p02 = r01;  pln("D " + p02); // E01Punkt    | E01Rechteck
      p02 = p01;  pln("E " + p02); // E01Punkt    | E01Punkt
                                   // -----------------------------

      pln("-------------------------------------------------------");
      p02 =               r01;     // Ohne Cast       erlaubt
//    r02 =               p02;     // Ohne Cast nicht erlaubt
      r02 = (E01Rechteck) p02;     // Mit  Cast       erlaubt

      r02 =               q01;     // Ohne Cast       erlaubt
//    q02 =               r02;     // Ohne Cast nicht erlaubt
      q02 = (E01Quadrat)  r02;     // Mit  Cast       erlaubt

//    e02 =               q01;     // Ohne Cast nicht erlaubt
//    e02 = (E01Ellipse)  q01;     // Mit  Cast nicht erlaubt
//    q02 =               e01;     // Ohne Cast nicht erlaubt
//    q02 = (E01Quadrat)  e01;     // Mit  Cast nicht erlaubt

      p02 =               r01;     // Ohne Cast       erlaubt
//    q02 =               p02;     // Ohne Cast nicht erlaubt
      q02 = (E01Quadrat)  p02;     // Mit  Cast       erlaubt aber
                                   // Laufzeitfehler (ClassCastException)
      pln("-------------------------------------------------------");
      pln("E01Ansendung03: Das war's erstmal!");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class E01Anwendung03
/* ------------------------------------------------------------------------
Fehlermeldungen des Compilers jikes von IBM, wenn die entsprechenden Zeilen
keine Kommentare sind:

    62.     r02 =               p02;     // Ohne Cast nicht erlaubt
            <--------------------->
*** Error: The type of the left-hand side in this assignment, "E01Rechteck",
is not compatible with the type of the right-hand side expression, "E01Punkt".


    66.     q02 =               r02;     // Ohne Cast nicht erlaubt
            <--------------------->
*** Error: The type of the left-hand side in this assignment, "E01Quadrat",
is not compatible with the type of the right-hand side expression, "E01Rechteck".


    69.     e02 =               q01;     // Ohne Cast nicht erlaubt
            <--------------------->
*** Error: The type of the left-hand side in this assignment, "E01Ellipse",
is not compatible with the type of the right-hand side expression, "E01Quadrat".


    70.     e02 = (E01Ellipse)  q01;     // Mit  Cast nicht erlaubt
                                <->
*** Error: An expression of type "E01Quadrat" cannot be cast into
type "E01Ellipse".


    71.     q02 =               e01;     // Ohne Cast nicht erlaubt
            <--------------------->
*** Error: The type of the left-hand side in this assignment, "E01Quadrat",
is not compatible with the type of the right-hand side expression, "E01Ellipse".


    72.     q02 = (E01Quadrat)  e01;     // Mit  Cast nicht erlaubt
                                <->
*** Error: An expression of type "E01Ellipse" cannot be cast into
type "E01Quadrat".


    75.     q02 =               p02;     // Ohne Cast nicht erlaubt
            <--------------------->
*** Error: The type of the left-hand side in this assignment, "E01Quadrat",
is not compatible with the type of the right-hand side expression, "E01Punkt".
---------------------------------------------------------------------------
Ausgabe des Programms E01Anwendung03:

E01Anwendung03: Jetzt geht es los!
-------------------------------------------------------
A null
B Ellipse,  Mittelpunkt: (4.0, 5.0), Radien: 6.0|7.0
C Quadrat,  Mittelpunkt: (3.0, 4.0), Seite:  5.0
D Rechteck, Mittelpunkt: (2.0, 3.0), Seiten: 4.0|5.0
E Punkt: (1.0, 2.0)
-------------------------------------------------------
Exception in thread "main" java.lang.ClassCastException: E01Rechteck
   at E01Anwendung03.main(E01Anwendung03.java:76)
------------------------------------------------------------------------ */