// Datei E02Anwendung.java
/* ------------------------------------------------------------------------
Zeigt wie man verschiedene Objekte (Rechteck-, Quadrat- und Ellipsen-
Objekte) mit demselben Unterprogramm (urAbstaendeAusgeben, reihungBear-
beiten, reihungAusgeben, flaechenSummeAusgeben) bearbeiten kann,
ohne genau zu unterscheiden, um was fuer Objekte es sich handelt.
------------------------------------------------------------------------ */
class E02Anwendung {
   // ---------------------------------------------------------------------
   static private E02GeoFigur[] produziereReihung() {
      // Erzeugt und liefert eine Reihung, die eine "wilde Mischung" von
      // E02Quadrat-, E02Rechteck- und E02Ellipse-Objekten enthaelt:
      E02GeoFigur[] pr = new E02GeoFigur[5];
      pr[0] = new E02Ellipse (1.0, -2.0, 3.0, 4.0);
      pr[1] = new E02Quadrat (2.0, -3.0, 4.0);
      pr[2] = new E02Ellipse (3.0, -4.0, 5.0, 6.0);
      pr[3] = new E02Rechteck(4.0, -5.0, 6.0, 7.0);
      pr[4] = new E02Rechteck(5.0, -6.0, 7.0, 8.0);
//    pr[4] = new E02Kreis   (6.0, -7.0, 8.0);

      return pr;
   } // produziereReihung
   // ---------------------------------------------------------------------
   static void urAbstaendeAusgeben(E02GeoFigur[] gr, String name) {
      // Gibt die Urabstaende aller Komponenten von gr aus:
      printf("Die Urabstaende der %s-Komponenten:%n", name);
      for (int i=0; i<gr.length; i++) {
         printf("%s[%d].urAbstand: ", name, i);
         // Der double-Wert gr[i].urAbstand() wird mit 2 Stellen
         // vor und 2 Stellen nach dem Dezimalpunkt ausgegeben:
         printf("%5.3f%n", gr[i].urAbstand());
      }
   } // urAbstaendeAusgeben
   // ---------------------------------------------------------------------
   static void reihungBearbeiten(E02GeoFigur[] gr) {
      // Spiegelt alle Komponenten von pr am Ursprung:
      for (E02GeoFigur g: gr) {
         g.urSpiegeln();
      }
   } // reihungBearbeiten
   // ---------------------------------------------------------------------
   static void reihungAusgeben(E02GeoFigur[] gr, String name) {
      // Gibt alle Komponenten von pr aus:
      printf("Die Komponenten der Reihung %s:%n", name);
      for (int i=0; i<gr.length; i++) {
         printf("%s[%d]: ", name, i);
         printf(gr[i].toString()); // Oder kuerzer: printf(gr[i]);
         printf("%n");
      }
   } // reihungAusgeben
   // ---------------------------------------------------------------------
   static void flaechenSummeAusgeben(E02GeoFigur[] gr) {
      // Berechnet die Flaechen aller Komponenten der Reihung gr
      // und gibt die Summe dieser Flaechen aus.

      // Diese Methode kann aenderungsfreundlicher programmiert werden
      // als die gleichnamige Methode in der Klasse E01Anwendung,
      // weil die Komponenten der Reihung gr vom Typ E02GeoFigur sind
      // und deshalb jede eine Methode getFlaeche enthaelt.

      double summe = 0;

      for (E02GeoFigur g: gr) {
         summe += g.getFlaeche();
      }

      // Die summe wird mit 3 Stellen vor und 2 Stellen nach dem
      // Dezimalpunkt ausgegeben:
      printf("Die Summe aller Flaechen: %6.2f%n", summe);
   } // flaechenSummeAusgeben
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Eine Reihung von E02GeoFigur-Objekten wird erzeugt, ausgegeben,
      // mit der Methode reihungBearbeiten bearbeitet und erneut
      // ausgegeben.

      E02GeoFigur[] tab = produziereReihung();
      printf("E02Anwendung: Jetzt geht es los!%n");
      printf("-----------------------------------------------------------%n");
      printf("Vor der Bearbeitung der Reihung:%n");
      printf("-----------------------------------------------------------%n");
      reihungAusgeben       (tab, "tab");
      flaechenSummeAusgeben (tab);
      printf("-----------------------------------------------------------%n");
      urAbstaendeAusgeben   (tab, "tab");
      reihungBearbeiten     (tab);
      printf("-----------------------------------------------------------%n");
      printf("Nach der Bearbeitung der Reihung:%n");
      printf("-----------------------------------------------------------%n");
      reihungAusgeben       (tab, "tab");
      printf("-----------------------------------------------------------%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class E02Anwendung
/* ------------------------------------------------------------------------
Ausgabe des Programms E02Anwendung:

E02Anwendung: Jetzt geht es los!
-----------------------------------------------------------
Vor der Bearbeitung der Reihung:
-----------------------------------------------------------
Die Komponenten der Reihung tab:
tab[0]: Ellipse,  Mittelpunkt: (1.0, -2.0), Radien: 3.0|4.0
tab[1]: Quadrat,  Mittelpunkt: (2.0, -3.0), Seite:  4.0
tab[2]: Ellipse,  Mittelpunkt: (3.0, -4.0), Radien: 5.0|6.0
tab[3]: Rechteck, Mittelpunkt: (4.0, -5.0), Seiten: 6.0|7.0
tab[4]: Rechteck, Mittelpunkt: (5.0, -6.0), Seiten: 7.0|8.0
Die Summe aller Flaechen: 245,95
-----------------------------------------------------------
Die Urabstaende der tab-Komponenten:
tab[0].urAbstand: 2,236
tab[1].urAbstand: 3,606
tab[2].urAbstand: 5,000
tab[3].urAbstand: 6,403
tab[4].urAbstand: 7,810
-----------------------------------------------------------
Nach der Bearbeitung der Reihung:
-----------------------------------------------------------
Die Komponenten der Reihung tab:
tab[0]: Ellipse,  Mittelpunkt: (-1.0, 2.0), Radien: 3.0|4.0
tab[1]: Quadrat,  Mittelpunkt: (-2.0, 3.0), Seite:  4.0
tab[2]: Ellipse,  Mittelpunkt: (-3.0, 4.0), Radien: 5.0|6.0
tab[3]: Rechteck, Mittelpunkt: (-4.0, 5.0), Seiten: 6.0|7.0
tab[4]: Rechteck, Mittelpunkt: (-5.0, 6.0), Seiten: 7.0|8.0
-----------------------------------------------------------
------------------------------------------------------------------------ */