// Datei E02Ellipse.java
/* ------------------------------------------------------------------------
Jedes Objekt dieser Klasse repraesentiert eine Ellipse auf einer zwei-
dimensionalen Ebene. Die Hauptachsen der Ellipse liegen parallel zu den
Achsen des Koordinatensystems.

Jede mit der Anmerkung (annotation) @Override markierte Methode muss eine
geerbte Methode ueberschreiben, sonst meldet der Compiler einen Fehler.

Diese Klasse gehoert zu folgendem Beerbungsbaum:

Object
 |
 +----E02GeoFigur
        |
        +---E02Rechteck
        |     |
        |     +---E02Quadrat
        |
        +---E02Ellipse
              |
              +---E02Kreis
------------------------------------------------------------------------ */
class E02Ellipse extends E02GeoFigur {
   // Die Radien dieser Ellipse:
   private double radiusX; // Radius in x-Richtung
   private double radiusY; // Radius in y-Richtung

   public E02Ellipse(double x,       double y,
                     double radiusX, double radiusY) {
      super(x, y);
      this.radiusX = radiusX;
      this.radiusY = radiusY;
   } // Konstruktor E02Ellipse

   public double getRadiusX() {return radiusX;}
   public double getRadiusY() {return radiusY;}

   // Funktionen, die den Umfang bzw. die Flaeche dieser Figur
   // liefern:
   @Override public double getUmfang()  {
      double summ = radiusX + radiusY;
      double prod = radiusX * radiusY;
      return Math.PI * (1.5 * summ - Math.sqrt(prod));
   } // getUmfang

   @Override public double getFlaeche() {return Math.PI*radiusX*radiusY;}

   @Override public String toString() {
      // Liefert eine String-Repraesentation dieser Figur:
      return new String("Ellipse,  Mittelpunkt: " + text() +
         ", Radien: " + radiusX + "|" + radiusY);
   } // toString

} // class E02Ellipse