// Datei Ersetzen02.java
/* ------------------------------------------------------------------------
Def. *homonym* ("bezeichnungsgleich"):
Zwei Attribute sind homonym, wenn ihre Namen   gleich sind.
Zwei Methoden  sind homonym, wenn ihre Profile gleich sind.

Vereinbart man in einer Klasse ein Element, welches homonym zu einem
geerbten Element ist, so wird das geerbte Element durch das vereinbarte
Element *ersetzt*. Es gibt zwei Arten von ersetzen:
*verdecken* (engl. to hide) und *ueberschreiben* (engl. to override).

Merke: Ueberschreiben ist "endgueltiger" als verdecken, denn ein Verwender
der Unterklasse kann verdeckte Elemente z.B. mit Cast-Befehlen aufdecken,
aber ueberschriebene Elemente kann er nicht "entueberschreiben".

Ueber eine Referenzvariable kann man nicht nur auf Objektelemente
zugreifen, sondern auch auf Klassenmethoden und Klassenattribute.
Deshalb ist es sinnvoll zu fragen, ob eine geerbte Klassenmethode
durch eine homonyme Klassenmethode verdeckt oder ueberschrieben wird.

In diesem Programm wird demonstriert:

Eine Klassenmethode verdeckt eine (homonyme, geerbte) Klassenmethode.
(siehe dazu die Klassenmethode metKK).

Eine  Objektmethode darf nicht homonym zu einer geerbten Klassenmethode sein.
Eine Klassenmethode darf nicht homonym zu einer geerbten  Objektmethode sein.
(siehe dazu die Methoden metKO und metOK in diesem Programm).

Endet ein Elementname mit KK, dann wird in Ober ein Klassenelement und
in Unter eine homonymes Klassenelement vereinbart.
Endet der Elementname mit KO, dann wird in Ober ein Klassenelement und
in Unter ein homonymes Objektelement vereinbart.
Fuer die Endungen OK und OO gilt Entsprechendes.
------------------------------------------------------------------------ */
class Ober {
   static String metKK() {return " Ober.metKK()";}
   static String metKO() {return " Ober.metKO()";}
          String metOK() {return " Ober.metOK()";}
} // class Ober
// ========================================================================
// In der folgenden Klasse wird versucht, alle geerbten Methoden (durch
// das Vereinbaren von homonymen Methoden) zu ersetzen. Nur bei einer
// Methode ist das auch erlaubt:
class Unter extends Ober {
   static String metKK() {return "Unter.metKK(), verdeckt Ober.metKK()";}
//        String metKO() {return "Unter.metKO(), Compilezeit-Fehler!";}
// static String metOK() {return "Unter.metOK(), Compilezeit-Fehler!";}
} // class Unter
// ========================================================================
class Ersetzen02 {
   static public void main(String [] sonja) {
      // Ueber die Klassennamen Ober und Unter und ueber die Objektnamen
      // oOb und uOb wird auf Klassen- und Objektmethoden zugegriffen:

      Ober  oOb = new  Ober(); // Ein  Ober-Objekt
      Unter uOb = new Unter(); // Ein Unter-Objekt

      pln("------------------------------------");
      pln("A  Ober.metKK():       " +        Ober.metKK());
      pln("B  Ober.metKO():       " +        Ober.metKO());
      pln("C Unter.metKK():       " +       Unter.metKK());
      pln("------------------------------------");
      pln("D        oOb .metKK(): " +        oOb .metKK());
      pln("E        oOb .metKO(): " +        oOb .metKO());
      pln("F        oOb .metOK(): " +        oOb .metOK());
      pln("------------------------------------");
      pln("G        uOb .metKK(): " +        uOb .metKK());
      pln("H        uOb .metKO(): " +        uOb .metKO());
      pln("I        uOb .metOK(): " +        uOb .metOK());
      pln("------------------------------------");
      pln("J ((Ober)uOb).metKK(): " + ((Ober)uOb).metKK());
      pln("K ((Ober)uOb).metKO(): " + ((Ober)uOb).metKO());
      pln("L ((Ober)uOb).metOK(): " + ((Ober)uOb).metOK());
      pln("------------------------------------");
      pln("Ersetzen02: Das war's erstmal!");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Ersetzen02
/* ------------------------------------------------------------------------
Fehlermeldungen des Compilers javac von Sun, wenn die entsprechenden Zeilen
keine Kommentare sind (die zweite Fehlermeldung ist falsch formuliert):

D:\Quellen\Ersetzen02.java:47: metKO() in Unter cannot override metKO() in
Ober; overridden method is static
        String metKO() {return "Unter.metKO(), Compilezeit-Fehler!";}
               ^
D:\Quellen\Ersetzen02.java:48: metOK() in Unter cannot override metOK() in
Ober; metOK() and metOK() are static
 static String metOK() {return "Unter.metOK(), Compilezeit-Fehler!";}
               ^
---------------------------------------------------------------------------
Fehlermeldungen des Compilers jikes von IBM, wenn die entsprechenden Zeilen
keine Kommentare sind sind:

    47.         String metKO() {return "Unter.metKO(), Compilezeit-Fehler!";}
                       <----->
*** Error: The instance method "java.lang.String metKO();" cannot override
the static method "java.lang.String metKO();" declared in type "Ober".


    48.  static String metOK() {return "Unter.metOK(), Compilezeit-Fehler!";}
                       <----->
*** Error: The static method "java.lang.String metOK();" cannot hide
the instance method "java.lang.String metOK();" declared in "Ober".
---------------------------------------------------------------------------
Ausgabe des Programms Ersetzen02:

------------------------------------
A  Ober.metKK():        Ober.metKK()
B  Ober.metKO():        Ober.metKO()
C Unter.metKK():       Unter.metKK(), verdeckt Ober.metKK()
------------------------------------
D        oOb .metKK():  Ober.metKK()
E        oOb .metKO():  Ober.metKO()
F        oOb .metOK():  Ober.metOK()
------------------------------------
G        uOb .metKK(): Unter.metKK(), verdeckt Ober.metKK()
H        uOb .metKO():  Ober.metKO()
I        uOb .metOK():  Ober.metOK()
------------------------------------
J ((Ober)uOb).metKK():  Ober.metKK()
K ((Ober)uOb).metKO():  Ober.metKO()
L ((Ober)uOb).metOK():  Ober.metOK()
------------------------------------
Ersetzen02: Das war's erstmal!
------------------------------------------------------------------------ */