// Datei Ersetzen03.java
/* ------------------------------------------------------------------------
Def. homonym ("bezeichnungsgleich"):
Zwei Attribute sind homonym, wenn ihre Namen   gleich sind.
Zwei Methoden  sind homonym, wenn ihre Profile gleich sind.

Vereinbart man in einer Klasse ein Element, welches homonym zu einem
geerbten Element ist, so wird das geerbte Element durch das vereinbarte
Element ersetzt. Es gibt zwei Arten von ersetzen:
verdecken (to hide) und ueberschreiben (to override).

Merke: Ueberschreiben ist "endgueltiger" als verdecken, denn ein Verwender
der Unterklasse kann verdeckte Elemente z.B. mit Cast-Befehlen aufdecken,
aber ueberschriebene Elemente kann er nicht "entueberschreiben".

Ueber eine Referenzvariable kann man nicht nur auf Objektelemente
zugreifen, sondern auch auf Klassenmethoden und Klassenattribute.
Deshalb ist es sinnvoll zu fragen, ob z.B. ein geerbtes Klassenattribut
durch ein homonymes Klassenattribut verdeckt oder ueberschrieben wird.

In diesem Programm wird demonstriert:

Ein Attribut verdeckt alle homonymen, geerbten Attribute. Ob es sich
dabei um Klassen- oder Objektattribute handelt ist gleichgueltig.

Endet ein Elementname mit KK, dann wird in Ober ein Klassenelement und
in Unter eine homonymes Klassenelement vereinbart.
Endet der Elementname mit KO, dann wird in Ober ein Klassenelement und
in Unter ein homonymes Objektelement vereinbart.
Fuer die Endungen OK und OO gilt Entsprechendes.
------------------------------------------------------------------------ */
class Ober {
   static String attKK = " Ober.attKK";
   static String attKO = " Ober.attKO";
          String attOK = " Ober.attOK";
          String attOO = " Ober.attOO";
} // class Ober
// ------------------------------------------------------------------------
// In der folgenden Klasse werden alle geerbten Attribute (durch das
// Vereinbaren von homonymen Attributen) ersetzt:
class Unter extends Ober {
   static String attKK = "Unter.attKK";
          String attKO = "Unter.attKO";
   static String attOK = "Unter.attOK";
          String attOO = "Unter.attOO";
} // class Unter
// ------------------------------------------------------------------------
class Ersetzen03 {
   static public void main(String [] sonja) {
      // Ueber die Klassennamen Ober und Unter und ueber die Objektnamen
      // oOb und uOb wird auf Klassen- und Objektattribute zugegriffen:

      Ober  oOb = new  Ober(); // Ein  Ober-Objekt
      Unter uOb = new Unter(); // Ein Unter-Objekt

      pln("--------------------------------");
      pln("A  Ober.attKK:       " +        Ober.attKK);
      pln("B  Ober.attKO:       " +        Ober.attKO);
      pln("C Unter.attKK:       " +       Unter.attKK);
      pln("D Unter.attOK:       " +       Unter.attOK);
      pln("--------------------------------");
      pln("E oOb.attKK:         " +         oOb.attKK);
      pln("F oOb.attKO:         " +         oOb.attKO);
      pln("G oOb.attOK:         " +         oOb.attOK);
      pln("H oOb.attOO:         " +         oOb.attOO);
      pln("--------------------------------");
      pln("I uOb.attKK:         " +         uOb.attKK);
      pln("J uOb.attKO:         " +         uOb.attKO);
      pln("K uOb.attOK:         " +         uOb.attOK);
      pln("L uOb.attOO:         " +         uOb.attOO);
      pln("--------------------------------");
      pln("M ((Ober)uOb).attKK: " + ((Ober)uOb).attKK);
      pln("N ((Ober)uOb).attKO: " + ((Ober)uOb).attKO);
      pln("O ((Ober)uOb).attOK: " + ((Ober)uOb).attOK);
      pln("P ((Ober)uOb).attOO: " + ((Ober)uOb).attOO);
      pln("--------------------------------");
      pln("Ersetzen03: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Ersetzen03
/* ------------------------------------------------------------------------
Ausgabe des Programms Ersetzen03:

--------------------------------
A  Ober.attKK:        Ober.attKK
B  Ober.attKO:        Ober.attKO
C Unter.attKK:       Unter.attKK
D Unter.attOK:       Unter.attOK
--------------------------------
E oOb.attKK:          Ober.attKK
F oOb.attKO:          Ober.attKO
G oOb.attOK:          Ober.attOK
H oOb.attOO:          Ober.attOO
--------------------------------
I uOb.attKK:         Unter.attKK
J uOb.attKO:         Unter.attKO
K uOb.attOK:         Unter.attOK
L uOb.attOO:         Unter.attOO
--------------------------------
M ((Ober)uOb).attKK:  Ober.attKK
N ((Ober)uOb).attKO:  Ober.attKO
O ((Ober)uOb).attOK:  Ober.attOK
P ((Ober)uOb).attOO:  Ober.attOO
--------------------------------
Ersetzen03: Das war's erstmal!
------------------------------------------------------------------------ */