// Datei Ersetzen04.java
/* ------------------------------------------------------------------------
Auf die Klassenelemente (class members, static members) einer Klasse
kann man nicht nur ueber den Namen der Klasse zugreifen (z.B. Ober.att,
Oder.met, Unter.att, ... etc.), sondern auch ueber den Namens einer
Variable dieser Klasse (z.B. oOb.att, oOb.met, uOb.att, ... etc.).

Def. homonym ("bezeichnungsgleich"):
Zwei Attribute sind homonym, wenn ihre Namen   gleich sind.
Zwei Methoden  sind homonym, wenn ihre Profile gleich sind.

Vereinbart man in einer Klasse ein Element, welches homonym zu einem
geerbten Element ist, so wird das geerbte Element durch das vereinbarte
Element ersetzt. Es gibt zwei Arten von ersetzen:
verdecken (to hide) und ueberschreiben (to override).

Merke: Ueberschreiben ist "endgueltiger" als verdecken, denn ein Verwender
der Unterklasse kann verdeckte Elemente z.B. mit Cast-Befehlen aufdecken,
aber ueberschriebene Elemente kann er nicht "entueberschreiben".

In diesem Programm wird demonstriert:

Ein  Klassenattribut verdeckt ein  (homonymes, geerbtes) Klassenattribut.
Eine Klassenmethode  verdeckt eine (homonyme,  geerbte)  Klassenmethode.
------------------------------------------------------------------------ */
// Die Klasse Mittel erbt zwei Klassenelemente att und met und ersetzt sie.
// Beide geerbte Elemente (das Klassenattribut att und die Klassenmethode
// met) werden dadurch verdeckt (nicht ueberschrieben).
// Fuer die Klasse Unter gilt ganz Entsprechendes wie fuer Mittel.

class Ober {
   static String att = new String("   Ober-Attribut");
   static String met() {return    "   Ober-Methode ";}
} // class Ober

class Mittel extends Ober {
   static String att = new String(" Mittel-Attribut");
   static String met() {return    " Mittel-Methode ";}
} // class Mittel

class Unter  extends Mittel{
   static String att = new String("  Unter-Attribut");
   static String met() {return    "  Unter-Methode ";}
} // class Unter
// ------------------------------------------------------------------------
class Ersetzen04 {
   static public void main(String [] sonja) {
      // Ueber Referenzvariablen mit unterschiedlichen Typen wird auf
      // Klassenelemente der Klassen Ober, Mittel und Unter zugegriffen.

      // Referenzvariablen mit unterschiedlichen Typen
      // werden vereinbart:
      Ober   oOb = null; // Typ Ober,   Zieltyp void
      Mittel mOb = null; // Typ Mittel, Zieltyp void
      Unter  uOb = null; // Typ Unter,  Zieltyp void

      // Ueber diese Referenzvariablen wird auf die Elemente namens att
      // und met in den 3 Klassen Ober, Mittel und Unter zugegriffen
      // (die Cast-Befehle in den Zeilen D, H und L kann man auch
      // weglassen, sie bewirken "identische Umdeutungen"):
      pln("-----------------------------------");
      pln("A " +           oOb .att + ", " +           oOb .met());
      pln("B " +           mOb .att + ", " +           mOb .met());
      pln("C " +           uOb .att + ", " +           uOb .met());
      pln("-----------------------------------");
      pln("D " + ((  Ober) oOb).att + ", " + ((  Ober) oOb).met());
      pln("E " + ((Mittel) oOb).att + ", " + ((Mittel) oOb).met());
      pln("F " + (( Unter) oOb).att + ", " + (( Unter) oOb).met());
      pln("-----------------------------------");
      pln("G " + ((  Ober) mOb).att + ", " + ((  Ober) mOb).met());
      pln("H " + ((Mittel) mOb).att + ", " + ((Mittel) mOb).met());
      pln("I " + (( Unter) mOb).att + ", " + (( Unter) mOb).met());
      pln("-----------------------------------");
      pln("J " + ((  Ober) uOb).att + ", " + ((  Ober) uOb).met());
      pln("K " + ((Mittel) uOb).att + ", " + ((Mittel) uOb).met());
      pln("L " + (( Unter) uOb).att + ", " + (( Unter) uOb).met());

      // Zum Abschluss wird auch ueber die Klassennamen Ober, Mittel und
      // Unter auf die Klassenelemente der Klassen zugegriffen:
      pln("-----------------------------------");
      pln("M " +           Ober.att + ", " +           Ober.met());
      pln("N " +         Mittel.att + ", " +         Mittel.met());
      pln("O " +          Unter.att + ", " +          Unter.met());
      pln("-----------------------------------");
      pln("Ersetzen04: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Ersetzen04
/* ------------------------------------------------------------------------
Ausgabe des Programms Ersetzen04:

-----------------------------------
A    Ober-Attribut,    Ober-Methode
B  Mittel-Attribut,  Mittel-Methode
C   Unter-Attribut,   Unter-Methode
-----------------------------------
D    Ober-Attribut,    Ober-Methode
E  Mittel-Attribut,  Mittel-Methode
F   Unter-Attribut,   Unter-Methode
-----------------------------------
G    Ober-Attribut,    Ober-Methode
H  Mittel-Attribut,  Mittel-Methode
I   Unter-Attribut,   Unter-Methode
-----------------------------------
J    Ober-Attribut,    Ober-Methode
K  Mittel-Attribut,  Mittel-Methode
L   Unter-Attribut,   Unter-Methode
-----------------------------------
M    Ober-Attribut,    Ober-Methode
N  Mittel-Attribut,  Mittel-Methode
O   Unter-Attribut,   Unter-Methode
-----------------------------------
Ersetzen04: Das war's erstmal!
------------------------------------------------------------------------ */