// Datei Ersetzen05.java
/* ------------------------------------------------------------------------
Ein Objekt kann nicht nur (Objekt-) Methoden und (Objekt-) Attribute
als Elemente enthalten, sondern auch innere Klassen (d.h. nicht-statische
geschachtelte Klassen).

Zwei innere Klassen sind homonym, wenn ihre Namen gleich sind.

In diesem Programm wird demonstriert:
Eine innere Klasse verdeckt eine (homonyme, geerbte) innere Klasse.

Das bedeutet: Beim Ersetzen (von geerbten Elementen) werden innere
Klassen aehnlich wie Attribute behandelt (und nicht wie Methoden).
------------------------------------------------------------------------ */
// Jede der folgenden drei Klassen namens Ober, Mittel und Unter enthaelt
// eine innere Klasse namens Inner. Diese drei Inner-Klassen enthalten
// jeweils nur eine Objektmethode namens toString. Gibt man ein Objekt
// einer dieser Inner-Klassen aus (z.B. mit pln), wird das Ergebnis der
// betreffenden toString-Methode ausgegeben.

class Ober {
   class Inner {
      public String toString() {return "Inner in Ober!";}
   } // class Inner
} // class Ober

class Mittel extends Ober {
   class Inner {
      public String toString() {return "Inner in Mittel!";}
   } // class Inner
} // class Mittel

class Unter  extends Mittel{
   class Inner {
      public String toString() {return "Inner in Unter!";}
   } // class Inner
} // class Unter
// ------------------------------------------------------------------------
class Ersetzen05 {
   static public void main(String [] sonja) {
      // Ueber Referenzvariablen mit unterschiedlichen Typen (verein-
      // barten Typen), aber demselben Zieltyp (momentanen Typ) wird auf
      // die inneren Klassen in Unter-Objekten zugegriffen

      // Referenzvariablen mit unterschiedlichen Typen
      // werden vereinbart:
      Ober   oOb = new Unter(); // Typ Ober,   Zieltyp Unter
      Mittel mOb = new Unter(); // Typ Mittel, Zieltyp Unter
      Unter  uOb = new Unter(); // Typ Unter,  Zieltyp Unter

      // Objekte der verschiedenen Inner-Klassen (die als Elemente in
      // den Objekten oOb, mOb und uOb enthalten sind) werden erzeugt.
      // Statt   new oOb.Inner()         muss man hier komischerweise
      //         oOb.new Inner()         schreiben!
      // (die Cast-Befehle in den Zeilen D, H und L kann man auch
      // weglassen, sie bewirken "identische Umdeutungen"):
      pln("------------------------------");
      pln("A " +            oOb .new Inner());
      pln("B " +            mOb .new Inner());
      pln("C " +            uOb .new Inner());
      pln("------------------------------");
      pln("D " +  (  (Ober) oOb).new Inner());
      pln("E " +  ((Mittel) oOb).new Inner());
      pln("F " +  ( (Unter) oOb).new Inner());
      pln("------------------------------");
      pln("G " +  (  (Ober) mOb).new Inner());
      pln("H " +  ((Mittel) mOb).new Inner());
      pln("I " +  ( (Unter) mOb).new Inner());
      pln("------------------------------");
      pln("J " +  (  (Ober) uOb).new Inner());
      pln("K " +  ((Mittel) uOb).new Inner());
      pln("L " +  ( (Unter) uOb).new Inner());
      pln("------------------------------");
      pln("Ersetzen05: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Ersetzen05
/* ------------------------------------------------------------------------
Ausgabe des Programms Ersetzen05:

------------------------------
A Inner in Ober!
B Inner in Mittel!
C Inner in Unter!
------------------------------
D Inner in Ober!
E Inner in Mittel!
F Inner in Unter!
------------------------------
G Inner in Ober!
H Inner in Mittel!
I Inner in Unter!
------------------------------
J Inner in Ober!
K Inner in Mittel!
L Inner in Unter!
------------------------------
Ersetzen05: Das war's erstmal!
------------------------------------------------------------------------ */