// Datei Faden06Tst.java
/* ------------------------------------------------------------------------
Eine "synchronisierte" Version des Programms Faden05Tst (nur die beide
mit dem Kommentar " // <--- " gekennzeichneten Zeilen wurden eingefuegt).

Ein Faden namens A versucht, die globale Variable text mit lauter 'A's zu
fuellen und dann auszugeben. Nebenlufig dazu versucht ein Faden namens B,
dieselbe Variable text mit lauter 'B's zu fuellen und auszugeben. Die Aus-
gaben enthalten trotzdem nie inkonsistente Kombinationen von 'A's und 'B's,
weil der Rumpf der Methode fuelleUndDrucke mit dem synchronized-Befehl
ununterbrechbar gemacht wurde.
------------------------------------------------------------------------ */
import java.util.Random;

class Faden06 implements Runnable {
   // Die globale Variable text soll mit Daten gefuellt werden:
   static StringBuilder text = new StringBuilder("XXXXXXXXXXXXXXXXXXXX");
   // ---------------------------------------------------------------------
   static void fuelleUndDrucke(StringBuilder sb, char zeichen) {
      // Fuellt die Variable sb mit dem zeichen und gibt sie aus:
      synchronized(sb) {                                            // <---
         for (int j=0, laenge=sb.length(); j<laenge; j++) {
            sb.setCharAt(j, zeichen);
            schlaf(10);                             // maximal 10 Millis
         } // for
         p("Faden " + zeichen + ": " + text + " ");
      } // synchronized                                             // <---
   } // fuelleUndDrucke
   // ---------------------------------------------------------------------
   public void run() {
      String name = Thread.currentThread().getName(); // "A" bzw. "B"
      char   erst = name.charAt(0);                   // 'A' bzw. 'B'

      for (int i=1; i<=10; i++) {
         fuelleUndDrucke(text, erst);
         pln(i);
         schlaf(1000);                           // maximal 1000 Millis
      } // for
   } // run
   // ---------------------------------------------------------------------
   static Random r01 = new Random();
   static void schlaf(int maxMillis) {
      // Legt den aufrufenden Faden eine zufaellige Anzahl von Millisek.
      // (zwischen 0 und maxMillis) schlafen.
      try {
         Thread.sleep(r01.nextInt(maxMillis));
      } catch (InterruptedException ie) {
         pln(ie);
      } // catch
   } // schlaf
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Faden06
// ========================================================================
class Faden06Tst {
   static public void main(String [] sonja) {
      Runnable r1 = new Faden06();
      Runnable r2 = new Faden06();
      Thread   f1 = new Thread(r1, "A");
      Thread   f2 = new Thread(r2, "B");
      f1.start();
      f2.start();
   } // main
} // class Faden06Tst
/* ------------------------------------------------------------------------
Zwei moegliche Ausgabe des Programms Faden06Tst:

| Faden A: AAAAAAAAAAAAAAAAAAAA 1  | Faden A: AAAAAAAAAAAAAAAAAAAA 1  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 1  | Faden B: BBBBBBBBBBBBBBBBBBBB 1  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 2  | Faden A: AAAAAAAAAAAAAAAAAAAA 2  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 2  | Faden A: AAAAAAAAAAAAAAAAAAAA 3  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 3  | Faden B: BBBBBBBBBBBBBBBBBBBB 2  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 3  | Faden A: AAAAAAAAAAAAAAAAAAAA 4  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 4  | Faden B: BBBBBBBBBBBBBBBBBBBB 3  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 4  | Faden A: AAAAAAAAAAAAAAAAAAAA 5  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 5  | Faden B: BBBBBBBBBBBBBBBBBBBB 4  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 5  | Faden B: BBBBBBBBBBBBBBBBBBBB 5  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 6  | Faden A: AAAAAAAAAAAAAAAAAAAA 6  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 6  | Faden B: BBBBBBBBBBBBBBBBBBBB 6  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 7  | Faden A: AAAAAAAAAAAAAAAAAAAA 7  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 7  | Faden B: BBBBBBBBBBBBBBBBBBBB 7  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 8  | Faden B: BBBBBBBBBBBBBBBBBBBB 8  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 9  | Faden A: AAAAAAAAAAAAAAAAAAAA 8  |
| Faden A: AAAAAAAAAAAAAAAAAAAA 10 | Faden B: BBBBBBBBBBBBBBBBBBBB 9  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 8  | Faden A: AAAAAAAAAAAAAAAAAAAA 9  |
| Faden B: BBBBBBBBBBBBBBBBBBBB 9  | Faden B: BBBBBBBBBBBBBBBBBBBB 10 |
| Faden B: BBBBBBBBBBBBBBBBBBBB 10 | Faden A: AAAAAAAAAAAAAAAAAAAA 10 |
------------------------------------------------------------------------ */