// Datei Faden11Tst.java
/* ------------------------------------------------------------------------
Eine Methode, deren Rumpf nur aus einem synchronized-Block besteht (wie
bei der Methode met01) bewirkt genau das Gleiche wie eine synchronized-
Methode (wie etwa met02). Bei beiden Methoden erfolgt die Parameterueber-
gabe vor der Synchronization (d.h. bevor ein Faden eventuell schlafen
gelegt wird).
------------------------------------------------------------------------ */
import java.util.Random;

class Faden11 extends Thread {
   // ---------------------------------------------------------------------
   static Random r01 = new Random(); // Ein Zufallszahlengenerator
   // ---------------------------------------------------------------------
   public String mitSeitenEffekt(int i) {
      // Gibt den Namen des aktuellen Fadens etc. aus und liefert
      // einen (unwichtigen) String als Ergebnis. Das Ergebnis dieser
      // Funktion wird als Parameter an die Methoden met01 und met02
      // uebergeben. Dadurch soll auf dem Bildschirm sichtbar werden,
      // wann etwa die Parameteruebergabe erfolgte.
      String name = getName();
      pln(name + " hat Param-" + i + " uebergeben !");
      schlaf(500);
      return ""; // Das Ergebnis ist hier nicht wichtig
   } // mitSeitenEffekt
   // ---------------------------------------------------------------------
   void met01(String s, int i) {
      // Eine Methode, deren Rumpf nur aus einem synchronized-Block
      // besteht. Hat gleiche Wirkung wie met02.
      synchronized(this) {
         String name = getName();
         pln(name + " hat met01-" + i + " ausgefuehrt!");
         schlaf(500);
      } // synchronized
   } // met01
   // ---------------------------------------------------------------------
   synchronized void met02(String s, int i) {
      // Eine synchronized-Methode. Hat gleiche Wirkung wie met01.
      String name = getName();
      pln(name + " hat met02-" + i + " ausgefuehrt!");
      schlaf(500);
   } // met02
   // ---------------------------------------------------------------------
   public void run() {
      for (int i=11; i<=15; i++) {
         met02(mitSeitenEffekt(i), i);
      } // for

      for (int i=21; i<=25; i++) {
         met01(mitSeitenEffekt(i), i);
      } // for
   } // run
   // ---------------------------------------------------------------------
   void schlaf(int maxMillis) {
      // Bewirkt, dass der aktuelle Faden eine zufaellige Anzahl von
      // Millisekunden (zwischen 0 und maxMillis) schlaeft.
      try {
         // Eine zufaellige Ganzzahl zwischen
         // 0 und maxMillis nach millis bringen:
         int millis =  r01.nextInt(maxMillis);
         // Den aktuellen Faden mindestens
         // millis Millisekunden schlafen lassen:
         sleep(millis);
      } catch (InterruptedException ie) {
         pln(ie);
      } // try-catch
   } // schlaf
   // ---------------------------------------------------------------------
   // Zwei Konstruktoren:
   Faden11(String name) {super(name);}     // "Selbstgewaehlter" Name
   Faden11()            {super();}         // Name wird automatisch erzeugt
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
} // class Faden11
// ========================================================================
class Faden11Tst {
   static public void main(String [] sonja) {
      System.out.println("Faden11Tst: Jetz geht's los!");
      System.out.println("----------------------------");
      Faden11 f1 = new Faden11("XX");
      Faden11 f2 = new Faden11("OO");
      f1.start();
      f2.start();

      block11: synchronized(f1) {
         block12: synchronized(f2) {
         }
      }

      block21: synchronized(f1) {
         block22: synchronized(f2) {
         }
      }

   } // main
   // ---------------------------------------------------------------------
} // class Faden11Tst
/* ------------------------------------------------------------------------
Zwei moegliche Ausgabe des Programms Faden11Tst:

| Faden11Tst: Jetz geht's los! | Faden11Tst: Jetz geht's los! |
| ---------------------------- | ---------------------------- |
| XX hat Param-11 uebergeben ! | XX hat Param-11 uebergeben ! |
| OO hat Param-11 uebergeben ! | OO hat Param-11 uebergeben ! |
| XX hat met02-11 ausgefuehrt! | OO hat met02-11 ausgefuehrt! |
| XX hat Param-12 uebergeben ! | OO hat Param-12 uebergeben ! |
| OO hat met02-11 ausgefuehrt! | XX hat met02-11 ausgefuehrt! |
| XX hat met02-12 ausgefuehrt! | XX hat Param-12 uebergeben ! |
| XX hat Param-13 uebergeben ! | XX hat met02-12 ausgefuehrt! |
| XX hat met02-13 ausgefuehrt! | OO hat met02-12 ausgefuehrt! |
| OO hat Param-12 uebergeben ! | XX hat Param-13 uebergeben ! |
| XX hat Param-14 uebergeben ! | OO hat Param-13 uebergeben ! |
| XX hat met02-14 ausgefuehrt! | OO hat met02-13 ausgefuehrt! |
| OO hat met02-12 ausgefuehrt! | XX hat met02-13 ausgefuehrt! |
| OO hat Param-13 uebergeben ! | XX hat Param-14 uebergeben ! |
| XX hat Param-15 uebergeben ! | OO hat Param-14 uebergeben ! |
| OO hat met02-13 ausgefuehrt! | XX hat met02-14 ausgefuehrt! |
| XX hat met02-15 ausgefuehrt! | OO hat met02-14 ausgefuehrt! |
| XX hat Param-21 uebergeben ! | XX hat Param-15 uebergeben ! |
| OO hat Param-14 uebergeben ! | OO hat Param-15 uebergeben ! |
| OO hat met02-14 ausgefuehrt! | XX hat met02-15 ausgefuehrt! |
| XX hat met01-21 ausgefuehrt! | OO hat met02-15 ausgefuehrt! |
| OO hat Param-15 uebergeben ! | XX hat Param-21 uebergeben ! |
| XX hat Param-22 uebergeben ! | OO hat Param-21 uebergeben ! |
| OO hat met02-15 ausgefuehrt! | XX hat met01-21 ausgefuehrt! |
| OO hat Param-21 uebergeben ! | OO hat met01-21 ausgefuehrt! |
| XX hat met01-22 ausgefuehrt! | OO hat Param-22 uebergeben ! |
| OO hat met01-21 ausgefuehrt! | XX hat Param-22 uebergeben ! |
| XX hat Param-23 uebergeben ! | OO hat met01-22 ausgefuehrt! |
| OO hat Param-22 uebergeben ! | OO hat Param-23 uebergeben ! |
| OO hat met01-22 ausgefuehrt! | XX hat met01-22 ausgefuehrt! |
| XX hat met01-23 ausgefuehrt! | XX hat Param-23 uebergeben ! |
| OO hat Param-23 uebergeben ! | OO hat met01-23 ausgefuehrt! |
| XX hat Param-24 uebergeben ! | XX hat met01-23 ausgefuehrt! |
| XX hat met01-24 ausgefuehrt! | OO hat Param-24 uebergeben ! |
| OO hat met01-23 ausgefuehrt! | XX hat Param-24 uebergeben ! |
| XX hat Param-25 uebergeben ! | OO hat met01-24 ausgefuehrt! |
| OO hat Param-24 uebergeben ! | OO hat Param-25 uebergeben ! |
| XX hat met01-25 ausgefuehrt! | XX hat met01-24 ausgefuehrt! |
| OO hat met01-24 ausgefuehrt! | OO hat met01-25 ausgefuehrt! |
| OO hat Param-25 uebergeben ! | XX hat Param-25 uebergeben ! |
| OO hat met01-25 ausgefuehrt! | XX hat met01-25 ausgefuehrt! |
------------------------------------------------------------------------ */