// Datei FloatBits01.java
/* ------------------------------------------------------------------------
Ein Objekt der Klasse FloatBits01 wird auf dem Bildschirm als Fenster
(JFrame) mit folgendem Inhalt dargestellt:

1. Eine horizontale Leiste von 32 JCheckBox-Objekten
2. Fuenf  JTextField-Objekte
3. Zwoelf JButton-Objekte

Die 32 CheckBoxen stellen die Bits einer float-Zahl dar. Der Benutzer kann
diese Bits einzeln an- und ausklicken. In den TextFields werden Teile der
float-Zahl (Vorzeichen, Exponent, Mantisse) und die float-Zahl "im Ganzen"
dargestellt. Der Benutzer kann die Inhalte von 3 der TextFields editieren.
Ausser float-Literalen wie 1.5 oder +123.456e-23 kann er auch Namen (null,
eins, Infinity, -Infinity, NaN, NaNN, min, minn, max) fuer die entsprechen-
den float-Werte eingeben. Gross/Kleinschreibung spielt dabei keine Rolle.
Der Name min bezeichnet die kleinste (nicht-normalisierte) und minn die
kleinste normalisierte float-Zahl.

Die Knoepfe (JButton-Objekte) sollen dem Benutzer bestimmte Veraenderungen
der float-Zahl erleichtern, z.B. alle Bits auf 1 setzen oder alle Bits der
Charakteristik auf 0 setzen etc.. Besonders interessant sind die Knoepfe
"Naechste float-Zahl" und "Vorige float-Zahl".
---------------------------------------------------------------------------
Struktur der Grabo-Elemente eines FloatBits01-Objekts (welche Grabo-Ele-
mente enthalten welche anderen Grabo-Elemente? Einrueckung bedeutet hier
"ist enthalten in"):

Objekt:                                           Typ des Objekts:
-------                                           ----------------
Das ganze Objekt                                  FloatBits01 (d.h. JFrame)
   checkBoxBox.......................................Box (horizontale)
      checkBoxTab[0]                                    JCheckBox
      checkBoxTab[1]                                    JCheckBox
      ...                                               ...
      checkBoxTab[31]                                   JCheckBox
   boxBox.........................................Box (horizontale)
      textFieldBox                                   Box (vertikale)
         boxTab[0]                                      Box (horizontale)
            labelTab [0] ("Vorzeichen:")                   JLabel
            textField[0]                                   JTextField
         boxTab[1]                                      Box (horizontale)
            labelTab [1] ("Exponent ...:")                 JLabel
            textField[1]                                   JTextField
         ...                                               ...
         boxTab[4]                                      Box (horizontale)
            labelTab [4] ("Die Float-Zahl ...:")           JLabel
            textField[4]                                   JTextField
      button01Box....................................Box (vertikale)
         buttonTab[0]  ("Alle Bits an")                 JButton
         buttonTab[1]  ("Alle Bits aus")                JButton
         ...                                            ...
         buttonTab[5]  ("Mantissen-Bits aus")           JButton
      button02Box....................................Box (vertikale)
         buttonTab[6]  ("Naechste float-Zahl")          JButton
         buttonTab[7]  ("Vorige float-Zahl")            JButton
         ...                                            ...
         buttonTab[11] ("Durch 0.1")                    JButton
   ohneExBox......................................Box
      ohneExLab                                      JLabel
      ohneExTextField                                JTextField

Die meisten Grabo-Elemente sind als Komponenten von Reihungen (arrays)
realisiert, um ihre Bearbeitung mit Hilfe von Schleifen zu ermoeglichen.
------------------------------------------------------------------------ */
// Import-Befehle (ohne Sternchen!):
import java .awt        .Component;
import java .awt        .Dimension;
import java .awt        .Color;
import java .awt  .event.  ItemListener;
import java .awt  .event.ActionListener;
import java .awt  .event.WindowListener;
import java .awt  .event.WindowAdapter;
import java .awt  .event.  ItemEvent;
import java .awt  .event.ActionEvent;
import java .awt  .event.WindowEvent;
import javax.swing      .Box;
import javax.swing      .JButton;
import javax.swing      .JCheckBox;
import javax.swing      .JFrame;
import javax.swing      .JLabel;
import javax.swing      .JTextField;
import javax.swing      .SwingConstants;
import java .util       .Hashtable;
import java .text       .DecimalFormat;
import java .text       .ParseException;
// ------------------------------------------------------------------------
class FloatBits01 extends JFrame {
   /* ---------------------------------------------------------------------
   Struktur dieser Klassen-Vereinbarung:
   1.   Einige Globale Konstanten
   2.   Grabo-Elemente (und Zubehoer)
   2.1.   Fuenf Box-Objekte
   2.2.   Die Reihung checkBoxTab mit 32 JCheckBox-Komponenten
   2.3.   Initialisierer fuer die checkBoxTab
   2.4.   ActionListener-Objekte fuer die textFieldTab
   2.5.   Fuenf Reihungen namens ...Tab (JTextField-Objekte etc.)
   2.6.   Initialisierer fuer die fuenf Reihungen
   2.7.   Ein JTextField-Objekt, formatierte Ein-/Ausgabe der float-Zahl
   2.8.   ActionListener-Objekte fuer die buttonTab
   2.9.   Drei Reihungen namens ...Tab (JButton-Objekte etc.)
   2.10.  Initialisierer fuer die drei Reihungen
   3.   Ein Konstruktor
   4.   Alle Objektmethoden (nach Namen alphabetisch sortiert):
   4.1.   void   bearbeiteFloat(String s)
   4.2.   void   bearbeiteMantisse(String s)
   4.3.   void   eingabeCheckBoxen()
   4.4.   String floatNachString2(float f)
   4.5.   int    getBits()
   4.6.   int    getBits(int von, int bis)
   4.7.   float  getFloat()
   4.8.   String gruppiere  (String s)
   4.9.   String ohneTrenner(String s)
   4.10.  void   setBits(int maske)
   4.11.  void   setBits(int maske, int von, int bis)
   4.12.  void   setFloat(float f)
   4.13.  float  stringNachFloat1(String s)
   4.14.  float  stringNachFloat2(String s)
   4.14.  int    stringNachInt1  (String s)
   5. Die Klassenmethode main
   --------------------------------------------------------------------- */
   // 1. Globale Konstanten:
   final int   ANZAHL_BITS    = 32; // Laenge einer float-Zahl: 32 Bit
   final int     MANTISSE_VON =  0; // Mantisse:       von Bit  0
   final int     MANTISSE_BIS = 22; //                 bis Bit 22
   final int    CHARAKTER_VON = 23; // Charakteristik: von Bit 23
   final int    CHARAKTER_BIS = 30; //                 bis Bit 30
   final int   VORZEICHEN_VON = 31; // Vorzeichen:     von Bit 31
   final int   VORZEICHEN_BIS = 31; //                 bis Bit 31

   final int   ANZ_TEXTFIELDS =  5; // Anzahl JTextField-Objekte
   final int   ANZ_KNOEPFE    = 12; // Anzahl    JButton-Objekte
   final Color FARBE01        = Color.white;
   final Color FARBE02        = Color.green;
   final Color FARBE03        = Color.red;

   // 2. Grabo-Elemente (und Zubehoer)

   // 2.1. Das Gesamtfenster enthaelt 2 Box-Objekte, die zweite Box
   // enthalet drei weitere Box-Objekte:
   Box  checkBoxBox = Box.createHorizontalBox(); // Im Gesamtfenster
   Box       boxBox = Box.createHorizontalBox(); // Im Gesamtfenster
   Box textFieldBox = Box.createVerticalBox();   // In der boxBox
   Box  button01Box = Box.createVerticalBox();   // In der boxBox
   Box  button02Box = Box.createVerticalBox();   // In der boxBox

   // 2.2. Die 32 Bits der float-Zahl werden durch 32 JCheckBox-Objekte
   // dargestellt:
   JCheckBox[] checkBoxTab = new JCheckBox[ANZAHL_BITS];
   // ---------------------------------------------------------------------
   // Ein ItemListener-Objekt fuer die JCheckBox-Objekte in der checkBoxTab:
   ItemListener itemListenerFuerCheckBoxen = new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
         eingabeCheckBoxen();
      } // itemStateChanged
   }; // itemListenerFuerCheckBoxen
   // ---------------------------------------------------------------------
   { // 2.3. Initialisierer fuer die checkBoxTab:

      for (int i=ANZAHL_BITS-1; i>=0; i--) {
         // Zwei kleine Zwischenraeume zum Trennen der drei Teile
         // Vorzeichen, Exponent und Mantisse:
         if (i==ANZAHL_BITS-1 || i==CHARAKTER_BIS || i==MANTISSE_BIS) {
            checkBoxBox.add(Box.createHorizontalStrut(15));
         }
         // Die einzelnen JCheckBox-Objekte erzeugen, "einrichten" und in
         // die checkBoxBox einfuegen:
         checkBoxTab[i] = new JCheckBox();
         checkBoxTab[i].setBackground(FARBE01);
         checkBoxTab[i].setVerticalTextPosition  (SwingConstants.BOTTOM);
         checkBoxTab[i].setHorizontalTextPosition(SwingConstants.CENTER);
         checkBoxTab[i].setText("" + i);
         checkBoxTab[i].addItemListener(itemListenerFuerCheckBoxen);
         checkBoxBox.add(checkBoxTab[i]);
      } // for
   } // Initialisierer fuer die checkBoxTab
   // ---------------------------------------------------------------------
   // 2.4. ActionListener-Objekte fuer die TextFields (d.h. fuer die
   // JTextField-Komponenten der Reihung textFieldTab (siehe unten):

      ActionListener actionListenerVorzeichen = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Das JTextField fuer das Vorzeichen wurde veraendert:
            String s1         = textFieldTab[VORZEICHEN_INDEX].getText();
            int    vorzeichen = stringNachInt1(s1);
            setBits(vorzeichen, VORZEICHEN_VON, VORZEICHEN_BIS);
            eingabeCheckBoxen();
         } // actionPerformed
      }; // ActionListener actionListenerVorzeichen

      ActionListener actionListenerExponent = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Das JTextField fuer den Exponenten wurde veraendert:
            String s1       = textFieldTab[CHARAKTER_INDEX].getText();
            int    exponent = stringNachInt1(s1);

            // Falsche Eingaben durch 0 ersetzen:
            if ( (exponent < -127) || (+128 < exponent) ) exponent = 0;
            int charakteristik = exponent + 127;
            setBits(charakteristik, CHARAKTER_VON, CHARAKTER_BIS);
            eingabeCheckBoxen();
         } // actionPerformed
      }; // ActionListener actionListenerExponent

      ActionListener actionListenerFloatZahl = new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            // Das JTextField fuer die float-Zahl wurde veraendert:
            String s1 = textFieldTab[FLOAT_ZAHL_INDEX].getText();
            bearbeiteFloat(s1);
         } // actionPerformed
      }; // ActionListener actionListenerFloatZahl
   // ---------------------------------------------------------------------
   // 2.5. Fuenf Reihungen namens ...Tab:
   String        [] labelTextTab  = new String        [ANZ_TEXTFIELDS];
   JLabel        []     labelTab  = new JLabel        [ANZ_TEXTFIELDS];
   JTextField    [] textFieldTab  = new JTextField    [ANZ_TEXTFIELDS];
   Box           []       boxTab  = new Box           [ANZ_TEXTFIELDS];
   ActionListener[] textActionTab = new ActionListener[ANZ_TEXTFIELDS];

   // Konstanten fuer die Indizes der fuenf ...Tab-Reihungen:
   final int VORZEICHEN_INDEX = 0;
   final int  CHARAKTER_INDEX = 1;
   final int MANTISSEC0_INDEX = 2; // Mantisse mit Charakteristik == 0
   final int MANTISSEC1_INDEX = 3; // Mantisse mit Charakteristik != 0
   final int FLOAT_ZAHL_INDEX = 4; // Die float-Zahl insgesamt

   // Die Groesse der JLabel-Objekte in der labelTab:
   Dimension labelDimension = new Dimension(170, 20);
   // Die Groesse der JTextField-Objekte in der textFieldTab:
   Dimension textFieldDimension = new Dimension(150, 30);

   { // 2.6. Initialisierer fuer die fuenf ...Tab-Reihungen:

      // Die Reihung labelTextTab initialisieren:
      labelTextTab [VORZEICHEN_INDEX] = "Vorzeichen:  ";
      labelTextTab [ CHARAKTER_INDEX] = "Exponent zur Basis 2:  ";
      labelTextTab [MANTISSEC0_INDEX] = "Mantisse (mit c   gleich 0):  ";
      labelTextTab [MANTISSEC1_INDEX] = "Mantisse (mit c ungleich 0):  ";
      labelTextTab [FLOAT_ZAHL_INDEX] = "Die float-Zahl im Ganzen:  ";

      // Die Reihung textActionTab initialisieren:
      textActionTab[VORZEICHEN_INDEX] = actionListenerVorzeichen;
      textActionTab[ CHARAKTER_INDEX] = actionListenerExponent;
      textActionTab[MANTISSEC0_INDEX] = null;
      textActionTab[MANTISSEC1_INDEX] = null;
      textActionTab[FLOAT_ZAHL_INDEX] = actionListenerFloatZahl;

      // Die Reihungen labelTab, textFieldTab und boxTab initialisieren:
      for (int i=0; i<ANZ_TEXTFIELDS; i++) {
             labelTab[i] = new JLabel(labelTextTab[i]);
         textFieldTab[i] = new JTextField(10);
               boxTab[i] = Box.createHorizontalBox();
             labelTab[i].setMaximumSize(labelDimension);
             labelTab[i].setHorizontalAlignment(SwingConstants.RIGHT);
         textFieldTab[i].setMaximumSize(textFieldDimension);
         textFieldTab[i].setMinimumSize(textFieldDimension);
         textFieldTab[i].addActionListener(textActionTab[i]);
               boxTab[i].add              (labelTab     [i]);
               boxTab[i].add              (textFieldTab [i]);
         // Die neue Box mit dem TextField und dem Label darin wird in
         // die textFieldBox eingefuegt:
         textFieldBox.add(boxTab[i]);
      } // for
      textFieldTab[MANTISSEC0_INDEX].setEditable(false);
      textFieldTab[MANTISSEC1_INDEX].setEditable(false);
      textFieldTab[FLOAT_ZAHL_INDEX].setBackground(FARBE02);
   } // Initialisierer fuer die fuenf ...Tab-Reihungen, Ende
   // ---------------------------------------------------------------------
   // 2.7. Ein JTextField-Objekt fuer die formatierte Ein-/Ausgabe
   // der floatZahl
   JLabel     ohneExLab       = new JLabel("Die float-Zahl formatiert:  ");
   JTextField ohneExTextField = new JTextField(0);
   Box        ohneExBox       = Box.createHorizontalBox();
   Dimension  ohneExDim       = new Dimension(600, 30);

      //          0         1         2         3         4         5
   String form = "###################################################" +
                 "0.0" +
                 "###################################################";
   DecimalFormat df = new DecimalFormat(form);

   ActionListener actionListenerohneEx = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         String s = ohneExTextField.getText();
         setFloat(stringNachFloat2(s));
         eingabeCheckBoxen();
      } // actionPerformed
   }; // ActionListener actionListenerohneEx

   {  // Initialisierer fuer das JTextField ohneExTextField:
      ohneExTextField.addActionListener(actionListenerohneEx);
      ohneExTextField.setPreferredSize(ohneExDim);
      ohneExTextField.setMaximumSize  (ohneExDim);
      ohneExTextField.setBackground(FARBE02);;
      ohneExBox.add(ohneExLab);
      ohneExBox.add(ohneExTextField);
      this.getContentPane().add(ohneExBox, "South");
      df.setMaximumIntegerDigits (50);
      df.setMinimumIntegerDigits ( 1);
      df.setMaximumFractionDigits(50);
      df.setMinimumFractionDigits( 1);
   }  // Initialisierer fuer das JTextField ohneExTextField
   // ---------------------------------------------------------------------
   // 2.8. ActionListener-Objekte fuer die Knoepfe (d.h. die JButton-Kompo-
   // nenten der Reihung buttonTab (siehe unten):
   ActionListener alleAn = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0xFFFFFFFF);
      }
   };

   ActionListener alleAus = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0x00000000);
      }
   };

   ActionListener charakterAn = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0xFFFFFFFF, CHARAKTER_VON, CHARAKTER_BIS);
      }
   };

   ActionListener charakterAus = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0x00000000, CHARAKTER_VON, CHARAKTER_BIS);
      }
   };

   ActionListener mantisseAn = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0xFFFFFFFF, MANTISSE_VON, MANTISSE_BIS);
      }
   };

   ActionListener mantisseAus = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setBits(0x00000000, MANTISSE_VON, MANTISSE_BIS);
      }
   };

   ActionListener nextFloat = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         int intZahl = getBits();
         intZahl++;
         setBits(intZahl);
      }
   };

   ActionListener prevFloat = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         int intZahl = getBits();
         intZahl--;
         setBits(intZahl);
      }
   };

   ActionListener mal2 = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setFloat(getFloat() * 2.0F);;
      }
   };

   ActionListener durch2 = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setFloat(getFloat() / 2.0F);;
      }
   };

   ActionListener mal1Zehntel = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setFloat(getFloat() * 0.1F);;
      }
   };

   ActionListener durch1Zehntel = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
         setFloat(getFloat() / 0.1F);;
      }
   };
   // ---------------------------------------------------------------------
   // 2.9. Drei Reihungen namens ...Tab:
   ActionListener[] buttonActionTab = new ActionListener[ANZ_KNOEPFE];
   String        [] buttonTextTab   = new String        [ANZ_KNOEPFE];
   JButton       [] buttonTab       = new JButton       [ANZ_KNOEPFE];

   {  // 2.10. Initialisierer fuer die drei Reihungen namens ...Tab
      // (buttonTextTab, buttonActionTab und buttonTab):

      // Die Reihung buttonTextTab (mit den Beschriftungen fuer die
      // einzelnen Knoepfe darin) initialisieren:
      buttonTextTab[0]  = "Alle Bits an";
      buttonTextTab[1]  = "Alle Bits aus";
      buttonTextTab[2]  = "Charakteristik-Bits an";
      buttonTextTab[3]  = "Charakteristik-Bits aus";
      buttonTextTab[4]  = "Mantissen-Bits an";
      buttonTextTab[5]  = "Mantissen-Bits aus";
      buttonTextTab[6]  = "Naechste float-Zahl";
      buttonTextTab[7]  = "Vorige   float-Zahl";
      buttonTextTab[8]  = "Mal   2.0";
      buttonTextTab[9]  = "Durch 2.0";
      buttonTextTab[10] = "Mal   0.1";
      buttonTextTab[11] = "Durch 0.1";

      // Die Reihung buttonAction (mit den ActionListener-Objekten fuer die
      // einizelnen Knopfe darin) initialisieren:
      buttonActionTab[ 0] = alleAn;      buttonActionTab[ 1] = alleAus;
      buttonActionTab[ 2] = charakterAn; buttonActionTab[ 3] = charakterAus;
      buttonActionTab[ 4] = mantisseAn;  buttonActionTab[ 5] = mantisseAus;
      buttonActionTab[ 6] = nextFloat;   buttonActionTab[ 7] = prevFloat;
      buttonActionTab[ 8] = mal2;        buttonActionTab[ 9] = durch2;
      buttonActionTab[10] = mal1Zehntel; buttonActionTab[11] = durch1Zehntel;

      // Die Groesse der JButton-Objekte in der buttonTab:
      Dimension buttonDimension = new Dimension(180, 30);

      // Die buttonTab initialisieren und die JButton-Objekte in die
      // button01Box bzw. in die button02Box einfuegen:
      for (int i=0; i<buttonTab.length; i++) {
         buttonTab[i] = new JButton    (buttonTextTab  [i]);
         buttonTab[i].addActionListener(buttonActionTab[i]);
         buttonTab[i].setMaximumSize(buttonDimension);
         if (i<6) {
            button01Box.add(buttonTab[i]);
         } else {
            button02Box.add(buttonTab[i]);
         }
      } // for

   }  // Initialisierer fuer die drei Reihungen namens ...Tab
   // ---------------------------------------------------------------------
   // 3. Ein Konstruktor:
   public FloatBits01(String ueberschrift) {            //  <-- Konstruktor
      super(ueberschrift);

      // Den windowClosing-Knopf aktivieren:
      addWindowListener(new WindowAdapter() {
         public void windowClosing (WindowEvent evt) {
            System.exit(0);
         }
      });

      // Position und Groesse des Fensters festlegen:
      this.setBounds(20, 10, 770, 330);

      // Drei Box-Objekte in das Objekt boxBox einfuegen:
      boxBox.add(Box.createHorizontalStrut(20));
      boxBox.add(textFieldBox);
      boxBox.add(Box.createHorizontalStrut(20));
      boxBox.add(button01Box);
      boxBox.add(Box.createHorizontalStrut(20));
      boxBox.add(button02Box);
      boxBox.add(Box.createHorizontalStrut(20));

      // Zwei Box-Objekte in dieses Fenster einfuegen:
      this.getContentPane().add(checkBoxBox, "North");
      this.getContentPane().add(     boxBox, "Center");

      this.getContentPane().setBackground(FARBE01);

      // Die JTextField-Objekte initialisieren:
      eingabeCheckBoxen();
   } // Konstruktor FloatBits01                             <-- Konstruktor
   // ---------------------------------------------------------------------
   // Namen fuer spezielle float-Werte und die zugehoerigen Float-
   // Objekte (diese Hashtabelle kann erweitert werden):
   final Hashtable<String, Float> htab = new Hashtable<String, Float>();

   {  // Initialisierer fuer die htab:
      htab.put( "eins",      new Float( 1.0F));
      htab.put( "null",      new Float( 0.0F));
      htab.put("-eins",      new Float(-1.0F));
      htab.put("-null",      new Float(-0.0F));
      htab.put( "infinity",  new Float( Float.POSITIVE_INFINITY));
      htab.put("-infinity",  new Float( Float.NEGATIVE_INFINITY));
      htab.put( "nan",       new Float( Float.NaN));
      htab.put("-nan",       new Float(-Float.NaN));
      htab.put( "nann",      new Float( Float.intBitsToFloat(0xFFFFFFFF)));
      htab.put( "min",       new Float( Float.MIN_VALUE));
      htab.put( "minn",      new Float( Float.intBitsToFloat(0x00800000)));
      htab.put( "max",       new Float( Float.MAX_VALUE));
      htab.put("-min",       new Float(-Float.MIN_VALUE));
      htab.put("-minn",      new Float(-Float.intBitsToFloat(0x00800000)));
      htab.put("-max",       new Float(-Float.MAX_VALUE));
      // min  ist die (betragsmaessig) kleinste               float-Zahl
      // minn ist die (betragsmaessig) kleinste normalisierte float-Zahl
   } // Initialisierer fuer die htab

   void bearbeiteFloat(String s) {
      // 4.1. Der String s wird in einen float-Wert umgewandelt. Der wird
      // (als Bitkette) in die Reihung checkBoxTab uebertragen.
      // ------------------------------------------------------------------
      // In htab nach einem Float-Objekt mit Schluessel s suchen:
      Float flob = (Float) htab.get(s.toLowerCase());

      setFloat((flob==null) ? stringNachFloat1(s) : flob.floatValue());
      eingabeCheckBoxen();
   } // bearbeiteFloat
   // ---------------------------------------------------------------------
   void bearbeiteMantisse(String s) {
      // 4.2. Der String s wird in einen float-Wert umgewandelt. Dessen
      // Mantisse wird (als Bitkette) in die Reihung checkBoxTab ueber-
      // tragen.
      float mantFloat = stringNachFloat1(s);
      int   mantInt   = Float.floatToRawIntBits(mantFloat);
      setBits(mantInt, MANTISSE_VON, MANTISSE_BIS);
      eingabeCheckBoxen();
   } // bearbeiteMantisse
   // ---------------------------------------------------------------------
   void eingabeCheckBoxen() {
      // 4.3. Eine JCheckBox wurde vom Benutzer veraendert. Die Veraenderung
      // wird in die JTextField-Objekte in der textFieldTab uebertragen.

      final int maskeCharakteristik127 = 0x3F800000; // Exponent 0

      int intZahl    = getBits(); // Alle Bits nach intZahl
      int vorzeichen = getBits(VORZEICHEN_VON, VORZEICHEN_BIS);
      int charakter  = getBits( CHARAKTER_VON,  CHARAKTER_BIS);
      int mantisseC0 = getBits(  MANTISSE_VON,   MANTISSE_BIS);
      int mantisseC1 = mantisseC0 | maskeCharakteristik127;
      int exponent   = charakter - 127; // Exponent = Charakteristik - 127

      // Bitketten in float-Werte umwandeln:
      float mantisseF0 = Float.intBitsToFloat(mantisseC0);
      float mantisseF1 = Float.intBitsToFloat(mantisseC1);
      float floatZahl  = Float.intBitsToFloat(intZahl);

      // Die neuen Werte in die JTextField-Objekte bringen:
      textFieldTab[VORZEICHEN_INDEX].setText(vorzeichen + "");
      textFieldTab[ CHARAKTER_INDEX].setText(exponent   + "");
      textFieldTab[MANTISSEC0_INDEX].setText(mantisseF0 + "");
      textFieldTab[MANTISSEC1_INDEX].setText(mantisseF1 + "");
      textFieldTab[FLOAT_ZAHL_INDEX].setText(floatZahl  + "");

      // Je nach Charakteristik (ob gleich 0 oder ungleich 0) eines von
      // zwei JTextField-Objekten nicht-editierbar machen (und damit
      // grau hinterlegen):
      if (charakter == 0) {
         textFieldTab[MANTISSEC0_INDEX].setBackground(FARBE02);
         textFieldTab[MANTISSEC1_INDEX].setBackground(FARBE03);
      } else {
         textFieldTab[MANTISSEC0_INDEX].setBackground(FARBE03);
         textFieldTab[MANTISSEC1_INDEX].setBackground(FARBE02);
      }

      // Die float-Zahl formatiert ("ohne Exponent") in ein JTextField-
      // Objekt ausgeben:
      ohneExTextField.setText(floatNachString2(floatZahl));
   } // eingabeCheckBoxen
   // ---------------------------------------------------------------------
   String floatNachString2(float f) {
      // 4.4. Wandelt f in einen String um, in dem die Ziffern zu Gruppen
      // zusammengefasst sind (um die Lesbarkeit zu erhoehen).
      String erg = gruppiere(df.format(f));
      return erg;
   } // floatNachString2
   // ---------------------------------------------------------------------
   int getBits() {
      // 4.5. Liefert die Zustaende aller JCheckBox-Objekte in der Reihung
      // checkBoxTab als int-Wert:
      return getBits(0, ANZAHL_BITS-1);
   } // getBits
   // ---------------------------------------------------------------------
   int getBits(int von, int bis) {
      // 4.6. Liefert die Zustaende der JCheckBox-Objekte checkBoxTab[von],
      // checkBoxTab[von+1], ... , checkBoxTab[bis] in den entsprechend
      // vielen niedrigstwertigen ("rechtesten") Bits des Ergebnisses.

      int intZahl = 0;
      int  einBit = 1;            // 2 hoch 0, nur Bit 0 ist an.
      for (int i=von; i<=bis; i++) {
         if ((checkBoxTab[i]).isSelected()) {
            intZahl |= einBit;    // Ein Bit anknipsen
         }
         einBit <<= 1;
      }
      return intZahl;
   } // getBits
   // ---------------------------------------------------------------------
   float getFloat() {
      // 4.7. Liefert die Zustaende aller JCheckBox-Objekte in der Reihung
      // checkBoxTab als float-Wert:
      int    intZahl   = getBits();
      return Float.intBitsToFloat(intZahl);
   } // getFloat
   // ---------------------------------------------------------------------
   String gruppiere(String s) {
      // 4.8. Liefert eine Kopie von s, in der Gruppen zu je 3 Ziffern
      // durch ein Trennzeichen voneinander getrennt sind. Falls s ein
      // Dezimalkomma enthaelt, wird es dabei beruecksichtigt.
      final char MINUS   = '-'; // Minuszeichen
      final char TRENNER = '_'; // Unterstrich
      int        vor     = s.indexOf(",");       // Anzahl  Vorkommastellen
      int        nach    = s.length() - vor - 1; // Anzahl Nachkommastellen
      int        bis     = (s.charAt(0) == MINUS) ? 1 : 0;
      // Die Variable bis verhindert Formatierungen wie "-_123,0"

      StringBuffer erg = new StringBuffer(s);

      int ind = nach % 3;
      if (ind == 0) ind = 3;
      ind = s.length() - ind;
      // Von rechts nach links Trennzeichen einfuegen, erst
      // nach dem Komma, dann vor dem Komma:
      for (         ; ind>vor+1; ind -= 3) erg.insert(ind, TRENNER);
      for (ind=vor-3; ind>bis  ; ind -= 3) erg.insert(ind, TRENNER);

      return erg.toString();
   } // gruppiere
   // ---------------------------------------------------------------------
   String ohneTrenner(String s) {
      // 4.9. Liefert eine Kopie von s, aus der alle Trennzeichen entfernt
      // wurden.
      final char TRENNER1 = '_'; // Trennzeichen
      final char TRENNER2 = '.'; // Trennzeichen
      StringBuffer erg = new StringBuffer(s);
      for (int i=erg.length()-1; i>=0; i--) {
         char c = erg.charAt(i);
         if (c == TRENNER1 || c == TRENNER2) erg.deleteCharAt(i);
      }
      return erg.toString();
   } // ohneTrenner
   // ---------------------------------------------------------------------
   void setBits(int maske) {
      // 4.10. Setzt die Zustaende aller JCheckBox-Objekte in der Reihung
      // checkBoxTab entsprechend den Bits in der maske.
      setBits(maske, 0, ANZAHL_BITS-1);
   } // setBits
   // ---------------------------------------------------------------------
   void setBits(int maske, int von, int bis) {
      // 4.11. Setzt die Zustaende der JCheckBox-Objekte checkBoxTab[von],
      // checkBoxTab[von+1], ... , checkBoxTab[bis] entsprechend den
      // niedrigstwertigen ("rechtesten") Bits der maske.

      int einBit = 1; // 2 hoch 0, nur Bit 0 ist an
      for (int i=von; i<=bis; i++) {
         if ((maske & einBit) == 0) {
            checkBoxTab[i].setSelected(false);
         } else {
            checkBoxTab[i].setSelected(true);
         }
         einBit <<= 1;
      }
   } // setBits
   // ---------------------------------------------------------------------
   void setFloat(float f) {
      // 4.12. Setzt die Zustaende aller JCheckBox-Objekte in der Reihung
      // checkBoxTab entsprechend dem float-Wert f.
      int intZahl = Float.floatToRawIntBits(f);
      setBits(intZahl);
   } // setFloat
   // ---------------------------------------------------------------------
   float stringNachFloat1(String s) {
      // 4.13. Liefert einen dem String s entsprechenden float-Wert (oder
      // den Wert 0.0F, falls s Fehler enthaelt).
      float erg = 0.0F;
      try {
         erg = Float.parseFloat(s);
      }
      catch (NumberFormatException e) {};
      return erg;
   } // stringNachFloat1
   // ---------------------------------------------------------------------
   float stringNachFloat2(String s) {
      // 4.14. Enternt aus s alle Trennzeichen und wandelt das Ergebnis in eine
      // float-Zahl um. Falls ein Fehler auftritt, wird die Zahl 0.0F als
      // Ergebnis geliefert.
      String ohne = ohneTrenner(s);
      float floatZahl = 0.0F;
      try {
         floatZahl = df.parse(ohne).floatValue();
      }
      catch (ParseException e) {}
      return floatZahl;
   } // stringNachFloat2
   // ---------------------------------------------------------------------
   int stringNachInt1(String s) {
      // 4.15. Liefert einen dem String s entsprechenden int-Wert (oder
      // den Wert 0, falls s Fehler enthaelt).
      int erg = 0;
      try {
         erg = Integer.parseInt(s);
      }
      catch (NumberFormatException e) {}
      return erg;
   } // stringNachInt1
   // ---------------------------------------------------------------------
   static public void main(String[] susi) {
      // Erzeugt ein FloatBits01-Objekt und macht es sichtbar:
      FloatBits01 fb01 = new FloatBits01("Die Bits einer float-Zahl:");
      fb01.setVisible(true);
   } // main

} // class FloatBits01
// ------------------------------------------------------------------------