// Datei For02.java
/* ------------------------------------------------------------------------
Beispiele fuer typische for-Schleifen. Jede der 7 Methoden anzAhs, anzZiff,
anzBuchstaben, anzZiffern, einMalN, durch3TeilbareZahlen und zweierPotenzen
enthaelt eine for-Schleife.
------------------------------------------------------------------------ */
class For02 {
   // ---------------------------------------------------------------------
   static int anzAhs(String s) {
      // Wie oft kommt das Zeichen 'A' in s vor? Diese Funktion liefert
      // die Anwort.

      // Der Ausdruck s.length() bezeichnet die Laenge des Strings s.
      //
      // Die einzelnen Zeichen eines Strings sind mit 0 beginnend
      // durchnummeriert (oder: indiziert). Das letzte Zeichen des
      // Strings s hat somit den Index s.length() - 1.
      //
      // Der Ausdruck s.charAt(0) bezeichnet das 0. Zeichen von s.
      // Der Ausdruck s.charAt(1) bezeichnet das 1. Zeichen von s.
      // Der Ausdruck s.charAt(2) bezeichnet das 2. Zeichen von s.
      // ...
      // Der Ausdruck s.charAt(i) bezeichnet das i. Zeichen von s.

      int anzahl = 0;

      for (int i=0; i<s.length(); i++) {
         if (s.charAt(i) == 'A') anzahl++;
      } // for

      return anzahl;
   } // anzAhs
   // ---------------------------------------------------------------------
   static int anzZiff(String s) {
      // Wieviele Dezimalziffern ('0' bis '9') kommen in s vor? Diese
      // Funktion liefert die Antwort.

      // Das doppelte Ampersand && bezeichnet die und-Verknuepfung. Der
      // Ausdruck  '0' <= c && c <= '9' hat genau dann den Wert true,
      // wenn die Variablen c eine Dezimalziffer enthaelt.

      int anzahl = 0;

      for (int i=0; i<s.length(); i++) {
         char c = s.charAt(i);
         if ('0' <= c && c <= '9') anzahl++;
      } // for

      return anzahl;
   } // anzZiff
   // ---------------------------------------------------------------------
   static int anzBuchstaben() {
      // Es gibt 65536 char-Werte (Zeichen). Wieviele davon gelten als
      // Buchstaben, d.h. fuer wieviele davon liefert die Funktion
      // Character.isLetter den Wert true? Diese Funktion liefert die
      // Anwort.

      int anzahl = 0;

      // Achtung: In der folgenden Schleife werden nicht alle char-Werte
      // geprueft, sondern nur alle bis auf den groessten. Es ist nicht
      // ganz einfach (aber durchaus moeglich) die Schleife so zu aendern,
      // dass wirklich alle char-Werte geprueft werden.
      // Hinweis: Der groesste char-Wert (Character.MAX_VALUE) ist kein
      // Buchstabe!

      // Die Konstante Character.MIN_VALUE bezeichnet den kleinsten
      // Wert des Typs char (d.h. den Wert 0), die Konstante
      // Character.MAX_VALUE bezeichnet den groessten Wert des Typs
      // char (d.h. den Wert 65535).

      for (char c=Character.MIN_VALUE; c<Character.MAX_VALUE; c++) {
         if (Character.isLetter(c))  anzahl++;
      } // for

      return anzahl;
   } // anzBuchstaben
   // ---------------------------------------------------------------------
   static int anzZiffern() {
      // Es gibt 65536 char-Werte (Zeichen). Wieviele davon gelten als
      // Ziffern, d.h. fuer wieviele davon liefert die Funktion
      // Character.isDigit den Wert true? Diese Funktion liefert die
      // Anwort.

      int anzahl = 0;

      // Achtung: In der folgenden Schleife werden nicht alle char-Werte
      // geprueft, sondern nur alle bis auf den groessten. Es ist nicht
      // ganz einfach (aber durchaus moeglich) die Schleife so zu aendern,
      // dass wirklich alle char-Werte geprueft werden.
      // Hinweis: Der groesste char-Wert (Character.MAX_VALUE) ist keine
      // Ziffer!

      for (char c=Character.MIN_VALUE; c<Character.MAX_VALUE; c++) {
         if (Character.isDigit(c))  anzahl++;
      } // for

      return anzahl;
   } // anzZiffern
   // ---------------------------------------------------------------------
   static void einMalN(int n1) {
      // Gibt eine Zeile des kleinen Ein-mal-eins aus, mit n1 als
      // erstem Faktor.

      p(n1 + "|");
      for (int n2=1; n2<=9; n2++) {
         p("x" + n2 + "=" + (n1*n2) + "|");
      } // for
      pln();
   } // einMalN
   // ---------------------------------------------------------------------
   static void durch3TeilbareZahlen(int n) {
      // Gibt alle durch 3 teilbaren Zahlen zwischen n und n+30 aus.

      p("Ab " + n + ": ");

      // Der Ausdruck (n+2) / 3 * 3 bezeichnet die kleinste durch 3
      // teilbare Zahl, die groesser oder gleich n ist.
      //
      // m += 3 ist eine Abkuerzung fuer m = m + 3.

      for (int m= (n+2) / 3 * 3;  m<=n+30;  m += 3) {
         p(m + " ");
      } // for
      pln("");
   } // vielfacheVonN
   // ---------------------------------------------------------------------
   static void zweierPotenzen() {
      // Gibt alle Zweierpotenzen (1, 2, 4, 8, 16, ...) aus, die sich
      // als Werte des Typs long darstellen lassen.

      // Im Initialisierungsteil der folgenden for-Schleife werden zwei
      // Variablen (e wie "Exponent" und p wie "Potenz") vereinbart, und
      // nicht nur eine, wie in den voherigen for-Schleifen.
      // Im Fortschaltungsteil werden beiden Variablen e und p veraendert
      // ("fortgeschaltet").
      //
      // Der Pruefteil p>0 funktioniert so: Wenn man p sehr oft mit 2
      // multipliziert gibt es irgendwann einen Ueberlauf und p nimmt
      // einen negativen Wert an (Long.MAX_VALUE + 1 ist gleich
      // Long.MIN_VALUE und Long.MIN_VALUE - 1 ist gleich Long.MAX_VALUE).

      for (long e=0, p=1;  p>0;  p*=2, e++) {
         pln("2 hoch " + e + " ist gleich " + p);
      } // for
   } // zweierPotenzen

   // Hier noch unschoen: e wird erst als ein Zeichen (0 bis 9) und dann
   // als zwei Zeichen (10, 11, 12, ...) ausgegeben, statt immer in der
   // selben Laenge. Verbesserung sind mit dem Befehl System.out.printf
   // (siehe Format01 etc.) oder mit AM.p (siehe AM und AMTst) moeglich.
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die oben vereinbarten Methoden anzAhs, anzZiff etc. werden
      // aufgerufen.

      final char AZ = '"'; // Ein doppeltes Anfuehrungszeichen

      // Die Funktion anzAhs wird mehrmals aufgerufen:
      pln("-------------------------------------------------------------");
      pln("A anzAhs(" + AZ + "ABCAAB" + AZ + "): " + anzAhs("ABCAAB"));
      pln("B anzAhs(" + AZ + "AAAAAA" + AZ + "): " + anzAhs("AAAAAA"));
      pln("C anzAhs(" + AZ + "aaaaaa" + AZ + "): " + anzAhs("aaaaaa"));
      pln("D anzAhs(" + AZ + "A"      + AZ + "): " + anzAhs("A"));
      pln("E anzAhs(" + AZ + ""       + AZ + "): " + anzAhs(""));

      // Die Funktion anzZiff wird mehrmals aufgerufen:
      pln("-------------------------------------------------------------");
      pln("F anzZiff(" + AZ + "1a4b32" + AZ + "): " + anzZiff("1a4b32"));
      pln("G anzZiff(" + AZ + "123654" + AZ + "): " + anzZiff("123654"));
      pln("H anzZiff(" + AZ + "abcdef" + AZ + "): " + anzZiff("abcdef"));
      pln("I anzZiff(" + AZ + "8"      + AZ + "): " + anzZiff("8"));
      pln("J anzZiff(" + AZ + ""       + AZ + "): " + anzZiff(""));

      // Die Funktionen anzBuchstaben und anzZiffern werden
      // einmal aufgerufen:
      pln("-------------------------------------------------------------");
      pln("K Anzahl isLetter: " + anzBuchstaben());
      pln("L Anzahl isDigit : " + anzZiffern());

      // Die Prozedur einMalN wird mehrmals aufgerufen:
      pln("-------------------------------------------------------------");
      p("M "); einMalN(6);
      p("N "); einMalN(7);
      p("O "); einMalN(8);

      // Die Prozedur durch3TeilbareZahlen wird mehrmals aufgerufen:
      pln("-------------------------------------------------------------");
      p("P "); durch3TeilbareZahlen(10);
      p("Q "); durch3TeilbareZahlen(11);
      p("R "); durch3TeilbareZahlen(12);
      p("S "); durch3TeilbareZahlen(13);
      p("T "); durch3TeilbareZahlen(14);
      p("U "); durch3TeilbareZahlen(15);

      // Die Prozedur zweierPotenzen wird einmal aufgerufen:
      pln("-------------------------------------------------------------");
      zweierPotenzen();
      pln("-------------------------------------------------------------");
      pln("For02: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class For02
/* ------------------------------------------------------------------------
Ausgabe des Programms For02:

-------------------------------------------------------------
A anzAhs("ABCAAB"): 3
B anzAhs("AAAAAA"): 6
C anzAhs("aaaaaa"): 0
D anzAhs("A"): 1
E anzAhs(""): 0
-------------------------------------------------------------
F anzZiff("1a4b32"): 4
G anzZiff("123654"): 6
H anzZiff("abcdef"): 0
I anzZiff("8"): 1
J anzZiff(""): 0
-------------------------------------------------------------
K Anzahl isLetter: 45851
L Anzahl isDigit : 208
-------------------------------------------------------------
M 6|x1=6|x2=12|x3=18|x4=24|x5=30|x6=36|x7=42|x8=48|x9=54|
N 7|x1=7|x2=14|x3=21|x4=28|x5=35|x6=42|x7=49|x8=56|x9=63|
O 8|x1=8|x2=16|x3=24|x4=32|x5=40|x6=48|x7=56|x8=64|x9=72|
-------------------------------------------------------------
P Ab 10: 12 15 18 21 24 27 30 33 36 39
Q Ab 11: 12 15 18 21 24 27 30 33 36 39
R Ab 12: 12 15 18 21 24 27 30 33 36 39 42
S Ab 13: 15 18 21 24 27 30 33 36 39 42
T Ab 14: 15 18 21 24 27 30 33 36 39 42
U Ab 15: 15 18 21 24 27 30 33 36 39 42 45
-------------------------------------------------------------
2 hoch 0 ist gleich 1
2 hoch 1 ist gleich 2
2 hoch 2 ist gleich 4
2 hoch 3 ist gleich 8
2 hoch 4 ist gleich 16
2 hoch 5 ist gleich 32
2 hoch 6 ist gleich 64
2 hoch 7 ist gleich 128
2 hoch 8 ist gleich 256
2 hoch 9 ist gleich 512
2 hoch 10 ist gleich 1024
2 hoch 11 ist gleich 2048
2 hoch 12 ist gleich 4096
2 hoch 13 ist gleich 8192
2 hoch 14 ist gleich 16384
2 hoch 15 ist gleich 32768
2 hoch 16 ist gleich 65536
2 hoch 17 ist gleich 131072
2 hoch 18 ist gleich 262144
2 hoch 19 ist gleich 524288
2 hoch 20 ist gleich 1048576
2 hoch 21 ist gleich 2097152
2 hoch 22 ist gleich 4194304
2 hoch 23 ist gleich 8388608
2 hoch 24 ist gleich 16777216
2 hoch 25 ist gleich 33554432
2 hoch 26 ist gleich 67108864
2 hoch 27 ist gleich 134217728
2 hoch 28 ist gleich 268435456
2 hoch 29 ist gleich 536870912
2 hoch 30 ist gleich 1073741824
2 hoch 31 ist gleich 2147483648
2 hoch 32 ist gleich 4294967296
2 hoch 33 ist gleich 8589934592
2 hoch 34 ist gleich 17179869184
2 hoch 35 ist gleich 34359738368
2 hoch 36 ist gleich 68719476736
2 hoch 37 ist gleich 137438953472
2 hoch 38 ist gleich 274877906944
2 hoch 39 ist gleich 549755813888
2 hoch 40 ist gleich 1099511627776
2 hoch 41 ist gleich 2199023255552
2 hoch 42 ist gleich 4398046511104
2 hoch 43 ist gleich 8796093022208
2 hoch 44 ist gleich 17592186044416
2 hoch 45 ist gleich 35184372088832
2 hoch 46 ist gleich 70368744177664
2 hoch 47 ist gleich 140737488355328
2 hoch 48 ist gleich 281474976710656
2 hoch 49 ist gleich 562949953421312
2 hoch 50 ist gleich 1125899906842624
2 hoch 51 ist gleich 2251799813685248
2 hoch 52 ist gleich 4503599627370496
2 hoch 53 ist gleich 9007199254740992
2 hoch 54 ist gleich 18014398509481984
2 hoch 55 ist gleich 36028797018963968
2 hoch 56 ist gleich 72057594037927936
2 hoch 57 ist gleich 144115188075855872
2 hoch 58 ist gleich 288230376151711744
2 hoch 59 ist gleich 576460752303423488
2 hoch 60 ist gleich 1152921504606846976
2 hoch 61 ist gleich 2305843009213693952
2 hoch 62 ist gleich 4611686018427387904
-------------------------------------------------------------
For02: Das war's erstmal!
------------------------------------------------------------------------ */
