// Datei For03.java
/* ------------------------------------------------------------------------
for-Schleifen, bei denen die Indexvariablen (hier wie ueblich i genannt)
nur zum Zaehlen gebraucht werden und im Rumpf nicht vorkommen.

Problem: Es sollen "Minuszeilen" angebbarer Laenge ausgegeben werden.

Hier drei Loesungen dieses Problems, zwei mit for-Schleifen und
eine ohne Schleife. Die dritte Loesung (plnMinusZeileC) ist ziemlich
kompliziert und nur dann noetig, wenn die zweite Loesung (plnMinusZeileB)
zu langsam ist.
------------------------------------------------------------------------ */
class For03 {
   // ---------------------------------------------------------------------
   static void plnMinusZeileA(int laenge) {
      // Gibt laenge viele Minuszeichen zur Standardausgabe aus und
      // rueckt den Cursor dann zum Anfang der naechsten Zeile vor.

      for (int i=1; i<=laenge; i++) p('-');
      pln();

   } // plnMinusZeileA
   // ---------------------------------------------------------------------
   static void plnMinusZeileB(int laenge) {
      // Gibt laenge viele Minuszeichen zur Standardausgabe aus und
      // rueckt den Cursor dann zum Anfang der naechsten Zeile vor.

      StringBuffer sb = new StringBuffer();

      for (int i=1; i<=laenge; i++) sb.append('-');
      pln(sb);

   } // plnMinusZeileB
   // ---------------------------------------------------------------------
   // Normalerweise ist plnMinusZeileB eine ausreichend gute Loesung des
   // Minuszeilen-Problems.
   // Nur wenn Zeitmessungen zeigen, dass die Prozedur plnMinusZeilenB
   // fuer ein bestimmten Anwendung zu langsam ist, sollte man die
   // folgende, ziemlich komplizierte aber moeglicherweise schnellere
   // Loesung in Erwaegung ziehen.
   //
   // Die Prozedur plnMinusZeileC greift auf eine globale Konstante
   // MINUS_STRING zu und vermeidet mit Hilfe der substring-Funktion
   // eine (moeglicherweise zeitaufwendige) Schleife.

   // Eine "offene Konstante" (blank final variable), deren Wert nicht
   // sofort sondern erst ein bisschen spaeter festgelegt wird:
   static private final String MINUS_STRING;

   // Ein statischer Initialisierer zum Initialisieren der
   // Konstanten MINUS_STRING.
   static {
      char[] minus_String = new char[250];
      java.util.Arrays.fill(minus_String, '-');
      MINUS_STRING = new String(minus_String);
   } // Ende des statischen Initialisierers

   static void plnMinusZeileC(int laenge) {
      // Gibt laenge viele Minuszeichen (maximal 250) zur Standardausgabe
      // aus und rueckt den Cursor dann zum Anfang der naechsten Zeile vor.
      laenge = Math.max(laenge, 0);
      laenge = Math.min(laenge, MINUS_STRING.length());
      pln(MINUS_STRING.substring(0, laenge));
   } // plnMInusC
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   static void p  (char   c)  {System.out.print  (c);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die oben vereinbarten Methoden plnMinusZeileA bis C werden
      // aufgerufen.

      p("A "); plnMinusZeileA(10);
      p("B "); plnMinusZeileB(15);
      p("C "); plnMinusZeileC(10);
      pln("-------------------------");
      pln("For03: Das war's erstmal!");   } // main
   // ---------------------------------------------------------------------
} // class For03
/* ------------------------------------------------------------------------
Ausgabe des Programms For03:

A ----------
B ---------------
C ----------
-------------------------
For03: Das war's erstmal!
------------------------------------------------------------------------ */