// Datei For04.java
/* ------------------------------------------------------------------------
Eine im Initialisierungsteil einer for-Schleife vereinbarte Indexvariable
wird vom Ausfuehrer zerstoert, sobald er die for-Schleife fertig ausge-
fuehrt hat. Wenn der Programmierer moechte, dass die Indexvariable auch
nach Ausfuehrung der Schleife noch existiert, muss er sie vor der Schleife
vereinbaren. Initialisieren kann er sie trotzdem im Initialisierungsteil
der for-Schleife.
------------------------------------------------------------------------ */
class For04 {
   // ---------------------------------------------------------------------
   static int teileDurchMinTeiler(int n) {
      // Ermittelt den kleinsten Teiler t von n und liefert die
      // Zahl n / t als Ergebnis.

      // Ein paar Sonderfaelle erledigen (n liegt zwischen -3 und +3):
      if (n == 0) return 1;
      int betrag = Math.abs(n);
      if (betrag <= 3) return n/betrag; // Ergebnis 1 oder -1

      // Jetzt die normalen Faelle bearbeiten.
      // Die Indexvariable teiler wird vor der for-Schleife vereinbart
      // (und im Initialisierungsteil der Schleife initialisert).
      // Die for-Schleife hat einen leeren Rumpf! Sie dient "nur" dazu,
      // die Indexvariable teiler geeignet zu veraendern.
      int teiler;
      for (teiler=2; n % teiler != 0; teiler++) {}
      // Jetzt enthaelt die Variable teiler den kleinsten Teiler
      // der Zahl n.

      return n / teiler;
   } // teileDurchMinTeiler
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die oben vereinbarten Methoden werden aufgerufen.

      pln("---------------------------------------------------------");
      pln("A  teileDurchMinTeiler(210):\t" + teileDurchMinTeiler(210));
      pln("B  teileDurchMinTeiler(105):\t" + teileDurchMinTeiler(105));
      pln("C  teileDurchMinTeiler( 35):\t" + teileDurchMinTeiler( 35));
      pln("D  teileDurchMinTeiler(  7):\t" + teileDurchMinTeiler(  7));
      pln("---------------------------------------------------------");
      int g1 = 512;
      int g2;

      while (Math.abs(g1) != 1) {
         g2 = teileDurchMinTeiler(g1);
         pln("E  teileDurchMinTeiler(" + g1 + "):\t" + g2);
         g1 = g2;
      } // while
      pln("---------------------------------------------------------");
      pln("For04: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class For04
/* ------------------------------------------------------------------------
Ausgabe des Programms For04:

---------------------------------------------------------
A  teileDurchMinTeiler(210):  105
B  teileDurchMinTeiler(105):  35
C  teileDurchMinTeiler( 35):  7
D  teileDurchMinTeiler(  7):  1
---------------------------------------------------------
E  teileDurchMinTeiler(512):  256
E  teileDurchMinTeiler(256):  128
E  teileDurchMinTeiler(128):  64
E  teileDurchMinTeiler(64):   32
E  teileDurchMinTeiler(32):   16
E  teileDurchMinTeiler(16):   8
E  teileDurchMinTeiler(8): 4
E  teileDurchMinTeiler(4): 2
E  teileDurchMinTeiler(2): 1
---------------------------------------------------------
For04: Das war's erstmal!
------------------------------------------------------------------------ */