// Datei For06.java
/* ------------------------------------------------------------------------
Mit den (seit der Java Version 5.0) neuen for-Schleifen kann man
leicht auf alle Komponenten einer Reihung zugreifen. Dabei kann man
aber die *Werte* der Reihungskomponenten *nicht veraendern*, nur "lesend
bearbeiten". Die Zielwerte der Reihungskomponenten (falls welche vorhanden
und veraenderbar sind) kann man dagegen veraendern.
------------------------------------------------------------------------ */
class For06 {
   // ---------------------------------------------------------------------
   static public void gibAusLongReihung(long[] r) {
      // Gibt die Reihung r in lesbarer Form aus.
      // Durch die Anweisung n = n / 2; im Schleifenrumpf werden keine
      // Werte von Reihungskomponenten veraendert (diese Anweisung ist
      // hier also "wirkungslos" und sollte entfernt werden).

      titel("long", r.length);

      for (long n: r) {
         p(n + " ");
         n = n / 2;     // *Werte* von Reihungskomponenten werden hier
      }                 // nicht veraendert!
      pln();
   } // gibAusLongReihung
   // ---------------------------------------------------------------------
   static public void gibAusStringBuilderReihung(StringBuilder[] r) {
      // Gibt die Reihung r in lesbarer Form aus.
      // Durch die Anweisung sb.reverse(); im Schleifenrumpf werden keine
      // *Werte* von Reihungskomponenten veraendert, wohl aber die
      // *Zielwerte* der Reihungskomponenten (diese Anweisung ist hier
      // also nicht "wirkungslos"!)
      titel("StringBuilder", r.length);

      for (StringBuilder sb: r) {
         p(sb + " ");
         sb.reverse();  // *Werte* von Reihungskomponenten werden hier
      }                 // nicht veraendert (aber *Zielwerte*!)
      pln();
   } // gibAusStringBuilderReihung
   // ---------------------------------------------------------------------
   static public void titel(String text, int laenge) {
      pln("---------------------------------------------");
      pln("Eine " + text + "-Reihung mit " + laenge + " Komponenten:");
   } // titel
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      pln("For06: Jetzt geht es los!");

      // Eine Reihung mit primitiven Komponenten und
      // eine Reihung mit Referenzvariablen als Komponenten:
      long[]          v1 = {10, 20, 30, 40, 50};
      StringBuilder[] v2 = {new StringBuilder("AB"),
                            new StringBuilder("cdef"),
                            new StringBuilder("123"),
                           };

      // Die Methode gibAusLongReihung gibt ihren Reihungsparameter aus,
      // veraendert ihn aber nicht:
      gibAusLongReihung(v1);
      gibAusLongReihung(v1);

      // Die Methode gibAusStringBuilderReihung gibt ihren Reihungs-
      // parameter aus und veraendert die Zielwerte der Komponenten:
      gibAusStringBuilderReihung(v2);
      gibAusStringBuilderReihung(v2);
      gibAusStringBuilderReihung(v2);

      pln("---------------------------------------------");
      pln("For06: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class For06
/* ------------------------------------------------------------------------
Ausgabe des Programms For06:

For06: Jetzt geht es los!
---------------------------------------------
Eine long-Reihung mit 5 Komponenten:
10 20 30 40 50
---------------------------------------------
Eine long-Reihung mit 5 Komponenten:
10 20 30 40 50
---------------------------------------------
Eine StringBuilder-Reihung mit 3 Komponenten:
AB cdef 123
---------------------------------------------
Eine StringBuilder-Reihung mit 3 Komponenten:
BA fedc 321
---------------------------------------------
Eine StringBuilder-Reihung mit 3 Komponenten:
AB cdef 123
---------------------------------------------
For06: Das war's erstmal!
------------------------------------------------------------------------ */
