// Datei Format07.java
/* ------------------------------------------------------------------------
Ein paar typische printf-Befehle, mit denen Ganzzahlen, Bruchzahlen und
Strings formatiert und ausgegeben werden.
------------------------------------------------------------------------ */
import java.util.Locale;

class Format07 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Format07: Jetzt geht es los!%n");

      // Ein paar Variablen gaengiger Type:
      int    i01  = 123456;
      int    i02  = 23;
      int    i03  = 50;
      double d01  = 123.45678;
      String vor  = "Moritz";
      String nach = "Meyer";

      // printf-Befehle mit einem einzigen Parameter:
      printf("--------------------------------------%n");
      printf("A Whiskey enthaelt 54%% Alkohol!%n");

      // printf-Befehle mit zwei Parametern, Ganzzahlen:
      printf("--------------------------------------%n");
      printf("B Der Preis betraegt %d Euro.%n",   23);
      printf("C Der Preis betraegt %d Euro.%n",  i01);
      printf("D Der Preis betraegt %7d Euro.%n", i02);
      printf("E Der Preis betraegt %7d Euro.%n", i01);
      printf("F Der Preis betraegt %7d Euro.%n", 123456789);

      // printf-Befehle mit mehr Parametern, Ganzzahlen:
      printf("--------------------------------------%n");
      printf("G Das kostet %d Euro und %d Cents.%n", i02, i03);
      printf("H 3 Zahlen: %d, %d, %f.%n",            i02, i03, d01);

      // printf-Befehle mit zwei Parametern, Bruchzahlen:
      printf("--------------------------------------%n");
      printf("I Ein Euro kostet %f US-Cents.%n",      d01);
      printf("J Ein Euro kostet %10.4f US-Cents.%n",  d01);
      printf("K Ein Euro kostet %10.2f US-Cents.%n",  d01);
      printf("L Ein Euro kostet %-10.4f US-Cents.%n", d01);
      printf("M Ein Euro kostet %-10.2f US-Cents.%n", d01);
      printf("N Ein Euro kostet %3.1f US-Cents.%n",   d01);

      // printf-Befehle mit verschiedenen Vorzeichenregeln:
      printf("--------------------------------------%n");
      printf("O Negativ: %d. Positiv: %d.%n",   -123, +123);
      printf("P Negativ: %+d. Positiv: %+d.%n", -123, +123);
      printf("Q Negativ: % d. Positiv: % d.%n", -123, +123);
      printf("R Negativ: %(d. Positiv: %(d.%n", -123, +123);

      // printf-Befehle mit mehr Parametern, auch Strings:
      printf("--------------------------------------%n");
      printf("S Nach-,Vorname: %s, %s, Alter: %d%n", nach, vor, i02);
      printf("T Nach-,Vorname: %s, %s, Alter: %s%n", nach, vor, i02);

      // Der Formatstring muss kein Literal sein:
      String f1 = "U Nach-,Vorname: %6s, %6s, Alter: %d%n";
      String f2 = "V Nach-,Vorname: %6.2s, %6.3s, Alter: %.1s%n";
      printf("--------------------------------------%n");
      printf(f1, vor, nach, i02);
      printf(f2, vor, nach, i02);

      // printf-Befehle, die nach den "Sitten anderer Laender" formatieren:
      Locale loc01 = Locale.ENGLISH;
      Locale loc02 = Locale.FRENCH;
      Locale loc03 = Locale.ITALIAN;
      printf("--------------------------------------%n");
      printf(loc01, "W Englisch    : %,f%n", 1234567.890123);
      printf(loc02, "X Franzsisch : %,f%n", 1234567.890123);
      printf(loc03, "Y Italienisch : %,f%n", 1234567.890123);

      // printf-Befehle, die die zu formatierenden Parameter mit
      // Indizes adressieren und mehrmals formatieren:
      printf("--------------------------------------%n");
      printf("Z F%1$s%2$s%2$s, f%1$s%2$s%2$s.%n", "latter", "ta");
      printf("a F%s%s%<s, f%1$s%2$s%<s.%n",       "latter", "ta");

      // printf-Befehle mit Umwandlungen, in denen Genauigkeiten
      // sehr unterschiedliche Wirkungen haben:
      printf("--------------------------------------%n");
      printf("b 123.456 mit %%6.2f umgewandelt: %6.2f%n", 123.456);
      printf("c 123.456 mit %%6.2s umgewandelt: %6.2s%n", 123.456);
      printf("--------------------------------------%n");

      printf("Format07: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Zwei Methoden mit kurzen Namen:
   static void printf(          String f, Object... v) {
      System.out.printf(f, v);
   }
   static void printf(Locale l, String f, Object... v) {
      System.out.printf(l, f, v);
   }
} // Format07
/* ------------------------------------------------------------------------
Ausgabe des Programms Format07:

Format07: Jetzt geht es los!
--------------------------------------
A Whiskey enthaelt 54% Alkohol!
--------------------------------------
B Der Preis betraegt 23 Euro.
C Der Preis betraegt 123456 Euro.
D Der Preis betraegt      23 Euro.
E Der Preis betraegt  123456 Euro.
F Der Preis betraegt 123456789 Euro.
--------------------------------------
G Das kostet 23 Euro und 50 Cents.
H 3 Zahlen: 23, 50, 123,456780.
--------------------------------------
I Ein Euro kostet 123,456780 US-Cents.
J Ein Euro kostet   123,4568 US-Cents.
K Ein Euro kostet     123,46 US-Cents.
L Ein Euro kostet 123,4568   US-Cents.
M Ein Euro kostet 123,46     US-Cents.
N Ein Euro kostet 123,5 US-Cents.
--------------------------------------
O Negativ: -123. Positiv: 123.
P Negativ: -123. Positiv: +123.
Q Negativ: -123. Positiv:  123.
R Negativ: (123). Positiv: 123.
--------------------------------------
S Nach-,Vorname: Meyer, Moritz, Alter: 23
T Nach-,Vorname: Meyer, Moritz, Alter: 23
--------------------------------------
U Nach-,Vorname: Moritz,  Meyer, Alter: 23
V Nach-,Vorname:     Mo,    Mey, Alter: 2
--------------------------------------
W Englisch    : 1,234,567.890123
X Franzsisch : 1234567,890123
Y Italienisch : 1.234.567,890123
--------------------------------------
Z Flattertata, flattertata.
a Flattertata, flattertata.
--------------------------------------
b 123.456 mit %6.2f umgewandelt: 123,46
c 123.456 mit %6.2s umgewandelt:     12
--------------------------------------
Format07: Das war's erstmal!
------------------------------------------------------------------------ */
