// Datei Format08.java
/* ------------------------------------------------------------------------
Die Klasse Calendar ist zwar abstract, enthaelt aber eine Fabrikmethode
getInstance, die ein Objekt (ihrer konkreten Unterklasse GregorianCalendar)
liefert. Dieses Objekt repraesentiert den Zeitpunkt "jetzt" (d.h. den
Zeitpunkt seiner Erzeugung).

Die Klasse GregorianCalendar ist eine konkrete Unterklass von Calendar
und besitzt 7 Konstruktoren. Der Standardkonstruktor (mit 0 Parametern)
initialisiert ebenfalls ein "jetzt-Objekt". Ein anderer Konstruktor
hat 6 Parameter (Jahr, Monat, Tag, Stunde, Minute und Sekunde) und
initialisiert das neue Objekt entsprechend.

Statt durch ein Calendar-Objekt kann man einen bestimmten Zeitpunkt t
auch durch einen long-Wert darstellen, der die "Millisekunde innerhalb
unserer Epoche" von t beschreibt. Damit ist die Anzahl der Millisekunden
gemeint, die zwischen dem 1. Januar 1970 (null Uhr UTC) und dem Zeit-
punkt t vergangen sind. Alle anderen Teile des Datums kann man aus
diesem einen long-Wert berechnen. Entsprechendes gilt auch fuer Huell-
objekte der Klasse Long.

Die speziellen Umwandlungsbefehle fuer Datums- und Zeitangaben
(im Formatstring eines printf-Befehls) darf man deshalb auf
Parameter der Typen Calender, Long und long anwenden.

Dieses Programm kann verschiedene Datum-und-Zeit-Objekte (der Typen
Calendar, GregorianCalendar und Long) entsprechend den Braeuchen
verschiedener Laender auf alle moeglichen Weisen formatieren. Damit
das Programm nicht zuviele Zeilen ausgibt, sind die meisten Komponenten
der Reihungen duzR und locR auskommentiert.
------------------------------------------------------------------------ */
import java.util.Locale;
import java.util.Calendar;
import java.util.GregorianCalendar;

class Format08 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      // ------------------------------------------------------------------
      // Eine Reihung von Datum-und-Zeit-Objekten. Die ersten beiden
      // sind "jetzt-Objekte" (sie repraesentieren den Zeitpunkt ihrer
      // Erzeugung), die letzten beiden repraesentieren zufaellig :-)
      // den selben Zeitpunkt (den 8. September 2004, 07:06:05 Uhr):
      Object[] duzR = {
//       Calendar.getInstance(),                     // Jetzt
//       new GregorianCalendar(),                    // Jetzt
//       new GregorianCalendar(2004, 9, 8, 7, 6, 5), // 08.10.2004
         new Long(1108451237095L),                   // 08.10.2004
      }; //duzR
      // ------------------------------------------------------------------
      // Eine Reihung von Objekten, die Informationen ueber die Braeuche
      // bestimmter Laender (z.B. beim Formatieren von Datums- und
      // Zeitangaben) enthalten:
      Locale[] locR = {
         Locale.ENGLISH,
//       Locale.FRENCH,
//       Locale.ITALIAN,
         Locale.GERMAN,
//       Locale.JAPANESE,
//       Locale.CHINESE,
//       Locale.KOREAN,
      }; // locR
      // ------------------------------------------------------------------
      printf("Format08: Jetzt geht es los!%n");

      // Jetzt wird jedes Datum-und-Zeit-Objekt entsprechend jeder
      // Lokalitaet auf alle moegliche Weisen formatiert:
      for (Object duz: duzR) {
         for (Locale loc: locR) {
            formatiereDatumUndZeit(loc, duz);
         }
      }
      printf("===============================================%n");
      printf("Format08: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   static void formatiereDatumUndZeit(Locale loc, Object duz) {
      // Formatiert das Datum-und-Zeit-Objekt duz entsprechend den
      // im Objekt loc beschriebenen Braeuchen auf alle moeglichen
      // Weisen (mit den 31 Unwandlungsbefehlen %tH, %tI, ... , %tc)
      // und gibt die Unwandlungsbefehle und ihre Ergebnisse aus.
      printf("===============================================%n");
      printf("Lokalitaet: %s%n", loc);
      printf("===============================================%n");
      printf("    Formatierungen einer Zeit                  %n");
      printf(loc, "%%tH Stunde des 24-Stunden-Tages: %tH%n", duz);
      printf(loc, "%%tI Stunde des 12-Stunden-Tages: %tI%n", duz);
      printf(loc, "%%tk Stunde des 24-Stunden-Tages: %tk%n", duz);
      printf(loc, "%%tl Stunde des 12-Stunden-Tages: %tl%n", duz);
      printf(loc, "%%tM Minute der Stunde          : %tM%n", duz);
      printf(loc, "%%tS Sekunde der Minute         : %tS%n", duz);
      printf(loc, "%%tL Millisekunde der Sekunde   : %tL%n", duz);
      printf(loc, "%%tN  Nanosekunde der Sekunde   : %tN%n", duz);
      printf(loc, "%%tp Vor-/Nachmittag-Markierung : %tp%n", duz);
      printf(loc, "%%tz Abstand von Zeitzone GMT   : %tz%n", duz);
      printf(loc, "%%tZ Abkuerzung fuer Zeitzone   : %tZ%n", duz);
      printf(loc, "%%ts      Sekunde der Epoche    : %ts%n", duz);
      printf(loc, "%%tQ Millisekunde der Epoche    : %tQ%n", duz);
      printf("-----------------------------------------------%n");
      printf("    Formatierungen eines Datums                %n");
      printf(loc, "%%tB Voller       Monatsname    : %tB%n", duz);
      printf(loc, "%%tb Abgekuerzter Monatsname    : %tb%n", duz);
      printf(loc, "%%th Abgekuerzter Monatsname    : %th%n", duz);
      printf(loc, "%%tA Voller       Tagesname     : %tA%n", duz);
      printf(loc, "%%ta Abgekuerzter Tagesname     : %ta%n", duz);
      printf(loc, "%%tC Jahr (vordere Ziffern)     : %tC%n", duz);
      printf(loc, "%%tY Jahr                       : %tY%n", duz);
      printf(loc, "%%ty Jahr (hintere Ziffern)     : %ty%n", duz);
      printf(loc, "%%tj Tag    des Jahres          : %tj%n", duz);
      printf(loc, "%%tm Monat  des Jahres          : %tm%n", duz);
      printf(loc, "%%td Tag    des Monats (2stell) : %td%n", duz);
      printf(loc, "%%te Tag    des Monats          : %te%n", duz);
      printf("-----------------------------------------------%n");
      printf("    Formatierungen von Datum/Zeit-Kombinationen%n");
      printf(loc, "%%tR Zeit des 24-Stunden-Tages  : %tR%n", duz);
      printf(loc, "%%tT Zeit des 24-Stunden-Tages  : %tT%n", duz);
      printf(loc, "%%tr Zeit des 12-Stunden-Tages  : %tr%n", duz);
      printf(loc, "%%tD Datum                      : %tD%n", duz);
      printf(loc, "%%tF Datum (ISO 8601)           : %tF%n", duz);
      printf(loc, "%%tc Datum und Zeit             : %tc%n", duz);
   } // formatiereDatumUndZeit
   // ---------------------------------------------------------------------
   // Zwei Methoden mit kurzen Namen:
   static void printf(          String f, Object... v) {
      System.out.printf(f, v);
   }
   static void printf(Locale l, String f, Object... v) {
      System.out.printf(l, f, v);
   }
} // Format08
/* ------------------------------------------------------------------------
Ausgabe des Programms Format08 (wobei in der Reihung duzR nur ein und
in der Reihung locR nur zwei Komponenten nicht auskommentiert waren):

Format08: Jetzt geht es los!
===============================================
Lokalitaet: en
===============================================
    Formatierungen einer Zeit
%tH Stunde des 24-Stunden-Tages: 08
%tI Stunde des 12-Stunden-Tages: 08
%tk Stunde des 24-Stunden-Tages: 8
%tl Stunde des 12-Stunden-Tages: 8
%tM Minute der Stunde          : 07
%tS Sekunde der Minute         : 17
%tL Millisekunde der Sekunde   : 095
%tN  Nanosekunde der Sekunde   : 095000000
%tp Vor-/Nachmittag-Markierung : am
%tz Abstand von Zeitzone GMT   : +0100
%tZ Abkuerzung fuer Zeitzone   : GMT+01:00
%ts      Sekunde der Epoche    : 1108451237
%tQ Millisekunde der Epoche    : 1108451237095
-----------------------------------------------
    Formatierungen eines Datums
%tB Voller       Monatsname    : February
%tb Abgekuerzter Monatsname    : Feb
%th Abgekuerzter Monatsname    : Feb
%tA Voller       Tagesname     : Tuesday
%ta Abgekuerzter Tagesname     : Tue
%tC Jahr (vordere Ziffern)     : 20
%tY Jahr                       : 2005
%ty Jahr (hintere Ziffern)     : 05
%tj Tag    des Jahres          : 046
%tm Monat  des Jahres          : 02
%td Tag    des Monats (2stell) : 15
%te Tag    des Monats          : 15
-----------------------------------------------
    Formatierungen von Datum/Zeit-Kombinationen
%tR Zeit des 24-Stunden-Tages  : 08:07
%tT Zeit des 24-Stunden-Tages  : 08:07:17
%tr Zeit des 12-Stunden-Tages  : 08:07:17 AM
%tD Datum                      : 02/15/05
%tF Datum (ISO 8601)           : 2005-02-15
%tc Datum und Zeit             : Tue Feb 15 08:07:17 GMT+01:00 2005
===============================================
Lokalitaet: de
===============================================
    Formatierungen einer Zeit
%tH Stunde des 24-Stunden-Tages: 08
%tI Stunde des 12-Stunden-Tages: 08
%tk Stunde des 24-Stunden-Tages: 8
%tl Stunde des 12-Stunden-Tages: 8
%tM Minute der Stunde          : 07
%tS Sekunde der Minute         : 17
%tL Millisekunde der Sekunde   : 095
%tN  Nanosekunde der Sekunde   : 095000000
%tp Vor-/Nachmittag-Markierung : am
%tz Abstand von Zeitzone GMT   : +0100
%tZ Abkuerzung fuer Zeitzone   : GMT+01:00
%ts      Sekunde der Epoche    : 1108451237
%tQ Millisekunde der Epoche    : 1108451237095
-----------------------------------------------
    Formatierungen eines Datums
%tB Voller       Monatsname    : Februar
%tb Abgekuerzter Monatsname    : Feb
%th Abgekuerzter Monatsname    : Feb
%tA Voller       Tagesname     : Dienstag
%ta Abgekuerzter Tagesname     : Di
%tC Jahr (vordere Ziffern)     : 20
%tY Jahr                       : 2005
%ty Jahr (hintere Ziffern)     : 05
%tj Tag    des Jahres          : 046
%tm Monat  des Jahres          : 02
%td Tag    des Monats (2stell) : 15
%te Tag    des Monats          : 15
-----------------------------------------------
    Formatierungen von Datum/Zeit-Kombinationen
%tR Zeit des 24-Stunden-Tages  : 08:07
%tT Zeit des 24-Stunden-Tages  : 08:07:17
%tr Zeit des 12-Stunden-Tages  : 08:07:17 AM
%tD Datum                      : 02/15/05
%tF Datum (ISO 8601)           : 2005-02-15
%tc Datum und Zeit             : Di Feb 15 08:07:17 GMT+01:00 2005
===============================================
Format08: Das war's erstmal!
------------------------------------------------------------------------ */
