// Datei Format09.java
/* ------------------------------------------------------------------------
In ein Formatter-Objekt f kann man mit der Methode f.format Daten
formatiert hineinschreiben. Wo die Daten dadurch "hinkommen" (das
Datenziel des Formatter-Objekts) kann man beim Erzeugen von f fest-
legen. Es folgen ein paar moegliche Datenziele:

- Ein internes StringBuilder-Objekt
- Ein beliebiges Appendable-Objekt, z.B.
  - Ein ausdruecklich angegebenes StringBuilder-Objekt
  - Ein PrintWriter-Objekt wie z.B. System.out oder System.err
  - Ein  FileWriter-Objekt
- Eine Datei (in Form eines Pfadnamens oder eines File-Objekts)

In diesem Programm werden 5 Formatter-Objekte mit unterschiedlichen
Datenzielen vereinbart und benutzt.
---------------------------------------------------------------------------
Allgemeiner Hinweis: Die Methode System.out.printf benutzt intern ein
Formatter-Objekt mit der Standardausgabe System.out als Datenziel.
------------------------------------------------------------------------ */
import java.util.Formatter;
import java.util.Calendar;
import java.io.FileWriter;
import java.io.FileReader;
import java.io.IOException;

class Format09 {
   // ---------------------------------------------------------------------
   static int rechNr = 0; // Rechnungen werden fortlaufend numeriert
   static void formatiereEineRechnung(Formatter f)  {
      // Formatiert und schreibt die Daten einer kleinen Rechnung in das
      // Formatter-Objekt f:
      Calendar heute = Calendar.getInstance();
      f.format("   +-----------------------------------+%n");
      f.format("   |Rechnung %d vom %20tD|%n", ++rechNr, heute);
      f.format("   |1 USB-Stick:       %3d Euro %2d Cent|%n",  72, 50);
      f.format("   |1 Flachbildschirm: %3d Euro %2d Cent|%n", 499, 80);
      f.format("   +-----------------------------------+%n");
   } // formatiereEineRechnung
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) throws IOException {
      pln("Format09: Jetzt geht es los!%n");       // Hier ist %n unsinnig!

      // Ein paar Datenziele fuer Formatter-Objekte:
      Appendable sb02 = new StringBuilder();
      Appendable pw03 = System.out;  // Ein PrintWriter-Objekt
      Appendable fw04 = new FileWriter("tmpA.tmp");

      // Ein paar Formatter-Objekte mit unterschiedlichen Datenzielen:
      Formatter f01 = new Formatter();          // Interner StringBuilder
      Formatter f02 = new Formatter(sb02);      // Expliziter StringBuilder
      Formatter f03 = new Formatter(pw03);      // PrintWriter
      Formatter f04 = new Formatter(fw04);      // FileWriter
      Formatter f05 = new Formatter("tmpB.tmp");// Eine Datei

      // Jetzt wird in jedes Formatter-Objekt eine kleine Rechnung
      // geschrieben und anschliessend auf dem Bildschirm sichtbar
      // gemacht:
      pln("=========================================");
      formatiereEineRechnung(f01);
      pln("Der Inhalt des Formatter-Objekts f01:");
      p  (f01);

      pln("=========================================");
      formatiereEineRechnung(f02);
      pln("Der Inhalt des StringBuilder-Objekts sb02:");
      p  (sb02);

      pln("=========================================");
      pln("Direkte Ausgabe zum Bildschirm:");
      formatiereEineRechnung(f03);

      pln("=========================================");
      pln("Ausgabe in die Datei tmpA.tmp:");
      formatiereEineRechnung(f04);
      f04.flush(); // Muss sein!

      // Die Datei tmpA.tmp lesen und zum Bildschirm ausgeben:
      FileReader fr04 = new FileReader("tmpA.tmp");
      char[]     cb04 = new char[250];
      fr04.read(cb04);
      p  (new String(cb04).substring(0, 210));
      fr04.close();

      pln("=========================================");
      pln("Ausgabe in die Datei tmpB.tmp:");
      formatiereEineRechnung(f05);
      f05.flush(); // Muss nicht sein!

      // Die Datei tmpB.tmp lesen und zum Bildschirm ausgeben:
      FileReader fr05 = new FileReader("tmpB.tmp");
      char[]     cb05 = new char[250];
      fr05.read(cb04);
      p  (new String(cb04).substring(0, 210));
      fr05.close();
      pln("=========================================");

      pln("Format09: Das war's erstmal!%n");       // Hier ist %n unsinnig!
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Format09
/* ------------------------------------------------------------------------
Ausgabe des Programms Format09 zur Standardausgabe (zusaetzlich werden
zwei Dateien namens tmpA.tmp und tmpB.tmp erzeugt):

Format09: Jetzt geht es los!%n
=========================================
Der Inhalt des Formatter-Objekts f01:
   +-----------------------------------+
   |Rechnung 1 vom             02/15/05|
   |1 USB-Stick:        72 Euro 50 Cent|
   |1 Flachbildschirm: 499 Euro 80 Cent|
   +-----------------------------------+
=========================================
Der Inhalt des StringBuilder-Objekts sb02:
   +-----------------------------------+
   |Rechnung 2 vom             02/15/05|
   |1 USB-Stick:        72 Euro 50 Cent|
   |1 Flachbildschirm: 499 Euro 80 Cent|
   +-----------------------------------+
=========================================
Direkte Ausgabe zum Bildschirm:
   +-----------------------------------+
   |Rechnung 3 vom             02/15/05|
   |1 USB-Stick:        72 Euro 50 Cent|
   |1 Flachbildschirm: 499 Euro 80 Cent|
   +-----------------------------------+
=========================================
Ausgabe in die Datei tmpA.tmp:
   +-----------------------------------+
   |Rechnung 4 vom             02/15/05|
   |1 USB-Stick:        72 Euro 50 Cent|
   |1 Flachbildschirm: 499 Euro 80 Cent|
   +-----------------------------------+
=========================================
Ausgabe in die Datei tmpB.tmp:
   +-----------------------------------+
   |Rechnung 5 vom             02/15/05|
   |1 USB-Stick:        72 Euro 50 Cent|
   |1 Flachbildschirm: 499 Euro 80 Cent|
   +-----------------------------------+
=========================================
Format09: Das war's erstmal!%n
------------------------------------------------------------------------ */
