// Datei Format11.java
/* ------------------------------------------------------------------------
Der Programmierer einer Klasse kann selbst festlegen, wie Objekte seiner
Klassen formatiert werden sollen, wenn man format- oder printf-Befehle
darauf anwendet. Dazu muss er in seinen Klassen die Schnittstelle
Formattable implementieren.

Die Schnittstelle Formattable enthaelt eine einzige Methode namens
formatTo. Ist fob ein Formattable-Objekt, dann bewirkt ein Aufruf wie
printf("%s", fob); dass de Methode fob.formatTo aufgerufen wird
und der printf-Befehl das Resultat dieses Aufrufs ausgibt.
------------------------------------------------------------------------ */
import java.util.Formattable;
import java.util.Formatter;
import java.math.BigDecimal;

import static java.util.FormattableFlags.LEFT_JUSTIFY;
import static java.util.FormattableFlags.ALTERNATE;
import static java.util.FormattableFlags.UPPERCASE;

/* =======================================================================-
Ein Problem und seine Loesung:
Jedes MyDouble-Objekt myd enthaelt den Wert eines Double-Objekts dob, aber
in Form eines BigDecimal-Objekts. Ein Befehl wie
printf("%30.21f", dob); gibt den Wert von dob zwar mit 21 Nachpunktstellen
aus, aber einige davon sind mathematisch nicht korrekt. Ein Befehl wie
printf("%30.21s", myd); gibt den Wert dagegen mit 21 korrekten Nachpunkt-
stellen aus. Dieser zweite printf-Befehl bewirkt, dass die Methode
myd.formatTo aufgerufen wird (und das BigDecimal-Objekt myd.big formatiert).
Siehe die Ausgabezeilen I und J bzw. K und L des Programms Format11
------------------------------------------------------------------------ */
class MyDouble implements Formattable {
   private BigDecimal big;
   // ---------------------------------------------------------------------
   public MyDouble(Double d) {big = new BigDecimal(d);}
   // ---------------------------------------------------------------------
   public void formatTo(Formatter fmt, int schalter,
                                       int breite,
                                       int genauigkeit)
   {
      /* ------------------------------------------------------------------
      Wenn man ein MyDouble-Objekt myd mit format (oder printf) und einem
      %s-Umwandlungsbefehl formatiert, wird diese Methode formatTo aufge-
      rufen und formatiert das Attribut myd.big mit einem "entsprechenden"
      %f-Befehl. "Entsprechend" soll heissen:

      Der %f-Befehl uebernimmt die Breite und Genauigkeit und (falls er
      angegeben wurde) den linksbuendig-Schalter Minus (-) des %s-Befehls.
      Falls im %s-Befehl der Alternativ-Schalter # gesetzt ist, wird
      im %f-Befehl der Vorzeichen-Schalter Plus (+) gesetzt. Falls im
      %s-Befehl (statt klein s gross) S angegeben wurde, wird im %f-Befehl
      der Gruppenseparations-Schalter Komma (,) gesetzt.
      ------------------------------------------------------------------ */
      StringBuffer sbg = new StringBuffer();
      if ((schalter & LEFT_JUSTIFY) != 0) sbg.append("-"); // - --> -
      if ((schalter & ALTERNATE   ) != 0) sbg.append("+"); // # --> +
      if ((schalter & UPPERCASE   ) != 0) sbg.append(","); // S --> , und f
      if (breite      != -1)              sbg.append(breite);
      if (genauigkeit != -1)              sbg.append("." + genauigkeit);

      fmt.format("%" + sbg + "f", big);
   } // formatTo
   // ---------------------------------------------------------------------
   public String toString() {
      // Liefert big als String in eckige Klammern eingeschlossen (damit
      // man das Ergebnis dieser toString-Funktion vom Ergebnis der
      // format-Funktion leicht unterscheiden kann).
      return "[" + big + "]";
   } // toString
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class MyDouble
// ========================================================================
class Format11 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      /* ------------------------------------------------------------------
      Produziert 23 Ausgabezeilen (A bis W).
      Die Zeilen A bis H sollen zeigen, dass ein Objekt new MyDouble(d)
      ganz entsprechend formatiert wird wie das Double-Objekt d.

      Die Zeilen I bis L sollen zeigen, dass ein Objekt mewMyDouble(d)
      genauer formatiert wird als das Double-Objekt d.

      Die Zeile M bis O zeigen die Wirkung der Schalter Linksbuendig -
      und Alternaitve # und die Wirkung von gross S (statt klein s).

      Die Zeilen P bis S zeigen, dass die formatTo-Methode eines
      MyDouble-Objekts ncht aufgerufen wird, wenn das Objekt mit einem
      %h-, %H-, %b- oder %B-Umwandlungsbefehl formatiert wird.

      In den Zeilen T bis W werden Double- und MyDouble-Objekte in
      Strings umgewanelt (implizit mit ihrer Methode toString) und
      dann mit einem %s-Umwandlungsbefehl formatiert.
      ------------------------------------------------------------------ */
      // Zwei Double- und zwei entsprechende MyDouble-Objekte:
      Double   d01 = new Double(1234567.89);
      Double   d02 = new Double( 0.1  );

      MyDouble m01 = new MyDouble(d01);
      MyDouble m02 = new MyDouble(d02);

      printf("Format11: Jetzt geht es los!%n");
      printf("-----------------------------%n");
      printf("A printf(\"%%f%%n\",         d01): %f%n",          d01);
      printf("B printf(\"%%s%%n\",         m01): %s%n",          m01);
      printf("C printf(\"%%f%%n\",         d02): %f%n",          d02);
      printf("D printf(\"%%s%%n\",         m02): %s%n",          m02);
      printf("-----------------------------%n");
      printf("E printf(\"%%15f%%n\",       d01): %15f%n",        d01);
      printf("F printf(\"%%15s%%n\",       m01): %15s%n",        m01);
      printf("G printf(\"%%15f%%n\",       d02): %15f%n",        d02);
      printf("H printf(\"%%15s%%n\",       m02): %15s%n",        m02);
      printf("-----------------------------%n");
      printf("I printf(\"%%30.21f%%n\",    d01): %30.21f%n",     d01);
      printf("J printf(\"%%30.21s%%n\",    m01): %30.21s%n",     m01);
      printf("K printf(\"%%30.21f%%n\",    d02): %30.21f%n",     d02);
      printf("L printf(\"%%30.21s%%n\",    m02): %30.21s%n",     m02);
      printf("-----------------------------%n");
      printf("M printf(\"%%-14.1s%%n\",    m01): %-14.1s%n",     m01); // LEFT_JUSTIFY -
      printf("N printf(\"%%#14.1s%%n\",    m01): %#14.1s%n",     m01); // ALTERNATE #
      printf("O printf(\"%%#14.1S%%n\",    m01): %#14.1S%n",     m01); // ALTERNATE # und UPPERCASE S
      printf("-----------------------------%n");
      printf("P printf(\"%%h%%n\",         m01): %h%n",          m01);
      printf("Q printf(\"%%H%%n\",         m01): %H%n",          m01);
      printf("R printf(\"%%b%%n\",         m01): %b%n",          m01);
      printf("S printf(\"%%B%%n\",         m01): %B%n",          m01);
      printf("-----------------------------%n");
      printf("T printf(\"%%s30.30%%n\", \"\"+d01): %30.30s%n", ""+d01);
      printf("U printf(\"%%s30.30%%n\", \"\"+m01): %30.30s%n", ""+m01);
      printf("V printf(\"%%s30.24%%n\", \"\"+d02): %30.24s%n", ""+d02);
      printf("W printf(\"%%s30.24%%n\", \"\"+m02): %30.24s%n", ""+m02);
      printf("-----------------------------%n");
      printf("Format11: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Format11
/* ------------------------------------------------------------------------
Ausgabe des Programms Format11:

Format11: Jetzt geht es los!
-----------------------------
A printf("%f%n",         d01): 1234567,890000
B printf("%s%n",         m01): 1234567,890000
C printf("%f%n",         d02): 0,100000
D printf("%s%n",         m02): 0,100000
-----------------------------
E printf("%15f%n",       d01):  1234567,890000
F printf("%15s%n",       m01):  1234567,890000
G printf("%15f%n",       d02):        0,100000
H printf("%15s%n",       m02):        0,100000
-----------------------------
I printf("%30.21f%n",    d01):  1234567,890000000000000000000
J printf("%30.21s%n",    m01):  1234567,889999999897554516792
K printf("%30.21f%n",    d02):        0,100000000000000000000
L printf("%30.21s%n",    m02):        0,100000000000000005551
-----------------------------
M printf("%-14.1s%n",    m01): 1234567,9       // LEFT_JUSTIFY -
N printf("%#14.1s%n",    m01):     +1234567,9  // ALTERNATE #
O printf("%#14.1S%n",    m01):   +1.234.567,9  // ALTERNATE # und UPPERCASE S
-----------------------------
P printf("%h%n",         m01): de6ced
Q printf("%H%n",         m01): DE6CED
R printf("%b%n",         m01): true
S printf("%B%n",         m01): TRUE
-----------------------------
T printf("%s30.30%n", ""+d01):                     1234567.89
U printf("%s30.30%n", ""+m01): [1234567.889999999897554516792
V printf("%s30.24%n", ""+d02):                            0.1
W printf("%s30.24%n", ""+m02):       [0.100000000000000005551
-----------------------------
Format11: Das war's erstmal!
------------------------------------------------------------------------ */
