// Datei GenTypen02.java
/* ------------------------------------------------------------------------
Eine generische Klasse PaarD wird so vereinbart, dass fuer jeden Typ
PaarD<S, T> gilt: Der Typ T ist ein Untertyp ("eine Erweiterung") des
Typs S. Daraus folgt fuer jedes Objekt p eines solchen Typs PaarD<S, T>:
Die Zuweisung p.k1 = p.k2; ist typenmaessig garantiert erlaubt.
------------------------------------------------------------------------ */
class PaarD<S, T extends S> { // T muss eine Erweiterung von S sein!
   S k1;       // Komponente 1
   T k2;       // Komponente 2
   // -----------------------------------------------------------
   PaarD(S k1, T k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarD
   // -----------------------------------------------------------
   PaarD(PaarD<S, T> p) {
      this.k1 = p.k1;
      this.k2 = p.k2;
   } // Konstruktor PaarD
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
   // Die Zuweisung in der folgenden Methode ist typenmaessig
   // immer erlaubt:
   void weiseZu() {k1 = k2;}
   // -----------------------------------------------------------
   void bearbeite(String name) {
      printf("------------------------------------%n");
      printf(name + ": %s%n", this.toString());
      this.weiseZu();
      printf(name + ": %s%n", this.toString());
   } // bearbeite
   // -----------------------------------------------------------
   // Eine generische Klassenmehtode (diese Methode PaarD.bearbeite
   // leistet genau das Gleiche wie die Methode GenTypen02.bearbeite,
   // siehe unten):
   static <U, V extends U>
   void bearbeite(PaarD<U, V> paar, String name) {
      printf("------------------------------------%n");
      printf(name + ": %s%n", paar);
      paar.weiseZu();
      printf(name + ": %s%n", paar);
   } // bearbeite
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // Klasse PaarD
// ==============================================================
class GenTypen02 {
   // Eine generische Klassenmehtode (diese Methode
   // GenTypen02.bearbeite lestet genau das Gleiche wie die Methode
   // PaarD.bearbeite, siehe oben):
   static <U, V extends U>
   void bearbeite(PaarD<U, V> paar, String name) {
      printf("------------------------------------%n");
      printf(name + ": %s%n", paar);
      paar.weiseZu();
      printf(name + ": %s%n", paar);
   } // bearbeite
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen02: Jetzt geht es los!%n");
      /* ------------------------------------------------------------------
      Zur Erinnerung: Die abstrakte Klasse Number ist ein Obertyp von
      Double und Integer etc. und (wie alle Typen) ein Untertyp von Object:

      Object <--- Number <--+-- Double
                            |
                            +-- Integer
      ------------------------------------------------------------------ */
      // Objekte, die als Komponenten von Paaren verwendet werden sollen:

      Object  o1 = new Object();
      Object  o2 = new Object();

      Number  n1 = new Double (1.2);
      Number  n2 = new Integer(201);

      Double  d1 = new Double (2.1);
      Integer i1 = new Integer(102);

      // Vier mal vier PaarD-Objekte:
      PaarD<Number, Integer> p1a = new PaarD<Number, Integer>(n1, i1);
      PaarD<Number, Integer> p1b = new PaarD<Number, Integer>(n1, i1);
      PaarD<Number, Integer> p1c = new PaarD<Number, Integer>(n1, i1);
      PaarD<Number, Integer> p1d = new PaarD<Number, Integer>(n1, i1);

      PaarD<Number, Double>  p2a = new PaarD<Number, Double> (n2, d1);
      PaarD<Number, Double>  p2b = new PaarD<Number, Double> (n2, d1);
      PaarD<Number, Double>  p2c = new PaarD<Number, Double> (n2, d1);
      PaarD<Number, Double>  p2d = new PaarD<Number, Double> (n2, d1);

      PaarD<Object, Number>  p3a = new PaarD<Object, Number> (o1, n1);
      PaarD<Object, Number>  p3b = new PaarD<Object, Number> (o1, n1);
      PaarD<Object, Number>  p3c = new PaarD<Object, Number> (o1, n1);
      PaarD<Object, Number>  p3d = new PaarD<Object, Number> (o1, n1);

      PaarD<Object, Number>  p4a = new PaarD<Object, Number> (o2, i1);
      PaarD<Object, Number>  p4b = new PaarD<Object, Number> (o2, i1);
      PaarD<Object, Number>  p4c = new PaarD<Object, Number> (o2, i1);
      PaarD<Object, Number>  p4d = new PaarD<Object, Number> (o2, i1);

      // Falsche Vereinbarungen von PaarD-Objekten:
//    PaarD<Double,  Integer> p1f; // Integer ist kein Untertyp von Double
//    PaarD<Integer, Double>  p2f; // Double  ist kein Untertyp von Integer
//    PaarD<Integer, Object>  p3f; // Object  ist kein Untertyp von Integer
      // ------------------------------------------------------------------
      // Jetzt werden die vier mal vier PaarD-Objekte bearbeitet:

      // Die vier a-Objekte (p1a, p2a, ...) mit den Methoden
      // printf und weiseZu bearbeiten:
      printf("------------------------------------%n");
      printf("p1a: %s%n", p1a);
      p1a.weiseZu();
      printf("p1a: %s%n", p1a);
      printf("------------------------------------%n");
      printf("p2a: %s%n", p2a);
      p2a.weiseZu();
      printf("p2a: %s%n", p2a);
      printf("------------------------------------%n");
      printf("p3a: %s%n", p3a);
      p3a.weiseZu();
      printf("p3a: %s%n", p3a);
      printf("------------------------------------%n");
      printf("p4a: %s%n", p4a);
      p4a.weiseZu();
      printf("p4a: %s%n", p4a);
      printf("====================================%n");

      // Die vier b-Objekte (p1b, p2b, ...) mit der Objektmethode
      // bearbeite bearbeiten:
      p1b.bearbeite("p1b");
      p2b.bearbeite("p2b");
      p3b.bearbeite("p3b");
      p4b.bearbeite("p4b");
      printf("====================================%n");

      // Die vier c-Objekte (p1c, p2c, ...) mit der Klassenmethode
      // PaarD.bearbeite bearbeiten:
      PaarD.bearbeite(p1c, "p1c");
      PaarD.bearbeite(p2c, "p2c");
      PaarD.bearbeite(p3c, "p3c");
      PaarD.bearbeite(p4c, "p4c");
      printf("====================================%n");

      // Die vier d-Objekte (p1d, p2d, ...) mit der Klassenmethode
      // GenTypen02.bearbeite bearbeiten:
      bearbeite(p1d, "p1d");
      bearbeite(p2d, "p2d");
      bearbeite(p3d, "p3d");
      bearbeite(p4d, "p4d");
      printf("====================================%n");

      printf("GenTypen02: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // GenTypen02
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen02:

GenTypen02: Jetzt geht es los!
------------------------------------
p1a: [1.2, 102]
p1a: [102, 102]
------------------------------------
p2a: [201, 2.1]
p2a: [2.1, 2.1]
------------------------------------
p3a: [java.lang.Object@1a8c4e7, 1.2]
p3a: [1.2, 1.2]
------------------------------------
p4a: [java.lang.Object@1172e08, 102]
p4a: [102, 102]
====================================
------------------------------------
p1b: [1.2, 102]
p1b: [102, 102]
------------------------------------
p2b: [201, 2.1]
p2b: [2.1, 2.1]
------------------------------------
p3b: [java.lang.Object@1a8c4e7, 1.2]
p3b: [1.2, 1.2]
------------------------------------
p4b: [java.lang.Object@1172e08, 102]
p4b: [102, 102]
====================================
------------------------------------
p1c: [1.2, 102]
p1c: [102, 102]
------------------------------------
p2c: [201, 2.1]
p2c: [2.1, 2.1]
------------------------------------
p3c: [java.lang.Object@1a8c4e7, 1.2]
p3c: [1.2, 1.2]
------------------------------------
p4c: [java.lang.Object@1172e08, 102]
p4c: [102, 102]
====================================
------------------------------------
p1d: [1.2, 102]
p1d: [102, 102]
------------------------------------
p2d: [201, 2.1]
p2d: [2.1, 2.1]
------------------------------------
p3d: [java.lang.Object@1a8c4e7, 1.2]
p3d: [1.2, 1.2]
------------------------------------
p4d: [java.lang.Object@1172e08, 102]
p4d: [102, 102]
====================================
GenTypen02: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\GenTypen02.java:105: type parameter java.lang.Integer is not
within its bound
    PaarD<Double,  Integer> p1f; // Integer ist kein Untertyp von Double
                   ^
D:\Java\GenTypen02.java:106: type parameter java.lang.Double is not
within its bound
    PaarD<Integer, Double>  p2f; // Double  ist kein Untertyp von Integer
                   ^
D:\Java\GenTypen02.java:107: type parameter java.lang.Object is not
within its bound
    PaarD<Integer, Object>  p3f; // Object  ist kein Untertyp von Integer
                   ^
------------------------------------------------------------------------ */
