// Datei GenTypen03.java
/* ------------------------------------------------------------------------
In einer nicht-generischen Klasse wie PaarF kann man generische Methoden
vereinbaren, die von "eigenen Typparametern" (hier: von N) abhaengen,
z.B.
1. eine  Objektmethode wie gibAusO1 (die von N abhaengt)
2. eine Klassenmethode wie gibAusK1 (die von N abhaengt).

In einer generischen Klasse wie PaarG<K> kann man generische
Methoden vereinbaren, die entweder von den Typparametern der Klasse
(hier: von K) oder von "eigenen Typparametern" (hier: von N) abhaengen,
z.B.
1. eine  Objektmethode wie gibAusO2 (die von K abhaengt)
2. eine  Objektmethode wie gibAusO3 (die von N abhaengt)
3. eine Klassenmethode wie gibAusK3 (die von N abhaengt)

In einer generischen Klasse wie PaarG<K> ist es *nicht erlaubt*, eine
generische Klassenmethode (static method) wie z.B. gibAusK2 zu verein-
baren, die von den Typparametern ihrer Klasse (hier: von K) abhaengt.
Der Typparameter K wird vom Ausfuehrer als ein Objektelement (a non static
member) der Klasse PaarG<K> betrachtet, welches *nicht* im Zusammenhang
mit Klassenelementen (static members) benutzt werden darf.
---------------------------------------------------------------------------
Zur Erinnerung: Die abstrakte Klasse Number ist ein Obertyp von Double und
Integer etc. und (wie alle Typen) ein Untertyp von Object:

                           Object
                             |
                           Number
                             |
                     +-------+-------+
                     |               |
                  Double          Integer

------------------------------------------------------------------------ */
class PaarF {       // Eine nicht-generische Klasse
   Object k1;       // Komponente 1
   Object k2;       // Komponente 2
   // ---------------------------------------------------------------------
   PaarF(Object k1, Object k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarF
   // ---------------------------------------------------------------------
   // Eine generische Objektmethode, die von einem eigenen Typparameter
   // N abhaengt:
          <N extends Number> void gibAusO1(N n) {printf("A %s%n", n);}
   // ---------------------------------------------------------------------
   // Eine generische Klassenmethode, die von einem eigenen Typparamter
   // N abhaengt:
   static <N extends Number> void gibAusK1(N n) {printf("B %s%n", n);}
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // Klasse PaarF
// ==============================================================
class PaarG<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarG(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarG
   // -----------------------------------------------------------
   // Eine generische Objektmethode, die vom Typparameter K
   // ihrer Klasse abhaengt:
   void gibAusO2(K k) {printf("C %s%n", k);}
   // -----------------------------------------------------------
   // Eine generische Klassenmethode, die vom Typparameter K
   // ihrer Klasse abhaengt (nicht erlaubt!):
// static
// void gibAusK2(K k) {printf("D %s%n", k);} // Verboten!
   // -----------------------------------------------------------
   // Eine generische Objektmethode, die von einem eigenen
   // Typparameter N abhaengt:
   <N extends Number>
   void gibAusO3(N n) {printf("E %s%n", n);}
   // -----------------------------------------------------------
   // Eine generische Klassenmethode, die von einem eigenen
   // Typparameter N abhaengt:
   static <N extends Number>
   void gibAusK3(N n) {printf("F %s%n", n);}
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // Klasse PaarG
// ==============================================================
class GenTypen03 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen03: Jetzt geht es los!%n");

      // Zwei Objekte, die mit gibAus-Methoden ausgegeben werden sollen
      // (bei d1 klappt die Ausgabe, bei s1 nicht):
      String         s1 = new String        ("Hallo!");
      Double         d1 = new Double        (2.5);

      // Ein PaarF-Objekt e1 und ein PaarG<Integer>-Objekt f1:
      PaarF          e1 = new PaarF         (s1, d1);
      Integer        i1 = new Integer       (205);
      Integer        i2 = new Integer       (306);
      PaarG<Integer> f1 = new PaarG<Integer>(i1, i2);

      // Aufrufe von Objektmethoden des Objekts e1:
//       e1.gibAusO1(s1); // Verboten (weil s1 keine Number ist)
         e1.gibAusO1(d1);

      // Aufrufe von Klassenmethoden der Klasse PaarF:
//    PaarF.gibAusK1(s1); // Verboten (weil s1 keine Number ist)
      PaarF.gibAusK1(d1);

      // Aufrufe von Objektmethoden des Objekts f1:
//       f1.gibAusO2(d1); // Verboten (weil d1 kein Integer ist)
         f1.gibAusO2(i1);
//       f1.gibAusO3(s1); // Verboten (weil s1 keine Number ist)
         f1.gibAusO3(d1);

      // Aufrufe von Klassenmethoden der Klasse PaarG:
//    PaarG.gibAusK3(s1); // Verboten (weil s1 keine Number ist)
      PaarG.gibAusK3(d1);

      printf("GenTypen03: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class GenTypen03
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen03:

GenTypen03: Jetzt geht es los!
A 2.5
B 2.5
C 205
E 2.5
F 2.5
GenTypen03: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\GenTypen03.java:74: non-static class K
cannot be referenced from a static context
 void gibAusK2(K k) {printf("D %s%n", k);} // Verboten!
               ^
D:\Java\GenTypen03.java:108: <N>gibAusO1(N) in PaarF
cannot be applied to (java.lang.String)
       e1.gibAusO1(s1); // Verboten (weil s1 keine Number ist)
         ^
D:\Java\GenTypen03.java:112: <N>gibAusK1(N) in PaarF
cannot be applied to (java.lang.String)
    PaarF.gibAusK1(s1); // Verboten (weil s1 keine Number ist)
         ^
D:\Java\GenTypen03.java:116: gibAusO2(java.lang.Integer) in
PaarG<java.lang.Integer> cannot be applied to (java.lang.Double)
       f1.gibAusO2(d1); // Verboten (weil d1 kein Integer ist)
         ^
D:\Java\GenTypen03.java:118: <N>gibAusO3(N) in PaarG<java.lang.Integer>
cannot be applied to (java.lang.String)
       f1.gibAusO3(s1); // Verboten (weil s1 keine Number ist)
         ^
D:\Java\GenTypen03.java:122: <N>gibAusK3(N) in PaarG
cannot be applied to (java.lang.String)
    PaarG.gibAusK3(s1); // Verboten (weil s1 keine Number ist)
         ^
------------------------------------------------------------------------ */
