// Datei GenTypen04.java
/* ------------------------------------------------------------------------
Eine generische Klasse PaarH<K> wird vereinbart. Als "Zubehoer" werden
(in der Klasse GenTypen04) dazu sechs (Klassen-) Methoden gibAusA bis
gibAusF vereinbart, mit denen man Objekte von mehr oder weniger vielen
der Typen PaarH<K> ausgeben kann. Genauer:

A. Die Methode gibAusA kann man nur auf Objekte des Typs PaarH<Object>
anwenden, aber nicht auf Objekte der Typen PaarH<String> oder
PaarH<Number> etc. Grund: Obwohl String ein Untertyp von Object ist,
gilt PaarH<String> nicht als Untertyp von PaarH<Object> (und fuer
Number und andere Typen gilt Entsprechendes).

B. Die Methode gibAusB kann man auf Objekte der Typen PaarH<S> anwenden,
bei denen S gleich dem Typ Number oder gleich einem Untertyp von Number
ist. Zu diesen Typen gehoeren z.B. PaarH<Number>, PaarH<Double> und
PaarH<Integer>. Nicht dazu gehoeren z.B. die Typen PaarH<String> und
PaarH<Object> (weil String und Object keine Untertypen von Number sind).

C. Fuer die Methode gibAusC gilt genau das Gleiche wie fuer gibAusB.

D. Die Methode gibAusD kann man auf Objekte der Typen PaarH<S> anwenden,
bei denen S gleich dem Typ Number oder gleich einem Obertyp von Number
ist. Zu diesen Typen gehoeren nur die Typen PaarH<Number> und
PaarH<Object>, weil Object der einzige Obertyp von Number ist.

E. Die Methode gibAusE kann man auf alle Objekte aller Typen PaarH<S>
anwenden, wobei S ein beliebiger Typ sein darf.

F. Fuer die Methode gibAusF gilt genau das Gleiche wie fuer gibAusE.
---------------------------------------------------------------------------
Zur Erinnerung: Die abstrakte Klasse Number ist ein Obertyp von Double und
Integer etc. und (wie alle Typen) ein Untertyp von Object:

Object <--- Number <--+-- Double
                      |
                      +-- Integer
------------------------------------------------------------------------ */
class PaarH<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarH(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarH
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
} // Klasse PaarH
// ==============================================================
class GenTypen04 {
   // -----------------------------------------------------------
   // Die sechs (Klassen-) Methoden gibAusA bis gibAusF:
   static
   void gibAusA(PaarH<Object>           p) {printf("%s\n", p);}
   // --------------------------------------------------------
   static <S extends Number>
   void gibAusB(PaarH<S>                p) {printf("%s\n", p);}
   // --------------------------------------------------------
   static
   void gibAusC(PaarH<? extends Number> p) {printf("%s\n", p);}
   // --------------------------------------------------------
   static
   void gibAusD(PaarH<? super   Number> p) {printf("%s\n", p);}
   // --------------------------------------------------------
   static <S>
   void gibAusE(PaarH<S>                p) {printf("%s\n", p);}
   // --------------------------------------------------------
   static
   void gibAusF(PaarH<?>                p) {printf("%s\n", p);}
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen04: Jetzt geht es los!\n");

      // Objekte verschiedener Typen, die als Komponenten
      // von Paaren verwendet werden sollen:
      Object         o1    = new Object ();
      Object         o2    = new Object ();
      String         s1    = new String ("Eins");
      String         s2    = new String ("Zwei");
      Double         d1    = new Double (1.1);
      Double         d2    = new Double (2.2);
      Integer        i1    = new Integer(111);
      Integer        i2    = new Integer(222);

      // Objekte verschiedener PaarH-Typen:
      PaarH<Object>  pOb01 = new PaarH<Object> (o1, o2);
      PaarH<Object>  pOb02 = new PaarH<Object> (s1, s2);
      PaarH<Object>  pOb03 = new PaarH<Object> (d1, i1);
      PaarH<String>  pSt01 = new PaarH<String> (s1, s2);
      PaarH<Number>  pNu01 = new PaarH<Number> (d1, i1);
      PaarH<Double>  pDo01 = new PaarH<Double> (d1, d2);
      PaarH<Integer> pIn01 = new PaarH<Integer>(i1, i2);

      printf("----------------------------------\n");
      printf("A gibAusA(pOb01): "); gibAusA(pOb01);
      printf("B gibAusA(pOb02): "); gibAusA(pOb02);
      printf("C gibAusA(pOb03): "); gibAusA(pOb03);
//    printf("D gibAusA(pSt01): "); gibAusA(pSt01);
//    printf("E gibAusA(pNu01): "); gibAusA(pNu01);
      printf("----------------------------------\n");
      printf("F gibAusB(pNu01): "); gibAusB(pNu01);
      printf("G gibAusB(pDo01): "); gibAusB(pDo01);
      printf("H gibAusB(pIn01): "); gibAusB(pIn01);
//    printf("I gibAusB(pSt01): "); gibAusB(pSt01);
//    printf("J gibAusB(pOb02): "); gibAusB(pOb02);
      printf("----------------------------------\n");
      printf("K gibAusC(pNu01): "); gibAusC(pNu01);
      printf("L gibAusC(pDo01): "); gibAusC(pDo01);
      printf("M gibAusC(pIn01): "); gibAusC(pIn01);
//    printf("N gibAusC(pSt01): "); gibAusC(pSt01);
//    printf("O gibAusC(pOb02): "); gibAusC(pOb02);
      printf("----------------------------------\n");
      printf("P gibAusD(pNu01): "); gibAusD(pNu01);
      printf("Q gibAusD(pOb02): "); gibAusD(pOb02);
//    printf("R gibAusD(pDo01): "); gibAusD(pDo01);
//    printf("S gibAusD(pIn01): "); gibAusD(pIn01);
      printf("----------------------------------\n");
      printf("T gibAusE(pOb02): "); gibAusE(pOb02);
      printf("U gibAusE(pOb03): "); gibAusE(pOb03);
      printf("V gibAusE(pSt01): "); gibAusE(pSt01);
      printf("W gibAusE(pNu01): "); gibAusE(pNu01);
      printf("X gibAusE(pDo01): "); gibAusE(pDo01);
      printf("Y gibAusE(pIn01): "); gibAusE(pIn01);
      printf("----------------------------------\n");
      printf("Z gibAusF(pOb02): "); gibAusF(pOb02);
      printf("A gibAusF(pOb03): "); gibAusF(pOb03);
      printf("B gibAusF(pSt01): "); gibAusF(pSt01);
      printf("C gibAusF(pNu01): "); gibAusF(pNu01);
      printf("D gibAusF(pDo01): "); gibAusF(pDo01);
      printf("E gibAusF(pIn01): "); gibAusF(pIn01);
      printf("----------------------------------\n");


      printf("GenTypen04: Das war's erstmal!");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen04
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen04:

GenTypen04: Jetzt geht es los!
----------------------------------
A gibAusA(pOb01): [java.lang.Object@3e25a5, java.lang.Object@19821f]
B gibAusA(pOb02): [Eins, Zwei]
C gibAusA(pOb03): [1.1, 111]
----------------------------------
F gibAusB(pNu01): [1.1, 111]
G gibAusB(pDo01): [1.1, 2.2]
H gibAusB(pIn01): [111, 222]
----------------------------------
K gibAusC(pNu01): [1.1, 111]
L gibAusC(pDo01): [1.1, 2.2]
M gibAusC(pIn01): [111, 222]
----------------------------------
P gibAusD(pNu01): [1.1, 111]
Q gibAusD(pOb02): [Eins, Zwei]
----------------------------------
T gibAusE(pOb02): [Eins, Zwei]
U gibAusE(pOb03): [1.1, 111]
V gibAusE(pSt01): [Eins, Zwei]
W gibAusE(pNu01): [1.1, 111]
X gibAusE(pDo01): [1.1, 2.2]
Y gibAusE(pIn01): [111, 222]
----------------------------------
Z gibAusF(pOb02): [Eins, Zwei]
A gibAusF(pOb03): [1.1, 111]
B gibAusF(pSt01): [Eins, Zwei]
C gibAusF(pNu01): [1.1, 111]
D gibAusF(pDo01): [1.1, 2.2]
E gibAusF(pIn01): [111, 222]
----------------------------------
GenTypen04: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\GenTypen04.java:102: gibAusA(PaarH<java.lang.Object>)
in GenTypen04 cannot be applied to (PaarH<java.lang.String>)
    printf("D gibAusA(pSt01): "); gibAusA(pSt01);
                                  ^
D:\Java\GenTypen04.java:103: gibAusA(PaarH<java.lang.Object>)
in GenTypen04 cannot be applied to (PaarH<java.lang.Number>)
    printf("E gibAusA(pNu01): "); gibAusA(pNu01);
                                  ^
D:\Java\GenTypen04.java:108: <S>gibAusB(PaarH<S>)
in GenTypen04 cannot be applied to (PaarH<java.lang.String>)
    printf("I gibAusB(pSt01): "); gibAusB(pSt01);
                                  ^
D:\Java\GenTypen04.java:109: <S>gibAusB(PaarH<S>)
in GenTypen04 cannot be applied to (PaarH<java.lang.Object>)
    printf("J gibAusB(pOb02): "); gibAusB(pOb02);
                                  ^
D:\Java\GenTypen04.java:114: gibAusC(PaarH<? extends java.lang.Number>)
in GenTypen04 cannot be applied to (PaarH<java.lang.String>)
    printf("N gibAusC(pSt01): "); gibAusC(pSt01);
                                  ^
D:\Java\GenTypen04.java:115: gibAusC(PaarH<? extends java.lang.Number>)
in GenTypen04 cannot be applied to (PaarH<java.lang.Object>)
    printf("O gibAusC(pOb02): "); gibAusC(pOb02);
                                  ^
D:\Java\GenTypen04.java:119: gibAusD(PaarH<? super java.lang.Number>)
in GenTypen04 cannot be applied to (PaarH<java.lang.Double>)
    printf("R gibAusD(pDo01): "); gibAusD(pDo01);
                                  ^
D:\Java\GenTypen04.java:120: gibAusD(PaarH<? super java.lang.Number>)
in GenTypen04 cannot be applied to (PaarH<java.lang.Integer>)
    printf("S gibAusD(pIn01): "); gibAusD(pIn01);
                                  ^
------------------------------------------------------------------------ */
