// Datei GenTypen05.java
/* ------------------------------------------------------------------------
Eine generische Klasse PaarI<K> wird vereinbart. Als "Zubehoer" werden
(in der Klasse GenTypen05) dazu sechs (Klassen-) Methoden vertauscheEtcA
bis vertauscheEtcF vereinbart, mit denen man Objekte von mehr oder weniger
vielen der Typen PaarI<K> bearbeiten kann (siehe dazu auch das sehr aehn-
liche Programm GenTypen04). Ein Objekt p "bearbeiten" soll hier heissen:
Die Komponenten p.k1 und p.k2 des Paares p vertauschen und das veraenderte
Paar p ausgegeben.

Nur die vertauscheEtc-Methoden A, B und E funktionieren richtig, die
Methoden C, D und F werden vom Ausfuehrer abgelehnt. Es wird empfohlen,
die Fehlermeldungen des Compilers (siehe unten) genau zu lesen.
Im Programm GenTypen10 findet man verbesserte und funktionierende
Versionen der Methoden C, D und F.
---------------------------------------------------------------------------
Zur Erinnerung: Die abstrakte Klasse Number ist ein Obertyp von Double und
Integer etc. und (wie alle Typen) ein Untertyp von Object:

Object <--- Number <--+-- Double
                      |
                      +-- Integer
------------------------------------------------------------------------ */
class PaarI<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarI(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarI
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
} // Klasse PaarI
// ==============================================================
class GenTypen05 {
   // -----------------------------------------------------------
   // Sechs (Klassen-) Methoden vertauscheEtcA bis vertauscheEtcF:
   static
   void vertauscheEtcA(PaarI<Object>           p) {
      Object tmp = p.k1;
      p.k1       = p.k2;
      p.k2       = tmp;
      printf("A %s%n", p);
   } // vertauscheEtcA
   // ---------------------------------------------
   static <S extends Number>
   void vertauscheEtcB(PaarI<S>                p) {
      S tmp = p.k1;
      p.k1  = p.k2;
      p.k2  = tmp;
      printf("B %s%n", p);
   } // vertauscheEtcB
   // ---------------------------------------------
   static
   void vertauscheEtcC(PaarI<? extends Number> p) {
      Number tmp = p.k1;
//    p.k1       = p.k2; // Verboten!
//    p.k2       = tmp;  // Verboten!
      printf("C %s%n", p);
   } // vertauscheEtcC
   // ---------------------------------------------
   static
   void vertauscheEtcD(PaarI<? super   Number> p) {
      Object tmp = p.k1;
//    p.k1       = p.k2; // Verboten!
//    p.k2       = tmp;  // Verboten!
      printf("D %s%n", p);
   } // vertauscheEtcD
   // ---------------------------------------------
   static <S>
   void vertauscheEtcE(PaarI<S>                p) {
      S tmp = p.k1;
      p.k1  = p.k2;
      p.k2  = tmp;
      printf("E %s%n", p);
   }
   // ---------------------------------------------
   static
   void vertauscheEtcF(PaarI<?>                p) {
      Object tmp = p.k1;
//    p.k1       = p.k2; // Verboten!
//    p.k2       = tmp;  // Verboten!
      printf("F %s%n", p);
   } // vertauscheEtcF
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen05: Jetzt geht es los!%n");

      // Objekte verschiedener Typen, die als Komponenten
      // von Paaren verwendet werden sollen:
      Object         o1    = new Object ();
      Object         o2    = new Object ();
      String         s1    = new String ("Eins");
      String         s2    = new String ("Zwei");
      Double         d1    = new Double (1.1);
      Double         d2    = new Double (2.2);
      Integer        i1    = new Integer(111);
      Integer        i2    = new Integer(222);

      // Objekte verschiedener PaarI-Typen:
      PaarI<Object>  pOb01 = new PaarI<Object> (o1, o2);
      PaarI<Object>  pOb02 = new PaarI<Object> (s1, s2);
      PaarI<Object>  pOb03 = new PaarI<Object> (d1, i1);
      PaarI<String>  pSt01 = new PaarI<String> (s1, s2);
      PaarI<Number>  pNu01 = new PaarI<Number> (d1, i1);
      PaarI<Double>  pDo01 = new PaarI<Double> (d1, d2);
      PaarI<Integer> pIn01 = new PaarI<Integer>(i1, i2);

      // Die Methoden vertauscheEtcA, ...B und ...E werden jeweils
      // zweimal auf dasselbe PaarI-Objekt angewendet (damit der
      // Vertausche-Effekt besonders leicht erkennbar wird):
      printf("-------------------------------------%n");
      printf("vertauscheEtcA(pOb01): "); vertauscheEtcA(pOb01);
      printf("vertauscheEtcA(pOb01): "); vertauscheEtcA(pOb01);
      printf("vertauscheEtcA(pOb02): "); vertauscheEtcA(pOb02);
      printf("vertauscheEtcA(pOb02): "); vertauscheEtcA(pOb02);
      printf("vertauscheEtcA(pOb03): "); vertauscheEtcA(pOb03);
      printf("vertauscheEtcA(pOb03): "); vertauscheEtcA(pOb03);
      printf("-------------------------------------%n");
      printf("vertauscheEtcB(pNu01): "); vertauscheEtcB(pNu01);
      printf("vertauscheEtcB(pNu01): "); vertauscheEtcB(pNu01);
      printf("vertauscheEtcB(pDo01): "); vertauscheEtcB(pDo01);
      printf("vertauscheEtcB(pDo01): "); vertauscheEtcB(pDo01);
      printf("vertauscheEtcB(pIn01): "); vertauscheEtcB(pIn01);
      printf("vertauscheEtcB(pIn01): "); vertauscheEtcB(pIn01);
      printf("-------------------------------------%n");
      printf("vertauscheEtcE(pOb02): "); vertauscheEtcE(pOb02);
      printf("vertauscheEtcE(pOb02): "); vertauscheEtcE(pOb02);
      printf("vertauscheEtcE(pOb03): "); vertauscheEtcE(pOb03);
      printf("vertauscheEtcE(pOb03): "); vertauscheEtcE(pOb03);
      printf("vertauscheEtcE(pSt01): "); vertauscheEtcE(pSt01);
      printf("vertauscheEtcE(pSt01): "); vertauscheEtcE(pSt01);
      printf("vertauscheEtcE(pNu01): "); vertauscheEtcE(pNu01);
      printf("vertauscheEtcE(pNu01): "); vertauscheEtcE(pNu01);
      printf("vertauscheEtcE(pDo01): "); vertauscheEtcE(pDo01);
      printf("vertauscheEtcE(pDo01): "); vertauscheEtcE(pDo01);
      printf("vertauscheEtcE(pIn01): "); vertauscheEtcE(pIn01);
      printf("vertauscheEtcE(pIn01): "); vertauscheEtcE(pIn01);
      printf("-------------------------------------%n");

      printf("GenTypen05: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen05
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen05:

GenTypen05: Jetzt geht es los!
-------------------------------------
vertauscheEtcA(pOb01): A [java.lang.Object@47858e, java.lang.Object@19134f4]
vertauscheEtcA(pOb01): A [java.lang.Object@19134f4, java.lang.Object@47858e]
vertauscheEtcA(pOb02): A [Zwei, Eins]
vertauscheEtcA(pOb02): A [Eins, Zwei]
vertauscheEtcA(pOb03): A [111, 1.1]
vertauscheEtcA(pOb03): A [1.1, 111]
-------------------------------------
vertauscheEtcB(pNu01): B [111, 1.1]
vertauscheEtcB(pNu01): B [1.1, 111]
vertauscheEtcB(pDo01): B [2.2, 1.1]
vertauscheEtcB(pDo01): B [1.1, 2.2]
vertauscheEtcB(pIn01): B [222, 111]
vertauscheEtcB(pIn01): B [111, 222]
-------------------------------------
vertauscheEtcE(pOb02): E [Zwei, Eins]
vertauscheEtcE(pOb02): E [Eins, Zwei]
vertauscheEtcE(pOb03): E [111, 1.1]
vertauscheEtcE(pOb03): E [1.1, 111]
vertauscheEtcE(pSt01): E [Zwei, Eins]
vertauscheEtcE(pSt01): E [Eins, Zwei]
vertauscheEtcE(pNu01): E [111, 1.1]
vertauscheEtcE(pNu01): E [1.1, 111]
vertauscheEtcE(pDo01): E [2.2, 1.1]
vertauscheEtcE(pDo01): E [1.1, 2.2]
vertauscheEtcE(pIn01): E [222, 111]
vertauscheEtcE(pIn01): E [111, 222]
-------------------------------------
GenTypen05: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\GenTypen05.java:61: assigning to wildcard ? extends java.lang.Number
    p.k1       = p.k2; // Verboten!
                  ^
D:\Java\GenTypen05.java:62: assigning to wildcard ? extends java.lang.Number
    p.k2       = tmp;  // Verboten!
                 ^
D:\Java\GenTypen05.java:69: assigning from wildcard ? super java.lang.Number
    p.k1       = p.k2; // Verboten!
                  ^
D:\Java\GenTypen05.java:70: incompatible types
found   : java.lang.Object
required: ? super java.lang.Number
    p.k2       = tmp;  // Verboten!
                 ^
D:\Java\GenTypen05.java:85: assigning from wildcard ?
    p.k1       = p.k2; // Verboten!
                  ^
D:\Java\GenTypen05.java:86: assigning to wildcard ?
    p.k2       = tmp;  // Verboten!
                 ^
------------------------------------------------------------------------ */
