// Datei GenTypen06.java
/* ------------------------------------------------------------------------
Die generische Schnittstelle Comparable<K> aus der Java-Standardbibliothek
kann man so implementieren, dass man Aepfel nur mit Aepfeln,aber nicht
mit Birnen vergleichen kann.

Mit der von der Klasse Object geerbten equals-Methode kann man z.B.
Aepfel mit beliebigen Objekten (also auch mit Birnen) vergleichen.

Die compareTo- und die equals-Methode einer Klasse Apfel sind
konsistent, wenn fuer alle Apfel-Objekte a1 und a2 die beiden Ausdruecke
(compareTo(a1, a2) == 0) und (a1.equals((Object)a2)) gleiche Werte
(true oder false) haben. Wenn die Methoden nicht konsistent sind,
gibt es Probleme beim Sortieren von Apfel-Objekten und beim Sammeln von
Apfel-Objekten in Sammlungen der Typen SortedSet und SortedMap.
------------------------------------------------------------------------ */
// ==============================================================
class Apfel implements Comparable<Apfel>, java.io.Serializable {
   // Eine Klasse mit einer speziellen compareTo-Methode und
   // einer damit konsistenten equals-Methode.
   private int qualitaet;
   private int gewicht;

   public int compareTo(Apfel that) {
      // Beim Vergleich zweier Aepfel zaehlt zuerst nur die
      // Qualitaet. Nur bei gleicher Qualitaet entscheidet
      // das Gewicht, welcher Apfel "der Groessere" ist.
      if (this.qualitaet != that.qualitaet) {
         return this.qualitaet - that.qualitaet;
      } else {
         return this.gewicht   - that.gewicht;
      }
   } // compareTo

   public boolean equals(Object ob) {
      if (ob == null || !(ob instanceof Apfel)) {
         return false;
      }
      Apfel that = (Apfel) ob;
      return this.qualitaet == that.qualitaet &&
             this.gewicht   == that.gewicht;
   } // equals

   public Apfel(int qualitaet, int gewicht) {
      this.qualitaet = qualitaet;
      this.gewicht   = gewicht;
   } // Konstruktor Apfel

   public String toString() {
      return "Apfel, Qualitaet: " + qualitaet + ", Gewicht: " + gewicht;
   } // toString

} // class Apfel
// ==============================================================
class Birne implements Comparable<Birne> {
   // Diese Klasse unterscheidet sich nur durch ihren Namen
   // von der Klasse Apfel.
   private int qualitaet;
   private int gewicht;

   public int compareTo(Birne that) {
      // Beim Vergleich zweier Birnen zaehlt zuerst nur die
      // Qualitaet. Nur bei gleicher Qualitaet entscheidet
      // das Gewicht, welche Birne "die Groessere" ist.
      if (that == null) {
         throw new NullPointerException();
      } else if (this.qualitaet != that.qualitaet) {
         return this.qualitaet - that.qualitaet;
      } else {
         return this.gewicht   -that.gewicht;
      }
   } // compareTo

   public boolean equals(Object ob) {
      if (ob == null || !(ob instanceof Birne)) {
         return false;
      }
      Birne that = (Birne) ob;
      return this.qualitaet == that.qualitaet &&
             this.gewicht   == that.gewicht;
   } // equals

   public Birne(int qualitaet, int gewicht) {
      this.qualitaet = qualitaet;
      this.gewicht   = gewicht;
   } // Konstruktor Birne

   public String toString() {
      return "Birne, Qualitaet: " + qualitaet + ", Gewicht: " + gewicht;
   } // toString

} // class Birne
// ==============================================================
class GenTypen06 {

   static public void main(String [] sonja) {
      printf("GenTypen06: Jetzt geht es los!%n");

      Apfel a1 = new Apfel(20, 10);
      Apfel a2 = new Apfel(15, 20);
      Apfel a3 = new Apfel(15, 30);
      Apfel a4 = new Apfel(15, 30);

      Birne b1 = new Birne(20, 10);
      Birne b2 = new Birne(15, 30);
      Birne b3 = new Birne(15, 20);
      Birne b4 = new Birne(15, 20);

      printf("---------------------------------------%n");
      printf("A a1: %s%n", a1);
      printf("B a2: %s%n", a2);
      printf("C a3: %s%n", a3);
      printf("---------------------------------------%n");
      printf("D b1: %s%n", b1);
      printf("E b2: %s%n", b2);
      printf("F b3: %s%n", b3);
      printf("---------------------------------------%n");
      printf("G a1.compareTo(a2): %s%n", a1.compareTo(a2));
      printf("H a2.compareTo(a1): %s%n", a2.compareTo(a1));
      printf("I a2.compareTo(a3): %s%n", a2.compareTo(a3));
      printf("J a3.compareTo(a2): %s%n", a3.compareTo(a2));
      printf("K a3.compareTo(a4): %s%n", a3.compareTo(a4));
      printf("L a4.compareTo(a3): %s%n", a4.compareTo(a3));
//    printf("M a1.compareTo(b1): %s%n", a1.compareTo(b1)); // CF
      printf("---------------------------------------%n");
      printf("N a1.equals(a2)   : %s%n", a1.equals(a2)   );
      printf("O a3.equals(a4)   : %s%n", a3.equals(a4)   );
      printf("P a3.equals(a4)   : %s%n", a3.equals(a4)   );
      printf("Q a1.equals(b1)   : %s%n", a1.equals(b1)   );
      printf("---------------------------------------%n");
      printf("R b1.compareTo(b2): %s%n", b1.compareTo(b2));
      printf("S b2.compareTo(b1): %s%n", b2.compareTo(b1));
      printf("T b2.compareTo(b3): %s%n", b2.compareTo(b3));
      printf("U a3.compareTo(b2): %s%n", b3.compareTo(b2));
      printf("V a3.compareTo(b4): %s%n", b3.compareTo(b4));
      printf("W a4.compareTo(b3): %s%n", b4.compareTo(b3));
//    printf("X a1.compareTo(b1): %s%n", b1.compareTo(a1)); // CF
      printf("---------------------------------------%n");
      printf("Y b1.equals(b2)   : %s%n", b1.equals(b2)   );
      printf("Z b3.equals(b4)   : %s%n", b3.equals(b4)   );
      printf("A b3.equals(b4)   : %s%n", b3.equals(b4)   );
      printf("B b1.equals(a1)   : %s%n", b1.equals(a1)   );
      printf("---------------------------------------%n");

      printf("GenTypen06: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class GenTypen06
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen06:

GenTypen06: Jetzt geht es los!
---------------------------------------
A a1: Apfel, Qualitaet: 20, Gewicht: 10
B a2: Apfel, Qualitaet: 15, Gewicht: 20
C a3: Apfel, Qualitaet: 15, Gewicht: 30
---------------------------------------
D b1: Birne, Qualitaet: 20, Gewicht: 10
E b2: Birne, Qualitaet: 15, Gewicht: 30
F b3: Birne, Qualitaet: 15, Gewicht: 20
---------------------------------------
G a1.compareTo(a2): 5
H a2.compareTo(a1): -5
I a2.compareTo(a3): -10
J a3.compareTo(a2): 10
K a3.compareTo(a4): 0
L a4.compareTo(a3): 0
---------------------------------------
N a1.equals(a2)   : false
O a3.equals(a4)   : true
P a3.equals(a4)   : true
Q a1.equals(b1)   : false
---------------------------------------
R b1.compareTo(b2): 5
S b2.compareTo(b1): -5
T b2.compareTo(b3): 10
U a3.compareTo(b2): -10
V a3.compareTo(b4): 0
W a4.compareTo(b3): 0
---------------------------------------
Y b1.equals(b2)   : false
Z b3.equals(b4)   : true
A b3.equals(b4)   : true
B b1.equals(a1)   : false
---------------------------------------
GenTypen06: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\GenTypen06.java:124: compareTo(Apfel) in Apfel
cannot be applied to (Birne)
    printf("M a1.compareTo(b1): %s%n", a1.compareTo(b1)); // CF
                                         ^
D:\Java\GenTypen06.java:137: compareTo(Birne) in Birne
cannot be applied to (Apfel)
    printf("X a1.compareTo(b1): %s%n", b1.compareTo(a1)); // CF
                                         ^
------------------------------------------------------------------------ */
