// Datei GenTypen07.java
/* --------------------------------------------------------------
Objekte der OberKlasse sind Verzierbar (aber nicht Dehnbar).
Die UnterKlasse erweitert die OberKlasse und implementiert
die Schnittstelle Dehnbar. Somit sind die Objekte der
UnterKlasse Verzierbar und Dehnbar.

Die generische Klasse PaarJ hat einen Typparameter K.
Der darf nur ersetzt werden durch einen Typ der gleichzeitig
Untertyp von OberKlasse, Verzierbar und Dehnbar ist.
K darf also z.B. durch die UnterKlasse ersetzt werden, aber
nicht durch die OberKlasse (weil die die Schnittstelle Dehnbar
nicht implementiert) und nicht durch die Klasse String (weil
die keine der drei Bedingungen erfuellt).
-------------------------------------------------------------- */
// ==============================================================
interface Verzierbar {
   void verziere1();           // implizit public
   void verziere2();           // implizit public
} // interface Verzierbar
// ==============================================================
interface Dehnbar {
   void dehneUm(int prozent);  // implizit public
} // interface Dehnbar
// ==============================================================
class OberKlasse implements Verzierbar {
   String verzierung1 = "   "; // 3 Blanks als "leere Verzierung"
   String verzierung2 = "   "; // 3 Blanks als "leere Verzierung"
   double laenge;

   OberKlasse(double laenge) {this.laenge = laenge;}

   public String toString() {
      return verzierung1 + laenge + verzierung2;
   } // toString

   // Implementierung der Schnittstelle Verzierbar:
   public void verziere1() {verzierung1 = "-->";}
   public void verziere2() {verzierung2 = "<--";}

} // class OberKlasse
// ==============================================================
class UnterKlasse extends OberKlasse implements Dehnbar {
   UnterKlasse(double laenge) {super(laenge);}

   // Implementierung der Schnittstelle Dehnbar:
   public void dehneUm(int p) {
      // Dehnt die laenge um p Prozent (aber nur, wenn p
      // zwischen 1 und 200 liegt, sonst passiert nichts).

      // Dehnungen nur um 1 bis 200 % moeglich:
      if (p < 1 || 200 < p) return; // Mach nichts!

      laenge += laenge * p / 100.0; // Dehne die laenge um p % !
   } // dehneUm
} // class UnterKlasse
// ==============================================================
class PaarJ<K extends OberKlasse & Verzierbar & Dehnbar> {
   // K darf nur ersetzt werden durch einen Typ der gleichzeitig
   // Untertyp von OberKlasse, Verzierbar und Dehnbar ist.
   // Da OberKlasse eine Klasse ist und Verzierbar und Dehnbar
   // Schnittstellen sind, muss "der Ersatz fuer K" eine
   // (echte oder unechte) Unterklasse von OberKlasse sein, die
   // die Schnittstellen Verzierbar und Dehnbar implementiert.
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarJ(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarJ
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
} // Klasse PaarJ
// ==============================================================
class GenTypen07 {
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen07: Jetzt geht es los!%n");

      // Zwei Objekte, die zu einem Paar zusammengefasst
      // werden sollen:
      UnterKlasse        ob1   = new UnterKlasse(10.0);
      UnterKlasse        ob2   = new UnterKlasse(20.0);

      PaarJ<UnterKlasse> paar1 = new PaarJ<UnterKlasse>(ob1, ob2);
//    PaarJ<String>      paar2; // Fehlermeldung des Compilers!

      printf("---------------------------------%n");
      printf("A paar1: %s%n", paar1);
      ob1.verziere1();
      printf("B paar1: %s%n", paar1);
      ob1.verziere2();
      printf("C paar1: %s%n", paar1);
      ob1.dehneUm(10);
      printf("D paar1: %s%n", paar1);
      printf("---------------------------------%n");
      paar1.k2.verziere1();
      paar1.k2.verziere2();
      paar1.k2.dehneUm(50);
      printf("E paar1: %s%n", paar1);
      printf("---------------------------------%n");
      printf("GenTypen07: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // GenTypen07
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen07:

GenTypen07: Jetzt geht es los!
---------------------------------
A paar1: [   10.0   ,    20.0   ]
B paar1: [-->10.0   ,    20.0   ]
C paar1: [-->10.0<--,    20.0   ]
D paar1: [-->11.0<--,    20.0   ]
---------------------------------
E paar1: [-->11.0<--, -->30.0<--]
---------------------------------
GenTypen07: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die entsprechende Zeile
kein Kommentar ist:

D:\Java\GenTypen07.java:90: type parameter java.lang.String is not
within its bound
    PaarJ<String>      paar2; // Fehlermeldung des Compilers!
          ^
------------------------------------------------------------------------ */
