// Datei GenTypen09.java
/* ------------------------------------------------------------------------
Fuenf Methoden mit Parametern der Typen PaarL<? extends T> bzw.
PaarL<? super T>, d.h. p-Typen werden mit beschraenkten Joker-
Variablen parametrisiert.
---------------------------------------------------------------------------
Ein parametrisierter Typ wie PaarL<? extends T>
ist haeufig fuer Methodenparameter geeignet, auf die nur lesend
zugegriffen wird (siehe kopiereA, kopiereB und liesKomp1).

Ein parametrisierter Typ wie PaarL<? super T>
ist haeufig fuer Methodenparameter geeignet, aud die nur schreibend
zugegriffen wird (siehe kopiereA, kopiereC und schreibKomp2).
------------------------------------------------------------------------ */
// ==============================================================
class PaarL<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarL(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarL
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "] ";
   } // toString
   // -----------------------------------------------------------
} // Klasse PaarL
// ==============================================================
class GenTypen09 {
   // -----------------------------------------------------------
   static
   <T> void kopiereA(PaarL<? extends T> quelle,
                     PaarL<? super   T> ziel) {
      // Die Komponenten des Paares quelle werden in
      // die Komponenten des Paares ziel   kopiert
      // (d.h. auf quelle wird nur lesend
      //  und  auf ziel   wird nur schreibend zugegriffen).
      ziel.k1 = quelle.k1;
      ziel.k2 = quelle.k2;
   } // kopiereA
   // -----------------------------------------------------------
   static
   <T> void kopiereB(PaarL<? extends T> quelle,
                     PaarL<T>           ziel) {
      // Die Komponenten des Paares quelle werden in
      // die Komponenten des Paares ziel   kopiert
      // (d.h. auf quelle wird nur lesend
      //  und  auf ziel   wird nur schreibend zugegriffen).
      ziel.k1 = quelle.k1;
      ziel.k2 = quelle.k2;
   } // kopiereB
   // -----------------------------------------------------------
   static
   <T> void kopiereC(PaarL<T>           quelle,
                     PaarL<? super   T> ziel) {
      // Die Komponenten des Paares quelle werden in
      // die Komponenten des Paares ziel   kopiert
      // (d.h. auf quelle wird nur lesend
      //  und  auf ziel   wird nur schreibend zugegriffen).
      ziel.k1 = quelle.k1;
      ziel.k2 = quelle.k2;
   } // kopiereC
   // -----------------------------------------------------------
   static
   <T> T liesKomp1(PaarL<? extends T> p) {
      // Liefert die Komponente k1 des Paares p
      // (d.h. auf p wird nur lesend zugegriffen).
      T alt = p.k1; // Lesender Zugriff auf p.k1
      return alt;
   } // liesKomp1
   // -----------------------------------------------------------
   static
   <T> void schreibKomp2(PaarL<? super T> p, T neu) {
      // Schreibt neu in die Komponente des Paares p
      // (d.h. auf p wird nur schreibend zugegriffen).
      p.k2 = neu; // Schreibender Zugriff auf p.k2
   } // schreibKomp2
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen09: Jetzt geht es los!%n");

      // Objekte, die zu einem Paar zusammengefasst werden sollen:
      Double d1 = new Double(1.1);
      Double d2 = new Double(2.2);

      // Ein Paar-Objekt mit "richtigen" Komponenten:
      PaarL<Double> pd = new PaarL<Double>(d1, d2);

      // Viele Paar-Objekte mit null-Komponenten:
      PaarL<Double> pdA = new PaarL<Double>(null, null);
      PaarL<Double> pdB = new PaarL<Double>(null, null);
      PaarL<Double> pdC = new PaarL<Double>(null, null);

      PaarL<Number> pnA = new PaarL<Number>(null, null);
      PaarL<Number> pnB = new PaarL<Number>(null, null);
      PaarL<Number> pnC = new PaarL<Number>(null, null);

      PaarL<Object> poA = new PaarL<Object>(null, null);
      PaarL<Object> poB = new PaarL<Object>(null, null);
      PaarL<Object> poC = new PaarL<Object>(null, null);

      // Das Paar-Objekt pd mit den "richtigen" Komponenten
      // wird mit den drei Kopier-Methoden in alle Paar-Objekte
      // mit den null-Komponenten kopiert:
      kopiereA(pd, pdA); kopiereB(pd, pdB); kopiereC(pd, pdC);
      kopiereA(pd, pnA); kopiereB(pd, pnB); kopiereC(pd, pnC);
      kopiereA(pd, poA); kopiereB(pd, poB); kopiereC(pd, poC);

      printf("-------------------------------------------------%n");
      printf("A pdA, pnA, poA: %s%s%s%n", pdA, pnA, poA);
      printf("B pdB, pnB, poB: %s%s%s%n", pdB, pnB, poB);
      printf("C pdC, pnC, poC: %s%s%s%n", pdC, pnC, poC);
      printf("-------------------------------------------------%n");

      // Die Komponente k1 von drei Paar-Objekten wird gelesen:
      Double d = liesKomp1(pdA);
      Number n = liesKomp1(pnA);
      Object o = liesKomp1(poA);

      printf("D d:  %s%n", d);
      printf("E n:  %s%n", n);
      printf("F o:  %s%n", o);
      printf("-----------------%n");

      // In die Komponente k2 von drei Paar-Objekten wird
      // geschrieben
      schreibKomp2(pdA, d);
      schreibKomp2(pnA, n);
      schreibKomp2(poA, o);

      printf("G pdA: %s%n", pdA);
      printf("H pnA: %s%n", pnA);
      printf("I poA: %s%n", poA);
      printf("-----------------%n");

      printf("GenTypen09: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen09
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen09:

GenTypen09: Jetzt geht es los!
-------------------------------------------------
A pdA, pnA, poA: [1.1, 2.2] [1.1, 2.2] [1.1, 2.2]
B pdB, pnB, poB: [1.1, 2.2] [1.1, 2.2] [1.1, 2.2]
C pdC, pnC, poC: [1.1, 2.2] [1.1, 2.2] [1.1, 2.2]
-------------------------------------------------
D d:  1.1
E n:  1.1
F o:  1.1
-----------------
G pdA: [1.1, 1.1]
H pnA: [1.1, 1.1]
I poA: [1.1, 1.1]
-----------------
GenTypen09: Das war's erstmal!
------------------------------------------------------------------------ */
