// Datei GenTypen11.java
/* ------------------------------------------------------------------------
Keiner der parametrisierten Typen PaarN<Object>, PaarN<String>, ...
ist Ober- oder Untertyp eines anderen dieser Typen.

Alle parametrisierten Typen PaarN<Object>, PaarN<String>, ...
sind Untertypen des Joker-parametrisierten Typs PaarN<?>.

Der Joker-parametrisierte Typ PaarN<?>
ist ein Untertyp des rohen Typs PaarN.


                           PaarN
                             |
                          PaarN<?>
                             |
     +---------------+-------+-------+--------   ...
     |               |               |
PaarN<String>   PaarN<Object>   PaarN<Integer>   ...
---------------------------------------------------------------------------
Zur Erinnerung: Die abstrakte Klasse Number ist ein Obertyp von Double und
Integer etc. und (wie alle Typen) ein Untertyp von Object:

                           Object
                             |
                           Number
                             |
                     +-------+-------+
                     |               |
                  Double          Integer

------------------------------------------------------------------------ */
// ==============================================================
class PaarN<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarN(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarN
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
} // Klasse PaarN
// ==============================================================
class GenTypen11 {
   // -----------------------------------------------------------
   // Die folgende Methode kann man auf Objekte ALLER
   // parametrisierten PaarN-Typen anwenden. Die Methode
   // darf allerdings fast nur lesend auf die Komponenten
   // k1 und k2 ihres Parameters p zugreifen, weil diese
   // Komponenten zu "irgendeinem Typ ?" gehoeren.
   static String toStringA(PaarN<?> p) {
      Object ob = p.k2; // Lesender     Zugriff, erlaubt
//    p.k2      = ob;   // Schreibender Zugriff, verboten
      p.k1      = null; // Schreibender Zugriff, erlaubt
      return "--> " + p.toString();
   } // toStringA
   // -----------------------------------------------------------
   // Die folgende Methode kann man auf Objekte ALLER
   // PaarN-Typen anwenden, insbesondere auf Objekte des
   // rohen Typs PaarN. Wendet man sie auf Objekte an,
   // die nicht zum rohen Typ PaarN gehoeren, warnt der
   // Compiler:
   static String toStringB(PaarN p) {
      return "<-- " + p.toString();
   } // toStringB
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen11: Jetzt geht es los!%n");

      // Objekte verschiedener Typen, die als Komponenten
      // von Paaren verwendet werden sollen:
      String         s1    = new String ("Eins");
      String         s2    = new String ("Zwei");
      Double         d1    = new Double (1.1);
      Double         d2    = new Double (2.2);
      Integer        i1    = new Integer(111);
      Integer        i2    = new Integer(222);

      // Objekte verschiedener PaarN-Typen:
      PaarN<Object>  pOb01 = new PaarN<Object> (s1, i1);
      PaarN<String>  pSt01 = new PaarN<String> (s1, s2);
      PaarN<Number>  pNu01 = new PaarN<Number> (d1, i1);
      PaarN<Double>  pDo01 = new PaarN<Double> (d1, d2);
      PaarN<Integer> pIn01 = new PaarN<Integer>(i1, i2);

      // Der Typ PaarN<?> ist ein Obertyp aller
      // parametrisierten PaarN-Typen PaarN<Object>,
      // PaarN<String>, PaarN<Number> etc.
      PaarN<?>       p01   = pOb01;
      PaarN<?>       p02   = pSt01;
      PaarN<?>       p03   = pNu01;
      PaarN<?>       p04   = pNu01;
      PaarN<?>       p05   = pNu01;

      // Der rohe Typ PaarN ist ein Obertyp des
      // Typs PaarN<?>:
      PaarN          p11   = p01;
      PaarN          p12   = p02;
      PaarN          p13   = p03;
      PaarN          p14   = p04;
      PaarN          p15   = p05;

      printf("-----------------------------------%n");
      printf("A toStringA(p01): %s%n", toStringA(p01));
      printf("B toStringA(p02): %s%n", toStringA(p02));
      printf("C toStringA(p03): %s%n", toStringA(p03));
      printf("D toStringA(p04): %s%n", toStringA(p04));
      printf("E toStringA(p05): %s%n", toStringA(p05));
      printf("-----------------------------------%n");
      printf("F toStringA(p11): %s%n", toStringA(p11)); // Warnung
      printf("G toStringA(p12): %s%n", toStringA(p12)); // Warnung
      printf("H toStringA(p13): %s%n", toStringA(p13)); // Warnung
      printf("I toStringA(p14): %s%n", toStringA(p14)); // Warnung
      printf("J toStringA(p15): %s%n", toStringA(p15)); // Warnung
      printf("-----------------------------------%n");
      printf("K toStringB(p11): %s%n", toStringB(p11));
      printf("L toStringB(p12): %s%n", toStringB(p12));
      printf("M toStringB(p13): %s%n", toStringB(p13));
      printf("N toStringB(p14): %s%n", toStringB(p14));
      printf("O toStringB(p15): %s%n", toStringB(p15));
      printf("-----------------------------------%n");
      printf("GenTypen11: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen11
/* ------------------------------------------------------------------------
Warnungen des Compilers:

D:\Java\GenTypen11.java:115: warning: unchecked method invocation:
toStringA(PaarN<?>) in GenTypen11 is applied to (PaarN)
      printf("F toStringA(p11): %s%n", toStringA(p11)); // Warnung
                                    ^
D:\Java\GenTypen11.java:116: warning: unchecked method invocation:
toStringA(PaarN<?>) in GenTypen11 is applied to (PaarN)
      printf("G toStringA(p12): %s%n", toStringA(p12)); // Warnung
                                    ^
D:\Java\GenTypen11.java:117: warning: unchecked method invocation:
toStringA(PaarN<?>) in GenTypen11 is applied to (PaarN)
      printf("H toStringA(p13): %s%n", toStringA(p13)); // Warnung
                                    ^
D:\Java\GenTypen11.java:118: warning: unchecked method invocation:
toStringA(PaarN<?>) in GenTypen11 is applied to (PaarN)
      printf("I toStringA(p14): %s%n", toStringA(p14)); // Warnung
                                    ^
D:\Java\GenTypen11.java:119: warning: unchecked method invocation:
toStringA(PaarN<?>) in GenTypen11 is applied to (PaarN)
      printf("J toStringA(p15): %s%n", toStringA(p15)); // Warnung
                                    ^
---------------------------------------------------------------------------
Ausgabe des Programms GenTypen11:

GenTypen11: Jetzt geht es los!
-----------------------------------
A toStringA(p01): --> [null, 111]
B toStringA(p02): --> [null, Zwei]
C toStringA(p03): --> [null, 111]
D toStringA(p04): --> [null, 111]
E toStringA(p05): --> [null, 111]
-----------------------------------
F toStringA(p11): --> [null, 111]
G toStringA(p12): --> [null, Zwei]
H toStringA(p13): --> [null, 111]
I toStringA(p14): --> [null, 111]
J toStringA(p15): --> [null, 111]
-----------------------------------
K toStringB(p11): <-- [null, 111]
L toStringB(p12): <-- [null, Zwei]
M toStringB(p13): <-- [null, 111]
N toStringB(p14): <-- [null, 111]
O toStringB(p15): <-- [null, 111]
-----------------------------------
GenTypen11: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die betreffende Zeile kein Kommentar ist:

D:\Java\GenTypen11.java:58: assigning to wildcard ?
      p.k2      = ob;   // Schreibender Zugriff, verboten
                  ^
------------------------------------------------------------------------ */
