// Datei GenTypen12.java
/* ------------------------------------------------------------------------
Dieses Programm soll einige falsche und einige fragwuerdige Befehle
demonstrieren, die mit generischen Einheiten zu tun haben. Die
betreffenden Zeilen sind folgendermassen gekennzeichnet:

Fehler   Der Befehl ist falsch. Der Compiler gibt eine Fehler-
         meldung aus und lehnt das Programm ab.
Warnung  Der Befehl ist fragwuerdig. Der Compiler git eine
         Warnung aus, akzeptiert das Programm aber.
Erlaubt  Der Befehl ist in Ordnung. Der Compiler gibt weder
         eine Fehlermeldung noch eine Warnung aus.

Die Fehlermeldungen und Warnung des Compilers findet man als
Kommentar nach dem Programmtext.
------------------------------------------------------------------------ */
// ==============================================================
class PaarO<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarO(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarO
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
   PaarO<K> umgedrehteKopieA() {
      // Liefert eine Kopie dieses Objekts, aber mit
      // vertauschten Komponenten:
      Object ob = new PaarO<K>(this.k2, this.k1);
      return (PaarO<K>) ob;                      // Warnung
   } // umgedrehteKopieA
   // -----------------------------------------------------------
   PaarO<K> umgedrehteKopieB() {
      // Liefert eine Kopie dieses Objekts, aber mit
      // vertauschten Komponenten:
      return new PaarO<K>(this.k2, this.k1);     // Erlaubt
   } // umgedrehteKopieB
   // -----------------------------------------------------------
// K      k3 = new K();                          // Fehler
   Object ob = k1;
   K      k4 = (K) ob;                           // Warnung
   K[]    r1;                                    // Erlaubt
// K[]    r2 = new K[3];                         // Fehler
   // -----------------------------------------------------------
} // Klasse PaarO
// ==============================================================
class GenTypen12 {

   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen12: Jetzt geht es los!%n");

      // Objekte verschiedener Typen, die als Komponenten
      // von Paaren verwendet werden sollen:
      String         s1 = new String ("Eins");
      String         s2 = new String ("Zwei");
      Double         d1 = new Double (1.1);
      Double         d2 = new Double (2.2);

      // Objekte verschiedener PaarO-Typen:
      PaarO<String>  pS = new PaarO<String> (s1, s2);
      PaarO<Double>  pD = new PaarO<Double> (d1, d2);

      // --------------------------------------------------------
      // Darf man eine Reihung mit Komponenten eines
      // p-Typs wie PaarO<String> vereinbaren? Fast nie!
//    PaarO<String>[] r3;                        // Fehler
//    Object          r4 = new PaarO<String>[3]; // Fehler
      PaarO<?>     [] r5 = new PaarO<?>     [3]; // Erlaubt
      // --------------------------------------------------------

      printf("------------------------------------------------%n");
      printf("A            pS.umgedrehteKopieA(): "
                         + pS.umgedrehteKopieA() + "%n");
      printf("B            pD.umgedrehteKopieA(): "
                         + pD.umgedrehteKopieA() + "%n");
      printf("C            pS.umgedrehteKopieB(): "
                         + pS.umgedrehteKopieB() + "%n");
      printf("D            pD.umgedrehteKopieB(): "
                         + pD.umgedrehteKopieB() + "%n");
      printf("------------------------------------------------%n");
      printf("E                   pS.getClass() : "
                         +        pD.getClass() + "%n");
      printf("F                   pD.getClass() : "
                         +        pD.getClass() + "%n");
      printf("G (pS.getClass() == pD.getClass()): "
            + (pS.getClass() == pD.getClass()) // Fehler
             + "%n");

      Class cS = pS.getClass();
      Class cN = pD.getClass();

      printf("H                      (cS == cN) : " + (cS==cN)  + "%n");
      printf("------------------------------------------------%n");
      printf("I    (pS instanceof PaarO<String>): "
//               + (pS instanceof PaarO<String>) // Fehler
             + "%n");
      printf("J    (pS instanceof PaarO<Double>): "
//               + (pS instanceof PaarO<Double>) // Fehler
             + "%n");
      printf("K    (pD instanceof PaarO<String>): "
//               + (pD instanceof PaarO<String>) // Fehler
             + "%n");
      printf("L    (pD instanceof PaarO<Double>): "
//               + (pD instanceof PaarO<Double>) // Fehler
             + "%n");
      printf("------------------------------------------------%n");

      printf("GenTypen12: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen12
/* ------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die betreffenden Zeilen
keine Kommentare sind:

D:\Java\GenTypen12.java:73: generic array creation
    Object          r4 = new PaarO<String>[3]; // Fehler
                         ^
D:\Java\GenTypen12.java:101: illegal generic type for instanceof
                 + (pS instanceof PaarO<String>) // Fehler
                                       ^
D:\Java\GenTypen12.java:104: illegal generic type for instanceof
               + (pS instanceof PaarO<Double>) // Fehler
                                     ^
D:\Java\GenTypen12.java:107: illegal generic type for instanceof
               + (pD instanceof PaarO<String>) // Fehler
                                     ^
D:\Java\GenTypen12.java:110: illegal generic type for instanceof
                 + (pD instanceof PaarO<Double>) // Fehler
                                       ^
---------------------------------------------------------------------------
Warnungen des Compilers:

D:\Java\GenTypen12.java:35: warning: [unchecked] unchecked cast
found   : java.lang.Object
required: PaarO<K>
      return (PaarO<K>) ob;                      // Warnung
                        ^
D:\Java\GenTypen12.java:46: warning: [unchecked] unchecked cast
found   : java.lang.Object
required: K
   K      k4 = (K) ob;                           // Warnung
                   ^
---------------------------------------------------------------------------
Ausgabe des Programms GenTypen12:

GenTypen12: Jetzt geht es los!
------------------------------------------------
A            pS.umgedrehteKopieA(): [Zwei, Eins]
B            pD.umgedrehteKopieA(): [2.2, 1.1]
C            pS.umgedrehteKopieB(): [Zwei, Eins]
D            pD.umgedrehteKopieB(): [2.2, 1.1]
------------------------------------------------
E                   pS.getClass() : class PaarO
F                   pD.getClass() : class PaarO
G (pS.getClass() == pD.getClass()):
H                      (cS == cN) : true
------------------------------------------------
I    (pS instanceof PaarO<String>): true
J    (pS instanceof PaarO<Double>):
K    (pD instanceof PaarO<String>):
L    (pD instanceof PaarO<Double>): true
------------------------------------------------
GenTypen12: Das war's erstmal!
------------------------------------------------------------------------ */
