// Datei GenTypen13.java
/* ------------------------------------------------------------------------
Aus einem Objekt p der Klasse PaarP kann man mit den Methoden namens
toArray eine Reihung r erzeugen, die dieselben Objekte enthaelt wie p.
Die parameterlose Methode toArray liefert eine Reihung des Typs Object[]
und der Laenge 2. Diese toArray-Methode ist typsicher (d.h. sie loest
keine Ausnahmen aus).

Die zweite toArray-Methode hat einen Parameter r vom Typ K[],
speichert die Komponenten von p darin und liefert diese Reihung
als Ergebnis. Falls die Reihung r zu kurz ist, wird als Ersatz
fuer sie eine Reihung der Laenge 2 erzeugt, gefuellt und geliefert.
Diese toArray-Methode ist ziemlich beschraenkt (man kann damit z.B.
ein Paar von Double-Werten nicht eine Reihung von Number- oder von
Object-Objekten umwandeln), dafuer aber typsicher.

Aehnlich wie die PaarP-Objekte in diesem Beispielprogramm enthaelt
auch jedes Collection-Objekt zwei Methoden namens toArray. Die
zweite (generische) dieser Methoden ist nicht beschraenkt, dafuer
aber auch nicht typsicher (sie loest evtl. eine Ausnahme des Typs
ArrayStoreException aus).
------------------------------------------------------------------------ */
import java.lang.reflect.Array;
// ==============================================================
class PaarP<K> {    // Eine generische Klasse, 1 Parameter K
   K k1;            // Komponente 1
   K k2;            // Komponente 2
   // -----------------------------------------------------------
   PaarP(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarP
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
   Object[] toArray() {
      Object[] erg = new Object[] {k1, k2};
      return erg;
   } // toArray
   // -----------------------------------------------------------
   K[] toArray(K[] r) {
      // Falls r zu kurz ist, wird eine neue Reihung der
      // Laenge 2 mit Komponenten des Typs K erzeugt:
      if (r.length < 2) {
         Class ct = r.getClass().getComponentType();
         r        = (K[]) Array.newInstance(ct, 2); // Warnung
      }

      // Die Komponenten dieses Paares in die Reihung r
      // einfuegen:
      r[0] = k1;
      r[1] = k2;
      return r;
   } // toArray
   // -----------------------------------------------------------
} // Klasse PaarP
// ==============================================================
class GenTypen13 {

   // -----------------------------------------------------------
   static void pr(Object[] or) {
      // Gibt die Reihung or "in lesbarer Form" aus (erst den Namen des
      // momentanen Typs von r, dahinter die Komponenten von or).

      // Den Namen des momentanen Typs von or berechnen:
      String
      typName = or.getClass().getName();
      typName = typName.replace("[Ljava.lang.", "");
      typName = typName.replace(";", "[]");

      // Den Namen des Typs von or etc. ausgeben:
      printf("Typ: %s, Kompos: ", typName);
      // Die Komponenten von or ausgeben:
      for (Object o: or) {
         printf("%s, ", o);
      }
      printf("%n");
   } // pr
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen13: Jetzt geht es los!%n");

      // Objekte verschiedener Typen, die als Komponenten
      // von Paaren verwendet werden sollen:
      String         s1 = new String ("Eins");
      String         s2 = new String ("Zwei");
      Double         d1 = new Double (1.1);
      Double         d2 = new Double (2.2);

      // Objekte verschiedener PaarP-Typen:
      PaarP<String>  pS = new PaarP<String> (s1, s2);
      PaarP<Double>  pD = new PaarP<Double> (d1, d2);
      PaarP<Number>  pN = new PaarP<Number> (d1, d2);

      // Reihungen verschiedener Typen und Laengen:
      String[]       rS = new String[2];
      Double[]       rD = new Double[3];
      Number[]       rN = new Number[0];

      printf("---------------------------------------------------------%n");
      printf("A pS            : %14s (Paar von String)%n", pS);
      printf("B pD            : %14s (Paar von Double)%n", pD);
      printf("C pN            : %14s (Paar von Number)%n", pN);
      printf("---------------------------------------------------------%n");
      printf("D pS.toArray()  : "); pr(pS.toArray());
      printf("E pD.toArray()  : "); pr(pD.toArray());
      printf("F pN.toArray()  : "); pr(pN.toArray());
      printf("---------------------------------------------------------%n");
      printf("G pS.toArray(rS): "); pr(pS.toArray(rS));
      printf("H pD.toArray(rD): "); pr(pD.toArray(rD));
      printf("I pN.toArray(rN): "); pr(pN.toArray(rN));
      printf("---------------------------------------------------------%n");
//    printf("J pS.toArray(rN): "); pr(pS.toArray(rN));
//    printf("K pD.toArray(rN): "); pr(pD.toArray(rN));

      printf("GenTypen13: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class GenTypen13
/* ------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die betreffenden Zeilen
keine Kommentare sind:

D:\Java\GenTypen13.java:115: cannot find symbol
symbol  : method toArray(java.lang.Number[])
location: class PaarP<java.lang.String>
    printf("J pS.toArray(rN): "); pr(pS.toArray(rN));
                                       ^
D:\Java\GenTypen13.java:116: cannot find symbol
symbol  : method toArray(java.lang.Number[])
location: class PaarP<java.lang.Double>
    printf("K pD.toArray(rN): "); pr(pD.toArray(rN));
                                       ^
---------------------------------------------------------------------------
Warnung des Compilers:

D:\meineDateien\BspBuch\GenTypen13.java:48: warning: [unchecked] unchecked cast
found   : java.lang.Object
required: K[]
         r        = (K[]) Array.newInstance(ct, 2); // Warnung
                                           ^
---------------------------------------------------------------------------
Ausgabe des Programms GenTypen13:

GenTypen13: Jetzt geht es los!
---------------------------------------------------------
A pS            : [Eins, Zwei]   (Paar von String)
B pD            : [1.1, 2.2]     (Paar von Double)
C pN            : [1.1, 2.2]     (Paar von Number)
---------------------------------------------------------
D pS.toArray()  : Typ: Object[],  Kompos: Eins, Zwei,
E pD.toArray()  : Typ: Object[],  Kompos: 1.1, 2.2,
F pN.toArray()  : Typ: Object[],  Kompos: 1.1, 2.2,
---------------------------------------------------------
G pS.toArray(rS): Typ: String[],  Kompos: Eins, Zwei,
H pD.toArray(rD): Typ: Double[],  Kompos: 1.1, 2.2, null,
I pN.toArray(rN): Typ: Number[],  Kompos: 1.1, 2.2,
---------------------------------------------------------
GenTypen13: Das war's erstmal!
------------------------------------------------------------------------ */
