// Datei GenTypen15.java
/* --------------------------------------------------------------------------------------
Dieses Programm zeigt von einigen Typen wie K, K<T>, K<?>, K<Object> etc. welcher
ein Untertyp von welchen anderen ist.

Die generische Klasse K<T> enthaelt 7 KB-Methoden ("Klein-Buchstaben-Methoden")
namens ma bis mg und 7 GB-Methoden ("Gross-Buchstaben-Methoden") namens mA bis mG.
Jede dieser Methoden hat einen Parameter eines "interessanten Typs". Sich entsprechende
Methoden (z.B. ma und mA oder mb und mB etc.) haben Parameter des selben Typs.
Die KB-Methoden geben jeweils einen Buchstaben ("a" bis "g") aus. Jede GB-Methode mit
formalem Parameter p ruft jede KB-Methode mit p als aktuellem Parameter auf.
Einige dieser Aufrufe werden vom Ausfuehrer mit einer Fehlermeldung abgelehnt,
andere loesen eine Warnung aus. Alle abgelehnten Aufrufe wurden in diesem Programm
auskommentiert.

An den Methodenaufrufen und den Fehlermeldungen kann man erkennen, welcher der
"interessanten Typen" ein Untertyp von welchen anderen interessanten Typen ist.
-------------------------------------------------------------------------------------- */

class K<T> {
   // 7 KB-Methoden ("Klein-Buchstaben-Methoden") namens ma bis mg werden vereinbart:
                 void ma(K         p) {printf("a ");}
                 void mb(K<T>      p) {printf("b ");}
                 void mc(K<?>      p) {printf("c ");}
                 void md(K<Object> p) {printf("d ");}
                 void me(K<String> p) {printf("e ");}
   <N>           void mf(K<N>      p) {printf("f ");}
   <N extends K> void mg(K<N>      p) {printf("g ");}
   // -----------------------------------------------------------------------------------
   // 7 GB-Methoden ("Gross-Buchstaben-Methoden") namens mA bis mG werden vereinbart:
   void mA(K         p) {
      ma(p); // K           -> K
      mb(p); // K           -> K<T>        Warnung
      mc(p); // K           -> K<?>
      md(p); // K           -> K<Object>   Warnung
      me(p); // K           -> K<String>   Warnung
      mf(p); // K           -> K<N>        Warnung
      mg(p); // K           -> K<N ext. K> Warnung
   }
   // -----------------------------------------------------------------------------------
   void mB(K<T>      p) {
      ma(p); // K<T>        -> K
      mb(p); // K<T>        -> K<T>
      mc(p); // K<T>        -> K<?>
//    md(p); // K<T>        -> K<Object>   Fehler
//    me(p); // K<T>        -> K<String>   Fehler
//    mf(p); // K<T>        -> K<N>        Fehler
//    mg(p); // K<T>        -> K<N ext. K> Fehler
   }
   // -----------------------------------------------------------------------------------
   void mC(K<?>      p) {
      ma(p); // K<?>        -> K
//    mb(p); // K<?>        -> K<T>        Fehler
      mc(p); // K<?>        -> K<?>
//    md(p); // K<?>        -> K<Object>   Fehler
//    me(p); // K<?>        -> K<String>   Fehler
      mf(p); // K<?>        -> K<N>
//    mg(p); // K<?>        -> K<N ext. K> Fehler
   }
   // -----------------------------------------------------------------------------------
   void mD(K<Object> p) {
      ma(p); // K<Object>   -> K
//    mb(p); // K<Object>   -> K<T>        Fehler
      mc(p); // K<Object>   -> K<?>
      md(p); // K<Object>   -> K<Object>
//    me(p); // K<Object>   -> K<String>   Fehler
      mf(p); // K<Object>   -> K<N>
//    mg(p); // K<Object>   -> K<N ext. K> Fehler
   }
   // -----------------------------------------------------------------------------------
   void mE(K<String> p) {
      ma(p); // K<String>   -> K
//    mb(p); // K<String>   -> K<T>        Fehler
      mc(p); // K<String>   -> K<?>
//    md(p); // K<String>   -> K<Object>   Fehler
      me(p); // K<String>   -> K<String>
      mf(p); // K<String>   -> K<N>
//    mg(p); // K<String>   -> K<N ext. K> Fehler
   }
   // -----------------------------------------------------------------------------------
   <N>
   void mF(K<N>      p) {
      ma(p); // K<N>        -> K
//    mb(p); // K<N>        -> K<T>        Fehler
      mc(p); // K<N>        -> K<?>
//    md(p); // K<N>        -> K<Object>   Fehler
//    me(p); // K<N>        -> K<String>   Fehler
      mf(p); // K<N>        -> K<N>
//    mg(p); // K<N>        -> K<N ext. K> Fehler
   }
   // -----------------------------------------------------------------------------------
   <N extends K>
   void mG(K<N>      p) {
      ma(p); // K<N ext. K> -> K
//    mb(p); // K<N ext. K> -> K<T>        Fehler
      mc(p); // K<N ext. K> -> K<?>
//    md(p); // K<N ext. K> -> K<Object>   Fehler
//    me(p); // K<N ext. K> -> K<String>   Fehler
      mf(p); // K<N ext. K> -> K<N>
      mg(p); // K<N ext. K> -> K<N ext. K>
   }
   // -----------------------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------------------------------
} // class K<T>

class GenTypen15 {
   // -----------------------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("GenTypen15: Jetzt geht es los!%n");

      K         k1 = new K();
      K<Object> k2 = new K<Object>();
      K<String> k3 = new K<String>();

      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mA(new K())");
      new K().mA(new K()); printf("%n");
      printf("%-36s: ", "new K().mA(new K<Object>())");
      new K().mA(new K()); printf("%n");
      printf("%-36s: ", "new K().mA(new K<String>())");
      new K().mA(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mA(new K())");
      new K<Object>().mA(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mA(new K<Object>())");
      new K<Object>().mA(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mA(new K<String>())");
      new K<Object>().mA(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mA(new K())");
      new K<String>().mA(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mA(new K<Object>())");
      new K<String>().mA(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mA(new K<String>())");
      new K<String>().mA(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mB(new K())");
      new K().mB(new K()); printf("%n");
      printf("%-36s: ", "new K().mB(new K<Object>())");
      new K().mB(new K()); printf("%n");
      printf("%-36s: ", "new K().mB(new K<String>())");
      new K().mB(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mB(new K())");
      new K<Object>().mB(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mB(new K<Object>())");
      new K<Object>().mB(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mB(new K<String>())");
      new K<Object>().mB(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mB(new K())");
      new K<String>().mB(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mB(new K<Object>())");
      new K<String>().mB(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mB(new K<String>())");
      new K<String>().mB(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mC(new K())");
      new K().mC(new K()); printf("%n");
      printf("%-36s: ", "new K().mC(new K<Object>())");
      new K().mC(new K()); printf("%n");
      printf("%-36s: ", "new K().mC(new K<String>())");
      new K().mC(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mC(new K())");
      new K<Object>().mC(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mC(new K<Object>())");
      new K<Object>().mC(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mC(new K<String>())");
      new K<Object>().mC(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mC(new K())");
      new K<String>().mC(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mC(new K<Object>())");
      new K<String>().mC(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mC(new K<String>())");
      new K<String>().mC(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mD(new K())");
      new K().mD(new K()); printf("%n");
      printf("%-36s: ", "new K().mD(new K<Object>())");
      new K().mD(new K()); printf("%n");
      printf("%-36s: ", "new K().mD(new K<String>())");
      new K().mD(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mD(new K())");
      new K<Object>().mD(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mD(new K<Object>())");
      new K<Object>().mD(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mD(new K<String>())");
      new K<Object>().mD(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mD(new K())");
      new K<String>().mD(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mD(new K<Object>())");
      new K<String>().mD(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mD(new K<String>())");
      new K<String>().mD(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mE(new K())");
      new K().mE(new K()); printf("%n");
      printf("%-36s: ", "new K().mE(new K<Object>())");
      new K().mE(new K()); printf("%n");
      printf("%-36s: ", "new K().mE(new K<String>())");
      new K().mE(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mE(new K())");
      new K<Object>().mE(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mE(new K<Object>())");
      new K<Object>().mE(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mE(new K<String>())");
      new K<Object>().mE(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mE(new K())");
      new K<String>().mE(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mE(new K<Object>())");
      new K<String>().mE(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mE(new K<String>())");
      new K<String>().mE(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mF(new K())");
      new K().mF(new K()); printf("%n");
      printf("%-36s: ", "new K().mF(new K<Object>())");
      new K().mF(new K()); printf("%n");
      printf("%-36s: ", "new K().mF(new K<String>())");
      new K().mF(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mF(new K())");
      new K<Object>().mF(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mF(new K<Object>())");
      new K<Object>().mF(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mF(new K<String>())");
      new K<Object>().mF(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mF(new K())");
      new K<String>().mF(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mF(new K<Object>())");
      new K<String>().mF(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mF(new K<String>())");
      new K<String>().mF(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("%-36s: ", "new K().mG(new K())");
      new K().mG(new K()); printf("%n");
      printf("%-36s: ", "new K().mG(new K<Object>())");
      new K().mG(new K()); printf("%n");
      printf("%-36s: ", "new K().mG(new K<String>())");
      new K().mG(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<Object>().mG(new K())");
      new K<Object>().mG(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mG(new K<Object>())");
      new K<Object>().mG(new K()); printf("%n");
      printf("%-36s: ", "new K<Object>().mG(new K<String>())");
      new K<Object>().mG(new K()); printf("%n");
      printf("---%n");
      printf("%-36s: ", "new K<String>().mG(new K())");
      new K<String>().mG(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mG(new K<Object>())");
      new K<String>().mG(new K()); printf("%n");
      printf("%-36s: ", "new K<String>().mG(new K<String>())");
      new K<String>().mG(new K()); printf("%n");
      printf("-------------------------------------%n");
      printf("GenTypen15: Das war's erstmal!%n");
   } // main
   // -----------------------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------------------------------
} // class GenTypen15
/* ------------------------------------------------------------------------
Ausgabe des Programms GenTypen15:

GenTypen15: Jetzt geht es los!
-------------------------------------
new K().mA(new K())                 : a b c d e f g
new K().mA(new K<Object>())         : a b c d e f g
new K().mA(new K<String>())         : a b c d e f g
---
new K<Object>().mA(new K())         : a b c d e f g
new K<Object>().mA(new K<Object>()) : a b c d e f g
new K<Object>().mA(new K<String>()) : a b c d e f g
---
new K<String>().mA(new K())         : a b c d e f g
new K<String>().mA(new K<Object>()) : a b c d e f g
new K<String>().mA(new K<String>()) : a b c d e f g
-------------------------------------
new K().mB(new K())                 : a b c
new K().mB(new K<Object>())         : a b c
new K().mB(new K<String>())         : a b c
---
new K<Object>().mB(new K())         : a b c
new K<Object>().mB(new K<Object>()) : a b c
new K<Object>().mB(new K<String>()) : a b c
---
new K<String>().mB(new K())         : a b c
new K<String>().mB(new K<Object>()) : a b c
new K<String>().mB(new K<String>()) : a b c
-------------------------------------
new K().mC(new K())                 : a c f
new K().mC(new K<Object>())         : a c f
new K().mC(new K<String>())         : a c f
---
new K<Object>().mC(new K())         : a c f
new K<Object>().mC(new K<Object>()) : a c f
new K<Object>().mC(new K<String>()) : a c f
---
new K<String>().mC(new K())         : a c f
new K<String>().mC(new K<Object>()) : a c f
new K<String>().mC(new K<String>()) : a c f
-------------------------------------
new K().mD(new K())                 : a c d f
new K().mD(new K<Object>())         : a c d f
new K().mD(new K<String>())         : a c d f
---
new K<Object>().mD(new K())         : a c d f
new K<Object>().mD(new K<Object>()) : a c d f
new K<Object>().mD(new K<String>()) : a c d f
---
new K<String>().mD(new K())         : a c d f
new K<String>().mD(new K<Object>()) : a c d f
new K<String>().mD(new K<String>()) : a c d f
-------------------------------------
new K().mE(new K())                 : a c e f
new K().mE(new K<Object>())         : a c e f
new K().mE(new K<String>())         : a c e f
---
new K<Object>().mE(new K())         : a c e f
new K<Object>().mE(new K<Object>()) : a c e f
new K<Object>().mE(new K<String>()) : a c e f
---
new K<String>().mE(new K())         : a c e f
new K<String>().mE(new K<Object>()) : a c e f
new K<String>().mE(new K<String>()) : a c e f
-------------------------------------
new K().mF(new K())                 : a c f
new K().mF(new K<Object>())         : a c f
new K().mF(new K<String>())         : a c f
---
new K<Object>().mF(new K())         : a c f
new K<Object>().mF(new K<Object>()) : a c f
new K<Object>().mF(new K<String>()) : a c f
---
new K<String>().mF(new K())         : a c f
new K<String>().mF(new K<Object>()) : a c f
new K<String>().mF(new K<String>()) : a c f
-------------------------------------
new K().mG(new K())                 : a c f g
new K().mG(new K<Object>())         : a c f g
new K().mG(new K<String>())         : a c f g
---
new K<Object>().mG(new K())         : a c f g
new K<Object>().mG(new K<Object>()) : a c f g
new K<Object>().mG(new K<String>()) : a c f g
---
new K<String>().mG(new K())         : a c f g
new K<String>().mG(new K<Object>()) : a c f g
new K<String>().mG(new K<String>()) : a c f g
-------------------------------------
GenTypen15: Das war's erstmal!
------------------------------------------------------------------------ */
