// Datei GleitRech07.java
/* ------------------------------------------------------------------------
Dieses Programm soll bestimmte Gefahren demonstrieren, die beim Rechnen
mit Gleitpunktzahlen drohen (Rundungsfehler). Die Formeln
((1 + r) * x) - (r * x * x) und x + (r * x * (1 - x)) sind beide
mathematisch aequivalent zur Formel x + r*x * r*x*x (und somit auch zu-
einander aequivalent). Ihr Quotient sollte deshalb immer gleich 1 sein.
Zwei Variablen x1 und x2 werden mit 0.1 initialisiert und dann wird
ihr Wert wiederholt durch die folgenden Zuweisungen veraendert:

x1 = ((1 + 3) * x1) - (3 * x1 * x1); // Formel1
x2 = x2 + (3 * x2 * (1 - x2));       // Formel2

Die Werte von x1 und x2 und ihr Quotient x1 / x2 werden nach jeder
solchen Zuweisung ausgegeben. Aufgrund von Rundungsfehlern ist der
Quotient nicht immer gleich 1.0, wie er eigentlich sein sollte.

GleitRech07 rechnet mit float-Zahlen (GleitRech08 mit double-Zahlen)
------------------------------------------------------------------------ */
class GleitRech07 {
   static public void main(String [] sonja) throws java.io.IOException {
      // Ein paar Konstanten:
      final float SEHR_GROSS = 1000.000F; // Tausend
      final float SEHR_KLEIN =    0.001F; // Ein Tausendstel

      // Ein paar Variablen:
            float x1         =    0.100F; // Fuer Formel 1
            float x2         =    0.100F; // Fuer Formel 2
            float q;                      // Quotient x1/x2
      // ------------------------------------------------------------------
      printf("GleitRech07: Wie oft (1..500)? ");
      int wieOft = EM.liesInt();

      // Ueberschriften ausgeben:
      printf("%n");
      printf("%7s %16s %16s  %16s%n",
         "Schritt", "Zahl x1", "Zahl x2", "Quotient x1/x2%n");

      // Wiederholt x1, x2 und q berechnen und ausgeben:
      for (int i=1; i<=wieOft; i++) {
         x1 = ((1 + 3) * x1) - (3 * x1 * x1); // x1 nach Formel1 berech.
         x2 = x2 + (3 * x2 * (1 - x2));       // x2 nach Formel2 berech.
         q  = x1 / x2;

         printf("%7d %16.9f %16.9f  %16.9f%n", i, x1, x2, q);

         // Falls der Quotient q sehr gross oder sehr klein ist, wird die
         // Ausgabe angehalten, damit die Ausgbezeilen nicht ungelesen
         // "vor den Ausgen des Benutzers vorbeiflimmern":
         if (q < SEHR_KLEIN || SEHR_GROSS < q) {
            printf("Weiter mit Return-Taste: ");
            String s = EM.liesString();
         }
      } // for
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class GleitRech07
/* ------------------------------------------------------------------------
Auszuege aus einem Dialog mit dem Programm GleitRech07:

GleitRech07: Wie oft (1..500)? 500

Schritt          Zahl x1          Zahl x2   Quotient x1/x2
      1      0.370000005      0.370000005      1.000000000
      2      1.069299936      1.069299936      1.000000000
      3      0.846992731      0.846992671      1.000000119
      4      1.235780954      1.235780954      1.000000000
      5      0.361660004      0.361660123      0.999999642
      6      1.054246187      1.054246426      0.999999762
      7      0.882679701      0.882679105      1.000000715
      8      1.193348408      1.193349242      0.999999285
    ...      ...              ...              ...
    129      0.264641762      0.195566759      1.353204250
    130      0.848461270      0.667527914      1.271049857
    131      1.234185457      1.333331108      0.925640643
    132      0.367100716      0.000008941  41059.519531250
    ...      ...              ...              ...
    248      0.050074577      1.065626502      0.046990741
    249      0.192775920      0.855826497      0.225251168
    250      0.659615993      1.225988984      0.538027644
    251      1.333184242      0.394808948      3.376783133
    252      0.000596523      1.111613512      0.000536628
    ...      ...              ...              ...
    483      1.320042610      1.283196092      1.028714657
    484      0.052632809      0.193007708      0.272697955
    485      0.202220604      0.660274863      0.306267321
    486      0.686202884      1.333210707      0.514699519
    487      1.332188368      0.000490427   2716.384521484
    ...      ...              ...              ...
------------------------------------------------------------------------ */