// Datei GleitRech08.java
/* ------------------------------------------------------------------------
Dieses Programm soll bestimmte Gefahren demonstrieren, die beim Rechnen
mit Gleitpunktzahlen drohen (Rundungsfehler). Die Formeln
((1 + r) * x) - (r * x * x) und x + (r * x * (1 - x)) sind beide
mathematisch aequivalent zur Formel x + r*x * r*x*x (und somit auch zu-
einander aequivalent). Ihr Quotient sollte deshalb immer gleich 1 sein.
Zwei Variablen x1 und x2 werden mit 0.1 initialisiert und dann wird
ihr Wert wiederholt durch die folgenden Zuweisungen veraendert:

x1 = ((1 + 3) * x1) - (3 * x1 * x1); // Formel1
x2 = x2 + (3 * x2 * (1 - x2));       // Formel2

Die Werte von x1 und x2 und ihr Quotient x1 / x2 werden nach jeder
solchen Zuweisung ausgegeben. Aufgrund von Rundungsfehlern ist der
Quotient nicht immer gleich 1.0, wie er eigentlich sein sollte.

GleitRech08 rechnet mit double-Zahlen (GleitRech07 mit float-Zahlen)
------------------------------------------------------------------------ */
class GleitRech08 {
   static public void main(String[] susi) throws java.io.IOException {
      // Ein paar Konstanten:
      final double SEHR_GROSS = 1000.000; // Tausend
      final double SEHR_KLEIN =    0.001; // Ein Tausendstel

      // Ein paar Variablen:
            double x1         =    0.100; // Fuer Formel 1
            double x2         =    0.100; // Fuer Formel 2
            double q;                      // Quotient x1/x2
      // ------------------------------------------------------------------
      printf("GleitRech08: Wie oft (1..500)? ");
      int wieOft = EM.liesInt();

      // Ueberschriften ausgeben:
      printf("%n");
      printf("%7s %16s %16s  %16s%n",
         "Schritt", "Zahl x1", "Zahl x2", "Quotient x1/x2%n");

      // Wiederholt x1, x2 und q berechnen und ausgeben:
      for (int i=1; i<=wieOft; i++) {
         x1 = ((1 + 3) * x1) - (3 * x1 * x1); // x1 nach Formel1 berech.
         x2 = x2 + (3 * x2 * (1 - x2));       // x2 nach Formel2 berech.
         q  = x1 / x2;

         printf("%7d %16.9f %16.9f  %16.9f%n", i, x1, x2, q);

         // Falls der Quotient q sehr gross oder sehr klein ist, wird die
         // Ausgabe angehalten, damit die Ausgbezeilen nicht ungelesen
         // "vor den Ausgen des Benutzers vorbeiflimmern":
         if (q < SEHR_KLEIN || SEHR_GROSS < q) {
            printf("Weiter mit Return-Taste: ");
            String s = EM.liesString();
         }
      } // for
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class GleitRech08
/* ------------------------------------------------------------------------
Auszuege aus einem Dialog mit dem Programm GleitRech08:

GleitRech08: Wie oft (1..500)? 500

Schritt          Zahl x1          Zahl x2   Quotient x1/x2
      1      0.370000000      0.370000000      1.000000000
      2      1.069300000      1.069300000      1.000000000
      3      0.846992530      0.846992530      1.000000000
    ...      ...              ...              ...
     22      0.139466051      0.139466051      0.999999999
     23      0.499511867      0.499511867      0.999999999
     24      1.249511152      1.249511152      1.000000000
     25      0.314210251      0.314210250      1.000000003
     26      0.960656758      0.960656756      1.000000002
    ...      ...              ...              ...
    126      1.030466500      0.661619436      1.557491277
    127      0.936282377      1.333256910      0.702252034
    128      1.115255440      0.000305677   3648.477885378
    ...      ...              ...              ...
    267      0.668043862      1.029743277      0.648747971
    268      1.333327643      0.937859459      1.421671051
    269      0.000022760      1.112696741      0.000020455
    ...      ...              ...              ...
    305      0.943476353      0.671249849      1.405551681
    306      1.103462526      1.333270317      0.827636011
    307      0.760961465      0.000252055   3019.032187776
    ...      ...              ...              ...
    491      0.668369855      1.126694355      0.593213103
    492      1.333324631      0.698456910      1.908957606
    493      0.000034810      1.330301475      0.000026167
    ...      ...              ...              ...
------------------------------------------------------------------------ */