// Datei Huellen01.java
/* ------------------------------------------------------------------------
Verhuellen: Umwandlung primitiver Wert --> Objekt der Huellklasse
Enthuellen: Umwandlung primitiver Wert <-- Objekt der Huellkalsse
Huellen:    Verhuellen und/oder Enthuellen

Autohuellen (engl. auto boxing): Der Ausfuehrer verhuellt und/oder
enthuellt primitive Werte automatisch, wenn er die Notwendigkeit
dazu aus dem Zusammenhang erkennt.
------------------------------------------------------------------------ */
class Huellen01 {
   // ---------------------------------------------------------------------
   static int add1(int n) {
      // Eine Funktion mit einem primitiven Parameter
      // und einem primitiven Ergebnis:
      return n + 1;
   } // add1
   // ---------------------------------------------------------------------
   static Integer mul2(Integer ob) {
      // Eine Funktion mit einem Objekt als Parameter
      // und einem Objekt als Ergebnis:
      return ob * 2;

      // Per Autohuellen wird dieser return-Befehl in
      // den folgenden return-Befehl umgewandelt:

//    return new Integer(ob.intValue() * 2);

      // ob wird autoenthuellt, der enthuellte Wert mit 2 multipliziert,
      // das Ergebnis autoverhuellt und geliefert.
   } // mul2
   // ---------------------------------------------------------------------
   // Zwei Faelle von Autohuellen:

    short short1 = 17;
    Short Short1 = new Short(short1);

    short short2 = Short1; // Short1 wird autoenthuellt
    Short Short2 = short1; // short1 wird autoverhuellt
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      // ------------------------------------------------------------------
      // Ein paar Variablen pw1? mit primitiven Werten
      byte      pw1b =         111;
      char      pw1c =         'A';
      short     pw1s =         222;
      int       pw1i =         333;
      long      pw1l =         444;
      float     pw1f = (float) 5.5;
      double    pw1d =         6.6;
      boolean   pw1w =         true;
      // ------------------------------------------------------------------
      // Huellen (mit ausdruecklichen Befehlen des Programmierers,
      // ohne Auto)

      // Verhuellen (explizit, ohne Auto):
      // Primitiver Wert pw1? --> Objekt ob1? einer Huellklasse
      Byte      ob1b = new Byte     (pw1b);
      Character ob1c = new Character(pw1c);
      Short     ob1s = new Short    (pw1s);
      Integer   ob1i = new Integer  (pw1i);
      Long      ob1l = new Long     (pw1l);
      Float     ob1f = new Float    (pw1f);
      Double    ob1d = new Double   (pw1d);
      Boolean   ob1w = new Boolean  (pw1w);

      // Verhuellen in einem Reihungsinitialisierer:
      Long[]    rob1 = {new Long(pw1l), ob1l};   // pw1l wird verhuellt

      // Enthuellen (explizit, ohne Auto):
      // Primitiver Wert pw1? <-- Objekt ob1? einer Huellklasse
      byte      pw2b = ob1b.   byteValue();
      char      pw2c = ob1c.   charValue();
      short     pw2s = ob1s.  shortValue();
      int       pw2i = ob1i.    intValue();
      long      pw2l = ob1l.   longValue();
      float     pw2f = ob1f.  floatValue();
      double    pw2d = ob1d. doubleValue();
      boolean   pw2w = ob1w.booleanValue();

      // Enthuellen in einem Reihungsinitialisierer:
      long[]    rpw2 = {pw1l, ob1l.longValue()}; // ob1l wird enthuellt
      // ------------------------------------------------------------------
      // Autohuellen (auto boxing, auto wrapping, nicht auto rapping!):

      // Autoverhuellen:
      // Primitiver Wert pw1? --> Objekt ob2? einer Huellklasse
      Byte      ob2b = pw1b;
      Character ob2c = pw1c;
      Short     ob2s = pw1s;
      Integer   ob2i = pw1i;
      Long      ob2l = pw1l;
      Float     ob2f = pw1f;
      Double    ob2d = pw1d;
      Boolean   ob2w = pw1w;

      // Verhuellen in einem Reihungsinitialisierer:
      Long[]    rob2 = {pw1l, ob1l};             // pw1l wird autoverhuellt

      // Autoenthuellen
      // Primitiver Wert pw1? <-- Objekt ob2? einer Huellklasse
      byte      pw3b = ob1b;
      char      pw3c = ob1c;
      short     pw3s = ob1s;
      int       pw3i = ob1i;
      long      pw3l = ob1l;
      float     pw3f = ob1f;
      double    pw3d = ob1d;
      boolean   pw3w = ob1w;

      // Enthuellen in einem Reihungsinitialisierer:
      long[]    rpw3 = {pw1l, ob1l};             // ob1l wird autoenthuellt
      // ------------------------------------------------------------------
      printf("Huellen01: Jetzt geht es los!\n");
      printf("-----------------------------\n");

      // Beim Aufrufen von Methoden werden die aktuellen Parameter
      // automatisch ver- bzw. enthuellt wenn noetig:
      printf("add1(pw1i) : %d\n", add1(pw1i));
      printf("mul2(pw1i) : %d\n", mul2(pw1i)); // pw1i wird autoverhuellt
      printf("add1(ob1i) : %s\n", add1(ob1i)); // ob1i wird autoenthuellt
      printf("mul2(ob1i) : %s\n", mul2(ob1i));

      // Die Ergebnisse von Methoden werden
      // automatisch ver- bzw. enthuellt wenn noetig:
      Integer ergOb = add1(pw1i);           // Ergebnis wird autoverhuellt
      int     ergPw = mul2(ob1i);           // Ergebnis wird autoenthuellt
      printf("ergOb      : %s\n", ergOb);
      printf("ergPw      : %d\n", ergPw);
      printf("-----------------------------\n");
      printf("Huellen01: Das war's erstmal!\n");

      // Zuweisungen zwischen Reihungen mit primitiven Komponenten und
      // Reihungen von Objekten sind (trotz Autohuellens) nicht erlaubt:
//    rob2 = rpw2;
//    rpw2 = rob2;
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // Huellen01
/* ------------------------------------------------------------------------
Ausgabe des Programms Huellen01:

Huellen01: Jetzt geht es los!
-----------------------------
add1(pw1i) : 334
mul2(pw1i) : 666
add1(ob1i) : 334
mul2(ob1i) : 666
ergOb      : 334
ergPw      : 666
-----------------------------
Huellen01: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\Huellen01.java:121: incompatible types
found   : long[]
required: java.lang.Long[]
    rob2 = rpw2;
           ^
D:\Java\Huellen01.java:122: incompatible types
found   : java.lang.Long[]
required: long[]
    rpw2 = rob2;
           ^
------------------------------------------------------------------------ */
