// Datei Int.java
/* ------------------------------------------------------------------------
Beim Addieren, Subtrahieren, Multiplizieren, ... von int-Werten ist das
Ergebnis nicht immer mathematisch korrekt. Jedes Int-Objekt repraesen-
tiert einen int-Wert und enthaelt die 5 Objektmethoden add, sub, mul, div
und mod. Das Rechnen mit diesen Methoden ergibt entweder ein mathematisch
korrektes Ergebnis oder loest eine Ausnahme (ArithmeticException) aus.
---------------------------------------------------------------------------
Anmerkung (zum Rechnen mit Werten des primitiven Typs int):

Die Addition, Subtraktion oder Multiplikation zweier int-Werte liefern in
vielen Faellen ein mathematisch nicht-korrektes Ergebnis.

Die Division zweier int-Werte liefert fast immer ein mathematisch korrektes
Ergebnis, ausser in den folgenden beiden Faellen:
Fall1: Eine Division durch 0 loest eine Ausnahme (ArithmeticException) aus.
Fall2: Die  Division Integer.MIN_VALUE / -1 ergibt das falsche Ergebnis
       Integer.MIN_VALUE (statt -Integer.MIN_VALUE).

Die Restfunktion (Modfunktion, Modulofunktion, %) fuer int-Werte liefert
fast immer ein mathematisch korrektes Ergebnis, nur wenn der zweite Operand
gleich 0 ist, wird eine Ausnahme (ArithmeticException) ausgeloest.
------------------------------------------------------------------------ */
class Int {
   // ---------------------------------------------------------------------
   // Der Wert dieses Int-Objekts:
   private final int wert;

   // Zwei Konstruktoren:
   public Int(int n) {wert = n;}
   public Int()      {wert = 0;}
   // ---------------------------------------------------------------------
   // Rechenfunktionen mit einem Wert des primitiven Typs int als Parameter:
   // ---------------------------------------------------------------------
   public Int add(int n) throws ArithmeticException {
      long erg = (long) wert + (long) n; // Sichere Addition!
      return new Int(toInt(erg));
   } // add
   // ---------------------------------------------------------------------
   public Int sub(int n) throws ArithmeticException {
      long erg = (long) wert - (long) n; // Sichere Subtraktion!
      return new Int(toInt(erg));
   } // sub
   // ---------------------------------------------------------------------
   public Int mul(int n) throws ArithmeticException {
      long erg = (long) wert * (long) n; //Sichere Multiplikation!
      return new Int(toInt(erg));
   } // mul
   // ---------------------------------------------------------------------
   public Int div(int n) throws ArithmeticException {
      long erg = (long) wert / (long) n; // Sichere Division!
      return new Int(toInt(erg));
   } // div
   // ---------------------------------------------------------------------
   public Int mod(int n) throws ArithmeticException {
      long erg = (long) wert % (long) n; // Sichere Restberechnung!
      return new Int(toInt(erg));
   } // div
   // ---------------------------------------------------------------------
   // Rechenfunktionen mit einem Objekt der Klasse Int als Parameter:
   // ---------------------------------------------------------------------
   public Int add(Int obInt) throws ArithmeticException {
      return add(obInt.wert);
   } // add
   // ---------------------------------------------------------------------
   public Int sub(Int obInt) throws ArithmeticException {
      return sub(obInt.wert);
   } // sub
   // ---------------------------------------------------------------------
   public Int mul(Int obInt) throws ArithmeticException {
      return mul(obInt.wert);
   } // mul
   // ---------------------------------------------------------------------
   public Int div(Int obInt) throws ArithmeticException {
      return div(obInt.wert);
   } // div
   // ---------------------------------------------------------------------
   public Int mod(Int obInt) throws ArithmeticException {
      return mod(obInt.wert);
   } // div
   // ---------------------------------------------------------------------
   // Zwei weitere Funktionen:
   // ---------------------------------------------------------------------
   public String toString() {return wert + "";}
   // ---------------------------------------------------------------------
   private int toInt(long n) throws ArithmeticException {
      // Falls es zum long-Wert n keinen entsprechenden int-Wert gibt, wird
      // eine Ausnahme (ArithmeticException) geworfen. Sonst wird der int-
      // Wert geliefert, der dem long-Wert n entspricht.
      if (n < Integer.MIN_VALUE || Integer.MAX_VALUE < n) {
         throw new ArithmeticException(n + " ist kein int-Wert!");
      }
      return (int) n;
   } // toInt
   // ---------------------------------------------------------------------
} // class Int