// Datei Iterable01.java
/* ------------------------------------------------------------------------
Die Klasse PaarL ist nicht-generisch! Siehe auch Iterable02.

Jede Sammlung des Typs PaarL enthaelt zwei String-Komponenten.
Auf die Komponenten einer PaarL-Sammlung kann man mit vereinfachten
for-Schleifen zugreifen, weil die Klasse PaarL die Schnittstelle
Iterable<String> implementiert.

Die Schnittstelle Iterable<String> enthaelt nur eine Methode:
Iterator<String> iterator();

Dabei ist Iterator<String> eine Schnittstelle, die drei Methoden enthaelt:
boolean hasNext();
String  next();
void    remove();
------------------------------------------------------------------------ */
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.lang.UnsupportedOperationException;

class PaarL implements Iterable<String> { // Paare von String-Objekten
   String k1;
   String k2;
   // -----------------------------------------------------------
   PaarL(String k1, String k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarL
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
   // Implementierung der Schnittstelle Iterable:
   public Iterator<String> iterator() {
      return new Schrittmacher(this);
   } // iterator
   // -----------------------------------------------------------
   class Schrittmacher implements Iterator<String> {
      private PaarL    paar;
      private int      naechsteKomponente = 1;
      public Schrittmacher(PaarL paar) {this.paar = paar;}

      // Die Implementierung der Schnittstelle Iterator<String>:
      public boolean hasNext() {return naechsteKomponente <= 2;}

      public String next() {
         if (!hasNext()) {
            throw new NoSuchElementException();
         } else if (naechsteKomponente == 1) {
            naechsteKomponente++;
            return paar.k1;
         } else {
            naechsteKomponente++;
            return paar.k2;
         } // if
      } // next

      // Aus einer PaarL-Sammlung kann man keine Komponenten
      // entfernen (remove wird nicht unterstuetzt):
      public void remove() {
         throw new UnsupportedOperationException();
      } // remove
   } // class Schrittmacher
   // -----------------------------------------------------------
} // Klasse PaarL
// ==============================================================
class Iterable01 {
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Iterable01: Jetzt geht es los!\n");

      // Objekte, die als Komponenten von Paaren verwendet werden sollen:
      String  s01 = "Hallo";
      String  s02 = "Sonja!";
      String  s03 = "Wie";
      String  s04 = "geht's?";

      // Objekte verschiedener PaarL-Typen:
      PaarL        p01 = new PaarL       (s01, s02);
      PaarL        p02 = new PaarL       (s03, s04);

      // Die Komponenten aller Paare mit vereinfachten for-Schleifen
      // ausgeben:
      printf("------------------------------\n");
      for (String        s: p01) printf("%s ", s); printf("\n");
      for (String        s: p02) printf("%s ", s); printf("\n");
      printf("------------------------------\n");

      printf("Iterable01: Das war's erstmal!\n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class Iterable01
/* ------------------------------------------------------------------------
Ausgabe des Programms Iterable01:

Iterable01: Jetzt geht es los!
------------------------------
Hallo Sonja!
Wie geht's?
------------------------------
Iterable01: Das war's erstmal!
------------------------------------------------------------------------ */
