// Datei Iterable02.java
/* ------------------------------------------------------------------------
Die Klasse PaarM<K> ist generisch! Siehe auch Iterable01.

Auf die Komponenten einer PaarM<K>-Sammlung kann man mit vereinfachten
for-Schleifen zugreifen, weil die Klasse PaarM<K> die Schnittstelle
Iterable<K> implementiert.

Die Schnittstelle Iterable<K> enthaelt nur eine Methode:
Iterator<K> iterator();

Dabei ist Iterator<K> eine Schnittstelle, die drei Methoden enthaelt:
boolean hasNext();
K       next();
void    remove();
------------------------------------------------------------------------ */
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.lang.UnsupportedOperationException;

class PaarM<K> implements Iterable<K> {
   K k1;
   K k2;
   // -----------------------------------------------------------
   PaarM(K k1, K k2) {
      this.k1 = k1;
      this.k2 = k2;
   } // Konstruktor PaarM
   // -----------------------------------------------------------
   public String toString() {
      return "[" + k1 + ", " + k2 + "]";
   } // toString
   // -----------------------------------------------------------
   // Implementierung der Schnittstelle Iterable:
   public Iterator<K> iterator() {
      return new Schrittmacher(this);
   } // iterator
   // -----------------------------------------------------------
   class Schrittmacher implements Iterator<K> {
      private PaarM<K> paar;
      private int      naechsteKomponente = 1;
      public  Schrittmacher(PaarM<K> paar) {this.paar = paar;}

      // Die Implementierung der Schnittstelle Iterator<K>:
      public boolean hasNext() {return naechsteKomponente <= 2;}

      public K next() {
         if (!hasNext()) {
            throw new NoSuchElementException();
         } else if (naechsteKomponente == 1) {
            naechsteKomponente++;
            return paar.k1;
         } else {
            naechsteKomponente++;
            return paar.k2;
         } // if
      } // next

      // Aus einer PaarM-Sammlung kann man keine Komponenten
      // entfernen (remove wird nicht unterstuetzt):
      public void remove() {
         throw new UnsupportedOperationException();
      } // remove
   } // class Schrittmacher
   // -----------------------------------------------------------
} // Klasse PaarM
// ==============================================================
class Iterable02 {
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Iterable02: Jetzt geht es los!\n");

      // Objekte, die als Komponenten von Paaren verwendet werden sollen:
      String  s01 = "Hallo";
      String  s02 = "Sonja!";
      String  s03 = "Wie";
      String  s04 = "geht's?";

      Integer i01 = 17;
      Integer i02 = 35;

      // Objekte verschiedener PaarM-Typen:
      PaarM<String>        p01 = new PaarM<String>       (s01, s02);
      PaarM<String>        p02 = new PaarM<String>       (s03, s04);
      PaarM<Integer>       p03 = new PaarM<Integer>      (i01, i02);
      PaarM<PaarM<String>> p04 = new PaarM<PaarM<String>>(p01, p02);

      // Die Komponenten aller Paare mit vereinfachten for-Schleifen
      // ausgeben:
      printf("------------------------------\n");
      for (String        s: p01) printf("%s ", s); printf("\n");
      for (String        s: p02) printf("%s ", s); printf("\n");
      for (Integer       i: p03) printf("%d ", i); printf("\n");
      for (PaarM<String> p: p04) printf("%s ", p); printf("\n");
      printf("------------------------------\n");

      printf("Iterable02: Das war's erstmal!\n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class Iterable02
/* ------------------------------------------------------------------------
Ausgabe des Programms Iterable02:

Iterable02: Jetzt geht es los!
------------------------------
Hallo Sonja!
Wie geht's?
17 35
[Hallo, Sonja!] [Wie, geht's?]
------------------------------
Iterable02: Das war's erstmal!
------------------------------------------------------------------------ */
