// Datei Iterable03.java
/* ------------------------------------------------------------------------
Die Schnittstelle Iterable ist Voraussetzung fuer vereinfachte for-
Schleifen. Sie kann nicht nur von Sammlungsklassen implementiert
werden, sondern auch von einer Klasse wie MyLong in diesem Beispiel.

Ein Objekt der Klasse MyLong enthaelt ein Long-Objekt, auf dessen
19 Dezimalziffern man mit vereinfachten for-Schleifen zugreifen
kann. Man bekommt dann zuerst die Einerziffer und zuletzt die
10-Trillionen-Ziffer (die hoechstwertige Ziffer).

Die Schnittstelle Iterable<Character> enthaelt nur eine Methode:
Iterator<Character> iterator();

Dabei ist Iterator<Character> eine Schnittstelle, die drei Methoden
enthaelt:

boolean   hasNext();
Character next();
void      remove();
------------------------------------------------------------------------ */
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.lang.UnsupportedOperationException;

class MyLong implements Iterable<Character> {
   Long lob;
   // -----------------------------------------------------------
   MyLong(Long lob) {
      this.lob = lob;
   } // Konstruktor MyLong
   // -----------------------------------------------------------
   public String toString() {
      return lob.toString();
   } // toString
   // -----------------------------------------------------------
   // Implementierung der Schnittstelle Iterable:
   public Iterator<Character> iterator() {
      return new Schrittmacher(this);
   } // iterator
   // -----------------------------------------------------------
   class Schrittmacher implements Iterator<Character> {
      private long longWert;
      private long naechsteZiffer = 0;
      public Schrittmacher(MyLong mlob) {longWert = mlob.lob;}

      // Die Implementierung der Schnittstelle Iterator<MyLong>:
      public boolean hasNext() {return naechsteZiffer <= 18;}

      public Character next() {
         // Liefert die naechste Dezimalziffer von longWert
         // oder wirft eine Ausnahme (wenn schon 19 Ziffern
         // geliefert wurden).
         if (!hasNext()) {
            throw new NoSuchElementException();
         } else {
            long longZiffer = longWert % 10;
                 longWert   = longWert / 10;
            longZiffer = longZiffer<0 ? -longZiffer : longZiffer;
            Character erg = (char) (longZiffer + '0');
            naechsteZiffer++;
            return erg;
         } // if
      } // next

      // Aus einer PaarM-Sammlung kann man keine Komponenten
      // entfernen (remove wird nicht unterstuetzt):
      public void remove() {
         throw new UnsupportedOperationException();
      } // remove
   // Eine Methode mit einem kurzen Namen:
   void printf(String f, Object... v) {System.out.printf(f, v);}
   } // class Schrittmacher
   // -----------------------------------------------------------
} // Klasse PaarM
// ==============================================================
class Iterable03 {
   // -----------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Iterable03: Jetzt geht es los!\n");

      // Objekte, die als Komponenten von Paaren verwendet werden sollen:
      MyLong ml01 = new MyLong(12345L);
      MyLong ml02 = new MyLong(1234567890123456789L);
      MyLong ml03 = new MyLong(Long.MIN_VALUE);
      MyLong ml04 = new MyLong(Long.MAX_VALUE);

      // Die long-Werte der MyLong-Objekte ml01 bis ml04 werden
      // "im Ganzen" und dann mit einer vereinfachten for-Schleife
      // noch einmal zifferweise ausgegeben:
      printf("-------------------------------------------------------------\n");
      printf("A %+20d: ", ml01.lob);
      for (Character c:   ml01) printf("%c ", c);
      printf("\n");
      printf("B %+20d: ", ml02.lob);
      for (Character c:   ml02) printf("%c ", c);
      printf("\n");
      printf("C %+20d: ", ml03.lob);
      for (Character c:   ml03) printf("%c ", c);
      printf("\n");
      printf("D %+20d: ", ml04.lob);
      for (Character c:   ml04) printf("%c ", c);
      printf("\n");
      printf("-------------------------------------------------------------\n");

      printf("Iterable03: Das war's erstmal!\n");
   } // main
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // -----------------------------------------------------------
} // class Iterable03
/* ------------------------------------------------------------------------
Ausgabe des Programms Iterable03:

Iterable03: Jetzt geht es los!
-------------------------------------------------------------
A               +12345: 5 4 3 2 1 0 0 0 0 0 0 0 0 0 0 0 0 0 0
B +1234567890123456789: 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1
C -9223372036854775808: 8 0 8 5 7 7 4 5 8 6 3 0 2 7 3 3 2 2 9
D +9223372036854775807: 7 0 8 5 7 7 4 5 8 6 3 0 2 7 3 3 2 2 9
-------------------------------------------------------------
Iterable03: Das war's erstmal!
------------------------------------------------------------------------ */
