// Datei Katze.java
/* ------------------------------------------------------------------------
Eine Erweiterung der Klasse JFrame. Jedes Objekt dieser Klasse proto-
kolliert seine Mausereignisse durch Meldungen zur Standardausgabe.
Mit Hilfe von Knoepfen (des Typs JCheckBox) kann der Benutzer einzelne
Arten von Mausereignissen (z.B. die mouseClicked-Ereignisse oder die
mouseWheelMoved-Ereignisse etc.) von der Protokollierung ausschliessen
bzw. sie wieder in die Protokollierung einbeziehen. Neben jedem Knopf
werden die Koordinaten des letzten Mausereignisses (der Art, fuer die
der Knopf "zustaendig" ist) angezeigt.

Reihenfolge der Elemente in dieser Klasse Katze:
1.   Struktur der Grabo-Elemente eines Katze-Objekts (Kommentar)
2.   Alle Klassen-Attribute
2.1.    Diverse Konstanten (der Typen int, String, JLabel)
2.2.    Eine Reihung mit 8 Namen fuer Knoepfe
3.   Alle  Objekt-Attribute
3.1.    Drei Boxen
3.2.    Eine Reihung mit 8 JCheckBox-Objekten
3.3.    Eine Reihung mit 8 JLabel-Objekten
3.4.    Eine Reihung mit 8 RodentListener-Objekten
4.   Ein (kleiner) Konstruktor
5.   Alle Objekt-Methoden in alphabetischer Reihenfolge
6.   Die Klassen-Methode main

Aenderungen gegenueber Katze09: Objektmethoden namens
addMouseInputListener und removeMouseInputListener wurden vereinbart,
die ihren Parameter sowohl als MouseListener als auch als MouseMo-
tionListener an bzw. abmelden. Dadurch wird die laestige Unterscheidung
zwischen MouseListener- und MouseMotionListener-Ereignissen noch
unwichtiger und unauffaelliger.

Aenderungen gegenueber Katze08: Das Anmelden der ActionListener-Objekte
bei den 7 JCheckBox-Objekten (in der knopfReihung) wurde vereinfacht.
Einige Befehle aus dem Konstruktor in Initialisierer verschoben. Einige
Objekt-Attribute wurden in Klassen-Attribute umgewandelt.

Aenderungen gegenueber Katze07: Anstelle der Typen MouseListener,
MouseAdapter, MouseMotionListener und MouseMotionAdapter werden
die Typen MouseInputListener und MouseInputAdapter verwendet, die es
seit Java 1.3 gibt. Schade, jetzt fehlen noch (in der Klasse Component)
Methoden addMouseInputListener und removeMouseInputListener.
------------------------------------------------------------------------ */
import javax.swing      .JFrame;              // Wird hier erweitert
import javax.swing      .JCheckBox;           // Knoepfe
import javax.swing      .Box;                 // knopfLeiste
import javax.swing      .JLabel;
import javax.swing.event.MouseInputListener;  // Schnittstelle
import javax.swing.event.MouseInputAdapter;   // Klasse
import java .awt        .Dimension;           // Groesse der Label
import java .awt  .event.MouseAdapter;        // Klasse
import java .awt  .event.MouseMotionAdapter;  // Klasse
import java .awt  .event.MouseListener;       // Schnittstelle
import java .awt  .event.MouseMotionListener; // Schnittstelle
import java .awt  .event.MouseWheelListener;  // Schnittstelle
import java .awt  .event.MouseEvent;          // Klasse
import java .awt  .event.MouseWheelEvent;     // Klasse
import java .awt  .event.ActionListener;      // Schnittstelle
import java .awt  .event.ActionEvent;         // Klasse
import myja .awt  .event.RodentListener;      // Abstrakte Klasse

class Katze extends JFrame {
   // ---------------------------------------------------------------------
   // 0. Eine Versionskennzeichnung (erstellt mit dem Programm serialver):
   static final long serialVersionUID = -5494608404999699654L;
   // ---------------------------------------------------------------------
   // 1. Struktur der Grabo-Komponenten (GUI components) eines Katze-
   // Objekts. Die Einrueckung drueckt Enthaltensein aus, in Klammern
   // sind die Typen der Komponenten angegeben. Alle Grabo-Komponenten
   // muessen Objekt-Attribute sein, da eine Grabo-Komponente sich zu
   // jedem Zeitpunkt in hoechstens einem Behaelter (container object)
   // befinden kann:
   //
   // Ein Rahmen             (Katze, d.h. JFrame)
   //    beideBoxen          (Box      )
   //       knopfLeiste      (Box      )
   //          ueberSchrift1 (JLabel   )
   //          8 Knoepfe     (JCheckBox)
   //       labelLeiste      (Box      )
   //          ueberSchrift2 (JLabel   )
   //          8 Label       (JLabel   )
   // ---------------------------------------------------------------------
   // 2. Alle Klassen-Attribute:

   // 2.1. Diverse Konstanten (der Typen int, String, JLabel):

   static final int ANZAHL_KNOEPFE = 8; // 8 Arten von Mausereignissen

   // Fuer jede Art von Mausereignis eine int-Konstante, die als Index
   // fuer verschiedene Reihungen die Lesbarkeit verbessern soll. Die
   // Werte der Konstanten CLICKED, ENTERED etc. sind willkuerlich ge-
   // waehlt (und koennten auch permutiert werden):
   static final int CLICKED=0,  ENTERED=1, EXITED=2, PRESSED=3,
                    RELEASED=4, DRAGGED=5, MOVED=6,  WHEELMOVED=7;
   // ---------------------------------------------------------------------
   // Ueberschriften und andere Konstanten:
   static final String   leeresLabel   = " --- ";
          final JLabel   ueberSchrift1 = new JLabel("Knoepfe");
          final JLabel   ueberSchrift2 = new JLabel("Koordinaten");
   // ---------------------------------------------------------------------
   // 2.2. Eine Klassen-Reihung mit ANZAHL_KNOEPFE vielen Namen fuer Knoepfe:
   static final String   [] knopfName    = new String   [ANZAHL_KNOEPFE];

   // Klassen-Initialisierer: Initialisiert die Reihung knopfName:
   static {
     knopfName[CLICKED] = "Clicked "; knopfName[ENTERED]   = "Entered   ";
     knopfName[EXITED]  = "Exited  "; knopfName[PRESSED]   = "Pressed   ";
     knopfName[RELEASED]= "Released"; knopfName[DRAGGED]   = "Dragged   ";
     knopfName[MOVED]   = "Moved   "; knopfName[WHEELMOVED]= "WheelMoved";
   } // Initialisierer
   // ---------------------------------------------------------------------
   // 3. Alle Objekt-Attribute:

   // 3.1. Drei Boxen:
   Box beideBoxen  = Box.createHorizontalBox();
   Box knopfLeiste = Box.createVerticalBox();
   Box labelLeiste = Box.createVerticalBox();
   // ---------------------------------------------------------------------
   // 3.2. Eine Reihung mit ANZAHL_KNOEPFE vielen JCheckBox-Objekten:
   final JCheckBox[] knopfReihung = new JCheckBox[ANZAHL_KNOEPFE];

   // Objekt-Initialisierer: Initialisiert die knopfReihung und bringt
   // ihre JCheckBox-Komponenten in die knopfLeiste:
   {
      knopfLeiste.add(ueberSchrift1);

      // Die Knoepfe erzeugen, be-Namen und in die knopfLeiste tun. Bei
      // jedem Knopf ein (simples) ActionListener-Objekt anmelden:
      for (int i=0; i<knopfReihung.length; i++) {
         knopfReihung[i] = new JCheckBox(knopfName[i]);
         final int INDEX = i; // In der folgenden anonymen Klasse darf man
                              // nicht auf die Variable i, wohl aber auf
                              // die Konstante INDEX zugreifen!

         knopfReihung[i].addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
               meldeAnOderAb(INDEX); // Zugriff auf Konstante INDEX erlaubt
            } // actionPerformed
         });

         knopfLeiste.add(knopfReihung[i]);
      } // for
   } // Objekt-Initialisierer fuer die Reihung knopfReihung
   // ---------------------------------------------------------------------
   // 3.3. Eine Reihung mit ANZAHL_KNOEPFE vielen JLabel-Objekten:
   final JLabel[] labelReihung  = new JLabel[ANZAHL_KNOEPFE];

   // Objekt-Initialisierer: Initialisiert die labelReihung und bringt
   // ihre JLabel-Komponenten in die Box labelLeiste:
   {
      labelLeiste.add(ueberSchrift2);

      // Die Label sollen genau die gleiche Hoehe haben wie die Knoepfe:
      Dimension dimKnopf = knopfReihung[0].getPreferredSize();
      Dimension dimLabel = new Dimension(70, dimKnopf.height);
      // Die Label erzeugen, mit leeresLabel-Text versehen, ihre Hoehe
      // festlegen und in die labelLeiste tun:
      for (int i=0; i<labelReihung.length; i++) {
         labelReihung[i] = new JLabel(leeresLabel);
         labelReihung[i].setMinimumSize(dimLabel); // beide set-Befehle
         labelReihung[i].setMaximumSize(dimLabel); // sind notwendig!
         labelLeiste.add(labelReihung[i]);
      } // for
   } // Objekt-Initialsierer fuer die Reihung labelReihung
   // ---------------------------------------------------------------------
   // Alle Grabo-Elemente (GUI components) sind fertig vereinbart (muessen
   // aber noch "zusammengesteckt" werden).
   // ---------------------------------------------------------------------
   // 3.4. Eine Reihung rlr mit ANZAHL_KNOEPFE vielen Komponenten des
   // Typs RodentListener.
   // Jedes RodentListener-Objekt enthaelt ein MouseListener-Objekt, ein
   // MouseMotionListener-Objekt oder ein MouseWheelListener-Objekt und
   // zwei Methoden (addThisListener und removeThisListener), mit denen
   // man dieses enthaltende Listener-Objekt bei einem beliebigen
   // Component-Objekt an- bzw. abmelden kann.

   RodentListener[] rolR = new RodentListener[ANZAHL_KNOEPFE];

   // Objekt-Initialisierer, initialisiert die Reihung rolR:
   {

      rolR[CLICKED] = RodentListener.makeRL(
         (MouseListener) new MouseAdapter() {
            public void mouseClicked(MouseEvent me) {
               protoKolliere(me, labelReihung[CLICKED]);
            }
      });

      rolR[ENTERED] = RodentListener.makeRL(
         (MouseListener) new MouseAdapter() {
            public void mouseEntered(MouseEvent me) {
               protoKolliere(me, labelReihung[ENTERED]);
            }
      });

      rolR[EXITED] = RodentListener.makeRL(
         (MouseListener) new MouseAdapter() {
            public void mouseExited(MouseEvent me) {
               protoKolliere(me, labelReihung[EXITED]);
            }
      });

      rolR[PRESSED] = RodentListener.makeRL(
         (MouseListener) new MouseAdapter() {
            public void mousePressed(MouseEvent me) {
               protoKolliere(me, labelReihung[PRESSED]);
            }
      });

      rolR[RELEASED] = RodentListener.makeRL(
         (MouseListener) new MouseAdapter() {
            public void mouseReleased(MouseEvent me) {
               protoKolliere(me, labelReihung[RELEASED]);
            }
      });

      rolR[DRAGGED] = RodentListener.makeRL(
         new MouseMotionAdapter() {
            public void mouseDragged(MouseEvent me) {
               protoKolliere(me, labelReihung[DRAGGED]);
            }
      });

      rolR[MOVED] = RodentListener.makeRL(
         new MouseMotionAdapter() {
            public void mouseMoved(MouseEvent me) {
               protoKolliere(me, labelReihung[MOVED]);
            }
      });

      rolR[WHEELMOVED] = RodentListener.makeRL(
         new MouseWheelListener() {
            public void mouseWheelMoved(MouseWheelEvent me) {
               protoKolliere(me, labelReihung[WHEELMOVED]);
            }
      });

   } // Objekt-Initialisierer, initialisiert die Reihung rolR:
   // ---------------------------------------------------------------------
   // 4. Ein (kleiner) Konstruktor:
   Katze(String titel) {
      super(titel);

      // Die grossen Grabo-Elemente "zusammenstecken":
      beideBoxen.add(knopfLeiste);
      beideBoxen.add(labelLeiste);
      this.getContentPane().add(beideBoxen, "West");
      this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
   } // Konstruktor Katze
   // ---------------------------------------------------------------------
   // 5. Alle Objekt-Methoden in alphabetischer Reihenfolge:
   // ---------------------------------------------------------------------
   void addMouseInputListener(MouseInputListener mil) {
      this.addMouseListener(mil);
      this.addMouseMotionListener(mil);
   } // addMouseInputListener
   // ---------------------------------------------------------------------
   String dasWesentliche(MouseEvent me) {
      // Wandelt das Mausereignis me in einen String um, schneidet den
      // ueberfluessigen Anfang "java.awt.event.MouseEvent" bzw.
      // "java.awt.event.MouseMotionEvent" davon ab und liefert den
      // wesentlichen Rest als Ergebnis.

      final int MAX = 70; // Max. Anzahl Zeichen pro Zeile

      StringBuilder sb  = new StringBuilder(me.toString());
      int           ind = sb.indexOf("[");
      sb.delete(0,  ind);

      // Fall sb laenger als MAX ist, jeweils nach dem letzten Komma
      // innerhalb eines Abschnitts von MAX vielen Zeichen einen
      // Zeilenumbruch und ein paar Blanks zum Einruecken der naechsten
      // Zeile einfuegen:
      ind = 0;
      while (sb.length() - ind > MAX) {
         ind = sb.lastIndexOf(",", ind+MAX);
         sb.insert(ind+1, "\n     ");
      }

      return sb.toString();
   } // dasWesentliche
   // ---------------------------------------------------------------------
   void meldeAb(int i) {
      // Meldet das i-te MouseInputListener-Objekt bei
      // diesem (this) Objekt *ab*:

      rolR[i].removeThisListener(this);

      // Die letzten Koordinaten (fuer Mausereignisse der i-ten Art)
      // durch leeresLabel ersetzen:
      labelReihung[i].setText(leeresLabel);
   } // meldeAb
   // ---------------------------------------------------------------------
   void meldeAn(int i) {
      // Meldet das i-te MouseInputListener-Objekt bei
      // diesem (this) Objekt *an*:

      rolR[i].addThisListener(this);
   } // meldeAn
   // ---------------------------------------------------------------------
   void meldeAnOderAb(int i) {
      // Je nachdem, ob der i-te Knopf gewaehlt ist oder nicht, wird das
      // i-te MouseInputListener-Objekt (MouseListener- bzw- MouseMotionListe-
      // ner-Objekt) aus der Reihung lauscher bei diesem Katze-Objekt
      // angemeldet bzw. abgemeldet.
      if (knopfReihung[i].isSelected()) meldeAn(i); else meldeAb(i);
   } // meldeAnOderAb
   // ---------------------------------------------------------------------
   int  ereignisNr = 0; // Die Mausereignisse werden numeriert
   void pln(MouseEvent me) {
      // pln (Abkuerzung fuer println) git die wesentlichen Daten des
      // Mausereignisses me numeriert zur Standardausgabe aus:
      printf("%3d: %s%n", ++ereignisNr, dasWesentliche(me));
   } // pln
   // ---------------------------------------------------------------------
   void protoKolliere(MouseEvent me, JLabel lab) {
      // Gibt das Mausereignis me zur Standardausgabe aus und schreibt
      // seine Koordinaten (Ort des Mausereignisses) in das betreffende
      // Label lab (welches eine Komponente der Reihung labelReihung ist):
      pln(me);                             // Zur Standardausgabe
      int    x = me.getX();
      int    y = me.getY();
      String s = "(" + x + ", " + y + ")"; // Koordinaten "verzieren"
      lab.setText(s);
   } // protoKolliere
   // ---------------------------------------------------------------------
   void removeMouseInputListener(MouseInputListener mil) {
      this.removeMouseListener(mil);
      this.removeMouseMotionListener(mil);
   } // removeMouseInputListner
   // ---------------------------------------------------------------------
   // 6. Die Klassen-Methode main:
   static public void main(String [] sonja) {
      // Erzeugt zwei Katze-Objekte unterschiedlicher Groesse:
      printf("Katze: Jetzt geht es los!%n");
      Katze meinFenster_1 = new Katze("Katze, Rahmen 1");
      Katze meinFenster_2 = new Katze("Katze, Rahmen 2");
      meinFenster_1.setBounds(100, 50, 400, 300);
      meinFenster_2.setBounds(550, 50, 300, 300);
      meinFenster_1.setVisible(true);
      meinFenster_2.setVisible(true);
      printf("Katze: Das war's eigentlich schon!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Katze
/* ------------------------------------------------------------------------
D:\meineDateien\BspJaSp\Katze.java:179:
reference to makeRL is ambiguous, both method
makeRL(java.awt.event.MouseMotionListener) in myja.awt.event.RodentListener and method
makeRL(java.awt.event.MouseWheelListener)  in myja.awt.event.RodentListener match
      rolR[CLICKED] = RodentListener.makeRL(
                                    ^
D:\meineDateien\BspJaSp\Katze.java:186: reference to makeRL is ambiguous, both method
makeRL(java.awt.event.MouseMotionListener) in myja.awt.event.RodentListener and method
makeRL(java.awt.event.MouseWheelListener)  in myja.awt.event.RodentListener match
      rolR[ENTERED] = RodentListener.makeRL(
                                    ^
D:\meineDateien\BspJaSp\Katze.java:193: reference to makeRL is ambiguous, both method
makeRL(java.awt.event.MouseMotionListener) in myja.awt.event.RodentListener and method
makeRL(java.awt.event.MouseWheelListener)  in myja.awt.event.RodentListener match
      rolR[EXITED] = RodentListener.makeRL(
                                   ^
D:\meineDateien\BspJaSp\Katze.java:200: reference to makeRL is ambiguous, both method
makeRL(java.awt.event.MouseMotionListener) in myja.awt.event.RodentListener and method
makeRL(java.awt.event.MouseWheelListener)  in myja.awt.event.RodentListener match
      rolR[PRESSED] = RodentListener.makeRL(
                                    ^
D:\meineDateien\BspJaSp\Katze.java:207: reference to makeRL is ambiguous, both method
makeRL(java.awt.event.MouseMotionListener) in myja.awt.event.RodentListener and method
makeRL(java.awt.event.MouseWheelListener) in myja.awt.event.RodentListener match
      rolR[RELEASED] = RodentListener.makeRL(
                                     ^
------------------------------------------------------------------------ */
