// Datei Kommerziell02.java
/* ------------------------------------------------------------------------
Objekte des Typs BigDecimal repraesentieren Dezimalbrueche, bei denen man
die Anzahl der Nachpunktstellen genau festlegen und die man auf viele
verschiedene Weisen runden kann. Sie sind besonders fuer kommerzielle
Berechnungen geeignet.
------------------------------------------------------------------------ */
import java.math.BigDecimal;

class Kommerziell02 {
   // ---------------------------------------------------------------------
   static final String MINUSZEILE = "------------------------------" +
                                    "------------------------------";
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Kommerziell02: Jetzt geht es los!\n");
      printf("%s\n", MINUSZEILE);

      // Parametersatz fuer PrintRechnung (genau):
      BigDecimal preisProKg    = new BigDecimal( "4.90");
      BigDecimal gewicht       = new BigDecimal("22.400");
      // ------------------------------------------------------------------
      printRechnung(preisProKg, gewicht);

      printf("Kommerziell02: Das war's erstmal!\n");
   } // main
   // ---------------------------------------------------------------------
   static void printRechnung(BigDecimal preisProKg, BigDecimal gewicht) {
      // Gibt eine Rechnung mit anzahl vielen Posten (Zeilen) aus:

      BigDecimal anzahl         = new BigDecimal("10.00");
      BigDecimal summe          = new BigDecimal( "0.00");

      // Produkte berechnen:
      BigDecimal preisProStk    = preisProKg .multiply(gewicht);
      BigDecimal preisProPosten = preisProStk.multiply(anzahl);

      // Die Produkte auf 2 Stellen runden:
      preisProStk    = preisProStk   .setScale(2, BigDecimal.ROUND_HALF_UP);
      preisProPosten = preisProPosten.setScale(2, BigDecimal.ROUND_HALF_UP);

      // Ueberschriften ausgeben:
      String f1 = "%12s %11s %11s %11s %11s\n";
      printf(f1, "Preis/kg", "Gewicht", "Preis/Stk", "Anzahl", "Preis");
      printf(f1, "in Euro",  "in kg",   "in Euro",   "",       "in Euro");

      // anzahl viele Postenzeilen ausgeben:
      for (int i=0; i<anzahl.intValue(); i++) {
         printf(" %11.2f"  , preisProKg);
         printf(" %11.3f"  , gewicht);
         printf(" %11.2f"  , preisProStk);
         printf(" %11.0f"  , anzahl);
         printf(" %11.2f\n", preisProPosten);
         summe = summe.add(preisProPosten);
      }
      // Summenzeile ausgeben:
      printf("%-48s %11.2f\n", "Rechnungs-Summe:", summe);

      printf("%s\n", MINUSZEILE);
      printf("Der genaue Preis/kg: %9.6f\n", preisProKg);
      printf("Das genaue Gewicht:  %9.6f\n", gewicht);
      printf("%s\n", MINUSZEILE);

   } // printRechnung
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Kommerziell02
/* ------------------------------------------------------------------------
Ausgabe des Programms Kommerziell02:

Kommerziell02: Jetzt geht es los!
------------------------------------------------------------
    Preis/kg     Gewicht   Preis/Stk      Anzahl       Preis
     in Euro       in kg     in Euro                 in Euro
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
        4,90      22,400      109,76          10     1097,60
Rechnungs-Summe:                                    10976,00
------------------------------------------------------------
Der genaue Preis/kg:  4,900000
Das genaue Gewicht:  22,400000
------------------------------------------------------------
Kommerziell02: Das war's erstmal!
------------------------------------------------------------------------ */