// Datei Kovarianz01.java
/* ------------------------------------------------------------------------
Demonstriert das Kovarianz-Problem bei Java-Reihungen. Der Compiler
erlaubt, dass man die Methode liesUndSchreib auf Reihungen beliebiger Typen
anwendet, z.B. auf Reihungen der Typen Object[], String[], Integer[] und
Object[][]. In den letzten beiden Faellen (Integer[] und Object[][]) wird
aber zur Laufzeit eine Ausnahme geworfen.
------------------------------------------------------------------------ */
class Kovarianz01 {
   // ---------------------------------------------------------------------
   static void liesUndSchreib(Object[] pr) {
      // Diese Methode greift schreibend und lesend auf die Komponenten
      // der Reihung pr zu.

      // pr[0] hat den vereinbarten Typ Object. Einer Object-Variablen
      // darf man u.a. einen String zuweisen. Die folgende Zuweisung
      // sieht deshalb ganz harmlos aus. Wegen des Kovarianz-Problems
      // kann sie aber zur Laufzeit eine Ausnahme ausloesen ohne dass
      // der Ausfuehrer das bei der Uebergabe des Programms ("zur Com-
      // pilezeit") vorhersehen koennte.
      pr[0] = new String("Anfang!");

      // Harmlose, nur lesende Zugriffe auf die Komponenten der Reihung pr:
      for (Object ob: pr) {
         p(ob + " ");
      }
      pln("");
      pln("-------------------------------");
   } // liesUndSchreib
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {

      pln("Kovarianz01: Jetzt geht es los!");
      pln("-------------------------------");

      // Drei Reihungen mit unterschiedlichen Komponenten-Typen:
      Object []   or  = {null, new Integer(3), "Musketiere"};
      String []   sr  = {null, "Hallo", "Sonja!"};
      Integer[]   ir  = {null, new Integer(20), new Integer(30)};
      Object [][] orr = {null, {"AA", "BB", "CC"}, {"DD", "EE"}};

      // Jetzt werden die drei Reihungen bearbeitet:
      liesUndSchreib(or);  // Geht gut!
      liesUndSchreib(sr);  // Geht gerade noch gut!
      liesUndSchreib(ir);  // Loest eine Ausnahme aus!
      liesUndSchreib(orr); // Loest eine Ausnahme aus!

      pln("-------------------------------");
      pln("Kovarianz01: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Kovarianz01
/* ------------------------------------------------------------------------
Ausgabe des Programms Kovarianz01:

Kovarianz01: Jetzt geht es los!
-------------------------------
Anfang! 3 Musketiere
-------------------------------
Anfang! Hallo Sonja!
-------------------------------
Exception in thread "main" java.lang.ArrayStoreException: java.lang.String
   at Kovarianz01.liesUndSchreib(Kovarianz01.java:20)
   at Kovarianz01.main(Kovarianz01.java:42)
------------------------------------------------------------------------ */
