// Datei Kovarianz02.java
/* ------------------------------------------------------------------------
List<K> ist eine generische Schnittstelle mit einem Parameter K (wie
"Komponententyp"). Parametrisierte-List-Typen wie List<OT> und List<UT>
werden hier benutzt und mit Reihungstypen wie OT[] und UT[] verglichen.

Es wird gezeigt: Aus A: folgt B:
A:  OT       ist  ein Obertyp von UT.
B:  OT[]     ist  ein Obertyp von UT[].

Ausserdem wird gezeigt dass gilt:
C:  List<OT> ist kein Obertyp von List<UT> (auch wenn OT Obertyp von UT ist)
D:  List<UT> ist kein Obertyp von List<OT> (auch wenn OT Obertyp von UT ist)

Mit der generischen Funktion liesUndSchreib(List<? extends OT> L)
kann man Listen verschiedener parametrisierter-List-Typen (z.B. List<OT>
und List<UT>) bearbeiten.
---------------------------------------------------------------------------
Abkuerzungen:

OT  Obertyp, oo OT-Objekt, or OT-Reihung, ol bzw. oL OT-Liste
UT Untertyp, uo UT-Objekt, ur UT-Reihung, ul bzw. uL UT-Liste
------------------------------------------------------------------------ */
import java.util.List;       // Eine generische Schnittstelle
import java.util.ArrayList;  // Eine generische Klasse (implementiert List)
import java.util.Arrays;     // Eine Klasse (enthaelt Methode asList)
// ------------------------------------------------------------------------
// Die Klassen OT und UT stehen hier fuer zwei beliebige Typen, von denen
// der eine ein Obertyp des anderen ist.
class OT {                                                      // Ober-Typ
   static    int    anzahl = 0;
   static    int    getAnzahl()            {return anzahl;}

   protected String name;
   public    String toString ()            {return "oo " + name;}
   public           OT       (String name) {this.name=name; anzahl++;}
} // class OT
// ------------------------------------------------------------------------
class UT  extends OT {                                         // Unter-Typ
   static    int    anzahl = 0;
   static    int    getAnzahl()            {return anzahl;}

   public    String toString ()            {return "uo " + name;}
   public           UT       (String name) {super(name); anzahl++;}
} // class OT
// ------------------------------------------------------------------------
class Kovarianz02 {
   // ---------------------------------------------------------------------
   static void liesUndSchreibO(List<OT> oL) {
      // Mit dieser (nicht-generischen) Methode kann man nur Listen des
      // Typs List<OT> (aber keine des Typs List<UT>) bearbeiten.
      p("Eine Liste der Laenge " + oL.size() + ": ");
      for (OT oo: oL) {
         p(oo + ", ");
      }
      pln();

      // Neue Komponenten an die Liste oL anhaengen:
      oL.add(new OT("C1")); //       Erlaubt!
      oL.add(new UT("C2")); //       Erlaubt!
   } // liesUndSchreibO
   // ---------------------------------------------------------------------
   static void liesUndSchreibU(List<UT> uL) {
      // Mit dieser (nicht-generischen) Methode kann man nur Listen des
      // Typs List<UT> (aber keine des Typs List<OT>) bearbeiten.
      p("Eine Liste der Laenge " + uL.size() + ": ");
      for (UT uo: uL) {
         p(uo + ", ");
      }
      pln();

      // Neue Komponenten an die Liste uL anhaengen:
//    uL.add(new OT("D1")); // Nicht erlaubt!
      uL.add(new UT("D2")); //       Erlaubt!
   } // liesUndSchreibU
   // ---------------------------------------------------------------------
   static void liesUndSchreib (List<? extends OT> L) {
      // Mit dieser generischen Methode kann man Listen aller Typen
      // List<T> bearbeiten, bei denen T ein Untertyp von OT ist
      // (insbesondere also Listen der Typen List<OT> und List<UT>).
      p("Eine Liste der Laenge " + L.size() + ": ");
      for (OT o: L) {
         p(o + ", ");
      }
      pln();

      // Neue Komponenten an die Liste L anhaengen:
//    L.add(new OT("E1")); // Nicht erlaubt!
//    L.add(new UT("E2")); // Nicht erlaubt!
   } // liesUndSchreib
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      pln("Kovarianz02: Jetzt geht es los!");

      // Je 2 Reihungen der Reihungstypen OT[] und UT[]
      // zum Initialisieren der folgenden 4 List-Objekte.
      // Achtung: Die Reihung orB enthaelt OT- und UT-Objekte!
      OT[]     orA = new OT[]{new OT("A1"), new OT("A2"), new OT("A3")};
      OT[]     orB = new OT[]{new UT("B1"), new OT("B2"), new UT("B3")};

      UT[]     urA = new UT[]{new UT("A1"), new UT("A2"), new UT("A3")};
      UT[]     urB = new UT[]{new UT("B1"), new UT("B2"), new UT("B3")};

      // Je 2 List-Objekte der p-List-Typen List<OT> und  List<UT>:
      List<OT> olA = new ArrayList<OT>(Arrays.asList(orA));
      List<OT> olB = new ArrayList<OT>(Arrays.asList(orB));

      List<UT> ulA = new ArrayList<UT>(Arrays.asList(urA));
      List<UT> ulB = new ArrayList<UT>(Arrays.asList(urB));

      // Wieviel OT-Objekte und UT-Objekte wurden bisher erzeugt?
      // Achtung: Jedes UT-Objekt zaehlt auch als OT-Objekt!
      pln("--------------------------");
      pln("OT.getAnzahl(): " + OT.getAnzahl());
      pln("UT.getAnzahl(): " + UT.getAnzahl());

      // Alle List-Objekte direkt mit pln ausgeben:
      pln("--------------------------");
      pln("olA: " + olA);
      pln("olB: " + olB);
      pln("ulA: " + ulA);
      pln("ulB: " + ulB);

      // Versuch, alle List-Objekte mit den Methoden liesUndSchreibO,
      // liesUndSchreibU und liesUndSchreib auszugeben:
      pln("-----------------------------------------------------------");
      liesUndSchreibO(olA);
      liesUndSchreibO(olB);
//    liesUndSchreibO(ulA); // Nicht erlaubt!
//    liesUndSchreibO(ulB); // Nicht erlaubt!
      pln("-----------------------------------------------------------");
//    liesUndSchreibU(olA); // Nicht erlaubt!
//    liesUndSchreibU(olB); // Nicht erlaubt!
      liesUndSchreibU(ulA);
      liesUndSchreibU(ulB);
      pln("-----------------------------------------------------------");
      liesUndSchreib (olA);
      liesUndSchreib (olB);
      liesUndSchreib (ulA);
      liesUndSchreib (ulB);
      pln("-----------------------------------------------------------");


      // Erlaubte und nicht erlaubte Zuweisungen zwischen
      // Variablen verschiedener Reihungstypen:
      orA = orB; // OT[]     Obertyp von OT[]?     Ja!
      orA = urB; // OT[]     Obertyp von UT[]?     Ja!

//    urA = orB; // UT[]     Obertyp von OT[]?     Nein!
      urA = urB; // UT[]     Obertyp von UT[]?     Ja!

      // Erlaubte und nicht erlaubte Zuweisungen zwischen
      // Variablen verschiedener parametrisierter-List-Typen:
      olA = olB; // List<OT> Obertyp von List<OT>? Ja!
//    olA = ulB; // List<OT> Obertyp von List<UT>? Nein!

//    ulA = olB; // List<UT> Obertyp von List<OT>? Nein!
      ulA = ulB; // List<UT> Obertyp von List<UT>? Ja!

      pln("Kovarianz02: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
// ------------------------------------------------------------------------
} // class Kovarianz02
/* ------------------------------------------------------------------------
Fehlermeldungen des Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\Kovarianz02.java:74: cannot find symbol
symbol  : method add(OT)
location: interface java.util.List<UT>
    uL.add(new OT("D1")); // Nicht erlaubt!
      ^
D:\Java\Kovarianz02.java:89: cannot find symbol
symbol  : method add(OT)
location: interface java.util.List<? extends OT>
    L.add(new OT("E1")); // Nicht erlaubt!
     ^
D:\Java\Kovarianz02.java:90: cannot find symbol
symbol  : method add(UT)
location: interface java.util.List<? extends OT>
    L.add(new UT("E2")); // Nicht erlaubt!
     ^
D:\Java\Kovarianz02.java:130: liesUndSchreibO(java.util.List<OT>) in
Kovarianz02 cannot be applied to (java.util.List<UT>)
    liesUndSchreibO(ulA); // Nicht erlaubt!
    ^
D:\Java\Kovarianz02.java:131: liesUndSchreibO(java.util.List<OT>) in
Kovarianz02 cannot be applied to (java.util.List<UT>)
    liesUndSchreibO(ulB); // Nicht erlaubt!
    ^
D:\Java\Kovarianz02.java:133: liesUndSchreibU(java.util.List<UT>) in
Kovarianz02 cannot be applied to (java.util.List<OT>)
    liesUndSchreibU(olA); // Nicht erlaubt!
    ^
D:\Java\Kovarianz02.java:134: liesUndSchreibU(java.util.List<UT>) in
Kovarianz02 cannot be applied to (java.util.List<OT>)
    liesUndSchreibU(olB); // Nicht erlaubt!
    ^
D:\Java\Kovarianz02.java:150: incompatible types
found   : OT[]
required: UT[]
    urA = orB; // UT[]     Obertyp von OT[]?     Nein!
          ^
D:\Java\Kovarianz02.java:156: incompatible types
found   : java.util.List<UT>
required: java.util.List<OT>
    olA = ulB; // List<OT> Obertyp von List<UT>? Nein!
          ^
D:\Java\Kovarianz02.java:158: incompatible types
found   : java.util.List<OT>
required: java.util.List<UT>
    ulA = olB; // List<UT> Obertyp von List<OT>? Nein!
          ^
---------------------------------------------------------------------------
Ausgabe des Programms Kovarianz02:

Kovarianz02: Jetzt geht es los!
--------------------------
OT.getAnzahl(): 12
UT.getAnzahl(): 8
--------------------------
olA: [oo A1, oo A2, oo A3]
olB: [uo B1, oo B2, uo B3]
ulA: [uo A1, uo A2, uo A3]
ulB: [uo B1, uo B2, uo B3]
-----------------------------------------------------------
Eine Liste mit 3 Komponenten: oo A1 oo A2 oo A3
Eine Liste mit 3 Komponenten: uo B1 oo B2 uo B3
-----------------------------------------------------------
Eine Liste mit 3 Komponenten: uo A1 uo A2 uo A3
Eine Liste mit 3 Komponenten: uo B1 uo B2 uo B3
-----------------------------------------------------------
Eine Liste mit 5 Komponenten: oo A1 oo A2 oo A3 oo C1 uo C2
Eine Liste mit 5 Komponenten: uo B1 oo B2 uo B3 oo C1 uo C2
Eine Liste mit 4 Komponenten: uo A1 uo A2 uo A3 uo D2
Eine Liste mit 4 Komponenten: uo B1 uo B2 uo B3 uo D2
-----------------------------------------------------------
Kovarianz02: Das war's erstmal!
------------------------------------------------------------------------ */
