// Datei Layout09.java
/* ------------------------------------------------------------------------
Die Klassen GridBagLayout und GridBagConstraints sind ziemlich komplex.
Dieses Programm erlaubt es, mit den vielen Moeglichkeiten dieser
Klassen anschaulich zu experimentieren.
Das Programm oeffnet zwei Rahmen (JFrame-Objekte): Einen kontrol-
lierenden Rahmen und einen kontrollierten Rahmen. Der kontrollierte
Rahmen ist mit einem GridBaglayout versehen und zeigt 5 Knoepfe
(JButton). Im kontrollierenden Rahmen kann man die Groessen-Attribute
der Knoepfe (preferredSize, maximumSize, minimumSize) und alle 14 Attri-
bute ihrer GridBagConstraints-Objekte ansehen und veraendern.
Beim Veraendern von Knopf-Attributen kann man wahlweise Zahlen eingeben
oder die Namen der dafuer vorgesehenen Swing-Konstanten (RELATIVE, RE-
MAINDER, NONE, BOTH, HORIZONTAL, VERTICAL,CENTER, NORTH, NORTHEAST, ...).
Veraenderungen, die man im kontrollierenden Rahmen eingibt, wirken sich
(sobald man auf Return gedrueckt hat) auf die Darstellung der Knoepfe
im kontrollierten Rahmen aus. So kann man "ein Gefuehl fuer die Wirkung
der einzelnen Attribute" entwickeln.
Der Benutzer kann jeden einzelnen Knopf auch deaktivieren (und spaeter
wieder aktivieren). Solange ein Knopf deaktiviert ist, wird er im
kontrollierten Rahmen nicht dargestellt.
Mehrere Menus bieten verschiedene Moeglichkeiten, die Attribute aller
Knoepfe bequem zu veraendern. So kann man z.B. alle Knoepfe in einer
senkrechten, waagerechten oder diagonalen Linie anordnen lassen (dazu
werden die gridx- und gridy-Attribute entsprechend veraendert) oder alle
fill-Attribute auf NONE oder auf BOTH setzen lassen etc..
Der Menuepunkt drucken im Menue Datei erlaubt es, ein Abbild des kon-
trollierten Fensters und eine Liste aller Attribute aller Knoepfe aus-
zudrucken (a kind of screenshot function).
------------------------------------------------------------------------
Zur Implementierung: Fuer jeden Knopf, der (mit einem GridBagCon-
staints-Objekt versehen) im kontrollierten Rahmen angezeigt wird, ver-
waltet das Programm 24 sog. Zustands-Elemente. Diese Zustands-Elemente
beschreiben "den momentanen Zustand des Knopfes". Im Einzeln:
1. Den Namen des Knopfes und ob er gerade aktiv ist oder nicht
   (2 Zustands-Elemente).
2. Die Groessen-Attribute des Knopfes: preferred, maximum and minimum
   size (6 Zustands-Elemente)
3. Alle Attribute des GridBagConstraints-Objektes des Knopfes
   (14 Zustands-Elemente)

Die Zustands-Elemente gehoeren zu den konkreten Unterklassen der ab-
strakten Klasse ZElem. Jedes Zustands-Element enthaelt unter anderem
ein JTextField-Objekt, in dem das betreffende Element im kontrollier-
enden Fenster dem Benutzer gezeigt wird und von ihm editiert werden
kann. Ausserdem enthaelt ein Zustands-Element den momentanen Wert
des betreffenden Elementes. Dieser Wert ist vom Typ int, double,
boolean bzw. String. Fuer jeden dieser Typen gibt es eine konkrete
Unterklasse von ZElem (die Klassen IntElem, DoubleElem, BooleanElem
und StringElem). Insgesamt gibt es 120 Zustands-Elemente (Objekte)
(fuer 5 Knoepfe, 24 Elemente pro Knopf). Diese Zustands-Elemente sind
in der Reihung zElemR untergebracht.

Wenn man ein Zustands-Element erzeugt, kann man ein HashMap-Objekt an-
geben. Darin sollten Namen von Konstanten (Strings) und die zugehoe-
rigen Werte (als Objekte, z.B. der Klasse Integer) miteinander ver-
bunden werden. Mit Hilfe dieser HashMap-Objekte wird erreicht, dass
der Benutzer beim Editieren eines Elementes wahlweise einen Wert oder
den Namen der entsprechenden SwingConstants-Konstanten eingeben kann.
Z.B. kann er fuer das fill-Element wahlweise die Zahlen 0, 1, 2 oder 3
oder die Konstanten-Namen NONE, BOTH, HORIZONTAL oder VERTICAL angeben.
Fuer andere Elemente entsprechend.

Inhalt dieser Datei (nach den import-Befehlen):
1. Die abstrakte Klasse ZElem. Dann ihre konkreten Unterklassen:
2. Die Klasse     IntElem.
3. Die Klasse  DoubleElem.
4. Die Klasse BooleanElem.
5. Die Klasse  StringElem.
6. Dann die Hauptklasse Layout09 (mit main-Methode).
------------------------------------------------------------------------ */
// package layout;

import javax.swing      .JTextField;
import javax.swing      .JLabel;
import javax.swing      .JTabbedPane;
import javax.swing      .JButton;
import javax.swing      .JPanel;
import javax.swing      .JFrame;
import javax.swing      .Box;
import javax.swing      .BoxLayout;
import javax.swing      .SwingConstants;
import javax.swing      .JMenuBar;
import javax.swing      .JMenu;
import javax.swing      .JMenuItem;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;
import java .awt        .Container;
import java .awt        .Dimension;
import java .awt        .GridBagLayout;
import java .awt        .GridBagConstraints;
import java .awt        .Graphics;
import java .awt        .Graphics2D;
import java .awt        .Color;
import java .awt        .Font;
import java .awt  .event.ActionListener;
import java .awt  .event.ActionEvent;
import java .awt  .event.WindowEvent;
import java .awt  .event.WindowAdapter;
import java .awt  .event.ComponentListener;
import java .awt  .event.ComponentEvent;
import java .awt  .event.ComponentAdapter;
import java .awt  .print.Printable;
import java .awt  .print.PrinterJob;
import java .awt  .print.PrinterException;
import java .awt  .print.PageFormat;
import java .util       .HashMap;
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
abstract class ZElem {
   // Beschreibt Zustands-Elemente. Fuer verschiedene Typen (int, double,
   // boolean, String) gibt es konkrete Unterklassen dieser Klasse.
   // ---------------------------------------------------------------------
   HashMap    hashMap;            // Konstanten-Namen und Werte verbinden
   JLabel     beSchriftungsLabel; // Beschriftung des Text-Feldes
   JTextField textField;          // Das Text-Feld
   // ---------------------------------------------------------------------
   abstract void textFieldToWert() ;
   // Holt den momentanen Inhalt des textField und wandelt ihn in einen
   // Wert um. Der Typ dieses Wertes ist von Unterklasse zu Unterklasse
   // verschieden.
   abstract void wertToTextField() ;
   // Wandelt den momentanen Wert dieses Zustands-Elementes in einen
   // String um und setzt den in das textField.
   // ---------------------------------------------------------------------
   // Ein Konstruktor:
   ZElem(String beSchriftung) {
      // Wie soll das textField beschriftet werden?
      beSchriftungsLabel = new JLabel(beSchriftung);
      textField          = new JTextField();
   } // Konstruktor ZElem
   // ---------------------------------------------------------------------
} // class ZElem
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
class IntElem extends ZElem {
   // Zustands-Elemente mit int-Wert.
   // ---------------------------------------------------------------------
   // Momentaner Wert dieses Zustands-Elementes und Standardwert. Falls
   // eine Eingabe des Benuters fuer dieses Element nicht zu entschluesseln
   // ist, wird der Standardwert genommen.
   int wert    = 0;
   int stdWert = 0;
   // ---------------------------------------------------------------------
   // Zwei Konstruktoren, einer ohne und einer mit HashMap:
   IntElem(String beSchriftung, int stdWert) {
      super(beSchriftung);
      wert         = stdWert;
      this.stdWert = stdWert;
   } // Konstruktor IntElement

   IntElem(String beSchriftung, int stdWert, HashMap hashMap) {
      this(beSchriftung, stdWert);
      this.hashMap = hashMap;
   } // Konstruktor IntElement
   // ---------------------------------------------------------------------
   void textFieldToWert() {
      // Wandelt den momentanen Inhalt von textField (ein JTextField) in
      // einen int-Wert um und bringt den nach wert.
      String s = textField.getText();     // Hol den momentanen Inhalt.
      s = s.toUpperCase();                // Falls ein Konstanten-Name.

      try {                               // Laesst er sich in einen int-
         wert = Integer.parseInt(s);      // Wert umwandeln?
      }
      catch (NumberFormatException nfe) { // Wenn nein und
         if (hashMap == null) {           // keine hashMap vorhanden, dann
            wert = stdWert;               // Standardwert nehmen.
            return;
         } // if
                                          // Sonst in der hashMap nachsehen
         Integer ausHashMap = (Integer) hashMap.get(s);

         if (ausHashMap == null) {        // Wenn kein Eintrag vorhanden,
            wert = stdWert;               // Standardwert nehmen.
            return;
         } // if

         wert = ausHashMap.intValue();    // Sons Wert aus hashMap nehmen.
      } // catch
   } // textFieldToWert
   // ---------------------------------------------------------------------
   void wertToTextField() {
      // Wandelt den wert in einen String um und bringt den in das
      // textField (ein JTextField).
      String  s = null;
      if (hashMap != null) {              // Wenn eine hashMap vorhanden,
         Integer i = new Integer(wert);   // dann dort nach einem passenden
         s = (String) hashMap.get(i);     // Eintrag suchen
      }
      if (s == null) {                    // Falls keiner vorhanden
         s = String.valueOf(wert);        // wert einfach so in String
      }                                   // umwandeln.
      textField.setText(s);               // Sonst String aus hashMap
   } // wertToTextField                   // nehmen und nach textField.
   // ---------------------------------------------------------------------
} // class IntElem
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
class DoubleElem extends ZElem {
   // Zustands-Elemente mit double-Wert.
   // ---------------------------------------------------------------------
   // Momentaner Wert dieses Zustands-Elementes und Standardwert:
   double wert    = 1.0;
   double stdWert = 1.0;
   // ---------------------------------------------------------------------
   // Nur ein Konstruktor (fuer double-Werte gibt es keine SwingConstants-
   // Konstanten, also sind HashMap-Objekte hier nicht vorgesehen):
   DoubleElem(String beSchriftung, double stdWert) {
      super(beSchriftung);
      wert         = stdWert;
      this.stdWert = stdWert;
   } // Konstruktor DoubleElem
   // ---------------------------------------------------------------------
   void textFieldToWert() {
      String s = textField.getText();
      try {
         wert = Double.parseDouble(s);
      }
      catch (NumberFormatException nfe) {
         wert = stdWert;
      }
   } // textFieldToWert
   // ---------------------------------------------------------------------
   void wertToTextField() {
      textField.setText(String.valueOf(wert));
   } // wertToTextField
   // ---------------------------------------------------------------------
} // class DoubleElem
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
class BooleanElem extends ZElem {
   // Zustands-Elemente mit boolean-Wert.
   // ---------------------------------------------------------------------
   // Momentaner Wert dieses Zustands-Elementes und Standardwert:
   boolean wert    = true;
   boolean stdWert = true;
   // ---------------------------------------------------------------------
   // Nur ein Konstruktor:
   BooleanElem(String beSchriftung, boolean stdWert) {
      super(beSchriftung);
      wert         = stdWert;
      this.stdWert = stdWert;
   } // Konstruktor BooleanElem
   // ---------------------------------------------------------------------
   void textFieldToWert() {
      String s = textField.getText();
      // Warum fehlt die Methode Boolean.parseBoolean()? Aus Versehen!
      if        (s.equals("false")) {
         wert = false;
      } else if (s.equals("true"))  {
         wert = true;
      } else {
         wert = stdWert;
      } // if

   } // textFieldToWert
   // ---------------------------------------------------------------------
   void wertToTextField() {
      textField.setText(String.valueOf(wert));
   } // wertToTextField
   // ---------------------------------------------------------------------
} // class BooleanElem
// xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
class StringElem extends ZElem {
   // ---------------------------------------------------------------------
   // Momentaner Wert dieses Zustands-Elementes und Standardwert:
   String ob;
   String std;
   // ---------------------------------------------------------------------
   StringElem(String beSchriftung, String stdObjekt) {
      super(beSchriftung);
      std = stdObjekt;
      ob  = std;
   }
   // ---------------------------------------------------------------------
   void textFieldToWert() {
      ob = textField.getText();
   }
   // ---------------------------------------------------------------------
   void wertToTextField() {
      textField.setText(ob);
   }
   // ---------------------------------------------------------------------
} // class StringElem
// XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
public class Layout09 extends JFrame implements
   SwingConstants,
   Printable
{
   // Jedes Objekt dieser Klasse stellt einen sog. kontrollierenden Rahmen
   // dar, der zusaetzlich einen sog. kontrollierten Rahmen erzeugt.
   // ---------------------------------------------------------------------
   // Im kontrollierten Rahmen werden (maximal) 5 Knoepfe angezeigt und zu
   // jedem Knopf gibt es 24 Zustands-Elemente:
   final int ANZ_KNOEPFE =  5; //  5 Knoepfe
   final int ANZ_ELEM    = 24; // 24 ZElem-Objekte pro Knopf

   // Fuer jedes Zustands-Element gibt es eine Index-Konstante:
   final int
      NAME          =  0, // "Verwaltungsdaten" des Knopfes
      AKTIV         =  1,

      WIDTH_PREF    =  2, // Attribute des Knopfes selbst
      HEIGHT_PREF   =  3,
      WIDTH_MAX     =  4,
      HEIGHT_MAX    =  5,
      WIDTH_MIN     =  6,
      HEIGHT_MIN    =  7,
      WIDTH_AKT     =  8, // aktuelle ("tatsaechliche") Breite
      HEIGHT_AKT    =  9, // aktuelle ("tatsaechliche") Hoehe

      ANCHOR        = 10, // Attribute des GridBagConstraints des Knopfes
      FILL          = 11,
      GRIDX         = 12,
      GRIDY         = 13,
      GRIDWIDTH     = 14,
      GRIDHEIGHT    = 15,
      IPADX         = 16,
      IPADY         = 17,
      WEIGHTX       = 18,
      WEIGHTY       = 19,
      INSETS_TOP    = 20,
      INSETS_BOTTOM = 21,
      INSETS_LEFT   = 22,
      INSETS_RIGHT  = 23;
   // ---------------------------------------------------------------------
   int aktuellerKnopfIndex = 0; // Index (zwischen 0 und ANZ_KNOEPFE-1)
   // des Knopfes, dessen Daten gerade im kontrollierenden Rahmen sichtbar
   // sind. Wird vom ChangeListener charly aktualisiert.
   // ---------------------------------------------------------------------
   // Der kontrollierte Rahmen wird mit einem GridBagLayout-Objekt versehen:
   JFrame             derMitGBL = new JFrame("Kontrollierter Rahmen " +
                                             "mit GridBagLayout");
   GridBagLayout      meinGBL   = new GridBagLayout();
   GridBagConstraints meinGBC   = new GridBagConstraints();
   Container          thisCP    = this     .getContentPane();
   Container          gblCP     = derMitGBL.getContentPane();
   // ---------------------------------------------------------------------
   // Die Knoepfe, die im kontrollierten Rahmen angezeigt werden (sie
   // "bewirken nichts" und sollen nur per Layout angeordnet werden:
   JButton[] buttonR  = new JButton[ANZ_KNOEPFE];
   // Fuer jeden Knopf ein JPanel-Objekt mit zwei zwei Box-Objekten darin:
   JPanel [] panelR   = new JPanel [ANZ_KNOEPFE];
   Box    [] box1R    = new Box    [ANZ_KNOEPFE]; // f. Beschriftungs-Label
   Box    [] box2R    = new Box    [ANZ_KNOEPFE]; // f. JTextField-Objekte
   // ---------------------------------------------------------------------
   // Ein JMemuBar-Objekt mit 4 JMenu-Objekten darin und darin insgesamt
   // 14 JMenuItem-Objekte:
   JMenuBar  menuBar    = new JMenuBar();

   JMenu     menu1      = new JMenu("Datei");
   JMenu     menu2      = new JMenu("Anordnen");
   JMenu     menu3      = new JMenu("fill");
   JMenu     menu4      = new JMenu("anchor");
   JMenu     menu5      = new JMenu("weight");

   JMenuItem menuItem11 = new JMenuItem("Drucken");

   JMenuItem menuItem21 = new JMenuItem("Waagerecht");
   JMenuItem menuItem22 = new JMenuItem("Senkrecht");
   JMenuItem menuItem23 = new JMenuItem("Diagonal1");
   JMenuItem menuItem24 = new JMenuItem("Diagonal2");
   JMenuItem menuItem25 = new JMenuItem("Kreuz");
   JMenuItem menuItem26 = new JMenuItem("RELATIVE-x");
   JMenuItem menuItem27 = new JMenuItem("RELATIVE-y");

   JMenuItem menuItem31 = new JMenuItem("alle NONE");
   JMenuItem menuItem32 = new JMenuItem("alle BOTH");
   JMenuItem menuItem33 = new JMenuItem("alle HORIZONTAL");
   JMenuItem menuItem34 = new JMenuItem("alle VERTICAL");

   JMenuItem menuItem41 = new JMenuItem("alle CENTER");
   JMenuItem menuItem42 = new JMenuItem("alle NORTH");

   JMenuItem menuItem51 = new JMenuItem("alle 0.0");
   JMenuItem menuItem52 = new JMenuItem("alle 1.0");
   JMenuItem menuItem53 = new JMenuItem("alle 0.5");
   // ---------------------------------------------------------------------
   // Fuer jeden Knopf im kontrollierten Fenster eine "Karteikarte"
   // innerhalb eines JTabbedPane-Objekts:
   JTabbedPane     tabbedPane = new JTabbedPane(BOTTOM);
   // ---------------------------------------------------------------------
   // Die JTextField-Objekte und die dazugehoerigen JLabel-Objekte sollen
   // gleich gross sein (damit sie "auf gleicher Hoehe" erscheinen):
   final Dimension dim1 = new Dimension(100, 20);
   // ---------------------------------------------------------------------
   // Ein einheitliches ActionListener-Objekt fuer alle JTextField-Objekte
   // (wenn der Benutzer eines der Text-Felder editiert, werden *alle*
   // Zustands-Elemente des aktuellen Knopfes aktualisiert)
   ActionListener machtAlles = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         aktualisiereAlleElementeEinesKnopfes(aktuellerKnopfIndex);
         knoepfeNeuArrangieren();
      } // action Performed
   }; // machtAlles
   // ---------------------------------------------------------------------
   // Fuer einige Elemente eines GridBagConstraints-Objektes gibt es
   // spezielle SwingConstants-Konstanten (z.B. die Konstanten NONE, BOTH,
   // ... etc. fuer das fill-Element und NORTH, WEST, ... etc. fuer das
   // anchor-Element). Hier werden ein paar HashMap-Objekte vereinbart,
   // die die Namen dieser Konstanten mit entsprechenden Integer-Objekten
   // assoziieren. Mit diesen HashMap-Objekten kann man "hin- und zurueck-
   // uebersetzen, z.B. von NORTH nach 11 und von 11 nach NORTH etc..
   // ---------------------------------------------------------------------
   // "Kompakte Vereinbarung" einiger Integer-Objekte. "m1" soll "minus 1"
   // und "p1", "p2", ... soll "plus 1", "plus 2", ...  bedeuten.
   final Integer
      m1  = new Integer(-1), p0  = new Integer( 0), p1  = new Integer( 1),
      p2  = new Integer( 2), p3  = new Integer( 3), p10 = new Integer(10),
      p11 = new Integer(11), p12 = new Integer(12), p13 = new Integer(13),
      p14 = new Integer(14), p15 = new Integer(15), p16 = new Integer(16),
      p17 = new Integer(17), p18 = new Integer(18);

   // Hier muss man 4 Gruppen von SwingConstants-Konstanten unterscheiden:
   HashMap<Object, Object> anchorMap = new
   HashMap<Object, Object>(18, 1.0F); // fuer anchor
   HashMap<Object, Object> fillMap   = new
   HashMap<Object, Object>( 8, 1.0F); // fuer fill
   HashMap<Object, Object> gridxMap  = new
   HashMap<Object, Object>( 2, 1.0F); // fuer gridx, -y
   HashMap<Object, Object> gridwMap  = new
   HashMap<Object, Object>( 4, 1.0F); // fuer gridwidth, -height

   { // Initialisierer fuer die HashMap-Objekte (zweispaltig notiert):
      anchorMap.put("CENTER",    p10); anchorMap.put(p10, "CENTER");
      anchorMap.put("NORTH",     p11); anchorMap.put(p11, "NORTH");
      anchorMap.put("NORTHEAST", p12); anchorMap.put(p12, "NORTHEAST");
      anchorMap.put("EAST",      p13); anchorMap.put(p13, "EAST");
      anchorMap.put("SOUTHEAST", p14); anchorMap.put(p14, "SOUTHEAST");
      anchorMap.put("SOUTH",     p15); anchorMap.put(p15, "SOUTH");
      anchorMap.put("SOUTHWEST", p16); anchorMap.put(p16, "SOUTHWEST");
      anchorMap.put("WEST",      p17); anchorMap.put(p17, "WEST");
      anchorMap.put("NORTHWEST", p18); anchorMap.put(p18, "NORTHWEST");

        fillMap.put("NONE",      p0 );   fillMap.put(p0,  "NONE");
        fillMap.put("BOTH",      p1 );   fillMap.put(p1,  "BOTH");
        fillMap.put("HORIZONTAL",p2 );   fillMap.put(p2,  "HORIZONTAL");
        fillMap.put("VERTICAL",  p3 );   fillMap.put(p3,  "VERTICAL");

       gridxMap.put("RELATIVE",  m1 );  gridxMap.put(m1,  "RELATIVE");

       gridwMap.put("RELATIVE",  m1 );  gridwMap.put(m1,  "RELATIVE");
       gridwMap.put("REMAINDER", p0 );  gridwMap.put(p0,  "REMAINDER");
   } // Initialisierer
   // ---------------------------------------------------------------------
   // Die Reihung zElemR enthaelt alle Zustands-Elemente aller Knoepfe:

   ZElem[][] zElemR = new ZElem [ANZ_KNOEPFE][ANZ_ELEM];

   { // Initialisierer fuer die Reihung zElemR, erzeugt alle Zustands-
     // Elemente (Objekte des Typs ZElem):

      ZElem[][] zR = zElemR; // lokale Abkuerzung "zR" fuer "zElemR"
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         zR[kNr][NAME]         = new StringElem ("Name", "K"+(kNr+1));
         zR[kNr][AKTIV]        = new BooleanElem("Aktiv?", true);
         zR[kNr][ WIDTH_PREF]  = new IntElem    ("width  (preferred)", 100);
         zR[kNr][HEIGHT_PREF]  = new IntElem    ("height (preferred)",  30);
         zR[kNr][ WIDTH_MAX]   = new IntElem    ("width  (max)",       150);
         zR[kNr][HEIGHT_MAX]   = new IntElem    ("height (max)",        50);
         zR[kNr][ WIDTH_MIN]   = new IntElem    ("width  (min)",        70);
         zR[kNr][HEIGHT_MIN]   = new IntElem    ("height (min)",        20);
         zR[kNr][ WIDTH_AKT]   = new IntElem    ("width  (akt)",         0);
         zR[kNr][HEIGHT_AKT]   = new IntElem    ("height (akt)",         0);
         zR[kNr][ANCHOR]       = new IntElem    ("anchor",    10, anchorMap);
         zR[kNr][FILL]         = new IntElem    ("fill",       0,  fillMap);
         zR[kNr][GRIDX]        = new IntElem    ("gridx",     -1, gridxMap);
         zR[kNr][GRIDY]        = new IntElem    ("gridy",     -1, gridxMap);
         zR[kNr][GRIDWIDTH]    = new IntElem    ("gridwidth",  1, gridwMap);
         zR[kNr][GRIDHEIGHT]   = new IntElem    ("gridheigth", 1, gridwMap);
         zR[kNr][IPADX]        = new IntElem    ("ipadx",                0);
         zR[kNr][IPADY]        = new IntElem    ("ipady",                0);

         zR[kNr][WEIGHTX]      = new DoubleElem ("weightx",            1.0);
         zR[kNr][WEIGHTY]      = new DoubleElem ("weighty",            1.0);
         zR[kNr][INSETS_TOP]   = new IntElem    ("insets (top)",         0);
         zR[kNr][INSETS_BOTTOM]= new IntElem    ("insets (bottom)",      0);
         zR[kNr][INSETS_LEFT]  = new IntElem    ("insets (left)",        0);
         zR[kNr][INSETS_RIGHT] = new IntElem    ("insets (right)",       0);

      } // for
      // Weitere Initialisierungen im Konstruktor Layout09
   } // Initialisierer
   // ---------------------------------------------------------------------
   // Die ActionListener-Objekte fuer die Menues (JMenuItem-Objekte):
   // ---------------------------------------------------------------------
   // Menue "Datei" (menu1), Menuepunkt "drucken" (menuItem11):
   ActionListener drucken = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         System.out.println("Druckauftrag soll erteilt werden!");
         PrinterJob printerJob = PrinterJob.getPrinterJob();
         // Ohne Seiten-Dialog (page dialog):
         PageFormat pageFormat = printerJob.defaultPage();
         // Mit  Seiten-Dialog (page dialog):
       //PageFormat pageFormat = printerJob.pageDialog(defaultPage());
         printerJob.setPrintable(Layout09.this, pageFormat);

         try {
            // Ohne Drucker-Dialog (print dialog):
            printerJob.print();
            // Mit  Drucker-Dialog (print dialog):
          //if (printerJob.printDialog()) printerJob.print();
         }
         // Minimale Ausnahmebehandlung:
         catch (PrinterException pex) {
            System.out.println("In Layout09.actionPerformed (drucken):");
            System.out.println(pex);
            return;
         }
         System.out.println("Druckauftrag wurde erteilt!");
      }
   }; // drucken
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "Waagerecht" (menuItem21)
   ActionListener waageRecht = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDX].textField.setText("" + kNr);
            zElemR[kNr][GRIDY].textField.setText("3");
            aktualisiereEinElementAllerKnoepfe(GRIDX);
            aktualisiereEinElementAllerKnoepfe(GRIDY);
            knoepfeNeuArrangieren();
         }
      }
   }; // waageRecht
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "Senkrecht" (menuItem22)
   ActionListener senkRecht = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDX].textField.setText("3");
            zElemR[kNr][GRIDY].textField.setText("" + kNr);
            aktualisiereEinElementAllerKnoepfe(GRIDX);
            aktualisiereEinElementAllerKnoepfe(GRIDY);
            knoepfeNeuArrangieren();
         }
      }
   }; // senkRecht
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "Diagonal1" (menuItem23)
   ActionListener diaGonal1 = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDX].textField.setText("" + kNr);
            zElemR[kNr][GRIDY].textField.setText("" + kNr);
            aktualisiereEinElementAllerKnoepfe(GRIDX);
            aktualisiereEinElementAllerKnoepfe(GRIDY);
            knoepfeNeuArrangieren();
         }
      }
   }; // diaGonal1
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "Diagonal2" (menuItem24)
   ActionListener diaGonal2 = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDX].textField.setText("" +              kNr);
            zElemR[kNr][GRIDY].textField.setText("" + (ANZ_KNOEPFE-kNr));
            aktualisiereEinElementAllerKnoepfe(GRIDX);
            aktualisiereEinElementAllerKnoepfe(GRIDY);
            knoepfeNeuArrangieren();
         }
      }
   }; // diaGonal2
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "Kreus" (menuItem25)
   ActionListener kreuz = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         zElemR[1][GRIDX].textField.setText("0");
         zElemR[1][GRIDY].textField.setText("3");
         zElemR[2][GRIDX].textField.setText("1");
         zElemR[2][GRIDY].textField.setText("3");
         zElemR[3][GRIDX].textField.setText("2");
         zElemR[3][GRIDY].textField.setText("3");
         zElemR[0][GRIDX].textField.setText("1");
         zElemR[0][GRIDY].textField.setText("2");
         zElemR[4][GRIDX].textField.setText("1");
         zElemR[4][GRIDY].textField.setText("4");

         aktualisiereEinElementAllerKnoepfe(GRIDX);
         aktualisiereEinElementAllerKnoepfe(GRIDY);
         knoepfeNeuArrangieren();
      }
   }; // kreuz
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "RELATIVE-x" (menuItem26)
   ActionListener relativeX = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDX].textField.setText("RELATIVE");
         }
         aktualisiereEinElementAllerKnoepfe(GRIDX);
         knoepfeNeuArrangieren();
      }
   }; // relativeX
   // ---------------------------------------------------------------------
   // Menue "Anordnung" (menu2), Menuepunkt "RELATIVE-y" (menuItem27)
   ActionListener relativeY = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][GRIDY].textField.setText("RELATIVE");
         }
         aktualisiereEinElementAllerKnoepfe(GRIDY);
         knoepfeNeuArrangieren();
      }
   }; // relativeY
   // ---------------------------------------------------------------------
   // Fuer Menue "fill" (menu3), Menuepunkt "alle NONE" (menuItem31)
   ActionListener alleNone = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][FILL].textField.setText("NONE");
         }
         aktualisiereEinElementAllerKnoepfe(FILL);
         knoepfeNeuArrangieren();
      }
   }; // alleNone
   // ---------------------------------------------------------------------
   // Fuer Menue "fill" (menu3), Menuepunkt "alle BOTH" (menuItem32)
   ActionListener alleBoth = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][FILL].textField.setText("BOTH");
         }
         aktualisiereEinElementAllerKnoepfe(FILL);
         knoepfeNeuArrangieren();
      }
   }; // alleBoth
   // ---------------------------------------------------------------------
   // Fuer Menue "fill" (menu3), Menuepunkt "alle HORIZONTAL" (menuItem33)
   ActionListener alleHorizontal = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][FILL].textField.setText("HORIZONTAL");
         }
         aktualisiereEinElementAllerKnoepfe(FILL);
         knoepfeNeuArrangieren();
      }
   }; // alleHorizontal
   // ---------------------------------------------------------------------
   // Fuer Menue "fill" (menu3), Menuepunkt "alle VERTICAL" (menuItem34)
   ActionListener alleVertical = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][FILL].textField.setText("VERTICAL");
         }
         aktualisiereEinElementAllerKnoepfe(FILL);
         knoepfeNeuArrangieren();
      }
   }; // alleVertical
   // ---------------------------------------------------------------------
   // Fuer Menue "anchor" (menu4), Menuepunkt "alle CENTER" (menuItem41)
   ActionListener alleCenter = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][ANCHOR].textField.setText("CENTER");
         }
         aktualisiereEinElementAllerKnoepfe(ANCHOR);
         knoepfeNeuArrangieren();
      }
   }; // alleCenter
   // ---------------------------------------------------------------------
   // Fuer Menue "anchor" (menu4), Menuepunkt "alle NORTH" (menuItem42)
   ActionListener alleNorth = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][ANCHOR].textField.setText("NORTH");
         }
         aktualisiereEinElementAllerKnoepfe(ANCHOR);
         knoepfeNeuArrangieren();
      }
   }; // alleNorth
   // ---------------------------------------------------------------------
   // Fuer das Menue "weight" (menu5), Menupunkt "all 0.0" (menuItem51)
   ActionListener alleWeight0 = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][WEIGHTX].textField.setText("0.0");
            zElemR[kNr][WEIGHTY].textField.setText("0.0");
         }
         aktualisiereEinElementAllerKnoepfe(WEIGHTX);
         aktualisiereEinElementAllerKnoepfe(WEIGHTY);
         knoepfeNeuArrangieren();
      }
   }; // alleWeight0
   // ---------------------------------------------------------------------
   // Fuer das Menue "weight" (menu5), Menupunkt "all 1.0" (menuItem52)
   ActionListener alleWeight1 = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][WEIGHTX].textField.setText("1.0");
            zElemR[kNr][WEIGHTY].textField.setText("1.0");
         }
         aktualisiereEinElementAllerKnoepfe(WEIGHTX);
         aktualisiereEinElementAllerKnoepfe(WEIGHTY);
         knoepfeNeuArrangieren();
      }
   }; // alleWeight1
   // ---------------------------------------------------------------------
   // Fuer das Menue "weight" (menu5), Menupunkt "all 0.5" (menuItem53)
   ActionListener alleWeightHalb = new ActionListener() {
      public void actionPerformed(ActionEvent ae) {
         for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
            zElemR[kNr][WEIGHTX].textField.setText("0.5");
            zElemR[kNr][WEIGHTY].textField.setText("0.5");
         }
         aktualisiereEinElementAllerKnoepfe(WEIGHTX);
         aktualisiereEinElementAllerKnoepfe(WEIGHTY);
         knoepfeNeuArrangieren();
      }
   }; // alleWeightHalb

   // Damit sind alle ActionListener-Objekte fuer die Menue-Punkte fertig.
   // ---------------------------------------------------------------------
   // Immer wenn der Benuter auf einen "Reiter" (tab) von tabbedPane
   // klickt wird charly.stateChanged() aufgerufen und "merkt sich" den
   // Index des Knopfes, dessen Zustand jetzt im kontrollierenden Fenster
   // sichtbar ist und editiert werden kann. Falls der Benutzer jetzt
   // etwas aendert (in den JTextField-Objekten der Zustands-Elemente)
   // muessen nur die Elemente *dieses* Knopfes aktualisiert werden.
   ChangeListener charly = new ChangeListener() {
      public void stateChanged(ChangeEvent ce) {
         aktuellerKnopfIndex = tabbedPane.getSelectedIndex();
      } // stateChanged
   }; // charly
   // ---------------------------------------------------------------------

   // ---------------------------------------------------------------------
   void aktualisiereAlleElementeAllerKnoepfe() {
      // Bringt alle Zustands-Elemente aller Knoepfe auf den neusten Stand.
      // Holt die Inhalte der JTextField-Objekte und wandelt sie in Werte
      // um. Entfernt alle Knoepfe aus dem kontrollierten Fenster, bringt
      // ihre GridBagConstraints auf den neusten Stand und fuegt sie wieder
      // in das kontrollierte Fenster ein (falls sie noch aktiv sind):
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         aktualisiereAlleElementeEinesKnopfes(kNr);
      }
   } // aktualisiereAlleElementeAllerKnoepfe
   // ---------------------------------------------------------------------
   void aktualisiereAlleElementeEinesKnopfes(int kNr) {

      for (int elemNr=0; elemNr<ANZ_ELEM; elemNr++) {
         aktualisiereEinElementEinesKnopfes(kNr, elemNr);
      } // for
   } // aktualisiereAlleElementeEinesKnopfes
   // ---------------------------------------------------------------------
   void aktualisiereEinElementAllerKnoepfe(int elemNr) {
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         aktualisiereEinElementEinesKnopfes(kNr, elemNr);
      } // for
   } // aktualisiereEinElementAllerKnoepfe
   // ---------------------------------------------------------------------
   void aktualisiereEinElementEinesKnopfes(int kNr, int elemNr) {
      if        (elemNr == WIDTH_AKT) {
         aktualisiereWidthAkt(kNr, elemNr);
      } else if (elemNr == HEIGHT_AKT) {
         aktualisiereHeightAkt(kNr, elemNr);
      }
      zElemR[kNr][elemNr].textFieldToWert();
      zElemR[kNr][elemNr].wertToTextField();
   } // aktualisiereEinElementEinesKnopfes
   // ---------------------------------------------------------------------
   void aktualisiereWidthAkt(int kNr, int elemNr) {
      int aktuelleBreite = buttonR[kNr].getWidth();
      zElemR[kNr][elemNr].textField.setText("" + aktuelleBreite);
   } // aktualisiereWidthAkt
   // ---------------------------------------------------------------------
   void aktualisiereHeightAkt(int kNr, int elemNr) {
      int aktuelleHoehe = buttonR[kNr].getHeight();
      zElemR[kNr][elemNr].textField.setText("" + aktuelleHoehe);
   } // aktualisiereHeightAkt
   // ---------------------------------------------------------------------
   void knoepfeNeuArrangieren() {
      // Entfernt alle Knoepfe aus dem kontrollierten Rahmen, berechnet
      // ihre GridBagConstraints-Werte neu ("aufgrund der neusten Eingaben
      // des Benutzers") und fuegt die Knoepfe dann wieder in den kontrol-
      // lierten Rahmen ein (soweit sie noch aktiv sind):

      gblCP.removeAll();   // Alle Knoepfe entfernen
      derMitGBL.repaint(); // Kontrollierten Rahmen loeschen

      // Die aktiven Knoepfe mit einem (evtl. veraenderten) GridBag-
      // Constraints-Objekt versehen und wieder einfuegen:
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         setzeEinenKnopf(kNr);
         setzeMeinGBC   (kNr);

         boolean istAktiv = ((BooleanElem) zElemR[kNr][AKTIV]).wert;
         if (!istAktiv) continue; // inaktive Knoepfe ignorieren
         gblCP.add(buttonR[kNr]); // aktive Knoepfe wieder einfuegen
      } // for

      try {
         derMitGBL.setVisible(true); // Muss (sonst nur grau!)
         // Ausnahmen werden hier ausgeloest, wenn die GridBagConstraints-
         // Objekte der Knoepfe unerlaubte Zahlen enthalten, z.B.
         // wenn anchor kleiner als 10 oder groesser als 18 ist, oder
         // wenn der Betrag von gridx oder gridy deutlich ueber 500 liegt.
      }
      catch (Throwable ausNahme) {
         System.out.println("In Layout09.knoepfeNeuArrangieren:");
         System.out.println(ausNahme);
         korrigiereGBCs();
      }

      // Die tatsaechliche Groesse der Knoepfe koennte sich geaendert
      // haben und muss aktualisiert werden:
      aktualisiereEinElementAllerKnoepfe( WIDTH_AKT);
      aktualisiereEinElementAllerKnoepfe(HEIGHT_AKT);

   } // knoepfeNeuArrangieren
   // ---------------------------------------------------------------------
   void setzeEinenKnopf(int kNr) {
      // Setzt den Namen und die Groessen-Attribute des Knopfes "mit der
      // Nummer kNr" (d.h. des Knopfes buttonR[kNr]).

      // Daten zu einem Knopf aus der Reihung zElemR holen ...
      String name  = ( (StringElem) zElemR[kNr][NAME]).ob;
      int    wpref = ( (IntElem) zElemR[kNr][WIDTH_PREF] ).wert;
      int    hpref = ( (IntElem) zElemR[kNr][HEIGHT_PREF]).wert;
      int    wmax  = ( (IntElem) zElemR[kNr][WIDTH_MAX]  ).wert;
      int    hmax  = ( (IntElem) zElemR[kNr][HEIGHT_MAX] ).wert;
      int    wmin  = ( (IntElem) zElemR[kNr][WIDTH_MIN]  ).wert;
      int    hmin  = ( (IntElem) zElemR[kNr][HEIGHT_MIN] ).wert;

      // ... und "in den Knopf hineinpacken":
      JButton k = buttonR[kNr]; // k ist hier *der Knopf*
      k.setText(name);
      k.setPreferredSize(new Dimension(wpref, hpref));
      k.setMaximumSize  (new Dimension(wmax,  hmax));
      k.setMinimumSize  (new Dimension(wmin,  hmin));

   } // setzeEinenKnopf
   // ---------------------------------------------------------------------
   void setzeMeinGBC(int kNr) {
      // Bringt die entsprechenden Zustands-Elemente des Knopfes "mit der
      // Nummer kNr" in das GridBagConstraints-Objekt meinGBC:

      meinGBC.anchor       =    intWert(kNr, ANCHOR);
      meinGBC.fill         =    intWert(kNr, FILL);
      meinGBC.gridx        =    intWert(kNr, GRIDX);
      meinGBC.gridy        =    intWert(kNr, GRIDY);
      meinGBC.gridwidth    =    intWert(kNr, GRIDWIDTH);
      meinGBC.gridheight   =    intWert(kNr, GRIDHEIGHT);
      meinGBC.ipadx        =    intWert(kNr, IPADX);
      meinGBC.ipady        =    intWert(kNr, IPADY);
      meinGBC.weightx      = doubleWert(kNr, WEIGHTX);
      meinGBC.weighty      = doubleWert(kNr, WEIGHTY);
      meinGBC.insets.top   =    intWert(kNr, INSETS_TOP);
      meinGBC.insets.bottom=    intWert(kNr, INSETS_BOTTOM);
      meinGBC.insets.left  =    intWert(kNr, INSETS_LEFT);
      meinGBC.insets.right =    intWert(kNr, INSETS_RIGHT);

      meinGBL.setConstraints(buttonR[kNr], meinGBC);
   } // setzeMeinGBC
   // ---------------------------------------------------------------------
   void korrigiereGBCs() {
      // Wenn die Eingaben des Benutzers (fuer das GridBagConstraints-
      // Objekt eines Knopfes) so falsch sind, dass sie eine Ausnahme aus-
      // loesen, werden die kritischen GBC-Komponenten aller Knoepfe durch
      // Standardwerte ersetzt (die hoffentlich keine Ausnahme ausloesen):
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         zElemR[kNr][ANCHOR]    .textField.setText("CENTER");
         // fill ist eigentlich unkritisch, wird trotzdem "standardisiert":
         zElemR[kNr][FILL]      .textField.setText("NONE");
         zElemR[kNr][GRIDX]     .textField.setText("RELATIVE");
         zElemR[kNr][GRIDY]     .textField.setText("RELATIVE");
         zElemR[kNr][GRIDWIDTH] .textField.setText("1");
         zElemR[kNr][GRIDHEIGHT].textField.setText("1");
         zElemR[kNr][IPADX]     .textField.setText("0");
         zElemR[kNr][IPADY]     .textField.setText("0");
      } // for
      aktualisiereAlleElementeAllerKnoepfe();
      knoepfeNeuArrangieren();
   } // reInitialisiereEinenKnopf
   // ---------------------------------------------------------------------
   // Unterprogramme, die mit dem Drucken zu tun haben (die print-Methode,
   // dIntZ(), dDoubleZ(), pad(), intWert() un doubleWert()).
   // ---------------------------------------------------------------------
   // Die print-Methode der Printable-Schnittstelle:
   public int print(Graphics g1, PageFormat pf, int seitenNr) {

      // Es soll nur *eine* Seite gedruckt werden:
      if (seitenNr > 0) return Printable.NO_SUCH_PAGE;

      Graphics2D g2 = (Graphics2D) g1;

      // Den Ursprungs-Punkt von g2 in die obere linke Ecke der zu drucken-
      // den Seite legen:
      g2.translate(pf.getImageableX(), pf.getImageableY());

      // Die Texte sollen schwarz auf weiss gedruckt werden:
      g2.setPaint     (Color.black);
      g2.setBackground(Color.white);
      g2.setFont      (new Font("Monospaced", Font.PLAIN, 10));

      // Globale Variablen fuer die Methode dNZ ("drucke naechste Zeile")
      // initialisieren:
      druckX =   0.0F;  // Zeilen beginnen so weit links wie moeglich
      druckY = 300.0F;  // Platz fuer das Bild des kontrollierten Rahmens
                        // freilassen (ca. 300 Pixel, oben auf der Seite)
      deltaY =  10.0F;  // Abstand zwischen zwei Zeilen (in y-Richtung)
      g      =  g2;     // g2 "globalisieren" um einen Parameter zu sparen

      // Die Text-Informationen drucken:
      // Ueberschriftszeile:
      dNZ     ("                       K1      K2      K3      K4      K5");

      // Die uebrigen Text-Zeilen:
      dNZ     ("----------------");
      dIntZ   ("width  (pref.): ",  WIDTH_PREF); // drucke eine int-Zeile
      dIntZ   ("height (pref.): ", HEIGHT_PREF); // drucke eine int-Zeile
      dIntZ   ("width  (max):   ",  WIDTH_MAX);  // ... etc.
      dIntZ   ("height (max):   ", HEIGHT_MAX);
      dIntZ   ("width  (min):   ",  WIDTH_MIN);
      dIntZ   ("height (min):   ", HEIGHT_MIN);
      dIntZ   ("width  (akt):   ",  WIDTH_AKT);
      dIntZ   ("height (akt):   ", HEIGHT_AKT);
      dNZ     ("----------------");
      dIntZ   ("anchor:         ", ANCHOR);
      dIntZ   ("fill:           ", FILL);
      dIntZ   ("gridx:          ", GRIDX);
      dIntZ   ("gridy:          ", GRIDY);
      dIntZ   ("gridwidth:      ", GRIDWIDTH);
      dIntZ   ("gridheight:     ", GRIDHEIGHT);
      dIntZ   ("ipadx:          ", IPADX);
      dIntZ   ("ipady:          ", IPADX);
      dDoubleZ("weightx:        ", WEIGHTX);    // drucke eine double-Zeile
      dDoubleZ("weighty:        ", WEIGHTY);    // durcke eine double-Zeile
      dIntZ   ("insets (top):   ", INSETS_TOP); // drucke eine    int-Zeile
      dIntZ   ("insets (bottom):", INSETS_BOTTOM);
      dIntZ   ("insets (left):  ", INSETS_LEFT);
      dIntZ   ("insets (right): ", INSETS_RIGHT);
      dNZ     ("----------------");

      // Das Drucken des kontrollierten Rahmens "bringt das Graphics2D-
      // Objekt g2 voellig durcheinander". Deshalb muss dieser Rahmen
      // *nach* den Textzeilen gedruckt werden, obwohl er auf dem Papier
      // oberhalb von den Textzeilen erscheint:
      derMitGBL.paint(g2); // Bild des kontrollierten Rahmens drucken

      return Printable.PAGE_EXISTS;
   } // print
   // ---------------------------------------------------------------------
   void dIntZ(String s, int elemNr) {
      // Druckt eine Zeile von int-Werten (pro Knopf einen Wert).
      // s sollte "ein erklaerendes Wort" fuer den Anfang der Zeile sein:
      StringBuffer sb = new StringBuffer(s + " ");
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         String einWert = "" + intWert(kNr, elemNr); // int nach String
         sb.append(pad(einWert, 8)); // Fuer jeden Wert 8 Zeichen
      }
      dNZ(sb.toString()); // drucke naechste Zeile
   } // dIntZ
   // ---------------------------------------------------------------------
   void dDoubleZ(String s, int elemNr) {
      // Druckte eine Zeile von double-Werten (pro Knopf einen Wert).
      // s sollte "ein erklaerendes Wort" fuer den Anfang der Zeile sein.
      StringBuffer sb = new StringBuffer(s + " ");
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {
         String einWert = "" +doubleWert(kNr, elemNr); // double nach Str.
         sb.append(pad(einWert, 8)); // Fuer jeden Wert 8 Zeichen
      }
      dNZ(sb.toString()); // drucke naechste Zeile
   } // dDoubleZ
   // ---------------------------------------------------------------------
   String pad(String s, int len) {
      // "Polstert" den String s links mit Blanks, bis er die Lanege len
      // hat. Vorsicht: Bei unvorsichtiger Anwendung dieser Methode gehen
      // signifikante Stellen "verloren"!
      String bs = "          " + s; // 10 Blanks + s
      return bs.substring(bs.length()-len); // Die letzten len Zeichen
   } // pad
   // ---------------------------------------------------------------------
   int intWert(int kNr, int elemNr) {
      // Holt aus der Reihung zElemR den int-Wert des Zustands-Elements
      // mit der Nummer elemNr fuer den Knopf mit der Nummer kNr
      return ((IntElem) zElemR[kNr][elemNr]).wert;
   }
   // ---------------------------------------------------------------------
   double doubleWert(int kNr, int elemNr) {
      // Holt aus der Reihung zElemR den douboe-Wert des Zustands-Elements
      // mit der Nummer elemNr fuer den Knopf mit der Nummer kNr
      return ((DoubleElem) zElemR[kNr][elemNr]).wert;
   }
   // ---------------------------------------------------------------------
   // Globale Variablen fuer die Methode dNZ ("drucke naechste Zeile").
   // Muessen vor dem ersten Aufruf von dNZ initialisiert werden!
   float      druckX; // naechster Zeilenanfang, x-Koordinate
   float      druckY; // naechster Zeilenanfang, y-Koordinate
   float      deltaY; // Abstand zwischen zwei Zeilen (in y-Richtung)
   Graphics2D g;      // Global, um einen "trivialen Parameter zu sparen".
   // ---------------------------------------------------------------------
   void dNZ(String s) {
      // Druckt die naechste Zeile. Diese Methode verlaesst sich darauf,
      // dass die globalen Variablen druckX, druckY und deltaY vor ihrem
      // ersten Aufruf richtig initialisiert wurden und dann nur von
      // ihr selbst verandert werden.
      g.drawString(s, druckX, druckY); // Eine Zeile drucken
      druckY += deltaY;                // Zur naechsten Zeile vorruecken
   } // dNZ
   // ---------------------------------------------------------------------
   // Ende der Unterprogramme, die mit dem Drucken zu tun haben
   // ---------------------------------------------------------------------
   // Ein Konstruktor mit Titel:                         <----- Konstruktor
   public Layout09(String titel) {
      super(titel);

      // Wenn der Benutzer auf den window closing button eines Layout09-
      // Rahmens klickt, wird "das zugehoerige Programm" beendet.
      this.addWindowListener(new WindowAdapter() {
         public void windowClosing(WindowEvent we) {
            System.exit(0);       // Programm sang- und klanglos beenden!
         }
         //public void windowActivated(WindowEvent we) {
         //   aktualisiereEinElementAllerKnoepfe( WIDTH_AKT);
         //   aktualisiereEinElementAllerKnoepfe(HEIGHT_AKT);
         //} // windowOpened
      });

      derMitGBL.addWindowListener(new WindowAdapter() {
         public void windowActivated(WindowEvent we) {
            aktualisiereEinElementAllerKnoepfe( WIDTH_AKT);
            aktualisiereEinElementAllerKnoepfe(HEIGHT_AKT);
         } // windowOpened
         public void windowDeactivated(WindowEvent we) {
            aktualisiereEinElementAllerKnoepfe( WIDTH_AKT);
            aktualisiereEinElementAllerKnoepfe(HEIGHT_AKT);
         } // windowOpened
      });

      // Die Knoepfe fuer den kontrollierten Rahmen und andere "pro Knopf"-
      // Groessen (JPanel- und Box-Objekte) erzeugen:
      for (int kNr=0; kNr<ANZ_KNOEPFE; kNr++) {        // fuer jeden Knopf
         buttonR[kNr] = new JButton("K" + kNr);
         box1R  [kNr] = Box.createVerticalBox();
         box2R  [kNr] = Box.createVerticalBox();
         panelR [kNr] = new JPanel();
         panelR [kNr].add(box1R[kNr]);
         panelR [kNr].add(box2R[kNr]);

         tabbedPane.addTab("K" + (kNr+1), panelR[kNr]);

         // Die Zustands-Elemente eines Knopfes "richtig einstellen",
         // in ihre Box-Objekte tun, initialisieren etc.:
         for (int elemNr=0; elemNr<ANZ_ELEM; elemNr++) { // fuer j. Element
            int align = JTextField.RIGHT; // Rechtsbuendig
            zElemR[kNr][elemNr].textField.setHorizontalAlignment(align);
            zElemR[kNr][elemNr].textField.addActionListener(machtAlles);
            zElemR[kNr][elemNr].textField.setPreferredSize         (dim1);
            zElemR[kNr][elemNr].beSchriftungsLabel.setPreferredSize(dim1);
            zElemR[kNr][elemNr].beSchriftungsLabel.setMaximumSize  (dim1);
            zElemR[kNr][elemNr].beSchriftungsLabel.setMinimumSize  (dim1);
            zElemR[kNr][elemNr].wertToTextField();
            box1R [kNr].add(zElemR[kNr][elemNr].beSchriftungsLabel);
            box2R [kNr].add(zElemR[kNr][elemNr].textField);
            if (elemNr == 1 || elemNr == 9) {     // Optische Trenner
               box1R[kNr].add(new JLabel("   ")); // nach "Aktiv?" und
               box2R[kNr].add(new JLabel("   ")); // vor  "anchor"
            }
         } // for (fuer jeden Knopf)

         // Die Text-Felder fuer die tatsaechliche Breite und Hoehe der
         // Knoepfe darf der Benutzer nur lessen, nicht veraendern:
         zElemR[kNr][ WIDTH_AKT].textField.setEditable(false);
         zElemR[kNr][HEIGHT_AKT].textField.setEditable(false);

      } // for (fuer jedes Element)

      aktualisiereAlleElementeAllerKnoepfe();
      knoepfeNeuArrangieren();

      // Die MenuItem-Objekte mit ActionListener-Objekten versehen:
      menuItem11.addActionListener(drucken);

      menuItem21.addActionListener(waageRecht);
      menuItem22.addActionListener(senkRecht);
      menuItem23.addActionListener(diaGonal1);
      menuItem24.addActionListener(diaGonal2);
      menuItem25.addActionListener(kreuz);
      menuItem26.addActionListener(relativeX);
      menuItem27.addActionListener(relativeY);

      menuItem31.addActionListener(alleNone);
      menuItem32.addActionListener(alleBoth);
      menuItem33.addActionListener(alleHorizontal);
      menuItem34.addActionListener(alleVertical);

      menuItem41.addActionListener(alleCenter);
      menuItem42.addActionListener(alleNorth);

      menuItem51.addActionListener(alleWeight0);
      menuItem52.addActionListener(alleWeight1);
      menuItem53.addActionListener(alleWeightHalb);

      // Der kontrollierende Rahmen wird bereit gemacht:
      // Die Menuepunkte in die Menues:
      menu1.add(menuItem11);

      menu2.add(menuItem21); menu2.add(menuItem22);
      menu2.add(menuItem23); menu2.add(menuItem24);
      menu2.add(menuItem25); menu2.add(menuItem26);
      menu2.add(menuItem27);

      menu3.add(menuItem31); menu3.add(menuItem32);
      menu3.add(menuItem33); menu3.add(menuItem34);

      menu4.add(menuItem41); menu4.add(menuItem42);

      menu5.add(menuItem51); menu5.add(menuItem52);
      menu5.add(menuItem53);

      // Die Menues in die Menueleiste:
      menuBar.add(menu1);    menuBar.add(menu2);
      menuBar.add(menu3);    menuBar.add(menu4);
      menuBar.add(menu5);

      // Die Menueleiste in den kontrollierenden Rahmen:
      this.setJMenuBar(menuBar);

      tabbedPane.addChangeListener(charly);
      tabbedPane.setSelectedIndex(aktuellerKnopfIndex); // zur Sicherheit
      thisCP.add(tabbedPane);
      this.pack();
      this.setVisible(true);

      // Der kontrollierte Rahmen (der mit GridBagLayout und den 5
      // Knoepfen) wird bereit gemacht:
      gblCP.setLayout(meinGBL);              // mein GridBagLayout!
      derMitGBL.setBounds(300, 0, 450, 250); // nicht zu gross (drucken!)
      derMitGBL.setVisible(true);
   } // Konstruktor Layout09
   // ---------------------------------------------------------------------
   static public void main(String[] args) {
      // Erzeugt ein Objekt der Klasse Layout09 (zum Testen):
      System.out.println("Layout09: Jetzt geht es los!");
      Layout09 rahmen1 = new Layout09("Kontrollierender Rahmen");
      System.out.println("Layout09: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class Layout