// Datei Methoden05.java
/* ------------------------------------------------------------------------
Dieses Programm enthaelt Prozeduren, die verschiedene geometrische Figuren
(Rechtecke, Dreiecke, Rauten) ausgeben. In der main-Methode werden die
Prozeduren mehrfach aufgerufen.

Die Figuren bestehen aus schwarzen und weissen "Einheiten", d.h. aus
Paaren von Sternchen "**" und Paaren von Punkten "..". Eine solche Einheit
erscheint auf einem Bildschirm ungefaehr als Quadrat, wohingegen ein
einzelnes Zeichen nur ein laengliches Rechteck belegt. Quadrate sind
bessere "Grundeinheiten" fuer Figuren als laengliche Rechtecke.

Dies ist eine Variante ("Weiterentwicklung") des Programms Methoden04. Die
drucke-Prozeduren haben hier einen zusaetzlichen int-Parameter abstand
und die Figuren werden mit einem entsprechenden Abstand vom linken Rand
gezeichnet. Die zusaetzliche Prozedur druckeSpitzeRaute gibt Rauten aus,
die aus ungerade vielen Zeilen bestehen (und somit links und rechts
"spitzer" sind als die Ausgaben der alten Prozedur druckeRaute).
------------------------------------------------------------------------ */
class Methoden05 {
   // ---------------------------------------------------------------------
   static final String SCHWARZ = "**************************************";
   static final String WEISS   = "......................................";
   // ---------------------------------------------------------------------
   static String schwarzeEinheiten(int anzahl) {
      // Liefert einen String, der aus anzahl vielen schwarzen Einheiten
      // besteht. Eine schwarze Einheit besteht aus 2 Zeichen des Strings
      // SCHWARZ:
      return SCHWARZ.substring(0, 2*anzahl);
   } // schwarzeEinheiten

   static String weisseEinheiten(int anzahl) {
      // Liefert einen String, der aus anzahl vielen weissen Einheiten
      // besteht. Eine weisse Einheit besteht aus 2 Zeichen des Strings
      // WEISS:
      return WEISS.substring(0, 2*anzahl);
   } // weisseEinheiten
   // ---------------------------------------------------------------------
   static void druckeRechteck(int abstand, int breite, int hoehe) {
      // Gibt ein Rechteck aus schwarzen Einheiten aus. Das Rechteck
      // besteht aus hoehe vielen Zeilen mit je breite vielen schwarzen
      // Einheiten darauf und beginnt abstand viele Einheiten vom linken
      // Rand.

      for (int zeile=1; zeile<=hoehe; zeile++) {
         pln(weisseEinheiten(abstand) + schwarzeEinheiten(breite));
      } // for zeile

   } // druckeRechteck
   // ---------------------------------------------------------------------
   static void druckeDreieckA(int abstand, int hoehe) {
      // Gibt ein Dreieck aus, welches aus hoehe vielen Zeilen besteht.
      // Die erste Zeile enthaelt eine, die letzte Zeile 2*hoehe-1 viele
      // schwarze Einheiten, d.h. die Spitze des Dreiecks zeigt nach oben,
      // aehnlich wie bei einem A. Das Dreieck beginnt abstand viele Ein-
      // heiten vom linken Rand.

      int anzWE = hoehe-1; // Anz. weisse   Einheiten in der 1. Zeile
      int anzSE = 1;       // Anz. schwarze Einheiten in der 1. Zeile

      for (int zeile=1; zeile<=hoehe; zeile++) {
         p(  weisseEinheiten(abstand)); // Abstand vom linken Rand
         p(  weisseEinheiten(anzWE));   // Vorne    weisse   Einheiten
         p(schwarzeEinheiten(anzSE));   // Dahinter schwarze Einheiten
         pln();                         // Zeilenwechsel

         anzWE -= 1;
         anzSE += 2;
      } // for zeile

   } // druckeDreieckA
   // ---------------------------------------------------------------------
   static void druckeDreieckV(int abstand, int hoehe) {
      // Gibt ein Dreieck aus, welches aus hoehe vielen Zeilen besteht.
      // Die erste Zeile enthaelt 2*hoehe-1 viele, die letzte Zeile eine
      // schwarze Einheiten, d.h. die Spitze des Dreiecks zeigt nach unten,
      // aehnlich wie bei einem V. Das Dreieck beginnt abstand viele Ein-
      // heiten vom linken Rand.

      int anzWE = 0;         // Anz. weisse   Einheiten in der 1. Zeile
      int anzSE = 2*hoehe-1; // Anz. schwarze Einheiten in der 1. Zeile

      for (int zeile=1; zeile<=hoehe; zeile++) {
         p(  weisseEinheiten(abstand)); // Abstand vom linken Rand
         p(  weisseEinheiten(anzWE));   // Vorne    weisse   Einheiten
         p(schwarzeEinheiten(anzSE));   // Dahinter schwarze Einheiten
         pln();                         // Zeilenwechsel

         anzWE += 1;
         anzSE -= 2;
      } // for zeile

   } // druckeDreieckV
   // ---------------------------------------------------------------------
   static void druckeRaute(int abstand, int hoehe) {
      // Gibt eine Raute aus, die aus hoehe vielen Zeilen besteht (genauer:
      // aus 2*(hoehe/2) vielen Zeilen. Das ist eine gerade Zeilenzahl,
      // selbst wenn hoehe ungerade ist). Mit Raute ist hier "eine Art
      // Quadrat, das auf einer Spitze steht" gemeint.

      druckeDreieckA(abstand, hoehe/2);
      druckeDreieckV(abstand, hoehe/2);
   } // druckeRaute
   // ---------------------------------------------------------------------
   static void druckeSpitzeRaute(int abstand, int hoehe) {
      // Gibt eine Raute aus, die aus hoehe vielen Zeilen besteht (genauer:
      // aus 2*(hoehe/2)+1 vielen Zeilen. Das ist eine ungerade Zeilenzahl).
      // Mit Raute ist hier "eine Art Quadrat, das auf einer Spitze steht"
      // gemeint.

      druckeDreieckA(abstand,     hoehe/2 + 1);
      druckeDreieckV(abstand + 1, hoehe/2);
   } // druckeSpitzeRaute
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Gibt eine paar Rechtecke, Dreiecke und Rauten aus:

      pln("Rechteck 7x3:"); druckeRechteck   (12, 7, 3); pln();
      pln("Rechteck 5x5:"); druckeRechteck   (13, 5, 5); pln();
      pln("DreieckA 6:");   druckeDreieckA   (10, 6);    pln();
      pln("DreieckA 3:");   druckeDreieckA   (13, 3);    pln();
      pln("DreieckV 4:");   druckeDreieckV   (12, 4);    pln();
      pln("DreieckV 5:");   druckeDreieckV   (11, 5);    pln();

      pln("Raute 8:");      druckeRaute      (12, 8);    pln();
      pln("Raute 4:");      druckeRaute      (14, 4);    pln();
      pln("Raute 5:");      druckeRaute      (14, 5);    pln();

      pln("RauteB 8:");     druckeSpitzeRaute(11, 8);    pln();
      pln("RauteB 4:");     druckeSpitzeRaute(13, 4);    pln();
      pln("RauteB 5:");     druckeSpitzeRaute(13, 5);    pln();

      pln("------------------------------");
      pln("Methoden05: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Methoden05
/* ------------------------------------------------------------------------
Ausgabe des Programms Methoden05:

Rechteck 7x3:
........................**************
........................**************
........................**************

Rechteck 5x5:
..........................**********
..........................**********
..........................**********
..........................**********
..........................**********

DreieckA 6:
..............................**
............................******
..........................**********
........................**************
......................******************
....................**********************

DreieckA 3:
..............................**
............................******
..........................**********

DreieckV 4:
........................**************
..........................**********
............................******
..............................**

DreieckV 5:
......................******************
........................**************
..........................**********
............................******
..............................**

Raute 8:
..............................**
............................******
..........................**********
........................**************
........................**************
..........................**********
............................******
..............................**

Raute 4:
..............................**
............................******
............................******
..............................**

Raute 5:
..............................**
............................******
............................******
..............................**

RauteB 8:
..............................**
............................******
..........................**********
........................**************
......................******************
........................**************
..........................**********
............................******
..............................**

RauteB 4:
..............................**
............................******
..........................**********
............................******
..............................**

RauteB 5:
..............................**
............................******
..........................**********
............................******
..............................**

------------------------------
Methoden05: Das war's erstmal!
------------------------------------------------------------------------ */
