// Datei Methoden06.java
/* ------------------------------------------------------------------------
Dieses Programm enthaelt Prozeduren, die verschiedene geometrische Figuren
(Rechtecke, Dreiecke, Rauten) erzeugen und als Reihungen des Typs String[]
liefern. In der main-Methode werden die Pozeduren mehrfach aufgerufen und
ihre Ergebnisse werden ausgegeben.

Die Figuren bestehen aus schwarzen und weissen "Einheiten", d.h. aus
Paaren von Sternchen "**" und Paaren von Punkten "..". Eine solche Einheit
erscheint auf einem Bildschirm ungefaehr als Quadrat, wohingegen ein
einzelnes Zeichen nur ein laengliches Rechteck belegt. Quadrate sind
bessere "Grundeinheiten" fuer Figuren als laengliche Rechtecke.

Dies ist eine Variante ("Weiterentwicklung") des Programms Methoden05.
Anstelle der drucke-Prozeduren (die eine Figur gleich ausgeben) gibt
es hier liefere-Methoden, die eine Figur in Form einer Reihung des
Typs String[] liefern.
------------------------------------------------------------------------ */
import java.util.Arrays;    // fuer asList
import java.util.ArrayList;

class Methoden06 {
   // ---------------------------------------------------------------------
   static final String SCHWARZ = "**************************************";
   static final String WEISS   = "......................................";
   // ---------------------------------------------------------------------
   static String schwarzeEinheiten(int anzahl) {
      // Liefert einen String, der aus anzahl vielen schwarzen Einheiten
      // besteht. Eine schwarze Einheit besteht aus 2 Zeichen des Strings
      // SCHWARZ:
      return SCHWARZ.substring(0, 2*anzahl);
   } // schwarzeEinheiten

   static String weisseEinheiten(int anzahl) {
      // Liefert einen String, der aus anzahl vielen weissen Einheiten
      // besteht. Eine weisse Einheit besteht aus 2 Zeichen des Strings
      // WEISS:
      return WEISS.substring(0, 2*anzahl);
   } // weisseEinheiten
   // ---------------------------------------------------------------------
   static String[] liefereRechteck(int abstand, int breite, int hoehe) {
      // Liefert eine Reihung von Strings, die zusammen ein Rechteck
      // aus schwarzen Einheiten enthalten. Das Rechteck besteht aus
      // hoehe vielen Zeilen mit je breite vielen schwarzen Einheiten
      // darauf und beginnt abstand viele Einheiten vom linken Rand.

      String[] erg = new String[hoehe];
      for (int zeile=0; zeile<hoehe; zeile++) {
         erg[zeile] = weisseEinheiten(abstand) + schwarzeEinheiten(breite);
      } // for zeile

      return erg;
   } // liefereRechteck
   // ---------------------------------------------------------------------
   static String[] liefereDreieckA(int abstand, int hoehe) {
      // Liefert eine Reihung von Strings, die zusammen ein Dreieck
      // aus schwarzen Einheiten enthalten. Das Dreieck besteht aus
      // hoehe vielen Zeilen. Die erste Zeile enthaelt eine, die letzte
      // Zeile 2*hoehe-1 viele schwarze Einheiten, d.h. die Spitze des
      // Dreiecks zeigt nach oben, aehnlich wie bei einem A. Das Dreieck
      // beginnt abstand viele Einheiten vom linken Rand.

      String[] erg = new String[hoehe];

      int anzWE = hoehe-1; // Anz. weisse   Einheiten in der 1. Zeile
      int anzSE = 1;       // Anz. schwarze Einheiten in der 1. Zeile

      for (int zeile=0; zeile<hoehe; zeile++) {
         erg[zeile] =   weisseEinheiten(abstand + anzWE) +
                      schwarzeEinheiten(          anzSE);
         anzWE -= 1;
         anzSE += 2;
      } // for zeile

      return erg;
   } // liefereDreieckA
   // ---------------------------------------------------------------------
   static String[] liefereDreieckV(int abstand, int hoehe) {
      // Liefert eine Reihung von Strings, die zusammen ein Dreieck
      // aus schwarzen Einheiten enthalten. Das Dreieck besteht aus
      // hoehe vielen Zeilen. Die erste Zeile enthaelt 2*hoehe-1, die
      // letzte Zeile eine schwarze Einheit(en), d.h. die Spitze des
      // Dreiecks zeigt nach unten, aehnlich wie bei einem V. Das Dreieck
      // beginnt abstand viele Einheiten vom linken Rand.

      String[] erg = new String[hoehe];

      int anzWE = 0;         // Anz. weisse   Einheiten in der 1. Zeile
      int anzSE = 2*hoehe-1; // Anz. schwarze Einheiten in der 1. Zeile

      for (int zeile=0; zeile<hoehe; zeile++) {
         erg[zeile] =   weisseEinheiten(abstand + anzWE) +
                      schwarzeEinheiten(          anzSE);
         anzWE += 1;
         anzSE -= 2;
      } // for zeile

      return erg;
   } // liefereDreieckV
   // ---------------------------------------------------------------------
   static String[] liefereRaute(int abstand, int hoehe) {
      // Liefert eine Reihung von Strings, die zusammen eine Raute
      // enthalten. Die Raute besteht aus hoehe vielen Zeilen  (genauer:
      // aus 2*(hoehe/2) vielen Zeilen. Das ist eine gerade Zeilenzahl,
      // auch wenn hoehe ungerade ist). Mit Raute ist hier "eine Art
      // Quadrat, das auf einer Spitze steht" gemeint.

      hoehe          = 2*(hoehe/2);
      String[]  erg  = new String[hoehe];

      // Die obere und die untere Haelfte der Raute werden berechnet:
      String[]  oben  = liefereDreieckA(abstand, hoehe/2);
      String[]  unten = liefereDreieckV(abstand, hoehe/2);

      // Die Reihungen oben und unten werden in ArrayList-Objekte
      // umgewandelt:
      ArrayList<String> alOben  = new ArrayList<String>(Arrays.asList(oben));
      ArrayList<String> alUnten = new ArrayList<String>(Arrays.asList(unten));

      // Alle Komponenten von alUnten werden hinten an alOben angehaengt:
      alOben.addAll(alUnten);

      // Das ArrayList-Objekt alOben wird zurueck in eine Reihung umgewandelt,
      // in der Variablen erg gespeicher und als Ergebnis geliefert:
      return (String[]) alOben.toArray(erg);
   } // liefereRaute
   // ---------------------------------------------------------------------
   static String[] liefereSpitzeRauteB(int abstand, int hoehe) {
      // Liefert eine Reihung von Strings, die zusammen eine Raute
      // enthalten. Die Raute besteht aus hoehe vielen Zeilen  (genauer:
      // aus 2*(hoehe/2)+1 vielen Zeilen. Das ist eine ungerade Zeilenzahl,
      // auch wenn hoehe gerade ist). Mit Raute ist hier "eine Art
      // Quadrat, das auf einer Spitze steht" gemeint.

      String[]  erg  = new String[hoehe];

      // Die obere und die untere Haelfte der Raute werden berechnet:
      String[]  oben  = liefereDreieckA(abstand,     hoehe/2 + 1);
      String[]  unten = liefereDreieckV(abstand + 1, hoehe/2);

      // Die Reihungen oben und unten werden in ArrayList-Objekte
      // umgewandelt:
      ArrayList<String> alOben  = new ArrayList<String>(Arrays.asList(oben));
      ArrayList<String> alUnten = new ArrayList<String>(Arrays.asList(unten));

      // Alle Komponenten von alUnten werden hinten an alOben angehaengt:
      alOben.addAll(alUnten);

      // Das ArrayList-Objekt alOben wird zurueck in eine Reihung umgewandelt,
      // in der Variablen erg gespeicher und als Ergebnis geliefert:
      return (String[]) alOben.toArray(erg);
   } // liefereSpitzeRauteB
   // ---------------------------------------------------------------------
   static void druckeStringReihung(String[] sr) {
      // Gibt die Reihung sr aus (jede Komponente auf eine Zeile, am
      // Schluss eine zusaetzliche Leerzeile):
      for (int zeile = 0; zeile<sr.length; zeile++) {
         pln(sr[zeile]);
      }
      pln();
   } // druckeStringReihung
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Gibt eine paar Rechtecke, Dreiecke und Rauten aus:

      pln("Rechteck 7x3:"); druckeStringReihung(liefereRechteck (12, 7, 3));
      pln("Rechteck 5x5:"); druckeStringReihung(liefereRechteck (13, 5, 5));
      pln("DreieckA 6:");   druckeStringReihung(liefereDreieckA    (10, 6));
      pln("DreieckA 3:");   druckeStringReihung(liefereDreieckA    (13, 3));
      pln("DreieckV 4:");   druckeStringReihung(liefereDreieckV    (12, 4));
      pln("DreieckV 5:");   druckeStringReihung(liefereDreieckV    (11, 5));

      pln("Raute 8:");      druckeStringReihung(liefereRaute       (12, 8));
      pln("Raute 4:");      druckeStringReihung(liefereRaute       (14, 4));
      pln("Raute 5:");      druckeStringReihung(liefereRaute       (14, 5));

      pln("RauteB 8:");     druckeStringReihung(liefereSpitzeRauteB(11, 8));
      pln("RauteB 4:");     druckeStringReihung(liefereSpitzeRauteB(13, 4));
      pln("RauteB 5:");     druckeStringReihung(liefereSpitzeRauteB(13, 5));

      pln("------------------------------");
      pln("Methoden06: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Methoden06
/* ------------------------------------------------------------------------
Ausgabe des Programms Methoden06:

Rechteck 7x3:
........................**************
........................**************
........................**************

Rechteck 5x5:
..........................**********
..........................**********
..........................**********
..........................**********
..........................**********

DreieckA 6:
..............................**
............................******
..........................**********
........................**************
......................******************
....................**********************

DreieckA 3:
..............................**
............................******
..........................**********

DreieckV 4:
........................**************
..........................**********
............................******
..............................**

DreieckV 5:
......................******************
........................**************
..........................**********
............................******
..............................**

Raute 8:
..............................**
............................******
..........................**********
........................**************
........................**************
..........................**********
............................******
..............................**

Raute 4:
..............................**
............................******
............................******
..............................**

Raute 5:
..............................**
............................******
............................******
..............................**

RauteB 8:
..............................**
............................******
..........................**********
........................**************
......................******************
........................**************
..........................**********
............................******
..............................**

RauteB 4:
..............................**
............................******
..........................**********
............................******
..............................**

RauteB 5:
..............................**
............................******
..........................**********
............................******
..............................**

------------------------------
Methoden06: Das war's erstmal!
------------------------------------------------------------------------ */
