// Datei ObjektFabrik.java
/* ------------------------------------------------------------------------
Die Objekt-Methode erzeugeObjekt hat einen Parameter vom Typ Class und
liefert als Ergebnis ein Objekt dieser Klasse. Die dazu benoetigten Daten
(Parameter fuer Konstruktoren) werden mit Hilfe eines Zufallsgenerators
(eines Objekts der Klasse Random) erzeugt.

Das Erzeugen eines Objekts ob einer Klasse c mit einem Konstruktor k kann
aus verschiedenen Gruenden misslingen, vor allem aus den folgenden:
1. Das Java-Sicherheitssystem erlaubt diesem Programm nicht, den
   Konstruktor k aufzurufen (z.B. weil k private ist).
2. Der Konstruktor k wirft eine Ausnahme (weil er mit ungeeigneten
   Parametern aufgerufen wurde, z.B. mit negativen int-Werte wo
   nur positive int-Werte erlaubt sind)
3. Die Erzeugung der Parameter, die der Konstruktor k erwartet,
   misslingt.

Wenn das Erzeugen eines Objekts misslingt, liefert die Methode
erzeugeObjekt null als Ergebnis.
---------------------------------------------------------------------------
Die Methode erzeugeObjekt ist rekursiv: Um ein Objekt ob1 einer Klasse c1
zu erzeugen, werden eventuell Objekte ob11, ob12, ob13... als Parameter
fuer einen Konstruktor benoetigt und um ob11 zu erzeugen werden eventuell
Objekte ob111, ob112, ob113, ... benoetigt etc..

Die 8 primitiven Typen (byte, char, short, int, long, float, double und
boolean), die 8 Wickelklassen (Byte, Character, Short, Integer, Long,
Float, Double, Boolean) und die Klasse String werden hier als *einfache
Typen* bezeichnet. Objekte dieser einfachen Typen werden "direkt" (*ohne
rekursive Aufrufe* der Methode erzeugeObjekt) erzeugt.
------------------------------------------------------------------------ */
import java.lang.reflect.Constructor;
import java.lang.reflect.Array;
import java.util.List;
import java.util.ArrayList;
import java.util.Random;

public class ObjektFabrik {
   // ---------------------------------------------------------------------
   // Die wichtigste Methode (und fast die einzige public-Methode) dieser
   // Klasse:
   public Object erzeugeObjekt(Class c) throws Exception {
      // Liefert ein Objekt der Klasse c (welches mit Hilfe eines Zufalls-
      // generators erzeugt wird) oder null (wenn das Erzeugen misslingt).

      if (TEST01) {
         pln("----------------------------------------------------------");
         pln("A --> erzeugeObjekt(" + c.getName() + ")");
      }

      List<Object> verboteneTypen = new ArrayList<Object>(); // Anfangs ist kein Typ verboten
      return erzeugeObjekt(c, verboteneTypen);
   } // erzeugeObjekt
   // ---------------------------------------------------------------------
   public Object erzeugeObjekt(Class c, List<Object> verboteneTypen)
      throws Exception
   {
      // Liefert ein Objekt der Klasse c oder null (wenn das Erzeugen des
      // Objekts misslingt).
      // Der Parameter verboteneTypen dient dazu, Endlosrekursionen zu
      // verhindern (welche sonst auftreten wuerden, wenn z.B. versucht
      // wird, ein Objekt eines Typs c1 mit Hilfe eines Konstruktors zu
      // erzeugen, der einen Parameter des Typs c1 erwartet).

      if (TEST01) {
         p("B --> erzeugeObjekt(" + c.getName() + ")");
         pln(", Verboten: " + verboteneTypen);
      }

      // Jetzt werden drei Faelle unterschieden:
      if (istEinfach(c)) return erzeugeEinfachesObjekt(c);
      if (c.isArray () ) return erzeugeReihungsObjekt (c, verboteneTypen);
                         return erzeugeSonstigesObjekt(c, verboteneTypen);
   } // erzeugeObjekt
   // ---------------------------------------------------------------------
   // Ein Zufallsgenerator (zum Erzeugen von Daten verschiedener Typen):
   private Random rand = new Random(123);

   // Zum Testen:
   private final boolean TEST01 = false;
   private final boolean TEST02 = false;
   // ---------------------------------------------------------------------
   // Das List-Objekt einfacherTyp enthaelt die Class-Objekte aller
   // einfachen Typen. Das Objekt erleichtert den Test, ob ein Class-Objekt
   // einen einfachen Typ repraesentiert und macht switch-Anweisungen
   // ueber den Index eines Class-Objekts moeglich:

   static private final List<Object> einfacherTyp = new ArrayList<Object>();

   static { // statischer Initialisierer fuer die Reihung einfacherTyp:
      einfacherTyp.add(     Byte.class); einfacherTyp.add(     Byte.TYPE);
      einfacherTyp.add(Character.class); einfacherTyp.add(Character.TYPE);
      einfacherTyp.add(    Short.class); einfacherTyp.add(    Short.TYPE);
      einfacherTyp.add(  Integer.class); einfacherTyp.add(  Integer.TYPE);
      einfacherTyp.add(     Long.class); einfacherTyp.add(     Long.TYPE);
      einfacherTyp.add(    Float.class); einfacherTyp.add(    Float.TYPE);
      einfacherTyp.add(   Double.class); einfacherTyp.add(   Double.TYPE);
      einfacherTyp.add(  Boolean.class); einfacherTyp.add(  Boolean.TYPE);
      einfacherTyp.add(   String.class);
   } // statischer Initialisierer

   private boolean istEinfach(Class c) {
      // Liefert true, wenn c einen einfachen Typ repraesentiert:
      return einfacherTyp.contains(c);
   } // istEinfach
   // ---------------------------------------------------------------------
   private Object erzeugeEinfachesObjekt(Class c) {
      // Verlaesst sich darauf, dass c einen einfachen Typ repraesentiert.
      // Liefert ein Objekt dieses Typs bzw. (falls c primitiv ist) ein
      // Objekt der entsprechenden Wickelklasse:

      // Das Class-Objekt c wird in einen int-Wert (Index fuer das
      // List-Objekt einfacherTyp) uebersetzt, damit eine switch-Anweisung
      // moeglich ist:
      int indexDesEinfachenTyps = einfacherTyp.indexOf(c);

                                                   // c repraesentiert einen
      switch (indexDesEinfachenTyps) {             // der folgenden Typen:
         case  0: case  1: return randByte();      // byte    oder Byte
         case  2: case  3: return randCharacter(); // char    oder Character
         case  4: case  5: return randShort();     // short   oder Short
         case  6: case  7: return randInteger();   // int     oder Integer
         case  8: case  9: return randLong();      // long    oder Long
         case 10: case 11: return randFloat();     // float   oder Float
         case 12: case 13: return randDouble();    // double  oder Double
         case 14: case 15: return randBoolean();   // boolean oder Boolean
         case 16:          return randString();    // String
         default:          return null;            // Sollte nicht vorkommen
      } // switch
   } // erzeugeEinfachesObjekt
   // ---------------------------------------------------------------------
   private Object erzeugeReihungsObjekt
      (Class reihungsTyp, List<Object> verboteneTypenAlt) throws Exception
   {
      // Verlaesst sich darauf, dass reihungsTyp einen Reihungstyp reprae-
      // sentiert. Erzeugt und liefert ein Objekt dieses Typs bzw. null
      // falls die Erzeugung des Reihungs-Objekts misslingt

      if (TEST02) {
         p  ("C --> erzeugeReihungsObjekt: " + reihungsTyp);
         pln(", Verboten: "                  + verboteneTypenAlt);
      }

      Class komponentenTyp = reihungsTyp.getComponentType();

      if (verboteneTypenAlt.contains(komponentenTyp)) return null;
      List<Object> verboteneTypenNeu = new ArrayList<Object>(verboteneTypenAlt);

      // Die Laenge der Reihung wird per Zufall bestimmt:
      int    laenge = rand.nextInt(10) + 2; // Zwischen 2 und 11
      // Die Ergebnis-Reihung wird erzeugt. Sie wird aber nur dann
      // geliefert, wenn die Erzeugung aller ihrer Komponenten gelingt:
      Object erg    = Array.newInstance(komponentenTyp, laenge);

      // Die Komponenten der Reihung erg erzeugen:
      for (int i=0; i<laenge; i++) {
         Object eineKomponente =
            erzeugeObjekt(komponentenTyp, verboteneTypenNeu);

         // Wenn die Erzeugung einer Komponenten missglueckt ist:
         if (eineKomponente == null) {
            if (TEST01) pln("D --> Erzeug. einer Reih.-Komp. missglueckte!");
            return null; // Erzeugung des Reihungs-Objekts ist missglueckt
         }

         // Eine Komponente wird in die Reihung erg eingesetzt:
         Array.set(erg, i, eineKomponente);
      } // for

      return erg;
   } // erzeugeReihungsObjekt
   // ---------------------------------------------------------------------
   private Object erzeugeSonstigesObjekt
      (Class  sonstigerTyp, List<Object> verboteneTypenAlt) throws Exception
   {
      // Verlaesst sich darauf, dass sonstigerTyp keinen einfachen Typ und
      // keinen Reihungstyp repraesentiert. Erzeugt ein Objekt der Klasse
      // sonstigerTyp und liefert es als Ergebnis bzw. liefert null, falls
      // die Erzeugung des Objekts misslingt. Das List-Objekt
      // verboteneTypenAlt dient dazu, Endlosrekursionen zu verhindern
      // (wenn z.B. zur Erzeugung eines Objekts einer Klasse c ein Objekt
      // der Klasse c benoetigt wird):

      if (TEST02) {
         p  ("E --> erzeugeSonstigesObjekt: " + sonstigerTyp);
         pln(", Verboten: "                   + verboteneTypenAlt);
      }

      // Wenn wir gerade dabei sind, ein Objekt der Klasse sonstigerTyp zu
      // erzeugen, dann versuchen wir nicht, rekursiv ein Objekt des selben
      // Typs zu erzeugen sondern brechen sofort ab:
      if (verboteneTypenAlt.contains(sonstigerTyp)) return null;

      // (Indirekt) rekursive Aufrufe dieser Methode duerfen den Vektor
      // verboteneTypenAlt nicht veraendern. Deshalb darf an rekursive
      // Aufrufe nur eine erweiterte *Kopie* dieses Vektors uebergeben
      // werden:
      List<Object> verboteneTypenNeu = new ArrayList<Object>(verboteneTypenAlt);
      // Ab jetzt ist auch der Typ sonstigerTyp verboten:
      verboteneTypenNeu.add(sonstigerTyp);

      // Wenn moeglich (d.h. wenn nicht von einem SecurityManager-Objekt
      // verboten) *alle* Konstruktoren holen (mit getDeclaredConstructors)
      // sonst nur die public-Konstruktoren (mit getConstructors):
      Constructor[] konstruktor;

      try {
         konstruktor = sonstigerTyp.getDeclaredConstructors();
      } catch (Exception ex) {
         konstruktor = sonstigerTyp.getConstructors();
      }

      if (TEST01) {
         pln("F Reihung von Konstruktoren:");
         pln(Objects.toString(konstruktor));
//       PrintObject.print(konstruktor);
      }

      Object erg;

      // Mit jedem Konstruktor der Klasse sonstigerTyp wird versucht, ein
      // Objekt zu erzeugen. Das erste erfolgreich erzeugte Objekt wird
      // als Ergebnis geliefert.
      for (int i=0; i<konstruktor.length; i++) {
         erg = erzeugeSonstigesObjektMk(konstruktor[i], verboteneTypenNeu);
         if (erg != null) {
            if (TEST01) pln("G --> Obj.Erzeug. gelungen!");
            return erg;
         }

         if (TEST01) {
            pln("H --> Obj.Erzeug misslungen! Typ:" + sonstigerTyp.getName());
         } // if
      } // for i
      return null;
   } // erzeugeSonstigesObjekt
   // ---------------------------------------------------------------------
   private Object erzeugeSonstigesObjektMk
      (Constructor ktor, List<Object> verboteneTypen) throws Exception
   {
      // Versucht, mit dem Konstruktor ktor ein Objekt zu erzeugen.
      // Liefert dieses Objekt bzw. null (falls die Erzeugung misslingt).
      if (TEST01) {
         p("I --> Konstruktor: ");
         pln(Objects.toString(ktor));
//       PrintObject.print(ktor);
      }

      // Jetzt wird versucht, auch private-Konstruktoren aufrufbar zu
      // machen. Falls ein Security-Manager-Objekt das nicht erlaubt und
      // eine Ausnahme wirft, beschraenken wir uns einfach auf die
      // public-Konstruktoren:
      try {
         ktor.setAccessible(true);
      } catch (Exception ex) {
         // O.k., wir duerfen nur public-Konstruktoren aufrufen!
      } // try/catch

      Class[] paramTypen = ktor.getParameterTypes();

      // Falls der Konstruktor ktor einen Parameter eines verbotenen Typs
      // hat, wird null als Ergebnis geliefert
      for (int i=0; i<paramTypen.length; i++) {
         if (verboteneTypen.contains(paramTypen[i])) return null;
      }

      // Versuch, zu jedem Parametertyp in paramTypen ein entsprechendes
      // Parameterobjekt in param zu erzeugen:
      Object[] param = getParameter(paramTypen, verboteneTypen);
      // Falls die Erzeugung der Parameterobjekte missglueckt ist:
      if (param == null) return null;

      // Versuch, mit dem Konstruktor ktor und den Parameterobjekten in
      // param ein Objekt zu erzeugen und zu liefern. Falls die Erzeugung
      // missglueckt wird null geliefert:
      try {
         return ktor.newInstance(param); // Objekterzeugung geglueckt!
      } catch (Exception ex) {
         return null;                    // Objekterzeugung missglueckt!
      } // try/catch
   } // erzeugeSonstigesObjektMk
   // ---------------------------------------------------------------------
   private Object[] getParameter(Class[] paramTypen, List<Object>  verboteneTypen)
      throws Exception
   {
      // Versucht, zu jedem Typ in der Reihung paramTypen ein entsprechen
      // des Objekt in der Reihung param zu erzeugen. Falls das glueckt,
      // wird param und sonst null geliefert. Die Erzeugung der Objekte
      // misslingt u.a. dann, wenn dazu ein Objekt eines verbotenen Typs
      // benoetigt wuerde (damit werden Endlosrekursionen verhindert):

      // Die Reihung param muss natuerlich gleich lang sein wie paramTypen:
      Object[] param = new Object[paramTypen.length];

      // Versuch, alle Parameterobjekte zu erzeugen. Wenn die Erzeugung
      // eines Objekts misslingt wird der Versuch abgebrochen und null
      // als Ergebnis geliefert:
      for (int i=0; i<paramTypen.length; i++) {
         param[i] = erzeugeObjekt(paramTypen[i], verboteneTypen);
         if (param[i] == null) return null;
      }

      if (TEST01) {
         pln("J --> Reihung von Parametern:");
         pln(Objects.toString(param));
//       PrintObject.print(param);
      }

      return param;
   } // getParam
   // ---------------------------------------------------------------------
   private Byte randByte() {
      // Liefert ein Byte-Objekt mit einem zufaelligen Wert.
      byte b = (byte) (rand.nextInt(256) - 127);
      return new Byte(b);
   } // randByte
   // ---------------------------------------------------------------------
   private Character randCharacter() {
      // Liefert ein Char-Objekt mit einem zufaelligen Wert (aber einige
      // Char-Werte werden sicherlich nie geliefert).

      // Nur 7-Bit-Ascii-Zeichen (0 bis 127) erlauben:
      char c = (char) rand.nextInt(128);
      // Steuerzeichen und Blank (0 bis 32) ausschliessen
      if (c < 33) c += 33;
      return new Character(c);
   } // randCharacter
   // ---------------------------------------------------------------------
   private Short randShort() {
      // Liefert ein Short-Objekt mit einem zufaelligen Wert zwischen
      // 0 und 9:
      short s = (short) rand.nextInt(10);
      return new Short(s);
   } // randShort
   // ---------------------------------------------------------------------
   private Integer randInteger() {
      // Liefert ein Integer-Objekt mit einem zufaelligen Wert zwischen
      // 0 und 19:
      return new Integer(rand.nextInt(20));
   } // randInteger
   // ---------------------------------------------------------------------
   private Long randLong() {
      // Liefert ein Long-Objekt mit einem zufaelligen Wert zwischen
      // 0 und 99:
      return new Long(rand.nextInt(100));
   } // randLong
   // ---------------------------------------------------------------------
   private Float randFloat() {
      // Liefert ein Float-Objekt mit einem zufaelligen Wert zwischen
      // 0.0 und 0.999:
      return new Float(rand.nextInt(100) / 100.0);
   } // randFloat
   // ---------------------------------------------------------------------
   private Double randDouble() {
      // Liefert ein Double-Objekt mit einem zufaelligen Wert zwischen
      // 0.0 und 99.999:
      return new Double(rand.nextInt(100*1000) / 1000.0);
   } // randDouble
   // ---------------------------------------------------------------------
   private Boolean randBoolean() {
      // Liefert ein Boolean-Objekt mit einem zufaelligen Wert:
      return new Boolean(rand.nextBoolean());
   } // randBoolean
   // ---------------------------------------------------------------------
   private String randString() {
      // Liefert ein String-Objekt mit einer zufaelligen Laenge zwischen
      // 0 und 29 und zufaelligen Zeichen darin:
      int laenge = rand.nextInt(30); // Laenge zwischen 0 und 29
      if (TEST01) pln("K --> String laenge: " + laenge);

      // Eine char-Reihung (als Parameter fuer einen String-Konstruktor)
      // erzeugen und mit zufaelligen char-Werten fuellen:
      char[] cr = new char[laenge];
      for (int i=0; i<cr.length; i++) cr[i] = randCharacter().charValue();

      return new String(cr);
   } // erzeugeStringObjekt
   // ---------------------------------------------------------------------
   public String toString() {
      // Damit auch Objekte dieser Klasse ObjektFabrik erzeugt und
      // "lesbar" ausgegeben werden koennen:
      return "Ein Objekt der Klasse ObjektFabrik";
   } // toString
   // ---------------------------------------------------------------------
   // Zwei Methoden mit kurzen Namen (pln und p):
   static public void pln(Object ob) {System.out.println(ob);}
   static public void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
   static public void main(String[] susi) {
      pln("ObjektFabrik: Zum Testen dieser Klasse eignet sich z.B. das");
      pln("Programm ObjektFabrikTst!");
   } // main
   // ---------------------------------------------------------------------
} // ObjektFabrik
// ------------------------------------------------------------------------