// Datei Permutationen.java
/* ------------------------------------------------------------------------
Jedes Objekt dieser Klasse ist eine Quelle von Permutationen. Die
Funktion naechste() liefert bei jedem Aufruf die jeweils naechste
Permutation in Form einer Reihung von int-Werten.

Der Konstruktor mit einem int-Parameter ANZAHL bewirkt, dass die int-Werte
1 bis ANZAHL permutiert werden.

Der Konstruktor mit einem int[]-Parameter IR bewirkt, dass die
Komponenten (einer Kopie) dieser Reihung IR permutiert werden.

Weitere Objektmethoden:

anzahlPermutationen()  Liefert die Anzahl der verschiedenen Permutationen,
                       die die Methode naechste() liefert bevor sie "sich
                       wiederholt".
nrDerNaechsten()       Liefert die Nummer der naechsten Permutation
                       (ein int-Wert zwischen 1 und anzahlPermutationen())
toString()             Liefert die naechste Permutation als String.
print(String name)     Gibt den name gefolgt von der nachsten Permutation
                       zur Standardausgabe aus.
------------------------------------------------------------------------ */
class Permutationen {
   // ---------------------------------------------------------------------
   private       int[] permutation;  // Die aktuelle Permutation
   private       int[] tauschMitWem; // Informationen fuer die Berechnung
                                     // der naechsten Permutation
   private final int   MAX_INDEX;    // Fuer permutation und tauschMitWem
   private final int   ANZAHL_ELEM;  // Soviel Elemente werden permutiert
   private       int   lfdNr= 0;     // laufende Nr. der Permutationen
   // ---------------------------------------------------------------------
   // Zwei Konstruktoren:
   public Permutationen(final int[] IR) {
      // Initialisiert dieses Objekt so, dass die naechste-Funktion
      // permutierte Kopien der Reihung IR liefert.
      permutation  = (int[]) IR.clone();
      ANZAHL_ELEM  = permutation.length;
      MAX_INDEX    = ANZAHL_ELEM - 1;

      tauschMitWem = new int[ANZAHL_ELEM];

      // Die Reihung tauschMitWem initialisieren:
      for (int i=0; i<=MAX_INDEX; i++) {
         tauschMitWem[i] = i+1;
      }
   } // Konstruktor Permutationen
   // -------------------------------------------
   public Permutationen(final int ANZAHL) {
      // Initialisiert dieses Objekt so, dass die naechste-Funktion
      // permutierte Kopien einer int-Reihung mit den Werten 1 bis ANZAHL
      // liefert.
      ANZAHL_ELEM  = ANZAHL;
      MAX_INDEX    = ANZAHL_ELEM - 1;

      permutation  = new int[ANZAHL_ELEM];
      tauschMitWem = new int[ANZAHL_ELEM];

      // Die zu permutierenden int-Werte in die Reihung permutation
      // bringen und die Reihung tauschMitWem initialisieren
      for (int i=0; i<=MAX_INDEX; i++) {
         permutation [i] = i+1;
         tauschMitWem[i] = i+1;
      }
   } // Konstruktor Permutationen
   // ---------------------------------------------------------------------
   private void berechneNaechstePermutation() {
      // Vertauscht zwei Komponenten der Reihung permutation miteinander.
      // Welche Komponenten vertauscht werden wird anhand der Informationen
      // in der Reihung tauschMitWem ermittelt. Die Reihung tauschMitWem
      // wird so veraendert, dass permutation der Reihe nach alle moeg-
      // lichen Permutationen annimmt bevor sie wieder ihren Anfangszustand
      // erreicht.

      int temp;
      int stacktop = MAX_INDEX - 1;

      while (stacktop >= 0) {
         if (tauschMitWem[stacktop] == MAX_INDEX+1) {

            // Rotation nach links:
            temp = permutation[stacktop];
            for (int i=stacktop+1; i<=MAX_INDEX; i++) {
               permutation[i-1] = permutation[i];
            }
            permutation[MAX_INDEX] = temp;

            // Naechstes Mal wieder von vorn
            tauschMitWem[stacktop] = stacktop + 1;
            stacktop               = stacktop - 1;
         } else {
            // Vertausche zwei Komponenten:
            temp                  = permutation[stacktop];
            permutation[stacktop] = permutation[tauschMitWem[stacktop]];
            permutation[tauschMitWem[stacktop]] = temp;

            // Naechstes Mal soll die naechste Komponente vertauscht
            // werden:
            tauschMitWem[stacktop]++;
            return;
         }
      } // while
   } // berechneNaechstePermutation
   // ---------------------------------------------------------------------
   public int anzahlPermutationen() {
      // Mit n Elementen kann man n! (n Fakultaet) viele Permutationen
      // bilden. Diese Funktion liefert die Anzahl der Permutationen,
      // die mit diesem Objekt erzeugt werden koennen.

      int erg = 2;
      for (int i=3; i<=MAX_INDEX+1; i++) erg *= i;
      return erg;
   } // anzahlPermutationen
   // ---------------------------------------------------------------------
   public int[] naechste() {
      // Die wichtigste Methode dieses Objekts. Liefert die naechste
      // Permutation (in Form einer Kopie der Reihung permutation).
      int[] erg = (int[]) permutation.clone();
      berechneNaechstePermutation();
      lfdNr++;
      return erg;
   } // naechste
   // ---------------------------------------------------------------------
   public int nrDerNaechsten() {
      // Liefert die laufende Nummer der naechsten Permutation.
      // Diese Nummer liegt zwischen 1 und anzahlPermutationen().
       return (lfdNr % anzahlPermutationen()) + 1;
    } // nrDerNaechsten
   // ---------------------------------------------------------------------
   public String toString() {
      // Liefert die naechste Permutation als String
      StringBuffer erg = new StringBuffer();
      for (int i=0; i<permutation.length; i++) {
         erg.append(permutation[i] + " ");
      }
      return erg.toString();
   } // toString
   // ---------------------------------------------------------------------
   void print(String name) {
      // Gibt den name gefolgt von der naechsten Permutation zur
      // Standardausgabe aus:
      pln(name + ":\t" + toString());
   } // print
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Objekte der Klasse Permutationen werden ein bisschen getestet:

      pln("Permutationen: Jetzt geht es los!");

      int[] ir = {17, 81, 33};

      // Mit verschiedenen Konstruktoren Objekte der Klasse
      // Permutationen erzeugen und in der Reihung pr speichern:
      Permutationen[] pr = {
         new Permutationen(4),
         new Permutationen(ir),
      }; // pr

      // Alle Permutationen aller Permutationen-Objekte in
      // der Reihung pr zur Standardausgabe ausgeben:
      for (int i=0; i< pr.length; i++) {
         Permutationen p = pr[i];
         pln("---------------------------------");
         pln("anzahlPermutationen: " + p.anzahlPermutationen());
         for (int j=0; j<p.anzahlPermutationen()+1; j++) {

            // Die naechste Permutation von p ausgeben:
            p.print("P" + (i+1) + "_" + p.nrDerNaechsten());

            // Die naechste Permutation holen (aber nichts damit machen):
            int[] r = p.naechste();
         } // for j
      } // for i

      pln("---------------------------------");
      pln("Permutationen: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class Permutationen