// Datei RandomAccessDatei01.java
/* ------------------------------------------------------------------------
Eine Direktzugriffsdatei (a random access file) wird erzeugt, ihre Laenge
wird festgelegt, Daten verschiedener Typen (int, double, String) werden
ab einer bestimmten Stelle in die Datei geschrieben und dann wieder einge-
lesen und zur Standardausgabe ausgegeben. Die Datei wird wieder geloescht.
------------------------------------------------------------------------ */
import java.io.RandomAccessFile;
import java.io.File;
import java.io.IOException;
import java.io.EOFException;

class RandomAccessDatei01 {
   // ---------------------------------------------------------------------
   static public void main(String[] susi) throws IOException {
      pln("RandomAccessDatei01: Jetzt geht es los!");
      pln("---------------------------------------");

      // Ein File-Objekt dat erzeugen (damit wir die entsprechende Datei
      // weiter unten wieder loeschen koennen) und mit dem File-Objekt
      // ein Direktdatei raf erzeugen und zum Lesen und Schreiben ("rw")
      // oeffnen:
      File             dat  = new File("RandomAccessDatei01.raf");
      RandomAccessFile raf  = new RandomAccessFile(dat, "rw");

      // Die Laenge der Direktdatei (in Bytes) mehrmals ausgeben und
      // veraendern:
      pln("A raf.getFilePointer(): " + raf.getFilePointer());
      pln("B raf.length()        : " + raf.length());
      raf.setLength(20 * 1000);        // 20 Tausend Bytes
      pln("C raf.length()        : " + raf.length());
      raf.setLength(50 * 1000);        // 50 Tausend Bytes
      pln("D raf.length()        : " + raf.length());
      raf.setLength(10 * 1000);        // 10 Tausend Bytes
      pln("E raf.length()        : " + raf.length());
      pln("F raf.getFilePointer(): " + raf.getFilePointer());
      // Der Inhalt der Direktdatei ist jetzt undefiniert. Wenn man Glueck
      // hat, findet man in einer neuen Datei Daten einer alten, bereits
      // "geloeschten" Datei.

      // Ab Byte Nr. 145 ein paar Daten unterschiedlicher Typen
      // nacheinander in die Datei schreiben:
      raf.seek(145);
      raf.writeInt   (255);
      raf.writeDouble(0.1);
      raf.writeUTF   ("Hallo Sonja!");

      pln("---------------------------------------");
      pln("G raf.getFilePointer(): " + raf.getFilePointer());

      // Daten von verschiedenen Stellen der Datei lesen und
      // in Variablen t1, n1, d1 etc. speichern:
      raf.seek(157);
      String t1 = raf.readUTF();

      raf.seek(145);
      int    n1 = raf.readInt();
      double d1 = raf.readDouble();

      // Daten von einer "unbeschriebenen" Stelle der Datei lesen
      raf.seek(333);
      int    n2 = raf.readInt();

      // Den Anfang des UTF-Strings als short lesen:
      raf.seek(157);
      short  s1 = raf.readShort();

      // Zwei byte-Werte aus dem UTF-String lesen:
      raf.seek(159);
      byte   b1 = raf.readByte();
      raf.seek(170);
      byte   b2 = raf.readByte();

      // Daten, die als int-Wert geschrieben wurden als
      // float-Wert einlesen:
      raf.seek(145);
      float  f1 = raf.readFloat();

      // Die Werte der Variablen t1, n1, d1 etc. ausgeben:
      pln("---------------------------------------");
      pln("H String  ab Byte 157 : " + t1);
      pln("I int     ab Byte 145 : " + n1);
      pln("J double  ab Byte 261 : " + d1);
      pln("K int     ab Byte 333 : " + n2);
      pln("L short   ab Byte 157 : " + s1);
      pln("M byte    ab Byte 159 : " + b1);
      pln("N byte    ab Byte 159 : " + (char) b1);
      pln("O byte    ab Byte 170 : " + b2);
      pln("P byte    ab Byte 170 : " + (char) b2);
      pln("Q float   ab Byte 145 : " + f1);
      pln("---------------------------------------");

      // Hinter das letzte Byte der Datei weitere Daten schreiben:
      pln("R raf.length()        : " + raf.length());
      raf.seek(raf.length());
      raf.writeUTF("Das Hinterletzte!");
      pln("S raf.length()        : " + raf.length());
      pln("---------------------------------------");
      pln("RandomAccessDatei01: Das war's erstmal!");

      // Die Direktdatei raf schliessen und die Datei dat loeschen:
      raf.close();
      dat.delete();
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class RandomAccessDatei01
/* ------------------------------------------------------------------------
Ausgabe des Programms RandomAccessDatei01:

RandomAccessDatei01: Jetzt geht es los!
---------------------------------------
A raf.getFilePointer(): 0
B raf.length()        : 0
C raf.length()        : 20000
D raf.length()        : 50000
E raf.length()        : 10000
F raf.getFilePointer(): 0
---------------------------------------
G raf.getFilePointer(): 171
---------------------------------------
H String  ab Byte 157 : Hallo Sonja!
I int     ab Byte 145 : 255
J double  ab Byte 261 : 0.1
K int     ab Byte 333 : 0
L short   ab Byte 157 : 12
M byte    ab Byte 159 : 72
N byte    ab Byte 159 : H
O byte    ab Byte 170 : 33
P byte    ab Byte 170 : !
Q float   ab Byte 145 : 3.57E-43
---------------------------------------
R raf.length()        : 10000
S raf.length()        : 10019
---------------------------------------
RandomAccessDatei01: Das war's erstmal!
---------------------------------------------------------------------------
Die Datei RandomAccessDatei01.raf (Auszuege in hex-Darstellung):

000000  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  ................
...
000080  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  ................
000090  00 00 00 00  FF 3F B9 99  99 99 99 99  9A 00 0C 48  ....?..H
0000A0  61 6C 6C 6F  20 53 6F 6E  6A 61 21 00  00 00 00 00  allo Sonja!.....
0000B0  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  ................
...
002700  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  ................
002710  00 11 44 61  73 20 48 69  6E 74 65 72  6C 65 74 7A  ..Das Hinterletz
002720  74 65 21                                            te!
------------------------------------------------------------------------ */
