// Datei RandomAccessDatei02.java
/* ------------------------------------------------------------------------
Eine Direktzugriffsdatei (a random access file) wird erzeugt (wenn sie noch
nicht existiert) bzw. geoeffnet (wenn sie bereits existiert). Aus der Datei
werden int-Werte gelesen und veraendert zurueckgeschrieben.
------------------------------------------------------------------------ */
import java.io.RandomAccessFile;
import java.io.File;
import java.io.IOException;
import java.io.EOFException;

class RandomAccessDatei02 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws IOException {
      printf("RandomAccessDatei02: Jetzt geht es los!\n");
      printf("---------------------------------------\n");

      final int ANZ_INT = 100; // Anzahl der int-Werte in der Datei
      final int LEN_INT =   4; // Laenge eines int-Wertes in Byte

      // Eine Direktzugriffsdatei zum Lesen und Schreiben ("rw") oeffnen
      // (bzw. erzeugen, wenn sie noch nicht existiert):
      File             dat = new File("c:/meineDateien/RandomAccessDatei02.raf");
      RandomAccessFile raf = new RandomAccessFile(dat, "rw");

      // Die Datei so gross machen, dass ANZ_INT viele int-Werte
      // hineinpassen:
      raf.setLength(ANZ_INT * LEN_INT); // Groesse (in Bytes) festlegen

      // Was steht in der Datei, bevor etwas hineingeschrieben wurde?
      // int-Werte lesen bis zum Ende der Datei und zur Standardausgabe
      // ausgeben:
      int anzInt = 0; // Anzahl der gelesenen int-Werte
      while (true) {
         try {
            int n = raf.readInt(); // Einen int-Wert aus der Datei lesen

            // In der Standardausgabe evtl. neue Zeile beginnen und
            // Zeilen-Nummer ausgeben:
            if (anzInt % 5 == 0) {
               if (anzInt > 0) printf("\n");
               printf("%2d. ", anzInt / 5 + 1);
            }

            // Die gelesene Zahl n in der Breite 12 ausgeben:
            printf("%12d", n);
            anzInt++;
         } catch (EOFException ee) {

            break;
         }
      } // while

      printf("\n---------------------------------------\n");
      printf("Aus der Datei \n%s\nwurden ",  dat);
      printf("%d int-Werte gelesen.\n",      anzInt);
      printf("Dateilaenge (in Bytes): %d\n", raf.length());

      // Jeden int-Wert in der Datei um 1 erhoehen (di wie "delta i" ist
      // der Abstand des aktuellen int-Wertes vom Dateianfang, in Bytes):
      for (int i=0; i<ANZ_INT; i++) {
         int di = 4 * i;          // delta i berechnen
         raf.seek(di);            // Vor dem aktuellen Wert positionieren
         int n  = raf.readInt();  // Aktuellen Wert lesen
         raf.seek(di);            // Vor dem aktuellen Wert positionieren
         raf.writeInt(n%100 + 1); // Aktuellen Wert ueberschreiben
      } // for i

      raf.close();
      dat.delete();
      printf("---------------------------------------\n");
      printf("RandomAccessDatei02: Das war's erstmal!\n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf (f, v);}
   // ---------------------------------------------------------------------
} // class RandomAccessDatei02
/* ------------------------------------------------------------------------
Eine (besonders langweilige) Ausgabe des Programms RandomAccessDatei02:

RandomAccessDatei02: Jetzt geht es los!
---------------------------------------
 1.           13          13          13          13          13
 2.           13          13          13          13          13
 3.           13          13          13          13          13
 4.           13          13          13          13          13
 5.           13          13          13          13          13
 6.           13          13          13          13          13
 7.           13          13          13          13          13
 8.           13          13          13          13          13
 9.           13          13          13          13          13
10.           13          13          13          13          13
11.           13          13          13          13          13
12.           13          13          13          13          13
13.           13          13          13          13          13
14.           13          13          13          13          13
15.           13          13          13          13          13
16.           13          13          13          13          13
17.           13          13          13          13          13
18.           13          13          13          13          13
19.           13          13          13          13          13
20.           13          13          13          13          13
---------------------------------------
Aus der Datei
c:/meineDateien/RandomAccessDatei02.raf
wurden 100 int-Werte gelesen.
Dateilaenge (in Bytes): 400
---------------------------------------
RandomAccessDatei02: Das war's erstmal!
------------------------------------------------------------------------ */
