// Datei Reflekt01.java
/* ------------------------------------------------------------------------
Klassen, Attribute, Methoden und Konstruktoren koennen in Java durch
Objekte der Klassen Class, Field, Method bzw. Constructor reflektiert
(d.h. dargestellt oder repraesentiert) werden.

Hier wird eine simple Klasse Rek1 vereinbart. Dann wird mit
Reflektions-Objekten auf ihre Attribute zugegriffen und ihre Methoden
werden aufgerufen.
------------------------------------------------------------------------ */
import java.lang.reflect.Field;
import java.lang.reflect.Method;
// ------------------------------------------------------------------------
class Rek1 {
   // Eine Klasse mit folgenden Elementen:
   // 2 Attribute (1 Klassenattribut, 1 Objektattribut)
   // 4 Methoden  (1 Klassenmethode,  3 Objektmethoden ohne/mit Params)

   static        float  karl = 12.45F;            // Klassenattribut
          public short  otto = 12345;             // Objektattribut

   static void hallo1_kop() {                     // Klassenprozedur
      System.out.println("F Hallo1!");            // ohne Parameter (kop)
   }

   public void hallo2_oop() {                     // Objektprozedur
      System.out.println("G Hallo2!");            // ohne Parameter (oop)
   }

   void hallo3_omp(String s1, String s2) {        // Objektprozedur
      String s = "H Hallo3: " + s1 +" und " + s2; // mit Parametern (omp)
      System.out.println(s);
   }

   protected int add(int n1, int n2) {            // Objektfunktion
      String s = "I add:    " + n1 + " + " + n2;  // mit Parametern
      System.out.println(s);
      return n1 + n2;
   }
} // class Rek1
// ------------------------------------------------------------------------
class Reflekt01 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) throws
      java.lang.reflect.InvocationTargetException,
      InstantiationException,
      ClassNotFoundException,
      IllegalAccessException
   {
      Rek1   nob1  = new Rek1();              // Ein "normales" Objekt

      // Drei Moeglichkeiten, sich ein Class-Objekt zu verschaffen, welches
      // die Klasse Rek1 reflektiert (oder repraesentiert)
      Class    kob1 = Rek1.class;              // Ein Reflektions-Objekt
      Class    kob2 = nob1.getClass();         // Ein Reflektions-Objekt
      Class    kob3 = Class.forName("Rek1");   // Ein Reflektions-Objekt

      // Objekte der durch kob1 repraesentierten Klasse erzeugen (die
      // (werden mit dem geschenkten Standardkonstruktor initialisiert):
      Object nob2 =        kob1.newInstance();
      Rek1   nob3 = (Rek1) kob1.newInstance();

      // Reflektionen aller Attribute und Methoden holen, die in der durch
      // kob1 reflektierten Klasse vereinbart wurden:
      Field [] ra = kob1.getDeclaredFields();  // Reflektionen der Attribute
      Method[] rm = kob1.getDeclaredMethods(); // Reflektionen der Methoden

      // Reihungen mit Parameter-Objekten fuer die Methoden der von kob1
      // reflektierten Klasse:
      Object[] p01 = new Object[] {"Su", "Joe!"};
      Object[] p02 = new Object[] {17  ,  25   }; // Autohuellen

      pln("Reflekt01: Jetzt geht es los!");
      pln("---------------------------------------------------------------");
      pln("A kob1     : " + kob1);      // Das Reflek.-Objekt kob1 ausgeben
      pln("B ra.length: " + ra.length); // Anzahl der Attribute
      pln("C rm.length: " + rm.length); // Anzahl der Methoden
      pln("---------------------------------------------------------------");
      // Die Reflektionen aller Attribute ausgeben, die in der durch kob1
      // reflektierten Klasse vereinbart wurden:
      for (int i=0; i<ra.length; i++) {
         p  ("D ra[" + i + "]: " + ra[i]); // Name des Attributs
         pln(", Wert: " + ra[i].get(nob2));  // Wert des Attributs (in nob2)
      }
      pln("---------------------------------------------------------------");
      // Die Reflektionen aller Methoden ausgeben, die in der durch kob1
      // reflektierten Klasse vereinbart wurden:
      for (int i=0; i<rm.length; i++) {
         pln("E rm[" + i + "]: " + rm[i]); // Name und Parameter der Methode
      }
      pln("---------------------------------------------------------------");
      // Jede Methode, die in der durch kob1 reflektierten Klasse vereinbart
      // wurde, einmal aufrufen:
      Object[] NULL = null;     // Der genaue Typ verhindert Warnungen

      Object ergo =             // Funktionsergebnis nach ergo
      rm[0].invoke(nob2,  p02); //  Objektfunktion add,          2 Paramet.
      rm[1].invoke(null, NULL); // Klassenprozedur hallo1_kop(), 0 Paramet.
      rm[2].invoke(nob2, NULL); //  Objektprozedur hallo2_oop(), 0 Paramet.
      rm[3].invoke(nob2,  p01); //  Objektprozedur hallo3_omp(), 2 Paramet.

      pln("J add-Erg.: " +ergo);
      pln("---------------------------------------------------------------");
      // O.k., hier kann man das Gleiche auch ohne Reflektion erreichen
      // (allerdings nur mit nob3, nicht mit dem Object-Objekt nob3):
      Rek1.hallo1_kop();
      nob3.hallo2_oop();
      nob3.hallo3_omp("Su", "Joe");
      int ergi =
      nob3.add(17, 25);
      pln("J add-Erg.: " + ergi);
      pln("---------------------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Reflekt01
/* ------------------------------------------------------------------------
Ausgabe des Programms Reflekt01:

Reflekt01: Jetzt geht es los!
---------------------------------------------------------------
A kob1     : class Rek1
B ra.length: 2
C rm.length: 4
---------------------------------------------------------------
D ra[0]: static float Rek1.karl, Wert: 12.45
D ra[1]: public short Rek1.otto, Wert: 12345
---------------------------------------------------------------
E rm[0]: protected int Rek1.add(int,int)
E rm[1]: static void Rek1.hallo1_kop()
E rm[2]: public void Rek1.hallo2_oop()
E rm[3]: void Rek1.hallo3_omp(java.lang.String,java.lang.String)
---------------------------------------------------------------
I add:    17 + 25
F Hallo1!
G Hallo2!
H Hallo3: Su und Joe!
J add-Erg.: 42
---------------------------------------------------------------
F Hallo1!
G Hallo2!
H Hallo3: Su und Joe
I add:    17 + 25
J add-Erg.: 42
---------------------------------------------------------------
------------------------------------------------------------------------ */