// Datei Reflekt02.java
/* ------------------------------------------------------------------------
Von einer Klasse, von der nur der Name bekannt ist, werden (mit Hilfe von
Reflektion) alle Methoden aufgerufen, die genau einen double-Parameter
besitzen.
------------------------------------------------------------------------ */
import java.lang.reflect.Method;
// ------------------------------------------------------------------------
class Rek2 {
   // Diese Klasse enthaelt mehrere Methoden mit einem double-Parameter.
   static double plus1(double d) {return  1.0 + d;}
   static String  mal2(double d) {return (2.0 * d) + "";}
   static void   void3(double d) {}
          double plus4(double d) {return  4.0 + d;}
          void   void5(double d)  {}
} // Rek2
// ------------------------------------------------------------------------
class Reflekt02 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) throws ClassNotFoundException {

      pln("Reflekt02: Jetzt geht es los!");

      // Ein paar volle Klassennamen zur Auswahl (der Klassenname
      // koennte auch eingelesen werden):
//    String   klassenName = "java.lang.Math";
//    String   klassenName = "java.lang.String";
//    String   klassenName = "java.lang.StringBuffer";
//    String   klassenName = "java.util.ArrayList";
      String   klassenName = "Rek2"; // siehe oben in dieser Datei

      // Das Klassenobjekt und alle Methodenobjekte der Klasse holen:
      Class    kob         = Class.forName(klassenName);
      Method[] rm          = kob.getDeclaredMethods();

      pln("---------------------------------------------------------------");
      pln("Bearbeitet wird die Klasse " + kob.getName());
      pln("---------------------------------------------------------------");
      for (Method m: rm) pruefeUndRufeAuf(m);
      pln("---------------------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   static void pruefeUndRufeAuf(Method m) {
      // Prueft, ob m eine Methode repraesentiert, die genau einen
      // Parameter vom Typ double hat. Wenn ja, wird die Methode
      // aufgerufen.

      // Die Klassenobjekte der Typen aller Parameter der Methode m
      // nach rt holen:
      Class<?>[] rt  = m.getParameterTypes();
      String metName = m.getName();

      // Nur die Methoden aufrufen, die genau einen Parameter
      // vom Typ double haben. Double.TYPE ist das Class-Objekt
      // des primitiven Typs double.
      if (rt.length == 1 && rt[0] == Double.TYPE) {
         rufeAuf(m);
      }
   } // pruefeUndRufeAuf
   // ---------------------------------------------------------------------
   static void rufeAuf(Method m)  {
      // Verlaesst sich darauf, dass m eine Methode reflektiert, die
      // genau einen Parameter vom Typ double hat. Ruft m mit dem
      // double-Wert 2.5 als Parameter auf.
      String   metName = m.getName();
      double   dopa    = 2.5;

      // Versuch, m als Klassenmethode aufzurufen (der erste Parameter
      // von invoke ist kein Objekt sondern null):
      try {
         Object erg  = m.invoke(null, dopa);
         printf("%-10s(%3.2g) ist gleich %s\n", metName, dopa, erg);
         return;  // Wenn der Aufruf geklappt hat
      } catch (Throwable t) {
         pln("\nEin Aufruf der Methode " + metName );
         pln("loeste folgende Ausnahme aus:\n" + t + "\n");
      } // try/catch

      // Versuch, m als Objektmethode aufzurufen (der erste Parameter
      // von invoke ist das entsprechende Objekt):
      try {
         Object ob   = m.getDeclaringClass().newInstance();
         Object erg  = m.invoke(ob, dopa);
         printf("%-10s(%3.2g) ist gleich %s\n", metName, dopa, erg);
         return;  // Wenn der Aufruf geklappt hat
      } catch (Throwable t) {
         pln("\nEin Aufruf der Methode " + metName );
         pln("loeste folgende Ausnahme aus:\n" + t);
         pln("Ursache: " + t.getCause() + "\n");
      } // try/catch

   } // rufeAuf
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Reflekt02
/* ------------------------------------------------------------------------
Ausgabe des Programms Reflekt02 (mit klassenName gleich "Rek2"):

Reflekt02: Jetzt geht es los!
---------------------------------------------------------------
Bearbeitet wird die Klasse Rek2
---------------------------------------------------------------
plus1     (2,50) ist gleich 3.5
mal2      (2,50) ist gleich 5.0
void3     (2,50) ist gleich null

Ein Aufruf der Methode plus4
loeste folgende Ausnahme aus:
java.lang.NullPointerException

plus4     (2,50) ist gleich 6.5

Ein Aufruf der Methode void5
loeste folgende Ausnahme aus:
java.lang.NullPointerException


Ein Aufruf der Methode void5
loeste folgende Ausnahme aus:
java.lang.reflect.InvocationTargetException
Ursache: java.lang.ArithmeticException

---------------------------------------------------------------
------------------------------------------------------------------------ */