// Datei Reflekt03.java
/* ------------------------------------------------------------------------
Liest wiederholt den vollen Namen einer Klasse (z.B. java.lang.String oder
erben01.Punkt oder Reflekt03 etc.) oder einer Schnittstelle (z.B.
java.lang.Runnable) ein. Gibt alle Superklassen, inneren Klassen, Schnitt-
stellen, Attribute, Konstruktoren und Methoden der Klasse bzw. alle
Attribute und Methoden der Schnittstelle aus. Beendet sich, wenn der
Benutzer "q" oder eine leere Zeichenkette eingibt.
In dieser Version wird die Ausgabe mit Hilfe von Abkuerzungen namens
a0, a1, a2, etc. uebersichtlicher gestaltet.
------------------------------------------------------------------------ */
import java.util.Vector;
import java.util.StringTokenizer;

class Reflekt03 {
   static final String BLANKS = "   ";
   static final String KEINE  = "--- Keine vorhanden! ---";
   // ---------------------------------------------------------------------
   // Zwei Hilfsmethoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
   static public void main(String susi[]) {
      String kName;
      Class  kObjekt = null; // Initialisierung beruhigt den Ausfuehrer

      while (true) {
         p("Voller Name einer Klasse/Schnittstelle ('q' zum Beenden)? ");
         kName = EM.liesString();
         if (kName.equals("") || kName.charAt(0) == 'q') break;

         try {
            kObjekt= Class.forName(kName); // Class-Objekt fuer Klasse
         }                                 // namens kName.
         catch (ClassNotFoundException ex){
            pln(kName + " ist weder Klasse noch Schnittstelle! ");
            continue;
         }

         // Informationen ueber die Klasse/Schnittstelle kObjekt ausgeben:
         putKlasseOderSchnittstelle(kObjekt); pln("");
         printAbkuerzungen(kName);

         // Fuer die naechste Runde: Alle Abkuerzungen loeschen und die
         // Nr der naechsten Abkuerzung auf 0 zuruecksetzen:
         abkuerzungen.clear();
         abkuerzungsNr = 0;
      } // while (true)
      pln("Reflekt03: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   static void putKlasseOderSchnittstelle(Class kob){
      // Gibt alle Informationen ueber die durch kob reflektierte
      // Klasse oder Schnittstelle zur Standardausgabe aus.
      if(kob.isInterface()){   // kob reflektiert eine Schnittstelle
         putNamen  ("Schnittstelle ", kob);
         putReihung("Attribute     ", kob.getDeclaredFields());
         putReihung("Methoden      ", kob.getDeclaredMethods());

      } else {                     // kob reflektiert eine Klasse
         putNamen  ("Klasse        ", kob);
         putSuper  ("Superklassen  ", kob);
         putReihung("Schnittstellen", kob.getInterfaces());
         putReihung("Innere Klassen", kob.getClasses());
         putReihung("Attribute     ", kob.getDeclaredFields());
         putReihung("Konstruktoren ", kob.getDeclaredConstructors());
         putReihung("Methoden      ", kob.getDeclaredMethods());
      }
   } // putKlasseOderSchnittselle
   // ---------------------------------------------------------------------
   static void putNamen(String text, Class kob) {
      // Gibt den Namen der durch kob reflektierten Klasse oder
      // Schnittstell aus, mit Modifizierern wie public oder final davor.
      int modiInt  = kob.getModifiers(); // Alle Modifizierer-Bits
          modiInt &= 0xFFFFFDFF;         // Modifizierer "interface"
                                         // (512) entfernen
      String modiString = java.lang.reflect.Modifier.toString(modiInt);
      pln("-------------------------------------------------------------");
      pln(text + ": " + modiString + " " + kob);
      pln("-------------------------------------------------------------");
   } // putNamen
   // ---------------------------------------------------------------------
   static void putSuper(String text, Class kob) {
      // Gibt alle Superklassen der durch kob reflektierten Klasse (oder
      // den Text "Keine vorhanden") zur Standardausgabe aus.
      p(text + ": ");
      Class kobSuper = kob.getSuperclass();
      if (kobSuper == null) {
         pln(KEINE);
         return;
      }
      pln("");
      putSuper(kobSuper);
   } // putSuper(String, Class)
   // ---------------------------------------------------------------------
   static void putSuper(Class kob) {
      // Rekursive Hilfsmethode fuer putSuper(String, Class)
      if (kob == null) {
         return;
      } else {
         putSuper(kob.getSuperclass());
         String name             = kob.getName();
         String nameOhneVorspann = abkuerzungenEinfuehren(name);
         pln(BLANKS + nameOhneVorspann);
      } // if
   } // putSuper(Class)
   // ---------------------------------------------------------------------
   static void putReihung(String text, Object[] reihung) {
      // Gibt die Komponenten der reihung aus, jede auf einer neuen Zeile,
      // um BLANKS eingerueckt. Wenn eine Komponente mehr als MAX_ZEICHEN_-
      // PRO_ZEILE belegt, wird versucht, sie bei einem Blank oder Punkt
      // zu trennen. Fortsetzungszeilen werden um 2 mal BLANKS eingerueckt.
      final int MAX_ZEICHEN_PRO_ZEILE = 78 - 2*BLANKS.length();
      p(text + ": ");

      // Wenn die reihung leer ist:
      if (reihung.length == 0) { pln(KEINE); return; }

      // Wenn die reihung nicht leer ist:
      pln(reihung.length + " Stueck:");
      for (int i=0; i<reihung.length; i++) {
         String original   = reihung[i].toString();
         String mitAbkuerz = abkuerzungenEinfuehren(original);
         while (mitAbkuerz.length() > MAX_ZEICHEN_PRO_ZEILE) {
            int iBlank = mitAbkuerz.lastIndexOf(' ', MAX_ZEICHEN_PRO_ZEILE);
            int iPunkt = mitAbkuerz.lastIndexOf('.', MAX_ZEICHEN_PRO_ZEILE);
            int iMax   = 1 + Math.max(iBlank, iPunkt);
            if (iMax < 0) iMax = MAX_ZEICHEN_PRO_ZEILE;
            pln(BLANKS + mitAbkuerz.substring(0, iMax));
            mitAbkuerz = mitAbkuerz.substring(iMax);
            p(BLANKS); // 1. BLANKS fuer naechste Zeile
         } // while
         pln(BLANKS + mitAbkuerz);
      } // for

   } // putReihung
   // ---------------------------------------------------------------------
   static int          abkuerzungsNr = 0;   // Nr der naechsten Abkuerzung
   static final String PRAE          = "a"; // Praefix fuer alle Abkuerzungen
   static final Vector<String> abkuerzungen  = new Vector<String>();

   static String abkuerzungenEinfuehren(String s) {
      final char   PUNKT = '.';
      final String BLANK = " ";
      final String SEPARATOREN = " (),";
      StringBuffer sb = new StringBuffer();

      StringTokenizer st = new StringTokenizer(s, SEPARATOREN, true);
      while (st.hasMoreTokens()) {
         String token = st.nextToken();
         int    ind   = token.lastIndexOf(PUNKT);

         // Falls token keinen PUNKT enthaelt:
         if (ind < 0) {
            sb.append(token);
            continue;
         }

         // ind ist der Index des letzten PUNKTes in token:
         String vorspann = token.substring(0, ind); // bis zum letzten Punkt
         String name     = token.substring(ind+1);  // ab  dem letzten Punkt

         // vorspann in abkuerzungen merken und durch Abkuerzung
         // "a0", "a1", ... ersetzen:
         int vind = abkuerzungen.indexOf(vorspann);
         sb.append(PRAE);
         if (vind < 0) {
            abkuerzungen.add(vorspann);
            sb.append(abkuerzungsNr++);
         } else {
            sb.append(vind);
         } // if
         sb.append(PUNKT + name);

      } // while st.hasMoreTokens

      return sb.toString();
   } // abkuerzungenEinfuehren
   // ---------------------------------------------------------------------
   static void printAbkuerzungen(String kName) {
      // Gibt alle Abkuerzungen a0, a1, ... und das, wofuer sie stehen, aus:
      pln("Die Abkuerzungen a0, a1 etc. fuer die Klasse " + kName);
      for (int i=0; i<abkuerzungen.size(); i++) {
         pln(PRAE + i + ": " + abkuerzungen.get(i));
      }
      pln("-------------------------------------------------------------");
   } // printAbkuerzungen
   // ---------------------------------------------------------------------
} // class Reflekt03
/* ------------------------------------------------------------------------
Ein Dialog mit dem Programm Reflekt03:

Voller Name einer Klasse/Schnittstelle ('q' zum Beenden)? erben02.GeoFigur
-------------------------------------------------------------
Klasse        : public abstract class erben02.GeoFigur
-------------------------------------------------------------
Superklassen  :
   a0.Object
Schnittstellen: --- Keine vorhanden! ---
Innere Klassen: --- Keine vorhanden! ---
Attribute     : 2 Stueck:
   protected double a1.x
   protected double a1.y
Konstruktoren : 1 Stueck:
   public a2.GeoFigur(double,double)
Methoden      : 6 Stueck:
   public a0.String a1.toString()
   public double a1.urAbstand()
   public void a1.urSpiegeln()
   public a0.String a1.text()
   public abstract double a1.getUmfang()
   public abstract double a1.getFlaeche()

Die Abkuerzungen a0, a1 etc. fuer die Klasse erben02.GeoFigur
a0: java.lang
a1: erben02.GeoFigur
a2: erben02
-------------------------------------------------------------
Voller Name einer Klasse/Schnittstelle ('q' zum Beenden)? Reflekt03
-------------------------------------------------------------
Klasse        : class Reflekt03
-------------------------------------------------------------
Superklassen  :
   a3.Object
Schnittstellen: --- Keine vorhanden! ---
Innere Klassen: --- Keine vorhanden! ---
Attribute     : 4 Stueck:
   static final a0.String v4.BLANKS
   static final a0.String a1.KEINE
   static int a1.abkuerzungsNr
   static v5.Vector a1.abkuerzungen
Konstruktoren : 1 Stueck:
   public Reflekt03()
Methoden      : 9 Stueck:
   public static void a1.main(a0.String[])
   static void a1.pln(a0.Object)
   static void a1.p(a0.Object)
   static void a1.putKlasseOderSchnittstelle(a0.Class)
   static void a1.printAbkuerzungen(a0.String)
   static void a1.putNamen(a0.String,a0.Class)
   static void a1.putSuper(a0.Class)
   static void a1.putReihung(a0.String,a0.Object[])
   static a0.String a1.abkuerzungenEinfuehren(a0.String)

Die Abkuerzungen a0, a1 etc. fuer die Klasse Reflekt03
a0: java.lang
a1: Reflekt03
a2: java.util
-------------------------------------------------------------
Voller Name einer Klasse/Schnittstelle ('q' zum Beenden)? java.lang.Runnable
-------------------------------------------------------------
Schnittstelle : public abstract interface java.lang.Runnable
-------------------------------------------------------------
Attribute     : --- Keine vorhanden! ---
Methoden      : 1 Stueck:
   public abstract void a0.run()

Die Abkuerzungen a0, a1 etc. fuer die Klasse java.lang.Runnable
a0: java.lang.Runnable
-------------------------------------------------------------
Voller Name einer Klasse/Schnittstelle ('q' zum Beenden)? q
Reflekt03: Das war's erstmal!
------------------------------------------------------------------------ */