// Datei Reflekt06Tst.java
/* ------------------------------------------------------------------------
Testklassen mit Namen wie AAA01, AAA02, ... BBB01, ... etc. zum Testen
eines ObjektFabrik-Objekts obFab. Mit der Methode obFab.erzeugeObjekt
sollte man Objekte einiger dieser Testklassen erzeugen koennen, von
anderen Testklassen ist das eigentlich nicht moeglich.
Das eigentliche Testprogramm (Reflekt06Tst) steht weiter unten nach
allen Testklassen.
------------------------------------------------------------------------ */
import java.lang.reflect.Method;
import java.util.Vector;

class AAA01 {
   // Es ist unmoeglich, ein Objekt dieser Klasse zu erzeugen, weil der
   // einzige Konstruktor schon ein Objekt dieser Klasse als Parameter
   // erwartet:
   public AAA01(AAA01 ob) {}
} // class AAA01
// ------------------------------------------------------------------------
class AAA02 {
   // Es ist unmoeglich, ein Objekt dieser Klasse zu erzeugen, weil der
   // einzige Konstruktor ein Objekt der Klasse AAA01 als Parameter
   // erwartet:
   public AAA02(AAA01 ob) {}
} // class AAA02
// ------------------------------------------------------------------------
class AAA03 {
   // Es ist unmoeglich, ein Objekt dieser Klasse zu erzeugen, weil der
   // einzige Konstruktor ein Objekt der Klasse AAA02 als Parameter
   // erwartet:
   public AAA03(AAA02 ob) {}
} // class AAA02
// ------------------------------------------------------------------------
class BBB01 {
   // Objekte dieser Klasse CCC01 sollten automatisch erzeugt werden
   // koennen. Der (einzige) Konstruktor erwartet eine dreidimensionale
   // Reihung von int-Werten als Parameter.
   int[][][] r3;
   // ---------------------------------------------------------------------
   public BBB01(int[][][] r3) {
      this.r3 = r3;
   } // Konstruktor BBB01

   public String toString() {
      StringBuffer sb = new StringBuffer("BBB01-Objekt: ");
      sb.append('{');
      for (int i1=0; i1<r3.length; i1++) {
         sb.append('{');
         for (int i2=0; i2<r3[i1].length; i2++) {
            sb.append('{');
            for (int i3=0; i3<r3[i1][i2].length; i3++) {
               sb.append(r3[i1][i2][i3] + " ");
            }
            sb.append('}');
         }
         sb.append('}');
      }
      sb.append('}');
      return sb.toString();
   } // toString
   // ---------------------------------------------------------------------
} // class BBB01
// ------------------------------------------------------------------------
class CCC01 {
   // Objekte dieser Klasse CCC01 sollten automatisch erzeugt werden
   // koennen.
   static int letzteBelegteNr = 0;
   int nr;
   // ---------------------------------------------------------------------
   public CCC01() {nr = ++letzteBelegteNr;}
   public String toString() {return "CCC01-Obj" + nr;}
} // class CCC01
// ------------------------------------------------------------------------
class CCC02 {
   // Objekte dieser Klasse CCC01 sollten automatisch erzeugt werden
   // koennen. Der (einzige) Konstruktor dieser Klasse verlangt eine
   // Reihung von CCC01-Objekten als Parameter.
   CCC01[] r1;
   public CCC02(CCC01[] r1) {this.r1 = r1;}
   // ---------------------------------------------------------------------
   public String toString() {
      StringBuffer sb = new StringBuffer("CCC02-Objekt:\n");
      for (int i=0; i<r1.length; i++) {
         sb.append(r1[i].toString() + " | ");
      }
      return sb.toString();
   } // toString
   // ---------------------------------------------------------------------
} // class CC02
// ------------------------------------------------------------------------
class DDD01 {
   // Von den Klassen DDD01 bis DDD03 kann man keine Objekte erzeugen,
   // weil ihre Konstruktoren zyklischen voneinander abhaengig sind
   // (DDD01 von DDD03 von DDD02 von DDD01 von ...)
   DDD03 ob;
   public DDD01(DDD03 ob)  {this.ob = ob;}
} // class DDD01
// ------------------------------------------------------------------------
class DDD02 {
   // Von den Klassen DDD01 bis DDD03 kann man keine Objekte erzeugen,
   // weil ihre Konstruktoren zyklischen voneinander abhaengig sind
   // (DDD02 von DDD01 von DDD03 von DDD02 von ...)
   DDD01[] r1;
   public DDD02(DDD01[] r1) {this.r1 = r1;}
} // class CC02
// ------------------------------------------------------------------------
class DDD03 {
   // Von den Klassen DDD01 bis DDD03 kann man keine Objekte erzeugen,
   // weil ihre Konstruktoren zyklischen voneinander abhaengig sind
   // (DDD03 von DDD02 von DDD01 von DDD03 von ...)
   DDD02[] r1;
   public DDD03(DDD02[] r1) {this.r1 = r1;}
} // class CC02
// ------------------------------------------------------------------------
class EEE01 {
   // Diese Klasse hat einen privaten und einen oeffentlichen Konstruktor.
   // Die Parameter der Konstruktoren dienen nur dazu, sie zu unterscheiden
   // (und werden zu nichts anderem verwendet).
   // Welcher Konstruktor wird von der ObjektFabrik02 genommen?
   private String text;

   private EEE01(int n) {
      text = "EEE01-Obj., vom privaten Konstruktor erzeugt!";
   } // Konstruktor EEE01

   public  EEE01(long n) {
      text = "EEE01-Obj., vom public   Konstruktor erzeugt!";
   } // Konstruktor EEE01

   public String toString() {return text;};
} // class EEE01
// ------------------------------------------------------------------------
// Ende der Testklassen. Jetzt kommt das eigentliche Testprogramm:
// ------------------------------------------------------------------------

class Reflekt06Tst {
   // ---------------------------------------------------------------------
   // Das zu testende Objekt:
   static ObjektFabrik obFab = new ObjektFabrik();

   static private void erzeugeEinObjektUndGibEsAus(Class c)
      throws Exception
   {
      // Direkt getestet wird nur die Methode obFab.erzeugeObjekt. Mit
      // dieser Methode wird hier ein Objekt erzeugen:
      Object ob = obFab.erzeugeObjekt(c);

      // Das Ergebnis ausgeben:
      p("===> TYP: " + c.getName());
      pln(" / OBJEKT: " + Objects.toString(ob));
   } // erzeugeEinObjektUndGibEsAus
   // ---------------------------------------------------------------------
   static public void main(String[] susi) throws Exception {
      // Objekte verschiedener Typen werden erzeugt und ausgegeben. Dadurch
      // wird das ObjektFabrik-Objekt obFab getestet.

      pln("Reflekt06Tst: Jetzt geht es los!");
      pln("-------------------------------------");

      // Objekte aller Wickelklassen und einiger weiterer Klassen (String
      // Vector, Methode etc.) erzeugen und ausgeben:
      erzeugeEinObjektUndGibEsAus(         Byte.class);
      erzeugeEinObjektUndGibEsAus(         Byte.class);
      erzeugeEinObjektUndGibEsAus(         Byte.class);
      erzeugeEinObjektUndGibEsAus(    Character.class);
      erzeugeEinObjektUndGibEsAus(    Character.class);
      erzeugeEinObjektUndGibEsAus(    Character.class);
      erzeugeEinObjektUndGibEsAus(        Short.class);
      erzeugeEinObjektUndGibEsAus(      Integer.class);
      erzeugeEinObjektUndGibEsAus(         Long.class);
      erzeugeEinObjektUndGibEsAus(        Float.class);
      erzeugeEinObjektUndGibEsAus(       Double.class);
      erzeugeEinObjektUndGibEsAus(       String.class);
      erzeugeEinObjektUndGibEsAus(       String.class);
      erzeugeEinObjektUndGibEsAus(       String.class);
      erzeugeEinObjektUndGibEsAus(       String.class);
      erzeugeEinObjektUndGibEsAus(       String.class);
      erzeugeEinObjektUndGibEsAus(       Vector.class);
//    erzeugeEinObjektUndGibEsAus(erben01.Punkt.class);
//    erzeugeEinObjektUndGibEsAus(erben01.Kreis.class);
//    erzeugeEinObjektUndGibEsAus(erben02.Kreis.class);
      erzeugeEinObjektUndGibEsAus( ObjektFabrik.class);
      erzeugeEinObjektUndGibEsAus(       Method.class);

      // Objekte aller Reihungstypen mit einem primitiven Typ oder einer
      // Wickelklasse als Komponententyp erzeugen und ausgeben:
      erzeugeEinObjektUndGibEsAus(       byte[].class);
      erzeugeEinObjektUndGibEsAus(       char[].class);
      erzeugeEinObjektUndGibEsAus(      short[].class);
      erzeugeEinObjektUndGibEsAus(        int[].class);
      erzeugeEinObjektUndGibEsAus(       long[].class);
      erzeugeEinObjektUndGibEsAus(      float[].class);
      erzeugeEinObjektUndGibEsAus(     double[].class);
      erzeugeEinObjektUndGibEsAus(    boolean[].class);

      erzeugeEinObjektUndGibEsAus(       Byte[].class);
      erzeugeEinObjektUndGibEsAus(  Character[].class);
      erzeugeEinObjektUndGibEsAus(      Short[].class);
      erzeugeEinObjektUndGibEsAus(    Integer[].class);
      erzeugeEinObjektUndGibEsAus(       Long[].class);
      erzeugeEinObjektUndGibEsAus(      Float[].class);
      erzeugeEinObjektUndGibEsAus(     Double[].class);
      erzeugeEinObjektUndGibEsAus(    Boolean[].class);

      // Objekte der speziellen Testklassen AAA01, AAA02, ... etc.
      // erzeugen und ausgeben:
      erzeugeEinObjektUndGibEsAus(        AAA01.class);
      erzeugeEinObjektUndGibEsAus(        AAA02.class);
      erzeugeEinObjektUndGibEsAus(        AAA03.class);
      erzeugeEinObjektUndGibEsAus(        BBB01.class);
      erzeugeEinObjektUndGibEsAus(        CCC01.class);
      erzeugeEinObjektUndGibEsAus(        CCC02.class);
      erzeugeEinObjektUndGibEsAus(        DDD01.class);
      erzeugeEinObjektUndGibEsAus(        DDD02.class);
      erzeugeEinObjektUndGibEsAus(        DDD03.class);
      erzeugeEinObjektUndGibEsAus(        EEE01.class);

      pln("-------------------------------------");
      pln("Reflekt06Tst: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Zwei Methoden mit kurzen Namen (pln und p):
   static public void pln(Object ob) {System.out.println(ob);}
   static public void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Reflekt06Tst
/* ------------------------------------------------------------------------
Ausgabe des Programms Reflekt06Tst:

Reflekt06Tst: Jetzt geht es los!
-------------------------------------
===> TYP: java.lang.Byte / OBJEKT: Typ: Byte, Wert: 58
===> TYP: java.lang.Byte / OBJEKT: Typ: Byte, Wert: -67
===> TYP: java.lang.Byte / OBJEKT: Typ: Byte, Wert: 126
===> TYP: java.lang.Character / OBJEKT: Typ: Character, Wert: &
===> TYP: java.lang.Character / OBJEKT: Typ: Character, Wert: A
===> TYP: java.lang.Character / OBJEKT: Typ: Character, Wert: I
===> TYP: java.lang.Short / OBJEKT: Typ: Short, Wert: 4
===> TYP: java.lang.Integer / OBJEKT: Typ: Integer, Wert: 17
===> TYP: java.lang.Long / OBJEKT: Typ: Long, Wert: 85
===> TYP: java.lang.Float / OBJEKT: Typ: Float, Wert: 0.53
===> TYP: java.lang.Double / OBJEKT: Typ: Double, Wert: 34.639
===> TYP: java.lang.String / OBJEKT: Typ: String, Wert: [5*We"
===> TYP: java.lang.String / OBJEKT: Typ: String, Wert: @tM4<|:),)0,+6woVI!2?!}4
===> TYP: java.lang.String / OBJEKT: Typ: String, Wert: !d;"8?<=N3
===> TYP: java.lang.String / OBJEKT: Typ: String, Wert: 8X/
===> TYP: java.lang.String / OBJEKT: Typ: String, Wert: l*5y37xX5VjPTI$1c95$8
===> TYP: java.util.Vector / OBJEKT: Typ: java.util.Vector, Wert: []
===> TYP: ObjektFabrik / OBJEKT: Typ: ObjektFabrik, Wert: Ein Objekt der Klasse ObjektFabrik
===> TYP: java.lang.reflect.Method / OBJEKT: null-Referenz
===> TYP: [B / OBJEKT: Typ: byte[], Laenge: 7
    {29, 85, -26, 62, 84, 114, 84}
===> TYP: [C / OBJEKT: Typ: char[], Laenge: 3
    {), A, d}
===> TYP: [S / OBJEKT: Typ: short[], Laenge: 3
    {6, 3, 5}
===> TYP: [I / OBJEKT: Typ: int[], Laenge: 9
    {19, 11, 9, 0, 10, 18, 0, 7, 7}
===> TYP: [J / OBJEKT: Typ: long[], Laenge: 2
    {36, 82}
===> TYP: [F / OBJEKT: Typ: float[], Laenge: 6
    {0.2, 0.5, 0.26, 0.04, 0.76, 0.88}
===> TYP: [D / OBJEKT: Typ: double[], Laenge: 7
    {45.043, 65.094, 36.452, 12.219, 0.921, 33.132, 45.347}
===> TYP: [Z / OBJEKT: Typ: boolean[], Laenge: 7
    {false, true, true, false, true, false, true}
===> TYP: [Ljava.lang.Byte; / OBJEKT: Typ: Byte[], Laenge: 8
    [0] Typ: Byte, Wert: -88
    [1] Typ: Byte, Wert: -85
    [2] Typ: Byte, Wert: -65
    [3] Typ: Byte, Wert: 86
    [4] Typ: Byte, Wert: -56
    [5] Typ: Byte, Wert: -50
    [6] Typ: Byte, Wert: -78
    [7] Typ: Byte, Wert: -13
===> TYP: [Ljava.lang.Character; / OBJEKT: Typ: Character[], Laenge: 11
    [0] Typ: Character, Wert: ^
    [1] Typ: Character, Wert: `
    [2] Typ: Character, Wert: #
    [3] Typ: Character, Wert: L
    [4] Typ: Character, Wert: 7
    [5] Typ: Character, Wert: >
    [6] Typ: Character, Wert: }
    [7] Typ: Character, Wert: 0
    [8] Typ: Character, Wert: x
    [9] Typ: Character, Wert: 0
    [10] Typ: Character, Wert: l
===> TYP: [Ljava.lang.Short; / OBJEKT: Typ: Short[], Laenge: 11
    [0] Typ: Short, Wert: 7
    [1] Typ: Short, Wert: 7
    [2] Typ: Short, Wert: 0
    [3] Typ: Short, Wert: 1
    [4] Typ: Short, Wert: 9
    [5] Typ: Short, Wert: 9
    [6] Typ: Short, Wert: 6
    [7] Typ: Short, Wert: 3
    [8] Typ: Short, Wert: 1
    [9] Typ: Short, Wert: 9
    [10] Typ: Short, Wert: 1
===> TYP: [Ljava.lang.Integer; / OBJEKT: Typ: Integer[], Laenge: 3
    [0] Typ: Integer, Wert: 6
    [1] Typ: Integer, Wert: 17
    [2] Typ: Integer, Wert: 2
===> TYP: [Ljava.lang.Long; / OBJEKT: Typ: Long[], Laenge: 9
    [0] Typ: Long, Wert: 74
    [1] Typ: Long, Wert: 11
    [2] Typ: Long, Wert: 53
    [3] Typ: Long, Wert: 91
    [4] Typ: Long, Wert: 41
    [5] Typ: Long, Wert: 69
    [6] Typ: Long, Wert: 24
    [7] Typ: Long, Wert: 7
    [8] Typ: Long, Wert: 92
===> TYP: [Ljava.lang.Float; / OBJEKT: Typ: Float[], Laenge: 3
    [0] Typ: Float, Wert: 0.14
    [1] Typ: Float, Wert: 0.02
    [2] Typ: Float, Wert: 0.59
===> TYP: [Ljava.lang.Double; / OBJEKT: Typ: Double[], Laenge: 5
    [0] Typ: Double, Wert: 45.719
    [1] Typ: Double, Wert: 70.924
    [2] Typ: Double, Wert: 72.467
    [3] Typ: Double, Wert: 72.047
    [4] Typ: Double, Wert: 52.264
===> TYP: [Ljava.lang.Boolean; / OBJEKT: Typ: Boolean[], Laenge: 11
    [0] Typ: Boolean, Wert: true
    [1] Typ: Boolean, Wert: false
    [2] Typ: Boolean, Wert: true
    [3] Typ: Boolean, Wert: false
    [4] Typ: Boolean, Wert: true
    [5] Typ: Boolean, Wert: true
    [6] Typ: Boolean, Wert: true
    [7] Typ: Boolean, Wert: false
    [8] Typ: Boolean, Wert: true
    [9] Typ: Boolean, Wert: false
    [10] Typ: Boolean, Wert: true
===> TYP: AAA01 / OBJEKT: null-Referenz
===> TYP: AAA02 / OBJEKT: null-Referenz
===> TYP: AAA03 / OBJEKT: null-Referenz
===> TYP: BBB01 / OBJEKT: Typ: BBB01, Wert: BBB01-Objekt: {{{7 17 4 16 6 17 10 15 }{17
     10 17 14 2 9 15 17 17 }{9 0 11 1 2 }{1 16 18 18 16 11 0
     }{12 1 11 15 9 13 4 0 11 }{5 3 7 0 0 17 19 13 3 7 }{17
    4 1 }{15 11 15 9 14 15 0 }{17 6 5 18 16 0 15 10 12 11 17
     }}{{2 16 }{16 0 4 6 }{12 12 12 11 18 11 10 13 }{3 7 2 1
     7 }{0 1 12 2 16 5 0 10 18 }}{{7 16 }{12 11 }{11 1 }{3 4
     18 }{7 7 16 11 8 19 5 }}}
===> TYP: CCC01 / OBJEKT: Typ: CCC01, Wert: CCC01-Obj1
===> TYP: CCC02 / OBJEKT: Typ: CCC02, Wert: CCC02-Objekt:
CCC01-Obj2 | CCC01-Obj3 | CC
    C01-Obj4 | CCC01-Obj5 |
===> TYP: DDD01 / OBJEKT: null-Referenz
===> TYP: DDD02 / OBJEKT: null-Referenz
===> TYP: DDD03 / OBJEKT: null-Referenz
===> TYP: EEE01 / OBJEKT: Typ: EEE01, Wert: EEE01-Obj., vom public   Konstruktor erzeu
    gt!
-------------------------------------
Reflekt06Tst: Das war's erstmal!
------------------------------------------------------------------------ */
