// Datei Reflekt07.java
/* ------------------------------------------------------------------------
Mit den Klassenmehtoden erst, letzt, anzahl und naechster kann man sich
entsprechende Informationen ueber einen beliebigen Aufzaehlungstyp holen,
wenn man dess Namen als String angeben kann.
------------------------------------------------------------------------ */
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;
// ------------------------------------------------------------------------
enum Farbe {rot, gruen, blau}
enum Color {red, green, blue, striped}
// ------------------------------------------------------------------------
class Reflekt07 {
   // ---------------------------------------------------------------------
   static Enum[] valuesFuerName(String name) {
      // Falls name der (volle) Name eines enum-Typs ist, liefert diese
      // Funktion die Reihung aller Objekte ("Aufzaehlungswerte") dieses
      // Typs. Sonst wird null als Ergebnis geliefert.
      try {
         Class    cOb = Class.forName(name);             // Klassenobjekt
         Method   met = cOb.getDeclaredMethod("values"); // Methode values
         Object[] obr = (Object[]) met.invoke(null);     // values aufrufen
         return (Enum[]) obr;
      } catch (Throwable t) {
         pln("Ausnahme in valuesFuerName:\n" + t);       // Falls irgendwas
         return null;                                    // schiefging
      } // try/catch
   } // valuesFuerName
   // ---------------------------------------------------------------------
   static Enum erst(String aufzaehlungsName) {
      // Liefert den ersten Wert des angegebenen Aufzaehlungstyps
      Enum[] er = valuesFuerName(aufzaehlungsName);
      return (er==null ? null : er[0]);
   } // erst
   // ---------------------------------------------------------------------
   static Enum letzt(String aufzaehlungsName) {
      // Liefert den letzten Wert des angegebenen Aufzaehlungstyps
      Enum[] er = valuesFuerName(aufzaehlungsName);
      return (er==null ? null : er[er.length-1]);
   } // letzt
   // ---------------------------------------------------------------------
   static int anzahl(String aufzaehlungsName) {
      // Liefert die Anzahl der Werte des angegebenen Aufzaehlungstyps
      Enum[] er = valuesFuerName(aufzaehlungsName);
      return (er==null ? -1 : er.length);
   } // anzahl
   // ---------------------------------------------------------------------
   static Enum naechster(String aufzaehlungsName, Enum aw) {
      // Liefert den (zyklischen) Nachfolger des Wertes aw des
      // angegebenen Aufzaehlungstyps:
      Enum[] er = valuesFuerName(aufzaehlungsName);
      if (er==null) return null;
      for (int i=0; i<er.length; i++) {
         if (er[i] != aw) continue;
         return (i==er.length-1 ? er[0] : er[i+1]);
      }
      return null;
   } // anzahl
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      pln("Reflekt07: Jetzt geht es los!");
      pln("---------------------------------------");
      pln("erst      von            Farbe: " + erst     ("Farbe"));
      pln("erst      von            Color: " + erst     ("Color"));
      pln("---------------------------------------");
      pln("letzt     von            Farbe: " + letzt    ("Farbe"));
      pln("letzt     von            Color: " + letzt    ("Color"));
      pln("---------------------------------------");
      pln("anzahl    von            Farbe: " + anzahl   ("Farbe"));
      pln("anzahl    von            Color: " + anzahl   ("Color"));
      pln("---------------------------------------");
      pln("naechster von gruen   in Farbe: " + naechster("Farbe", Farbe.gruen));
      pln("naechster von blau    in Farbe: " + naechster("Farbe", Farbe.blau ));
      pln("naechster von rot     in Farbe: " + naechster("Farbe", Farbe.rot  ));
      pln("naechster von green   in Farbe: " + naechster("Farbe", Color.green));
      pln("---------------------------------------");
      pln("naechster von red     in Color: " + naechster("Color", Color.red    ));
      pln("naechster von green   in Color: " + naechster("Color", Color.green  ));
      pln("naechster von blue    in Color: " + naechster("Color", Color.blue   ));
      pln("naechster von striped in Color: " + naechster("Color", Color.striped));
      pln("naechster von gruen   in Color: " + naechster("Color", Farbe.gruen  ));
      pln("---------------------------------------");
      pln("erst      von          Couleur: " + erst     ("Couleur"));
      pln("anzahl    von          Couleur: " + anzahl   ("Couleur"));
      pln("---------------------------------------");
      pln("Reflekt07: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------

} // class Reflekt07
/* ------------------------------------------------------------------------
Ausgabe des Programms Reflekt07:

Reflekt07: Jetzt geht es los!
---------------------------------------
erst      von            Farbe: rot
erst      von            Color: red
---------------------------------------
letzt     von            Farbe: blau
letzt     von            Color: striped
---------------------------------------
anzahl    von            Farbe: 3
anzahl    von            Color: 4
---------------------------------------
naechster von gruen   in Farbe: blau
naechster von blau    in Farbe: rot
naechster von rot     in Farbe: gruen
naechster von green   in Farbe: null
---------------------------------------
naechster von red     in Color: green
naechster von green   in Color: blue
naechster von blue    in Color: striped
naechster von striped in Color: red
naechster von gruen   in Color: null
---------------------------------------
Ausnahme in valuesFuerName:
java.lang.ClassNotFoundException: Couleur
erst      von          Couleur: null
Ausnahme in valuesFuerName:
java.lang.ClassNotFoundException: Couleur
anzahl    von          Couleur: -1
---------------------------------------
Reflekt07: Das war's erstmal!
------------------------------------------------------------------------ */