// Datei Reihungen05.java
/* ------------------------------------------------------------------------
Eine Reihung srA, deren Komponenten Referenzvariablen sind, kann
man auf drei ganz verschiedene Weisen "kopieren":

Man kann den Wert der Variablen srA kopieren (siehe srB)

Flaches Kopieren: Man kopiert die Reihung srA, aber nicht die Objekte,
auf die die Komponenten von srA zeigen (siehe srC)

Tiefes Kopieren: Man kopiert die Objekte, auf die die Komponenten
von srA zeigen und fass diese Kopien zu einer neuen Reihung
zusammen (siehe srD)

Die Unterschiede zwischen diesen verschiedenen Weisen zu kopieren
werden z. B. dann deutlich, wenn man die Kopien veraendert und
durch geeignete Ausgaben prueft, auf welche Reihungen sich die
Aenderungen ausgewirkt haben.

Diese drei Weisen zu kopieren stehen in einer engen Beziehung
zu den drei Schritten, die zur Erzeugung einer Reihung von
Objekten notwendig sind (siehe Reihungen04).

Den Unterschied zwischen den drei Weisen zu kopieren kann man
sich z. B. dadurch klar machen, dass man das folgende Programm
"mit Papier und Bleistift" ausfuehrt und dabei die Variablen srA
bis srD als Bojen darstellt.

Siehe auch das Beispielprogramm Reihungen03.
------------------------------------------------------------------------ */
import java.util.Arrays;

class Reihungen05 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      pln("Reihungen05: Jetzt geht es los!");

      StringBuilder[] srA = {
         new StringBuilder("Hallo!"),
         new StringBuilder("Wie geht's?"),
         new StringBuilder("Gut!"),         // Komma hier erlaubt!
      };

      StringBuilder[] srB = srA;
      StringBuilder[] srC = new StringBuilder[srA.length];
      System.arraycopy(srA, 0, srC, 0, srA.length);
      StringBuilder[] srD = tiefeKopie(srA);

      pln("--------------------------------------------");
      pln("srA vorher : " + Arrays.toString(srA));
      pln("srB vorher : " + Arrays.toString(srB));
      pln("srC vorher : " + Arrays.toString(srC));
      pln("srD vorher : " + Arrays.toString(srD));

      // Das Original srA und die Kopien srB bis srD veraendern:
      srA[0].setCharAt(1, 'e');
      srB[0].setCharAt(5, '?');
      srC[2] = new StringBuilder("Bestens!");
      srD[2].setCharAt(3, '?');

      pln("--------------------------------------------");
      pln("srA nachher: " + Arrays.toString(srA));
      pln("srB nachher: " + Arrays.toString(srB));
      pln("srC nachher: " + Arrays.toString(srC));
      pln("srD nachher: " + Arrays.toString(srD));
      pln("--------------------------------------------");

      pln("Reihungen05: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   static public StringBuilder[] tiefeKopie(StringBuilder[] sr) {
      // Erzeugt Kopien der StringBuilder-Objekte sbr[0], sbr[1], ... etc.
      // und liefert eine Reihung dieser Kopien.
      // Die Klasse StringBuilder erbt von Object eine Methode clone,
      // aber diese clone-Methode ist nicht public (sondern protected)
      // und kann deshalb hier nicht zum Kopieren von StringBuilder-
      // Objeken verwendet werden.

      StringBuilder[] erg = new StringBuilder[sr.length];
      for (int i=0; i<erg.length; i++) {
         erg[i] = new StringBuilder(sr[i]);
      }
      return erg;
   } // tiefeKopie
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Reihungen05
/* ------------------------------------------------------------------------
Ausgabe des Programms Reihungen05:

Reihungen05: Jetzt geht es los!
--------------------------------------------
srA vorher : [Hallo!, Wie geht's?, Gut!]
srB vorher : [Hallo!, Wie geht's?, Gut!]
srC vorher : [Hallo!, Wie geht's?, Gut!]
srD vorher : [Hallo!, Wie geht's?, Gut!]
--------------------------------------------
srA nachher: [Hello?, Wie geht's?, Gut!]
srB nachher: [Hello?, Wie geht's?, Gut!]
srC nachher: [Hello?, Wie geht's?, Bestens!]
srD nachher: [Hallo!, Wie geht's?, Gut?]
--------------------------------------------
Reihungen05: Das war's erstmal!
------------------------------------------------------------------------ */
