// Datei Reihungen13.java
/* ------------------------------------------------------------------------
Der Modul (oder: die Klasse) java.util.Arrays enthaelt eine Reihe von
nuetzlichen Methoden zur Bearbeitung von Reihungen: fill, binarySearch,
equals, und sort. Diese Methoden werden hier auf Reihungen des Typs int[]
angewendet, d.h. auf Reihungen mit Komponenten eines primitiven Typs.
------------------------------------------------------------------------ */
import java.util.Arrays;

class Reihungen13 {
   static public void main(String[] sonja) {

      // Vier int-Reihungen als "Rohmaterial":
      int[] ir01 = new int[1000];
      int[] ir02 = new int[1000];
      int[] ir03 = new int[1000];
      int[] ir04 = {33, 25, 17, 22, 31};

      // Die Reihungen ir01 und ir02 werden mit je 1000 Werten 17 gefuellt:
      Arrays.fill(ir01, 17);
      Arrays.fill(ir02, 17);

      // Die Reihung ir03 wird zur Haelfte mit dem Wert 3 und
      //                       zur Haelfte mit dem Wert 5 gefuellt:
      Arrays.fill(ir03,   0, 499, 3);
      Arrays.fill(ir03, 500, 999, 5);

      // Ein Funktionsaufruf wie Arrays.equals(ir01, ir02) bewirkt, dass
      // die Zielwerte der Variablen ir01 und ir02 (die Reihungen ir01
      // und ir02) verglichen werden, nicht die Werte der Variablen (d.h.
      // nicht die Referenzen der Reihungen)

      // Ein Funktionsaufruf wie ir01.equals(ir02) bewirkt, dass die
      // Werte der Variablen ir01 und ir02 verglichen werden, nicht die
      // Reihungen selbst (d.h. nicht die Zielwerte der Variablen ir01
      // und ir02).
      pln("-------------------------------------------------------------");
      pln("A Arrays.equals(ir01, ir02): " + Arrays.equals(ir01, ir02)    );
      pln("B ir01.equals(ir02):         " + (ir01.equals(ir02) )         );
      pln("-------------------------------------------------------------");
      pln("C Arrays.equals(ir02, ir03): " + Arrays.equals(ir02, ir03)    );
      pln("D ir02.equals(ir03):         " + (ir01.equals(ir03) )         );

      // Mit der Prozedur Arrays.sort kann man eine Reihung aufsteigend
      // sortieren:
      pln("-------------------------------------------------------------");
      p  ("E "); druckeIntReihung_S(ir04, "ir04");
      pln("F Arrays.sort(ir04);"); Arrays.sort(ir04);
      p  ("G "); druckeIntReihung_S(ir04, "ir04");

      // Die Funktionen Arrays.binarySearch funktionieren nur dann richtig,
      // wenn man sie auf eine aufsteigend sortierte Reihung anwendet.

      // Ein Funktionsaufruf wie Arrays.binarySearch(ir01, 31) liefert
      // den Index, an dem der Wert 31 in der Reihung ir01 steht.

      // Ein Funktionsaufruf wie Arrays.binarySearch(ir01, 28) liefert
      // einen negativen Wert, weil 28 nicht in ir01 vorkommt. Genauer:
      // Das Ergebnis ist (- EinfuegeIndex - 1), wobei EinfuegeIndex die
      // Stelle in der Reihung ir01 bezeichnet, an der der Wert 28 einge-
      // fuegt werden muesste, damit die Sortierung erhalten bleibt.
      pln("-------------------------------------------------------------");
      pln("H Arrays.binarySearch(ir04, 31): " + Arrays.binarySearch(ir04, 31));
      pln("I Arrays.binarySearch(ir04, 28): " + Arrays.binarySearch(ir04, 28));
      pln("-------------------------------------------------------------");
      pln("Reihungen13: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   static void druckeIntReihung_S(int[] ir, String name) {
      // Mit sicheren und einfachen for-Schleifen:
      if (ir == null) {
         pln(name + ": null");
         return;
      } // if

      p(name + ": " + ir.length + " Komponenten: ");
      for (int i: ir) {
         p(i + " ");
      }
      pln();
   } // druckeIntReihung_S
   // ---------------------------------------------------------------------
   static void druckeIntReihung_A(int[] ir, String name) {
      // Mit alten for-Schleifen mit Index:
      if (ir == null) {
         pln(name + ": null");
         return;
      } // if

      p(name + ": " + ir.length + " Komponenten: ");
      for (int i=0; i<ir.length; i++) {
         p(ir[i] + " ");
      } // for i
      pln();
   } // druckeIntReihung_A
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // Reihungen13
/* ------------------------------------------------------------------------
Ausgabe des Programms Reihungen13:

-------------------------------------------------------------
A Arrays.equals(ir01, ir02): true
B ir01.equals(ir02):         false
-------------------------------------------------------------
C Arrays.equals(ir02, ir03): false
D ir02.equals(ir03):         false
-------------------------------------------------------------
E ir04: 5 Komponenten: 33 25 17 22 31
F Arrays.sort(ir04);
G ir04: 5 Komponenten: 17 22 25 31 33
-------------------------------------------------------------
H Arrays.binarySearch(ir04, 31): 3
I Arrays.binarySearch(ir04, 28): -4
-------------------------------------------------------------
Reihungen13: Das war's erstmal!
------------------------------------------------------------------------ */
