// Datei Reihungen15.java
/* ------------------------------------------------------------------------
Wenn man eine Reihung mit der Methode System.out.print "einfach so"
ausgibt, erscheint auf dem Bildschirm eine Zeichenkette, an der man
den Typ der Reihung erkennen kann, die ansonsten aber nicht sehr
informativ ist (siehe unten die Ausgabezeilen AA bis AE).

Mit der Methode Arrays.toString kann man eine einstufige Reihung
in einen String umwandeln, der die Komponenten der Reihung darstellt
(siehe Ausgabezeilen BA bis BC. Man kann diese Methode auch auf
mehrstufige Reihungen anwenden, die dann wie einstufige Reihungen
behandelt werden (siehe Ausgabezeilen BD und BE).

Mit der Methode Array.toDeepString kann man eine einstufige Reihung
von Objekten oder eine beliebige mehrstufige Reihung in einen
String umwandeln, der die elementaren Reihungskomponenten ("die,
die selbst keine Reihungen sind") darstellt (siehe Ausgabezeilen CC
bis CE). Diese Methode darf man nicht auf einstufige Reihungen mit
primitiven Komponenten anwenden (siehe Fehlermeldungen des Com-
pilers am Ende dieser Datei). Die Klasse Arrays gehoert zur
Standardbibliothek.

Mit der Methode Objects.toString kann man ein beliebiges Objekt in
einen String umwandeln. Falls das Objekt irgendeine Reihung ist,
enthaelt dieser String den Typ der Reihung und ihre Komponenten.
Falls auch die Komponenten Reihungen sind, enthaelt der String
auch deren Typen und Komponenten u.s.w. (siehe Ausgabezeilen DA
bis DE). Die Klasse Objects gehoert *nicht* zur Standardbibliothek
(ist aber frei verfuegbar).
------------------------------------------------------------------------ */
import java.util.Arrays;

class Reihungen15 {
   static public void main(String[] sonja) {
      // ------------------------------------------------------------------
      // Zwei einstufige Reihungen mit primitiven Komponenten:
      int[]     ir01 = {25, 17, 22, 33, 44, 55, 66};
      boolean[] br01 = {true, 3<5, false, true, 3==5};

      // Eine einstufige Reihung von Objekten:
      String[]  sr01 = {
         "Vom Eise befreit sind Strom und Bche",
         "Durch des Frhlings holden belebenden Blick.",
         "Im Tale blhet Hoffnungsglck."
      };

      // Eine Reihung, die zwei einstufige Reihungen, ein Strin-Objekt
      // und ein Long-Objekt als Komponenten enthaelt:
      Object[] or01 = {ir01, br01, "Kompo. mit Index 3", new Long(123)};

      // Eine zweistufige Reihung mit elementaren Komponenten des
      // primitiven Typs long (aber mit Komponenten des Referenztyps
      // long[], d.h. Reihung von long):
      long[][] rr01 = new long[3][35]; // Zweistufig!
      // ------------------------------------------------------------------
      // Gibt man Reihungen "einfach so" mit der Methode System.out.println
      // (alias pln) aus, erfaehrt man nichts ueber ihre Komponenten:
      pln("Reihungen15: Jetzt geht es los!");
      pln("=============================================================");
      pln("AA ir01: " + ir01);
      pln("-------------------------------------------------------------");
      pln("AB br01: " + br01);
      pln("-------------------------------------------------------------");
      pln("AC sr01: " + sr01);
      pln("-------------------------------------------------------------");
      pln("AD or01: " + or01);
      pln("-------------------------------------------------------------");
      pln("AE rr01: " + rr01);

      // Wandelt man die Reihungen vor dem Ausgeben mit der Methode
      // Arrays.toString in Strings um, erfaehrt man mehr:
      pln("=============================================================");
      pln("BA ir01: " + ir01 + "\n" + Arrays.toString(ir01));
      pln("-------------------------------------------------------------");
      pln("BB br01: " + br01 + "\n" + Arrays.toString(br01));
      pln("-------------------------------------------------------------");
      pln("BC sr01: " + sr01 + "\n" + Arrays.toString(sr01));
      pln("-------------------------------------------------------------");
      pln("BD or01: " + or01 + "\n" + Arrays.toString(or01));
      pln("-------------------------------------------------------------");
      pln("BE rr01: " + rr01 + "\n" + Arrays.toString(rr01));

      // Die Methode Arrays.deepToString darf man nicht auf einstufige
      // Reihungen mit primitiven Komponenten (wie z.B. ir01) anwenden,
      // aber auf einstufige Reihungen von Objekten (wie z.B. sr01) oder
      // und auf alle mehrstufigen Reihungen (wie z.B. or01 und rr01):
      pln("=============================================================");
//    pln("CA ir01: " + ir01 + "\n" + Arrays.deepToString(ir01));
//    pln("-------------------------------------------------------------");
//    pln("CB br01: " + br01 + "\n" + Arrays.deepToString(br01));
//    pln("-------------------------------------------------------------");
      pln("CC sr01: " + sr01 + "\n" + Arrays.deepToString(sr01));
      pln("-------------------------------------------------------------");
      pln("CD or01: " + or01 + "\n" + Arrays.deepToString(or01));
      pln("-------------------------------------------------------------");
      pln("CE rr01: " + rr01 + "\n" + Arrays.deepToString(rr01));

      // Wandelt man die Reihungen vor dem Ausgeben mit der Methode
      // Objects.toString in Strings um, erfaehrt man noch mehr:
      pln("=============================================================");
      pln("DA ir01: " + Objects.toString(ir01));
      pln("-------------------------------------------------------------");
      pln("DB br01: " + Objects.toString(br01));
      pln("-------------------------------------------------------------");
      pln("DC sr01: " + Objects.toString(sr01));
      pln("-------------------------------------------------------------");
      pln("DD or01: " + Objects.toString(or01));
      pln("-------------------------------------------------------------");
      Objects.setZeichenProERS(3); // Zeichen pro Einrueckstufe
      pln("DE rr01: " + Objects.toString(rr01));
      pln("=============================================================");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // Reihungen15
/* ------------------------------------------------------------------------
Ausgabe des Programms Reihungen15:

Reihungen15: Jetzt geht es los!
=============================================================
AA ir01: [I@10b62c9
-------------------------------------------------------------
AB br01: [Z@82ba41
-------------------------------------------------------------
AC sr01: [Ljava.lang.String;@923e30
-------------------------------------------------------------
AD or01: [Ljava.lang.Object;@130c19b
-------------------------------------------------------------
AE rr01: [[J@1f6a7b9
=============================================================
BA ir01: [I@10b62c9
[25, 17, 22, 33, 44, 55, 66]
-------------------------------------------------------------
BB br01: [Z@82ba41
[true, true, false, true, false]
-------------------------------------------------------------
BC sr01: [Ljava.lang.String;@923e30
[Vom Eise befreit sind Strom und Bche, Durch des Frhlings holden belebenden Blick., Im Tale blhet Hoffnungsglck.]
-------------------------------------------------------------
BD or01: [Ljava.lang.Object;@130c19b
[[I@10b62c9, [Z@82ba41, Kompo. mit Index 3, 123]
-------------------------------------------------------------
BE rr01: [[J@1f6a7b9
[[J@7d772e, [J@11b86e7, [J@35ce36]
=============================================================
CC sr01: [Ljava.lang.String;@923e30
[Vom Eise befreit sind Strom und Bche, Durch des Frhlings holden belebenden Blick., Im Tale blhet Hoffnungsglck.]
-------------------------------------------------------------
CD or01: [Ljava.lang.Object;@130c19b
[[25, 17, 22, 33, 44, 55, 66], [true, true, false, true, false], Kompo. mit Index 3, 123]
-------------------------------------------------------------
CE rr01: [[J@1f6a7b9
[[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
=============================================================
DA ir01: Typ: int[], Laenge: 7
    {25, 17, 22, 33, 44, 55, 66}
-------------------------------------------------------------
DB br01: Typ: boolean[], Laenge: 5
    {true, true, false, true, false}
-------------------------------------------------------------
DC sr01: Typ: String[], Laenge: 3
    [0] Typ: String, Wert: Vom Eise befreit sind Strom und B
        che
    [1] Typ: String, Wert: Durch des Frhlings holden belebe
        nden Blick.
    [2] Typ: String, Wert: Im Tale blhet Hoffnungsglck.
-------------------------------------------------------------
DD or01: Typ: Object[], Laenge: 4
    [0] Typ: int[], Laenge: 7
        {25, 17, 22, 33, 44, 55, 66}
    [1] Typ: boolean[], Laenge: 5
        {true, true, false, true, false}
    [2] Typ: String, Wert: Kompo. mit Index 3
    [3] Typ: Long, Wert: 123
-------------------------------------------------------------
DE rr01: Typ: long[][], Laenge: 3
   [0] Typ: long[], Laenge: 35
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
   [1] Typ: long[], Laenge: 35
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
   [2] Typ: long[], Laenge: 35
      {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
       0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
=============================================================
---------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die entsprechendeN ZeileN kein
Kommentar ist:

D:\Java\Reihungen15.java:88: cannot find symbol
symbol  : method deepToString(int[])
location: class java.util.Arrays
      pln("CA ir01: " + ir01 + "\n" + Arrays.deepToString(ir01));
                                            ^
D:\Java\Reihungen15.java:90: cannot find symbol
symbol  : method deepToString(boolean[])
location: class java.util.Arrays
      pln("CB br01: " + br01 + "\n" + Arrays.deepToString(br01));
                                            ^
------------------------------------------------------------------------ */
