// Datei Reihungen16.java
/* ------------------------------------------------------------------------
Ob zwei einstufige Reihungen gleich sind, kann man mit Hilfe von
drei Operationen/Methoden untersuchen:

1. Der Gleichheitsoperator == liefert nur dann true, wenn die verglichenen
   Reihungen identisch sind (siehe Ausgabezeilen A, E, H etc.).
2. Die Objektmethode equals, die in jedem Reihungsobjekt enthalten ist,
   liefert ebenfalls nur bei identischen Reihungen true (siehe Ausgabe-
   zeilen B, F, I etc.).
3. Die Methode Arrays.equals liefert true, wenn die verglichenen
   Reihungen gleiche Komponenten haben. Falls die Komponenten Objekte
   sind, werden sie dabei mit ihrer eigenen equals-Methode verglichen
   (siehe Ausgabezeilen C, G, J, N, R).

Ob zwei mehrstufige Reihungen gleich sind, kann man wie bei einstufigen
Reihungen mit 1. bis 3. und zusaetzlich mit 4. untersuchen:

4. Die Methode Arrays.deepEquals liefert true, wenn die verglichenen
   Reihungen gleiche Komponenten haben. Falls die Komponenten ebenfalls
   Reihungen sind, werden sie (nicht mit ihrer eigenen equals-Methode
   sondern) mit der Methode Arrays.deepEquals verglichen (siehe Aus-
   gabezeilen K, O, S und Z).

Merke: Die Methode Arrays.deepEquals darf man nicht auf einstufige
Reihungen mit primitiven Komponenten anwenden, wohl aber auf einstufige
Reihungen von Objekten und auf alle mehrstufigen Reihungen.
------------------------------------------------------------------------ */
import static java.util.Arrays.equals;
import static java.util.Arrays.deepEquals;
import java.util.Arrays;

class Reihungen16 {
   static public void main(String[] sonja) {
      // ------------------------------------------------------------------
      // Ein- und zweistufige Reihungen verschiedener Typen werden
      // vereinbart:

      // Zwei einstufige Reihungen mit gleichen Komponenten:
      int[]     ir11 = { 11, 22,   33, 44, 55};
      int[]     ir12 = { 11, 22,   33, 44, 55};

      // Zwei zweistufige Reihungen mit gleichen elementaren Komponenten:
      int[][]   ir21 = {{11, 22}, {33, 44, 55}};
      int[][]   ir22 = {{11, 22}, {33, 44, 55}};

      // Zwei Reihungen von Object-Objekten, die einen String, eine
      // einstufige und eine zweistufige Reihung als Komponenten
      // enthalten. Die Reihungen or21 und or22 sind somit
      // "ein- bis dreistufig". Sie haben gleiche elementare
      // Komponenten.
      Object[]  or21 = {"Hallo!", ir11, ir21};
      Object[]  or22 = {"Hallo!", ir11, ir21};

      // Zwei Reihungen mit nur einer Komponente. Vergleicht man diese
      // Komponenten-Objekte mit ihrer eigenen equals-Methode, so ergibt
      // sich true (siehe Ausgabezeilen T und U, vergleicht man sie mit
      // dem Gleichheitsoperator == so ergibt sich false (siehe Ausgabe-
      // zeile V):
      String    st01 = new String("Hallo!"); // st01.equals(st02) ist true
      String    st02 = new String("Hallo!"); // st01 == st02      ist false

      String[]  sr11 = {st01};
      String[]  sr12 = {st02};


      pln("Reihungen16: Jetzt geht es los!");
      pln("--------------------------------------");
      pln("A       ir11   ==   ir11       : " + (      ir11   ==   ir11       )); // true
      pln("B       ir11.equals(ir11)      : " + (      ir11.equals(ir11)      )); // true
      pln("C Arrays.    equals(ir11, ir11): " + (Arrays.    equals(ir11, ir11))); // true
//    pln("D Arrays.deepEquals(ir11, ir11): " + (Arrays.deepEquals(ir11, ir11)));
      pln("--------------------------------------");
      pln("E       ir11   ==   ir12       : " + (      ir11   ==   ir12       )); // false
      pln("F       ir11.equals(ir12)      : " + (      ir11.equals(ir12)      )); // false
      pln("G Arrays.    equals(ir11, ir12): " + (Arrays.    equals(ir11, ir12))); // true
      pln("--------------------------------------");
      pln("H       ir21   ==   ir21       : " + (      ir21   ==   ir21       )); // true
      pln("I       ir21.equals(ir21)      : " + (      ir21.equals(ir21)      )); // true
      pln("J Arrays.    equals(ir21, ir21): " + (Arrays.    equals(ir21, ir21))); // true
      pln("K Arrays.deepEquals(ir21, ir21): " + (Arrays.deepEquals(ir21, ir21))); // true
      pln("--------------------------------------");
      pln("L       ir21   ==   ir22       : " + (      ir21   ==   ir22       )); // false
      pln("M       ir21.equals(ir22)      : " + (      ir21.equals(ir22)      )); // false
      pln("N Arrays.    equals(ir21, ir22): " + (Arrays.    equals(ir21, ir22))); // false
      pln("O Arrays.deepEquals(ir21, ir22): " + (Arrays.deepEquals(ir21, ir22))); // true
      pln("--------------------------------------");
      pln("P       or21   ==   or22       : " + (      or21   ==   or22       )); // false
      pln("Q       or21.equals(or22)      : " + (      or21.equals(or22)      )); // false
      pln("R Arrays.    equals(or21, or22): " + (Arrays.    equals(or21, or22))); // true
      pln("S Arrays.deepEquals(or21, or22): " + (Arrays.deepEquals(or21, or22))); // true
      pln("--------------------------------------");
      pln("T    sr11[0].equals(sr12[0])   : " + (sr11[0].equals(sr12[0]))      ); // true
      pln("U    sr12[0].equals(sr11[0])   : " + (sr12[0].equals(sr11[0]))      ); // true
      pln("V    sr12[0]   ==   sr11[0]    : " + (sr12[0]   ==   sr11[0])       ); // false
      pln("--------------------------------------");
      pln("W       sr11   ==   sr12       : " + (      sr11   ==   sr12       )); // false
      pln("X       sr11.equals(sr12)      : " + (      sr11.equals(sr12)      )); // false
      pln("Y Arrays.    equals(sr11, sr12): " + (Arrays.    equals(sr11, sr12))); // true
      pln("Z Arrays.deepEquals(sr11, sr12): " + (Arrays.deepEquals(sr11, sr12))); // true
      pln("--------------------------------------");

   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // Reihungen16
/* ------------------------------------------------------------------------
Ausgabe des Programms Reihungen16:

Reihungen16: Jetzt geht es los!
--------------------------------------
A       ir11   ==   ir11       : true
B       ir11.equals(ir11)      : true
C Arrays.    equals(ir11, ir11): true
--------------------------------------
E       ir11   ==   ir12       : false
F       ir11.equals(ir12)      : false
G Arrays.    equals(ir11, ir12): true
--------------------------------------
H       ir21   ==   ir21       : true
I       ir21.equals(ir21)      : true
J Arrays.    equals(ir21, ir21): true
K Arrays.deepEquals(ir21, ir21): true
--------------------------------------
L       ir21   ==   ir22       : false
M       ir21.equals(ir22)      : false
N Arrays.    equals(ir21, ir22): false
O Arrays.deepEquals(ir21, ir22): true
--------------------------------------
P       or21   ==   or22       : false
Q       or21.equals(or22)      : false
R Arrays.    equals(or21, or22): true
S Arrays.deepEquals(or21, or22): true
--------------------------------------
T    sr11[0].equals(sr12[0])   : true
U    sr12[0].equals(sr11[0])   : true
V    sr12[0]   ==   sr11[0]    : false
--------------------------------------
W       sr11   ==   sr12       : false
X       sr11.equals(sr12)      : false
Y Arrays.    equals(sr11, sr12): true
Z Arrays.deepEquals(sr11, sr12): true
--------------------------------------
---------------------------------------------------------------------------
Fehlermeldung des Compilers, wenn die entsprechende Zeile kein
Kommentar ist:

D:\Java\Reihungen16.java:72:
deepEquals(java.lang.Object[],java.lang.Object[]) in java.util.Arrays
cannot be applied to (int[],int[])
    pln("D Arrays.deepEquals(ir11, ir11): " + (Arrays.deepEquals(ir11, ir11)));
                                                     ^
------------------------------------------------------------------------ */
