// Datei Return01.java
/* ------------------------------------------------------------------------
Prozeduren koennen normal oder durch return-Anweisungen beendet werden.
Funktionen duerfen nicht normal, sondern muessen mit return-Anweisungen
beendet werden. In einer Prozedur darf nach return kein Ausdruck stehen,
in einer Funktion muss nach return ein Ausdruck des Rueckgabetyps stehen.
------------------------------------------------------------------------ */
class Return01 {
   // -----------------------------------------------------------
   // Eine Prozedur, die immer normal beendet wird (d.h. nachdem
   // der Ausfuehrer den letzten Befehl des Rumpfes ausgefuehrt
   // hat, ohne auf einen return-Befehl zu stossen):
   static void proz01(int n) {
      if (n < 0) {
         pln("A  n ist negativ!");
      } else if (0 < n) {
         pln("B  n ist positiv!");
      } else {
         pln("C  n ist gleich 0!");
      }
   } // proz01
   // -----------------------------------------------------------
   // Eine Prozedur, die manchmal durch eine return-Anweisung
   // und manchmal normal beendet wird:
   static void proz02(int n) {
      String s = "n ist ";
      if (n < 0) {
         pln("D  " + s + "negativ!");
         return;
      }

      if (0 < n) {
         pln("E  " + s + "positiv!");
         return;
      }

      pln("F  " + s + "gleich 0!");
   } // proz02
   // -----------------------------------------------------------
   // Eine Funktion darf nie normal beendet werden.
   static String funk01(int n) {
      String s = "n ist ";
      if (n < 0) {
         return "G  " + s + "negativ!";
      }

      if (0 < n) {
         return "H  " + s + "positiv!";
      }

      return "I  " + s + "gleich 0!";
   } // funk01
   // -----------------------------------------------------------
   // Verschiedene erlaubte und nicht erlaubte Aufrufe von
   // Funktionen und Prozeduren:

   // Ein Funktionsaufruf ausserhalb aller Methoden, als Teil
   // einer Variablen-Vereinbarung:
   String s1 = funk01(17);    // Erlaubt,       sinnvoll
// proz02(17);                // Nicht erlaubt

   // Prozeduren duerfen (fast) nur innerhalb von Methoden
   // aufgerufen werden.
   static void irgendEineMethode() {
      proz02(17);             // Erlaubt,       sinnvoll
      funk01(17);             // Erlaubt, nicht sinnvoll
      String s2 = funk01(17); // Erlaubt,       sinnvoll
//    String s3 = proz02(17); // Nicht erlaubt
   } // irgendEineMethode
   // -----------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // -----------------------------------------------------------
   static public void main(String[] sonja) {
      pln("----------------------------");
      proz01(-3);
      proz01(+5);
      proz01( 0);
      pln("----------------------------");
      proz02(-3);
      proz02(+5);
      proz02( 0);
      pln("----------------------------");
      pln(funk01(-3));
      pln(funk01(+5));
      pln(funk01( 0));
      pln("----------------------------");
      pln("Return01: Das war's erstmal!");
   } // main
   // -----------------------------------------------------------
} // class Return01
/* ------------------------------------------------------------------------
Ausgabe des Programms Return01:

----------------------------
A  n ist negativ!
B  n ist positiv!
C  n ist gleich 0!
----------------------------
D  n ist negativ!
E  n ist positiv!
F  n ist gleich 0!
----------------------------
G  n ist negativ!
H  n ist positiv!
I  n ist gleich 0!
----------------------------
Return01: Das war's erstmal!
------------------------------------------------------------------------ */