// Datei Sammlungen02.java
/* ------------------------------------------------------------------------
Die Methoden addAll, containsAll, removeAll und retainsAll aus der
Schnittstelle Collection<K> werden auf Sammlungen der Klassen TreeSet
und PriorityQueue angewendet. TreeSet-Sammlungen sind sortiert und
duerfen keine Doppelgaenger enthalten. In einer PriorityQueue-Sammlung
steht die kleinste Komponente immer vorn ("Kopf-Komponente") und Doppel-
gaenger sind erlaubt.
------------------------------------------------------------------------ */
import java.util.Collection;
import java.util.TreeSet;
import java.util.PriorityQueue;
import java.util.Arrays;

class Sammlungen02 {
   static public void main(String[] sonja) {

      // Reihungen von String-Objekten, mit denen spaeter Sammlungen
      // initialisiert werden sollen:
      String[] sr01 = {"bb", "aa", "cc"};
      String[] sr02 = {"dd", "aa", "ee"};
      String[] sr03 = {"dd", "aa"};
      String[] sr04 = {"ff", "gg", "aa", "bb", "cc"};

      // Sammlungen (collection objects) werden erzeugt und mit
      // Hilfe von Reihungen initialisiert:
      Collection<String> c1 = new       TreeSet<String>(Arrays.asList(sr01));
      Collection<String> c2 = new       TreeSet<String>(Arrays.asList(sr02));
      Collection<String> c3 = new       TreeSet<String>(Arrays.asList(sr03));
      Collection<String> c4 = new       TreeSet<String>(Arrays.asList(sr04));

      Collection<String> p1 = new PriorityQueue<String>(Arrays.asList(sr01));
      Collection<String> p2 = new PriorityQueue<String>(Arrays.asList(sr02));
      Collection<String> p3 = new PriorityQueue<String>(Arrays.asList(sr03));
      Collection<String> p4 = new PriorityQueue<String>(Arrays.asList(sr04));

      // Die Sammlungen werden ausgegeben und bearbeitet. Die
      // Ergebnisse der Bearbeitung werden ebenfalls ausgegeben:
      pln("Sammlungen02: Jetzt geht es los!");
      pln("----------------------------------------------------");
      pln("A sr01                    : " + Arrays.toString(sr01));
      pln("B sr02                    : " + Arrays.toString(sr02));
      pln("C sr03                    : " + Arrays.toString(sr03));
      pln("D sr04                    : " + Arrays.toString(sr04));
      pln("----------------------------------------------------");
      pln("E c1                      : " + c1);
      pln("F c2                      : " + c2);
      pln("G c3                      : " + c3);
      pln("H c4                      : " + c4);
      pln("----------------------------------------------------");
      pln("I c1.containsAll(c2)      : " + c1.containsAll(c2));
      pln("J c2.containsAll(c3)      : " + c2.containsAll(c3));
      pln("----------------------------------------------------");
      c1.addAll(c2);
      pln("K c1 nach c1.addAll   (c2): " + c1);

      c1.removeAll(c2);
      pln("L c1 nach c1.removeAll(c2): " + c1);

      c1.addAll(c2);
      pln("M c1 nach c1.addAll   (c2): " + c1);

      c1.retainAll(c4);
      pln("N c1 nach c1.retainAll(c4): " + c1);
      pln("----------------------------------------------------");
      pln("O p1                      : " + p1);
      pln("P p2                      : " + p2);
      pln("Q p3                      : " + p3);
      pln("R p4                      : " + p4);
      pln("----------------------------------------------------");
      pln("S p1.containsAll(p2)      : " + p1.containsAll(p2));
      pln("T p2.containsAll(p3)      : " + p2.containsAll(p3));
      pln("----------------------------------------------------");
      p1.addAll(p2);
      pln("U p1 nach p1.addAll   (p2): " + p1);

      p1.removeAll(p2);
      pln("V p1 nach p1.removeAll(p2): " + p1);

      p1.addAll(p2);
      pln("W p1 nach p1.addAll   (p2): " + p1);

      p1.retainAll(p4);
      pln("X p1 nach p1.retainAll(p4): " + p1);
      pln("----------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // Sammlungen02
/* ------------------------------------------------------------------------
Ausgabe des Programms Sammlungen02:

Sammlungen02: Jetzt geht es los!
----------------------------------------------------
A sr01                    : [bb, aa, cc]              <-- Reihungen
B sr02                    : [dd, aa, ee]
C sr03                    : [dd, aa]
D sr04                    : [ff, gg, aa, bb, cc]
----------------------------------------------------
E c1                      : [aa, bb, cc]              <-- TreeSet-Sammlungen
F c2                      : [aa, dd, ee]
G c3                      : [aa, dd]
H c4                      : [aa, bb, cc, ff, gg]      <-- Kein Doppelgaenger
----------------------------------------------------
I c1.containsAll(c2)      : false
J c2.containsAll(c3)      : true
----------------------------------------------------
K c1 nach c1.addAll   (c2): [aa, bb, cc, dd, ee]
L c1 nach c1.removeAll(c2): [bb, cc]
M c1 nach c1.addAll   (c2): [aa, bb, cc, dd, ee]
N c1 nach c1.retainAll(c4): [aa, bb, cc]
----------------------------------------------------
O p1                      : [aa, bb, cc]              <-- PriorityQueue-Sammlungen
P p2                      : [aa, dd, ee]
Q p3                      : [aa, dd]
R p4                      : [aa, bb, ff, gg, cc]
----------------------------------------------------
S p1.containsAll(p2)      : false
T p2.containsAll(p3)      : true
----------------------------------------------------
U p1 nach p1.addAll   (p2): [aa, aa, cc, bb, dd, ee]  <-- Doppelgaenger!
V p1 nach p1.removeAll(p2): [bb, cc]
W p1 nach p1.addAll   (p2): [aa, cc, bb, dd, ee]
X p1 nach p1.retainAll(p4): [aa, cc, bb]
----------------------------------------------------
------------------------------------------------------------------------ */