// Datei Sammlungen04.java
/* ------------------------------------------------------------------------
Ist rD eine Reihung von z.B. Double-Objekten (und nicht eine Reihung von
primitiven Werten), dann liefert der Aufruf Arrays.asList(rD) eine List-
Sammlung fester Laenge, die die Komponenten von rD enthaelt, z.B. so:

List<Double> lD = Arrays.asList(rD);

Falls man die Liste lD (bzw. die Reihung rD) aendert, aendert man dadurch
automatisch auch die Reihung rD (bzw. die Liste lD). Methoden, die die
Laenge der Liste lD veraendern wuerden, funktionieren "nicht richtig",
sonder werfen nur eine Ausnahme des Typs UnsupportedOperationException.
---------------------------------------------------------------------------
Feinheiten: Ein Methodenaufruf wie lD.remove(2.0) loest nur dann eine
Ausnahme aus, wenn die Liste lD ein Double-Objekt mit dem Wert 2.0
enthaelt (denn andernfalls wuerde die Methode remove nichts entfernen
und damit auch nicht versuchen, die Laenge der Liste zu veraendern).

Fuer die anderen "potentiell laengenveraendernden" Methoden addAll,
removeAll, retainAll und clear gilt ganz entsprechendes.

Bei Sammlungen, die Doppelgaenger erlauben, wirkt die Methode add
immer "laengenveraendernd" und ein Aufruf wie lD.add(2.0) loest
deshalb immer eine Ausnahme aus.

Die Methode set ist zwar "potentiell veraendernd", aber nie "laengen-
veraendernd". Deshalb loest sie nie eine Ausnahme des Typs
UnsupportedOperationException aus. Die Methoden contains und containsAll
sind nicht veraendernd und loesen deshalb ebenfalls keine Ausnahme aus.
------------------------------------------------------------------------ */
import java.util.List;
import java.util.Arrays;

class Sammlungen04 {
   // ---------------------------------------------------------------------
   // Reihungen von Double-Objekten:
   static Double[] rD1={new Double(1.0), new Double(2.0), new Double(3.0)};
   static Double[] rD2={new Double(4.0), new Double(5.0), new Double(6.0)};
   static Double[] rD3={};

   // Mit der Methode Arrays.asList werden "List-Sichten auf die Reihungen"
   // erzeugt. Die List-Sichten (oder Listen) haben eine unveraenderbare
   // Laenge (wie die Reihungen).
   static List<Double> lD1 = Arrays.asList(rD1);
   static List<Double> lD2 = Arrays.asList(rD2);
   static List<Double> lD3 = Arrays.asList(rD3);
   // ---------------------------------------------------------------------
   static void wendeMethodeAn(int nr) {
      // Je nach dem Wert von nr wird eine bestimmte Meldung ausgegeben
      // und eine bestimmte Methode (add oder set oder remove etc.) auf
      // die Sammlung lD1 bzw. lD3 angewendet.

      switch(nr) {
         case  1: p("lD1.add        (2.0)  "); lD1.add        (2.0); break;
         case  2: p("lD1.add        (2.5)  "); lD1.add        (2.5); break;
         case  3: pln("--------------------");
                  p("lD1.set     (0, 1.0)  "); lD1.set     (0, 1.0); break;
         case  4: p("lD1.set     (0, 2.0)  "); lD1.set     (0, 2.0); break;
         case  5: pln("--------------------");
                  p("lD1.remove     (2.0)  "); lD1.remove     (2.0); break;
         case  6: p("lD1.remove     (2.5)  "); lD1.remove     (2.5); break;
         case  7: pln("--------------------");
                  p("lD1.contains   (2.0)  "); lD1.contains   (2.0); break;
         case  8: p("lD1.contains   (2.5)  "); lD1.contains   (2.5); break;
         case  9: pln("--------------------");
                  p("lD1.addAll     (lD2)  "); lD1.addAll     (lD2); break;
         case 10: p("lD1.addAll     (lD3)  "); lD1.addAll     (lD3); break;
         case 11: pln("--------------------");
                  p("lD1.removeAll  (lD1)  "); lD1.removeAll  (lD1); break;
         case 12: p("lD1.removeAll  (lD2)  "); lD1.removeAll  (lD2); break;
         case 13: pln("--------------------");
                  p("lD1.containsAll(lD2)  "); lD1.containsAll(lD2); break;
         case 14: p("lD1.containsAll(lD1)  "); lD1.containsAll(lD1); break;
         case 15: pln("--------------------");
                  p("lD1.retainAll  (lD2)  "); lD1.retainAll  (lD2); break;
         case 16: p("lD1.retainAll  (lD1)  "); lD1.retainAll  (lD1); break;
         case 17: pln("--------------------");
                  p("lD1.clear      ()     "); lD1.clear      ();    break;
         case 18: p("lD3.clear      ()     "); lD3.clear      ();    break;
      } // switch
   } // wendeMethodeAn
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Auf die Sammlung lD1 (deren Laenge unveraenderbar ist) werden
      // nacheinander verschiedene Methoden angewendet. Nach jeder
      // Anwendung wird ausgegeben, ob keine bzw. welche Ausnahme
      // ausgeloest wurde.

      pln("Sammlungen04: Jetzt geht es los!");
      pln("------------------------------------------------------------");
      pln("A lD1:                       " + lD1);
                      lD1.set(0, 9.0);
      pln("B lD1 nach lD1.set(0, 9.0):  " + lD1);
                      rD1[0] = 1.0;
      pln("C lD1 nach rD1[0] = 1.0   :  " + lD1);
      pln("------------------------------------------------------------");
      pln("D lD1:                       " + lD1);
      pln("E lD2:                       " + lD2);
      pln("F lD3:                       " + lD3);
      pln("------------------------------------------------------------");
      // Die Methode wendeMethodeAn 18 Mal aufrufen und dadurch
      //  eventuell ausgeloeste Ausnahmen durch Ausgeben behandeln:
      for (int nr=1; nr<=18; nr++) {
         try {
            wendeMethodeAn(nr);
            pln("Keine Ausnahme!");
         } catch (Throwable ex) {
            String erg = (ex + "").replace("java.lang.", "");
            pln(erg);     // Die gefangene Ausnahme ausgebe
         } // try-catch
      } // for
      pln("------------------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // Sammlungen04
/* ------------------------------------------------------------------------
Ausgabe des Programms Sammlungen04:

Sammlungen04: Jetzt geht es los!
------------------------------------------------------------
A lD1:                       [1.0, 2.0, 3.0]
B lD1 nach lD1.set(0, 9.0):  [9.0, 2.0, 3.0]
C lD1 nach rD1[0] = 1.0   :  [1.0, 2.0, 3.0]
------------------------------------------------------------
D lD1:                       [1.0, 2.0, 3.0]
E lD2:                       [4.0, 5.0, 6.0]
F lD3:                       []
------------------------------------------------------------
lD1.add        (2.0)  UnsupportedOperationException
lD1.add        (2.5)  UnsupportedOperationException
--------------------
lD1.set     (0, 1.0)  Keine Ausnahme!
lD1.set     (0, 2.0)  Keine Ausnahme!
--------------------
lD1.remove     (2.0)  UnsupportedOperationException
lD1.remove     (2.5)  Keine Ausnahme!
--------------------
lD1.contains   (2.0)  Keine Ausnahme!
lD1.contains   (2.5)  Keine Ausnahme!
--------------------
lD1.addAll     (lD2)  UnsupportedOperationException
lD1.addAll     (lD3)  Keine Ausnahme!
--------------------
lD1.removeAll  (lD1)  UnsupportedOperationException
lD1.removeAll  (lD2)  Keine Ausnahme!
--------------------
lD1.containsAll(lD2)  Keine Ausnahme!
lD1.containsAll(lD1)  Keine Ausnahme!
--------------------
lD1.retainAll  (lD2)  UnsupportedOperationException
lD1.retainAll  (lD1)  Keine Ausnahme!
--------------------
lD1.clear      ()     UnsupportedOperationException
lD3.clear      ()     Keine Ausnahme!
------------------------------------------------------------
------------------------------------------------------------------------ */