// Datei Sammlungen06.java
/* ------------------------------------------------------------------------
Jede Sammlung s enthaelt 2 Methoden namens toArray. Ein Aufruf wie
s.toArray() liefert eine neue Reihung des Typs Object[], die dieselben
Objekte enthaelt wie s.
---------------------------------------------------------------------------
Mit der zweiten toArray-Methode kann man die Komponenten von s in
eine schon vorhandene Reihung r einfuegen lassen (und damit verhindern,
dass eine neue Reihung erzeugt wird). Dabei darf r zu einem beliebigen
Reihungstyp T[] gehoeren.

Falls die Sammlung s eine Komponente enthaelt, die nicht zu einem
Untertyp von T gehoert, wird eine Ausnahme des Typs ArrayStoreException
geworfen.

Falls r "zu kurz" ist, wird eine neue Reihung vom selben Typ wie r und
mit der richtigen Laenge s.size() erzeugt und als Ergebnis geliefert.

Falls r "zu lang" ist, wird die erste "ueberfluessige" Komponente von r
auf null gesetzt. An dieser "Markierung" kann man erkennen, bis wohin
Objekte in die Reihung r eingefuegt wurden. Reihungskomponenten hinter
dieser Position bleiben unveraendert.
---------------------------------------------------------------------------
Man beachte: Aus einer Sammlung des Typs Collection<Object>, die nur
Double-Komponenten enthaelt, kann man mit der zweiten toArray-Methode
keine Reihung des Typs Integer[] erzeugen, aber Reihungen der Typen:
Object[], Number[] und Double[] (siehe Ausgabezeilen U, V, W und X).
------------------------------------------------------------------------ */
import java.util.Collection;
import java.util.ArrayList;

class Sammlungen06 {
   static public void main(String[] sonja) {

      // Drei Sammlungen unterschiedlicher Typen:
      Collection<Integer> sI = new ArrayList<Integer>();
      Collection<Double>  sD = new ArrayList<Double> ();
      Collection<Number>  sN = new ArrayList<Number> ();
      Collection<Object>  sO = new ArrayList<Object> ();

      // In jede Sammlung drei Objekte einfuegen:
      sI.add(111); sI.add(222); sI.add(333); // 3 Integer-Objekte
      sD.add(1.1); sD.add(2.2); sD.add(3.3); // 3  Double-Objekte
      sN.add(1.1); sN.add(2.2); sN.add(3.3); // 3  Double-Objekte
      sO.add(1.1); sO.add(2.2); sO.add(3.3); // 3  Double-Objekte

      // Drei Reihungen unterschiedlicher Typen und Laengen erzeugen:
      Integer[] rI  = new Integer[0]; // Zu kurz
      Double [] rD  = new Double [3]; // Genau passende Laenge
      Number [] rN  = new Number [5]; // Zu lang
      Object [] rO  = new Object [5]; // Zu lang

      rN[3] = 1.2; // Erste  "ueberfluessige" Komponente von rN
      rN[4] = 3.4; // Zweite "ueberfluessige" Komponente von rN
      rO[3] = 5.6; // Erste  "ueberfluessige" Komponente von rO
      rO[4] = 7.8; // Zweite "ueberfluessige" Komponente von rO

      printf("Sammlungen06: Jetzt geht es los!\n");
      printf("----------------  \n");
      printf("A             sI: " + sI + " (Sammlung von Integer)\n");
      printf("B             sD: " + sD + " (Sammlung von Double) \n");
      printf("C             sN: " + sN + " (Sammlung von Number) \n");
      printf("D             sO: " + sO + " (Sammlung von Object) \n");
      printf("----------------  \n");
      printf("E sI.toArray()  : "); pr(sI.toArray());
      printf("F sD.toArray()  : "); pr(sD.toArray());
      printf("G sN.toArray()  : "); pr(sN.toArray());
      printf("H sO.toArray()  : "); pr(sO.toArray());
      printf("----------------  \n");
      printf("I sI.toArray(rI): "); pr(sI.toArray(rI));
//    printf("J sI.toArray(rD): "); pr(sI.toArray(rD));
      printf("K sI.toArray(rN): "); pr(sI.toArray(rN));
      printf("L sI.toArray(rO): "); pr(sI.toArray(rO));
      printf("----------------  \n");
//    printf("M sD.toArray(rI): "); pr(sD.toArray(rI));
      printf("N sD.toArray(rD): "); pr(sD.toArray(rD));
      printf("O sD.toArray(rN): "); pr(sD.toArray(rN));
      printf("P sD.toArray(rO): "); pr(sD.toArray(rO));
      printf("----------------  \n");
//    printf("Q sN.toArray(rI): "); pr(sN.toArray(rI));
      printf("R sN.toArray(rD): "); pr(sN.toArray(rD));
      printf("S sN.toArray(rN): "); pr(sN.toArray(rN));
      printf("T sN.toArray(rO): "); pr(sN.toArray(rO));
      printf("----------------  \n");
//    printf("U sO.toArray(rI): "); pr(sO.toArray(rI));
      printf("V sO.toArray(rD): "); pr(sO.toArray(rD));
      printf("W sO.toArray(rN): "); pr(sO.toArray(rN));
      printf("X sO.toArray(rO): "); pr(sO.toArray(rO));
      printf("----------------  \n");
   } // main
   // ---------------------------------------------------------------------
   static void pr(Object[] rO) {
      // Gibt die Reihung rO "in lesbarer Form" aus (erst den Namen des
      // momentanen Typs von rO, dahinter die Komponenten von rO).

      // Den Namen des momentanen Typs von rO berechnen:
      String
      typName = rO.getClass().getName();
      typName = typName.replace("[Ljava.lang.", "");
      typName = typName.replace(";", "[]");

      // Den Namen des Typs von rO etc. ausgeben:
      printf("Typ: %9s, Kompos: ", typName);
      // Die Komponenten von rO ausgeben:
      for (Object o: rO) {
         printf(o + ", ");
      }
      printf("\n");
   } // pr
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // Sammlungen06
/* ------------------------------------------------------------------------
Ausgabe des Programms Sammlungen06:

Sammlungen06: Jetzt geht es los!
----------------
A             sI: [111, 222, 333] (Sammlung von Integer)
B             sD: [1.1, 2.2, 3.3] (Sammlung von Double)
C             sN: [1.1, 2.2, 3.3] (Sammlung von Number)
D             sO: [1.1, 2.2, 3.3] (Sammlung von Object)
----------------
E sI.toArray()  : Typ:  Object[], Kompos: 111, 222, 333,
F sD.toArray()  : Typ:  Object[], Kompos: 1.1, 2.2, 3.3,
G sN.toArray()  : Typ:  Object[], Kompos: 1.1, 2.2, 3.3,
H sO.toArray()  : Typ:  Object[], Kompos: 1.1, 2.2, 3.3,
----------------
I sI.toArray(rI): Typ: Integer[], Kompos: 111, 222, 333,
K sI.toArray(rN): Typ:  Number[], Kompos: 111, 222, 333, null, 3.4,
L sI.toArray(rO): Typ:  Object[], Kompos: 111, 222, 333, null, 7.8,
----------------
N sD.toArray(rD): Typ:  Double[], Kompos: 1.1, 2.2, 3.3,
O sD.toArray(rN): Typ:  Number[], Kompos: 1.1, 2.2, 3.3, null, 3.4,
P sD.toArray(rO): Typ:  Object[], Kompos: 1.1, 2.2, 3.3, null, 7.8,
----------------
R sN.toArray(rD): Typ:  Double[], Kompos: 1.1, 2.2, 3.3,
S sN.toArray(rN): Typ:  Number[], Kompos: 1.1, 2.2, 3.3, null, 3.4,
T sN.toArray(rO): Typ:  Object[], Kompos: 1.1, 2.2, 3.3, null, 7.8,
----------------
V sO.toArray(rD): Typ:  Double[], Kompos: 1.1, 2.2, 3.3,
W sO.toArray(rN): Typ:  Number[], Kompos: 1.1, 2.2, 3.3, null, 3.4,
X sO.toArray(rO): Typ:  Object[], Kompos: 1.1, 2.2, 3.3, null, 7.8,
----------------
---------------------------------------------------------------------------
Fehlermedlungen des Programms, wenn die entsprechenden Zeilen
keine Kommentare sind:

J sI.toArray(rD): java.lang.ArrayStoreException
   at java.lang.System.arraycopy(Native Method)
   at java.util.ArrayList.toArray(ArrayList.java:305)
   at Sammlungen06.main(Sammlungen06.java:70)
Exception in thread "main"

M sD.toArray(rI): java.lang.ArrayStoreException
   at java.lang.System.arraycopy(Native Method)
   at java.util.ArrayList.toArray(ArrayList.java:305)
   at Sammlungen06.main(Sammlungen06.java:74)
Exception in thread "main"

Q sN.toArray(rI): java.lang.ArrayStoreException
   at java.lang.System.arraycopy(Native Method)
   at java.util.ArrayList.toArray(ArrayList.java:305)
   at Sammlungen06.main(Sammlungen06.java:79)
Exception in thread "main"

U sO.toArray(rI): java.lang.ArrayStoreException
   at java.lang.System.arraycopy(Native Method)
   at java.util.ArrayList.toArray(ArrayList.java:305)
   at Sammlungen06.main(Sammlungen06.java:84)
Exception in thread "main"
------------------------------------------------------------------------ */