// Datei Scanner01.java
/* ------------------------------------------------------------------------
Mit Hilfe eines Scanner-Objekts kann man z.B. int-Werte, float-Werte,
boolean-Werte oder Strings von der Standardeingabe einlesen.
Mit den Methoden hasNextInt, hasNextfloat, hasNextBoolean etc. kann man
pruefen, ob der naechste Teil der Eingabe als int-Wert, float-Wert oder
boolean-Wert etc. interpretiert werden kann.
Mit den Methoden nextInt, nextFloat, nextBoolean etc. kann man dann einen
Wert des betreffenden Typs einlesen.

Die Klasse Scanner wurde (erst, endlich) mit Java 5 eingefuehrt.
------------------------------------------------------------------------ */
import java.util.Scanner;

class Scanner01 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      printf("Scanner01: Jetzt geht es los!%n");
      printf("-----------------------------%n");

      Scanner scin = new Scanner(System.in);

      while (true) {
         printf("Eine Eingabe (q zum Beenden): ");
         if        (scin.hasNextInt()) {
            int i = scin.nextInt();
            printf("Das war eine Ganzzahl       : %d%n", i);
         } else if (scin.hasNextFloat()) {
            float f = scin.nextFloat();
            printf("Das war eine Bruchzahl      : %.2f%n", f);
         } else if (scin.hasNextBoolean()) {
            boolean b = scin.nextBoolean();
            printf("Das war ein Wahrheitswert   : %b%n", b);
         } else if (scin.hasNext()) {
            String s = scin.next();
            printf("Das war ein String          : %s%n", s);
            if (s.equals("q")) break;
         }
      }

      printf("-----------------------------%n");
      printf("Scanner01: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Scanner01
/* ------------------------------------------------------------------------
Ein Dialog mit diesem Programm:

Scanner01: Jetzt geht es los!
-----------------------------
Eine Eingabe (q zum Beenden): 888888888
Das war eine Ganzzahl       : 888888888
Eine Eingabe (q zum Beenden): 8888888888
Das war eine Bruchzahl      : 8888889344,00
Eine Eingabe (q zum Beenden): 1,235
Das war eine Bruchzahl      : 1,24
Eine Eingabe (q zum Beenden): 2,235
Das war eine Bruchzahl      : 2,23
Eine Eingabe (q zum Beenden): 1,0e38
Das war eine Bruchzahl      : 99999996802856920000000000000000000000,00
Eine Eingabe (q zum Beenden): 1,0e39
Das war eine Bruchzahl      : Infinity
Eine Eingabe (q zum Beenden): NaN
Das war eine Bruchzahl      : NaN
Eine Eingabe (q zum Beenden): nan
Das war ein String          : nan
Eine Eingabe (q zum Beenden): true
Das war ein Wahrheitswert   : true
Eine Eingabe (q zum Beenden): FaLsE
Das war ein Wahrheitswert   : false
Eine Eingabe (q zum Beenden): Hallo!
Das war ein String          : Hallo!
Eine Eingabe (q zum Beenden): "Hallo Sonja"
Das war ein String          : "Hallo
Eine Eingabe (q zum Beenden): Das war ein String          : Sonja"
Eine Eingabe (q zum Beenden): q
Das war ein String          : q
-----------------------------
Scanner01: Das war's erstmal!
---------------------------------------------------------------------------
Erlaeuterungen:
1. Die Zahl 8888888888 (zehn achten) ist groesser als der groesste
   int-Wert (2147483648) und wird von diesem Programm deshalb als
   float-Wert eingelesen. Der "nchste float Wert zu 8888888888"
   ist allerdings um 456 zu gross.
2. Die Bruchzahl 1,235 wird bei der Ausgabe zu 1,24 gerundet, aber
   2,235 wird nicht aufgerundet. Regel: Es kommt auf den genauen
   float-Wert an, nicht auf seine ungefaehre dezimale Darstellung!
3. Die Bruchzahl 1,0e38 kann natuerlich nur naeherungsweise als
   float-Wert dargestellt werden. Der von Java verwendete Naeherungs-
   wert ist allerdings um etwa 3 Nonillionen (eine 3 gefolgt von
   30 Nullen) zu klein. Als absolute Zahl betrachtet ist dieser
   Fehler ziemlich gross. Als relativer Fehler ist er trotzdem ziemlich
   klein: weniger als ein Promille von einem Promille. In vielen
   Zusammenhaengen kann man relative Fehler dieser Groessenordnung
   vernachlaessigen.
4. Die Bruchzahl 1,0e39 ist groesser als der groesste float-Wert
   (etwa 3,402e38) und wird deshalb durch den speziellen float-Wert
   Infinity (unendlich) dargestellt.
5. Eine NaN ("not a number") ist ein float-Wert, der keine Zahl darstellt.
   Zum Typ float gehoeren etwa 16,8 Millionen NaNs (das sind ungefaehr
   3,9 Promille aller float-Werte). Die uebrigen float-Werte (996,1 Pro-
   mille aller float-Werte) stellen Zahlen dar.
6. Wenn man eine NaN eingeben will, muss man auf GROSS/klein-Schreibung
   achten: "NaN" wird als float-Eingabe akzeptiert, "nan" und "NAN" etc.
   nur als String-Eingaben.
7. Wenn man die Wahrheitswerte true und false eingeben will, braucht
   man nicht auf GROSS/klein-Schreibung zu achten.
8. Beim Einlesen eines Strings mit der Methode getNext eines Scanner-
   Objekts beendet das erste transparente Zeichen (engl. white space
   character, hier Blanks und Tab-Zeichen) den String. Das bedeutet:
   Mit getNext kann man nicht ohne weiteres einen String einlesen, der
   transparente Zeichen enthaelt. Auch mit doppelten Anfuehrungszeichen
   kann man daran nichts aendern.
------------------------------------------------------------------------ */
