// Datei Serial01.java
/* ------------------------------------------------------------------------
Drei Objekte (unterschiedlicher Typen) werden mit Hilfe eines Stroms des
Typs ObjectOutputStream in eine Datei geschrieben und mit Hilfe eines
Stroms des Typ ObjectInputStream von dort wieder eingelesen und zur
Standardausgabe ausgegeben.
------------------------------------------------------------------------ */
import java.io  .IOException;
import java.io  .FileNotFoundException;
import java.io  .EOFException;
import java.io  .File;
import java.io  .FileInputStream;
import java.io  .FileOutputStream;
import java.io  .ObjectInputStream;
import java.io  .ObjectOutputStream;
import java.io  .Serializable;
import java.lang.ClassNotFoundException; // Zur Dokumentation
// ------------------------------------------------------------------------
class Serial01 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws
      FileNotFoundException,
      IOException,
      ClassNotFoundException
   {
      // ------------------------------------------------------------------
      String pfad  = "Serial01.tmp";

      // Zwei Ausgabestroeme und eine Datei erzeugen und zusmmenfuegen:
      FileOutputStream   fos = new FileOutputStream  (pfad);
      ObjectOutputStream oos = new ObjectOutputStream(fos);
      // Datenfluss: Datei Serial01.tmp <-- fos <-- oos <-- Programm

      // Drei Objekte in die Datei schreiben:
      oos.writeObject(new String       ("Hallo Sonja!"));
      oos.writeObject(new StringBuilder("Wie geht's?"));
      oos.writeObject(new Integer      (17));

      oos.close();
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fis = new FileInputStream  (pfad);
      ObjectInputStream ois = new ObjectInputStream(fis);
      // Datenfluss: Datei Serial01.tmp --> fis --> ois --> Programm

      // Alle Objekte aus der Datei lesen und zur Standardausgabe ausgeben.
      // Das Ende der Datei wird per Ausnahme EOFException erkannt:
      printf("--------------------------------------\n");
      try {
         while (true) {
            Object ob = ois.readObject();
            String
            typName = ob.getClass().getName();
            typName = typName.substring(typName.lastIndexOf(".")+1);
            printf("%-12s (Typ: %s)\n", ob, typName);
         }
      }  catch (EOFException e) {
         printf("Ende der Datei " + pfad + " erreicht!\n");
      } // try/catch

      ois.close();
      printf("--------------------------------------\n");
      printf("Serial01: Das war's erstmal!\n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Serial01
/* ------------------------------------------------------------------------
Ausgabe des Programms Serial01 zur Standardausgabe (Bildschirm):

--------------------------------------
Hallo Sonja! (Typ: String)
Wie geht's?  (Typ: StringBuilder)
17           (Typ: Integer)
Ende der Datei Serial01.tmp erreicht!
--------------------------------------
Serial01: Das war's erstmal!
===========================================================================
Inhalt der Datei Serial01.tmp (in hexadezimaler Darstellung):

000000  AC ED 00 05  74 00 0C 48  61 6C 6C 6F  20 53 6F 6E  ..t..Hallo Son
000010  6A 61 21 73  72 00 17 6A  61 76 61 2E  6C 61 6E 67  ja!sr..java.lang
000020  2E 53 74 72  69 6E 67 42  75 69 6C 64  65 72 3C D5  .StringBuilder<
000030  FB 14 5A 4C  6A CB 03 00  00 78 70 77  04 00 00 00  .ZLj...xpw....
000040  0B 75 72 00  02 5B 43 B0  26 66 B0 E2  5D 84 AC 02  .ur..[C&f].
000050  00 00 78 70  00 00 00 1B  00 57 00 69  00 65 00 20  ..xp.....W.i.e.
000060  00 67 00 65  00 68 00 74  00 27 00 73  00 3F 00 00  .g.e.h.t.'.s.?..
000070  00 00 00 00  00 00 00 00  00 00 00 00  00 00 00 00  ................
000080  00 00 00 00  00 00 00 00  00 00 00 00  00 00 78 73  ..............xs
000090  72 00 11 6A  61 76 61 2E  6C 61 6E 67  2E 49 6E 74  r..java.lang.Int
0000A0  65 67 65 72  12 E2 A0 A4  F7 81 87 38  02 00 01 49  eger.⠤.8...I
0000B0  00 05 76 61  6C 75 65 78  72 00 10 6A  61 76 61 2E  ..valuexr..java.
0000C0  6C 61 6E 67  2E 4E 75 6D  62 65 72 86  AC 95 1D 0B  lang.Number..
0000D0  94 E0 8B 02  00 00 78 70  00 00 00 11               ...xp....
------------------------------------------------------------------------ */