// Datei Serial02.java
/* ------------------------------------------------------------------------
Objekte der Klasse Artikel werden mit einem Strom der Klasse
ObjectOutputStream in eine Datei namens Serial02.tmp geschrieben
und mit einem Strom der Klasse ObjectInputStream von dort wieder
eingelesen.

Ein Objekt "serialisieren" bedeutet, das Objekt "in eine Serie von
Bytes umwandeln, die man in eine Datei schreiben und spaeter von
dort wieder einlesen kann".

Serialisiert werden koennen nur Objekte von Klassen, die die Schnitt-
stelle Serializable implementieren. Serializable ist eine sogenannte
Markierungsschnittstelle, d.h. sie ist leer (sie enthaelt keine
Methoden und Attribute). Trotzdem kann geprueft werden, ob ein Objekt
otto diese Schnittstlle implementiert (ob der boolean-Ausdruck
(otto instanceof Serializable) true liefert oder false).
------------------------------------------------------------------------ */
import java.io  .IOException;
import java.io  .FileNotFoundException;
import java.io  .EOFException;
import java.io  .File;
import java.io  .FileInputStream;
import java.io  .FileOutputStream;
import java.io  .ObjectInputStream;
import java.io  .ObjectOutputStream;
import java.io  .Serializable;
import java.lang.ClassNotFoundException; // Zur Dokumentation
// ------------------------------------------------------------------------
// Objekte der folgenden Klasse werden ("serialisiert" und) in eine Datei
// geschrieben und spaeter von dort wieder eingelesen:
class Artikel implements Serializable {
   // Jedes Objekt dieser Klasse beschreibt einen Artikel (seine
   // bezeichnung und die anzahl, in der er momentan vorhanden ist)
   String bezeichnung;
   int    anzahl;

   public String toString() {
      return "Bez.: " + bezeichnung + ", Anz.: " + anzahl;
   } // toString

   Artikel(String bezeichnung, int anzahl) {
      this.bezeichnung = bezeichnung;
      this.anzahl      = anzahl;
   } // Konstruktor Artikel
} // Artikel
// ------------------------------------------------------------------------
class Serial02 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws
      FileNotFoundException,
      IOException,
      ClassNotFoundException
   {
      // Eine Reihung von Artikel-Objekten:
      Artikel[] tab = {
         new Artikel("Knoepfe", 65),
         new Artikel("Socken ", 17),
         new Artikel("Hemden ", 24)
      };
      // ------------------------------------------------------------------
      String pfad  = "Serial02.tmp";
      File   dat01 = new File(pfad);
      pln("Die Datei " + pfad + " wird  erstellt!");

      // Zwei Ausgabestroeme und eine Datei erzeugen und zusmmenfuegen:
      FileOutputStream    fos = new FileOutputStream  (dat01);
      ObjectOutputStream  oos = new ObjectOutputStream(fos);
      // Datenfluss: Datei Serial02.tmp <-- fos <-- oos <-- Programm

      // Ein paar Objekte in die Datei schreiben:
      for (Artikel a: tab) oos.writeObject(a);
      oos.close();
      pln("Die Datei " + pfad + " wurde erstellt!");
      pln("--------------------------------------");
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fis = new FileInputStream  (dat01);
      ObjectInputStream ois = new ObjectInputStream(fis);
      // Datenfluss: Datei Serial02.tmp --> fis --> ois --> Programm

      // Alle Objekte aus der Datei lesen und zur Standardausgabe ausgeben.
      // Das Ende der Datei wird per Ausnahme EOFException erkannt:
      try {
         while (true) {
            Artikel art = (Artikel) ois.readObject();
            pln(art);
         }
      }  catch (EOFException e) {
         pln("Ende der Datei erreicht!");
      }  finally {
         ois.close();
         dat01.delete(); // Die Datei dat01 loeschen
         pln("--------------------------------------");
         pln("Serial02: Das war's erstmal!");
      } // try/catch/finally
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
} // class Serial02
/* ------------------------------------------------------------------------
Ausgabe des Progrmms Serial02 zur Standardausgabe (Bildschirm):

Die Datei Serial02.tmp wird  erstellt!
Die Datei Serial02.tmp wurde erstellt!
--------------------------------------
Bez.: Knoepfe, Anz.: 65
Bez.: Socken , Anz.: 17
Bez.: Hemden , Anz.: 24
Ende der Datei erreicht!
--------------------------------------
Serial02: Das war's erstmal!
===========================================================================
Inhalt der Datei Serial02.tmp (in hexadezimaler Darstellung):

000000  AC ED 00 05  73 72 00 07  41 72 74 69  6B 65 6C C6  ..sr..Artikel
000010  B3 E8 98 56  25 8F 4D 02  00 02 49 00  06 61 6E 7A  V%.M...I..anz
000020  61 68 6C 4C  00 0B 62 65  7A 65 69 63  68 6E 75 6E  ahlL..bezeichnun
000030  67 74 00 12  4C 6A 61 76  61 2F 6C 61  6E 67 2F 53  gt..Ljava/lang/S
000040  74 72 69 6E  67 3B 78 70  00 00 00 41  74 00 07 4B  tring;xp...At..K
000050  6E 6F 65 70  66 65 73 71  00 7E 00 00  00 00 00 11  noepfesq.~......
000060  74 00 07 53  6F 63 6B 65  6E 20 73 71  00 7E 00 00  t..Socken sq.~..
000070  00 00 00 18  74 00 07 48  65 6D 64 65  6E 20        ....t..Hemden
------------------------------------------------------------------------ */