// Datei Serial03.java
/* ------------------------------------------------------------------------
In einen ObjectOutputStream kann man auch Werte primitiver Typen schreiben
und aus einem ObjectInputStream kann man solche Werte lesen.
Aber Vorsicht: Diese Art zu schreiben und zu lesen ist nicht typsicher!

Hier werden drei int-Werte und zwei double-Werte in eine Datei geschrieben
und als sieben int-Werte wieder gelesen.

Der Aufruf ois.available(); liefert die "Netto-Anzahl" der Bytes im
Strom ois, ohne Beruecksichtigung von "Verwaltungsdaten".
------------------------------------------------------------------------ */
import java.io  .IOException;
import java.io  .FileNotFoundException;
import java.io  .EOFException;
import java.io  .File;
import java.io  .FileInputStream;
import java.io  .FileOutputStream;
import java.io  .ObjectInputStream;
import java.io  .ObjectOutputStream;
import java.io  .Serializable;
import java.lang.ClassNotFoundException; // Zur Dokumentation

class Serial03 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws
      FileNotFoundException,
      IOException,
      ClassNotFoundException
   {
      // ------------------------------------------------------------------
      String pfad  = "Serial03.tmp";
      File   dat01 = new File(pfad);
      pln("Die Datei " + pfad + " wird erstellt!");

      // Zwei Ausgabestroeme und eine Datei erzeugen und zusmmenfuegen:
      FileOutputStream    fos = new FileOutputStream  (dat01);
      ObjectOutputStream  oos = new ObjectOutputStream(fos);
      // Datenfluss: Datei Serial03.tmp <-- fos <-- oos <-- Programm

      pln("-----------------------------------------------");
      // Ein paar int-Werte in die Datei und zur Std.Ausg. ausgeben:
      oos.writeInt   (16);  p(16 + " ");
      oos.writeInt   (32);  p(32 + " ");
      oos.writeInt   (48);  p(48 + " ");

      // Ein paar double-Werte in die Datei und zur Std.Ausg. ausgeben:
      oos.writeDouble(0.0); p(0.0 + " ");
      oos.writeDouble(1.1); p(1.1 + " ");

      pln();
      pln("3 int- und 2 double-Werte in Datei geschrieben!");
      pln("-----------------------------------------------");
      oos.close();
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fis = new FileInputStream  (dat01);
      ObjectInputStream ois = new ObjectInputStream(fis);
      // Datenfluss: Datei Serial03.tmp --> fis --> ois --> Programm

      // Anzahl der Daten-Bytes in der Datei ausgeben:
      pln(pfad + " enthaelt " + ois.available() + " Bytes");

      // Alle Werte aus der Datei lesen und zur Standardausgabe ausgeben.
      // Das Ende der Datei wird mit available() erkannt:
      pln("-----------------------------------------------");
      int anz = 0;
      while (ois.available() > 0) {
         int ein = ois.readInt();
         p(ein + " ");
         anz++;
      }
      pln();
      pln(anz + " int-Werte aus Datei gelesen!");
      ois.close();
      dat01.delete(); // Die Datei dat01 loeschen
      pln("-----------------------------------------------");
      pln("Serial03: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
} // class Serial03
/* ------------------------------------------------------------------------
Ausgabe des Programms Serial03 zur Standardausgabe (Bildschirm):

Die Datei Serial03.tmp wird erstellt!
-----------------------------------------------
16 32 48 0.0 1.1
3 int- und 2 double-Werte in Datei geschrieben!
-----------------------------------------------
Serial03.tmp enthaelt 28 Bytes
-----------------------------------------------
16 32 48 0 0 1072798105 -1717986918
7 int-Werte aus Datei gelesen!
-----------------------------------------------
Serial03: Das war's erstmal!
===========================================================================
Inhalt der Datei Serial03.tmp (in hexadezimaler Darstellung):

000000  AC ED 00 05  77 1C 00 00  00 10 00 00  00 20 00 00  ..w........ ..
000010  00 30 00 00  00 00 00 00  00 00 3F F1  99 99 99 99  .0........?
000020  99 9A                                               
------------------------------------------------------------------------ */