// Datei Serial04.java
/* ------------------------------------------------------------------------
Beliebig viele Objekte, die "zusammenhaengen", kann man mit einem einzigen
writeObject-Befehl in einen Strom des Typs ObjectOutputStream schreiben und
mit einem einzigen readObject-Befehl aus einem Strom des Typs ObjectInput-
Stream wieder einlesen.

Jedes Objekt der Klasse Perle enthaelt unter anderem ein Attribut next,
welches auf eine weitere Perle zeigen (oder den Wert null haben) kann.
Perle-Objekte koennen somit "zusammenhaengen".
------------------------------------------------------------------------ */
import java.io  .IOException;
import java.io  .FileNotFoundException;
import java.io  .File;
import java.io  .FileInputStream;
import java.io  .FileOutputStream;
import java.io  .ObjectInputStream;
import java.io  .ObjectOutputStream;
import java.io  .Serializable;
import java.lang.ClassNotFoundException; // Zur Dokumentation
// ------------------------------------------------------------------------
class Perle implements Serializable {
   // Jedes Perle-Objekt hat einen Namen und zeigt moeglicherweise
   // auf ein weiteres Perle-Objekt ("Perlen koennen Ketten bilden")
   String name;
   Perle  next;

   public String toString() {
      if (next == null) {
         return name;
      } else {
         return name + "-" + next.toString();
      }
   } // toString

   Perle(String name, Perle next) {
      this.name = name;
      this.next = next;
   } // Konstruktor Serial04
} // Perle
// ------------------------------------------------------------------------
class Serial04 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws
      FileNotFoundException,
      IOException,
      ClassNotFoundException
   {
      // Eine Kette von 5 Perlen vereinbaren:
      Perle ob01 = new Perle ("Alf",      null); // Letzt Perle der Kette
      Perle ob02 = new Perle ("Berta",    ob01);
      Perle ob03 = new Perle ("Carlos",   ob02);
      Perle ob04 = new Perle ("Dora",     ob03);
      Perle ob05 = new Perle ("Elfriede", ob04); // Erste Perle der Kette
      // ------------------------------------------------------------------
      // Die Kette, die mit dem Objekt ob05 beginnt, ausgeben:
      pln("-----------------------------------------------------");
      pln("Ausgegeben wurde ob05: " + ob05);

      String pfad  = "Serial04.tmp";
      File   dat01 = new File(pfad);

      // Zwei Ausgabestroeme und eine Datei erzeugen und zusmmenfuegen:
      FileOutputStream    fos = new FileOutputStream  (dat01);
      ObjectOutputStream  oos = new ObjectOutputStream(fos);
      // Datenfluss: Datei Serial04.tmp <-- fos <-- oos <-- Programm

      // Das Objekt ob05 ("und alle daranhaengenden Objekte") mit einem
      // einzigen writeObject-Befehl in die Datei schreiben:
      oos.writeObject(ob05);

      oos.close();
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fis = new FileInputStream  (dat01);
      ObjectInputStream ois = new ObjectInputStream(fis);
      // Datenfluss: Datei Serial04.tmp --> fis --> ois --> Programm

      // Ein Objekt ("und alle daranhaengenden Objekte") mit einem einzigen
      // readObject-Befehl aus der Datei in die Variable ob15 einlesen:
      Perle ob15 = (Perle) ois.readObject();
      ois.close();
      dat01.delete(); // Die Datei dat01 loeschen

      // Die Kette, die mit dem Objekt ob15 beginnt, ausgeben:
      pln("Eingelesen wurde ob15: " + ob15);
      pln("-----------------------------------------------------");
      pln("Serial04: Das war's erstmal!");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
} // class Serial04
/* ------------------------------------------------------------------------
Ausgabe des Programms Serial04 zur Standardausgabe (Bildschirm):

-----------------------------------------------------
Ausgegeben wurde ob05: Elfriede-Dora-Carlos-Berta-Alf
Eingelesen wurde ob15: Elfriede-Dora-Carlos-Berta-Alf
-----------------------------------------------------
Serial04: Das war's erstmal!
===========================================================================
Inhalt der Datei Serial04 (in hexadezimaler Darstellung):

000000  AC ED 00 05  73 72 00 05  50 65 72 6C  65 41 82 90  ..sr..PerleA
000010  C8 C0 95 3C  DA 02 00 02  4C 00 04 6E  61 6D 65 74  ++<+...L..namet
000020  00 12 4C 6A  61 76 61 2F  6C 61 6E 67  2F 53 74 72  ..Ljava/lang/Str
000030  69 6E 67 3B  4C 00 04 6E  65 78 74 74  00 07 4C 50  ing;L..nextt..LP
000040  65 72 6C 65  3B 78 70 74  00 08 45 6C  66 72 69 65  erle;xpt..Elfrie
000050  64 65 73 71  00 7E 00 00  74 00 04 44  6F 72 61 73  desq.~..t..Doras
000060  71 00 7E 00  00 74 00 06  43 61 72 6C  6F 73 73 71  q.~..t..Carlossq
000070  00 7E 00 00  74 00 05 42  65 72 74 61  73 71 00 7E  .~..t..Bertasq.~
000080  00 00 74 00  03 41 6C 66  70                        ..t..Alfp
------------------------------------------------------------------------ */