// Datei StringBuffer03.java
/* ------------------------------------------------------------------------
Dieses Programm fuehrt Zeitmessungen durch mit den Methoden append und
insert der Klasse StringBuffer und mit der Konkatenations-Operation + fuer
Strings.

Jede einzelne Zeitmessung wird nach MAX_MILLIS Millisekunden oder nach
MAX_ANZAHL vielen Ausfuehrungen des zu vermessenden Befehls abgebrochen
(je nachdem, welche Bedingung frueher erfuellt ist). Danach wird die
Anzahl der Befehlsausfuehrungen pro Sekunde berechnet.

Der jeweils zu vermessende Befehl steht in einer doppelten for-Schleife.
Damit soll der verfaelschende Einfluss von "Verwaltungsbefehlen" auf das
Messergebnis verringert werden.
------------------------------------------------------------------------ */
class StringBuffer03 {
   // Konstanten zur Erhoehung der Lesbarkeit:
   static final int TAUSEND    = 1000;
   static final int MILLION    = TAUSEND * TAUSEND;

   // Jeder zu vermessende Befehl wird hochstens MAX_MILLIS viele Milli-
   // sekunden und hoechstens MAX_ANZAHL Mal ausgefuehrt:
   static final int MAX_MILLIS =   10 * TAUSEND;
   static final int MAX_ANZAHL =  100 * TAUSEND;
   // ---------------------------------------------------------------------
   static long append() {
      // Misst und berechnet, wie oft der Befehl sb.append('X')
      // pro Sekunde ausgefuehrt werden kann.
      StringBuffer sb = new StringBuffer();

      // Die Anfangszeit merken und den maximalen Endzeitpunt berechnen:
      long anfZeit = System.currentTimeMillis();
      long maxZeit = anfZeit + MAX_MILLIS;
      int  anzahl  = 0;

      // Die innere Schleife fuehrt den zu vermessenden Befehl TAUSEND
      // Mal aus. Die aeussere Schleife fuehrt ihren Rumpf solange aus,
      // bis der maximale Endzeitpunkt erreicht oder der zu vermessende
      // Befehl (etwa) MAX_ANZAHL Mal ausgefuehrt wurde.
      for (; anzahl<=MAX_ANZAHL; anzahl += TAUSEND) {
         for (int i=1; i<=TAUSEND; i++) sb.append('X');
         if (System.currentTimeMillis() >= maxZeit) break;
      }

      long dauer  = System.currentTimeMillis() - anfZeit;
      return anzahl * TAUSEND / dauer;
   } // append
   // ---------------------------------------------------------------------
   static long insert() {
      // Misst und berechnet, wie oft der Befehl sb.insert(0, 'X')
      // pro Sekunde ausgefuehrt werden kann. Der Befehl insert(0, ...)
      // (ganz vorn einfuegen) ist der teuerste insert-Befehl!
      StringBuffer sb = new StringBuffer();

      // Die Anfangszeit merken und den maximalen Endzeitpunt berechnen:
      long anfZeit = System.currentTimeMillis();
      long maxZeit = anfZeit + MAX_MILLIS;
      int  anzahl  = 0;

      // Die innere Schleife fuehrt den zu vermessenden Befehl TAUSEND
      // Mal aus. Die aeussere Schleife fuehrt ihren Rumpf solange aus,
      // bis der maximale Endzeitpunkt erreicht oder der zu vermessende
      // Befehl (etwa) MAX_ANZAHL Mal ausgefuehrt wurde.
      for (; anzahl<=MAX_ANZAHL; anzahl += TAUSEND) {
         for (int i=1; i<=TAUSEND; i++) sb.insert(0, 'X');
         if (System.currentTimeMillis() >= maxZeit) break;
      }

      long dauer  = System.currentTimeMillis() - anfZeit;
      return anzahl * TAUSEND / dauer;
   } // insert
   // ---------------------------------------------------------------------
   static long konkat() {
      // Misst und berechnet, wie oft der Befehl s = s + 'X'
      // pro Sekunde ausgefuehrt werden kann.
      String s = new String();

      // Die Anfangszeit merken und den maximalen Endzeitpunt berechnen:
      long anfZeit = System.currentTimeMillis();
      long maxZeit = anfZeit + MAX_MILLIS;
      int  anzahl  = 0;

      // Die innere Schleife fuehrt den zu vermessenden Befehl TAUSEND
      // Mal aus. Die aeussere Schleife fuehrt ihren Rumpf solange aus,
      // bis der maximale Endzeitpunkt erreicht oder der zu vermessende
      // Befehl (etwa) MAX_ANZAHL Mal ausgefuehrt wurde.
      for (; anzahl<=MAX_ANZAHL; anzahl += TAUSEND) {
         for (int i=1; i<=TAUSEND; i++) s = s + 'X';
         if (System.currentTimeMillis() >= maxZeit) break;
      }

      long dauer  = System.currentTimeMillis() - anfZeit;
      return anzahl * TAUSEND / dauer;
   } // konkat
   // ---------------------------------------------------------------------
   static public void main(String[] sigfried) {
      // Die Zeitmessungs-Methoden append, insert und konkat werden aufge-
      // rufen und ihre Ergebnisse werden ausgegeben:

      pln("A append : pro Sekunde: " + append());
      pln("B insert : pro Sekunde: " + insert());
      pln("C konkat : pro Sekunde: " + konkat());
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class StringBuffer03
/* ------------------------------------------------------------------------
Ausgabe des Programms StingBuffer01 (ausgefuehrt mit einem Pentium III mit
500MHz unter Windows98):

A append : pro Sekunde: 2020000
B insert : pro Sekunde: 4840
C konkat : pro Sekunde: 1351
------------------------------------------------------------------------ */