// Datei Stroeme01.java
/* ------------------------------------------------------------------------
Demonstriert zeichenorientierte Ein-/Ausgabe von/zu Dateien.

Das Ende einer Zeile in einer Textdatei ist plattformabhaengig und
besteht aus folgenden Zeichen:
Unter aelteren Macintosh-Systemen:             Ein CR-Zeichen (hex: 0D)
Unter Unix (z.B. Linux):                       Ein LF-Zeichen (hex: 0A)
Unter Dos/Windows:                 Ein CR- und ein LF-Zeichen (hex: 0D 0A)

Die Methode newLine in einem BufferedWriter-Objekt (siehe unten bruno)
schreibt eine Zeilenende-Markierung der gerade verwendeten Plattform.

Die Methode readLine in einem BufferedReader-Objekt (siehe unten britta)
liest "eine Zeile", d.h. alle Zeichen bis zur naechsten Zeilenende-Mar-
kierung (und liefert diese Zeichen als String, aber *ohne* die Zeilenende-
Markierung). Dieser String kann leer oder nicht-leer sein (0 oder mehr als
0 Zeichen enthalten). Ist das Ende der Datei erreicht, liefert die Methode
readLine anstelle eines (leeren oder nicht-leeren) Strings den Wert null.
------------------------------------------------------------------------ */
import java.io .FileOutputStream;
import java.io .FileInputStream;
import java.io .File;
import java.io .OutputStreamWriter;
import java.io .InputStreamReader;
import java.io .BufferedWriter;
import java.io .BufferedReader;
import java.io .InputStream;
import java.net.URL;
// ------------------------------------------------------------------------
class Stroeme01 {
   static public void main(String [] sonja) throws java.io.IOException {

      // Pfadnamen kann man wahlweise wie unter DOS mit Rueckwaertsschraeg-
      // strichen ('\') oder wie unter Unix/Linux   mit            Schraeg-
      // strichen ('/') angeben. Rueckwaertsschraegstriche muss man doppelt
      // notieren, damit sie nicht als Fluchtsymbole wirken.
//    String pfad  = "D:\\meineDateien\\BspJaSp\\Stroeme01.tmp";
      String pfad  = "D:/meineDateien/BspJaSp/Stroeme01.tmp";
      File   dat01 = new File(pfad);
      // ------------------------------------------------------------------
      // Strings in eine Datei schreiben (und durch Zeilenende-Markierungen
      // voneinander trennen)
      // ------------------------------------------------------------------
      // Eine Datei und drei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream   felix = new FileOutputStream  (dat01);
      OutputStreamWriter oskar = new OutputStreamWriter(felix);
      BufferedWriter     bruno = new BufferedWriter    (oskar);
      // Datenfluss: Datei <-- felix <-- oskar <-- bruno <-- Programm

      // Vier Strings und zwei Zeilenende-Markierungen (durch die Stroeme)
      // in die Datei schreiben. Nach der letzten Zeile ist eine
      // Zeilenende-Markierung nicht unbedingt noetig (aber erlaubt):
      bruno.write("A Hallo Sonja! ");    // Zeile 1, Anfang
      bruno.write("Wie geht's?");        // Zeile 1, Fortsetzung
      bruno.write('?');                  // Zeile 1, Fortsetzung
      bruno.newLine();
      bruno.write("B Danke, gut!");      // Zeile 2
      bruno.newLine();
      bruno.write("C Freut mich.");      // Zeile 3
      bruno.close();
      // ------------------------------------------------------------------
      // Zeilen (als Strings) aus einer Datei lesen, und zwar entweder mit
      // Variante A aus der durch die URL beschriebenen Datei aus dem
      //            Internet oder mit
      // Variante B aus der durch den pfad beschriebenen Datei im lokalen
      //            Dateisystem.
      // ------------------------------------------------------------------
      URL url = new URL("http://www.tfh-berlin.de/~grude/" +              // Variante A
                        "JavaIstEineSprache/Stroeme01.tmp");              // Variante A
      InputStream       fiona  = url.openStream();                        // Variante A
      // ------------------------------------------------------------------
      // Drei Eingabestroeme erzeugen und mit der Datei verbinden:
//    FileInputStream   fiona  = new FileInputStream  (dat01);             // Variante B
      InputStreamReader ilse   = new InputStreamReader(fiona);
      BufferedReader    britta = new BufferedReader   (ilse);
      // Datenfluss: Datei --> fional --> ilse -->  britta --> Programm

      // Alle Zeilen (durch die Stroeme) aus der Datei lesen und zur
      // Standardausgabe ausgeben:
      String einZeile;
      while (true) {
         einZeile = britta.readLine();
         if (einZeile == null) break;     // Ende der Datei erreicht
         pln(einZeile);
      }
      pln("-----------------------------");
      pln("Stroeme01: Das war's erstmal!");

      britta.close();
//    dat01.delete(); // Datei dat01 loeschen
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
} // class Stroeme01
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme01 zur Standardausgabe (Bildschirm):

A Hallo Sonja! Wie geht's??
B Danke, gut!
C Freut mich.
-----------------------------
Stroeme01: Das war's erstmal!
===========================================================================
Inhalt der Datei Stroeme01.tmp (in hexadezimaler Darstellung):

000000  48 61 6C 6C  6F 20 53 6F  6E 6A 61 21  20 57 69 65  Hallo Sonja! Wie
000010  20 67 65 68  74 27 73 3F  3F 0D 0A 44  61 6E 6B 65   geht's??..Danke
000020  2C 20 67 75  74 21 0D 0A  46 72 65 75  74 20 6D 69  , gut!..Freut mi
000030  63 68 2E                                            ch.
------------------------------------------------------------------------ */