// Datei Stroeme02.java
/* ------------------------------------------------------------------------
Demonstriert zeichenorientierte Ein-/Ausgabe von/zu Dateien.

Eine Zeilenende-Markierung in einer Textdatei besteht
unter Unix/Linux      aus einem LF-Zeichen               (hex.: 0A),
unter dem alten MacOS aus einem CR-Zeichen               (hex.: 0D) und
unter DOS/Windows     aus einem CR- und einem LF-Zeichen (hex.: 0D 0A).

Die Methode readLine in einem BufferedReader-Objekt (siehe unten britta)
kann alle drei Zeilenende-Markierungen "richtig verstehen".
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.io.BufferedWriter;
import java.io.BufferedReader;
// ------------------------------------------------------------------------
class Stroeme02 {
   static public void main(String[] salome) throws java.io.IOException {

      String pfad = "D:/meineDateien/BspJaSp/Stroeme02.tmp";
      File   dat01 = new File(pfad);

      final char CR = 0x0D; // Das Zeichen carriage return
      final char LF = 0x0A; // Das Zeichen line feed
      // ------------------------------------------------------------------
      // Strings in eine Datei schreiben (und durch Zeilenende-Markierungen
      // voneinander trennen)
      // ------------------------------------------------------------------
      // Eine Datei und drei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream   felix = new FileOutputStream  (dat01);
      OutputStreamWriter oskar = new OutputStreamWriter(felix);
      BufferedWriter     bruno = new BufferedWriter    (oskar);
      // Datenfluss: Datei <-- felix <-- oskar <-- bruno <-- Programm

      // Drei Strings mit unterschiedlichen Zeilenende-Markierungen
      // in die Datei schreiben:                // Zeilenende-Markierung
                                                // wie unter
      bruno.write("Zeile 1 ABCDEFG" + LF);      // Unix/Linux
      bruno.write("Zeile 2 abcdefg" + CR);      // altem Mac-OS
      bruno.write("Zeile 3 123456"  + CR + LF); // DOS/Windows
      bruno.close();
      // ------------------------------------------------------------------
      // Zeilen (als Strings) aus einer Datei lesen
      // ------------------------------------------------------------------
      // Drei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fiona  = new FileInputStream  (pfad);
      InputStreamReader ilse   = new InputStreamReader(fiona);
      BufferedReader    britta = new BufferedReader   (ilse);
      // Datenfluss: Datei --> fiona --> ilse --> britta --> Programm

      // Alle Zeilen (durch die Stroeme) aus der Datei lesen und zur
      // Standardausgabe ausgeben:
      String einZeile;
      while (true) {
         einZeile = britta.readLine();
         if (einZeile == null) break;     // Ende der Datei erreicht
         pln(einZeile);
      }
      pln("-----------------------------");
      pln("Stroeme02: Das war's erstmal!");


      britta.close();
      dat01.delete(); // Die Datei dat01 loeschen
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Stroeme02
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme02 zur Standardausgabe (Bildschirm):

Zeile 1 ABCDEFG
Zeile 2 abcdefg
Zeile 3 123456
-----------------------------
Stroeme02: Das war's erstmal!
===========================================================================
Inhalt der Datei Stroeme02.tmp (in hexadezimaler Darstellung):

000000  5A 65 69 6C  65 20 31 20  41 42 43 44  45 46 47 0A  Zeile 1 ABCDEFG.
000010  5A 65 69 6C  65 20 32 20  61 62 63 64  65 66 67 0D  Zeile 2 abcdefg.
000020  5A 65 69 6C  65 20 33 20  31 32 33 34  35 36 0D 0A  Zeile 3 123456..
------------------------------------------------------------------------ */