// Datei Stroeme03.java
/* ------------------------------------------------------------------------
Demonstriert zeichenorientierte Ein-/Ausgabe von/zu Dateien.

Mit einem StreamTokenizer-Objekt namens stella wird eine Textdatei
in sogenannte Token zerlegt, die durch frei waehlbare Trennzeichen
(hier: Kommas und Ausrufezeichen) voneinander getrennt sind. Mit
dem Befehl stella.nextToken liest man das naechste Token und sein
Text steht danach in der String-Variablen stella.sval!
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import java.io.OutputStreamWriter;
import java.io.InputStreamReader;
import java.io.BufferedWriter;
import java.io.StreamTokenizer;

class Stroeme03 {
   static public void main(String[] salome) throws java.io.IOException {

      String pfad  = "D:/meineDateien/BspJaSp/Stroeme03.tmp";
      File   dat01 = new File(pfad);
      // ------------------------------------------------------------------
      // Eine Datei und drei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream   felix = new FileOutputStream  (dat01);
      OutputStreamWriter oskar = new OutputStreamWriter(felix);
      BufferedWriter     bruno = new BufferedWriter    (oskar);
      // Datenfluss: Datei <-- felix <-- oskar <-- bruno <-- Programm

      // Sieben durch Kommas bzw. Ausrufezeichen voneinander getrennte
      // Zeichenketten (und dazwischen ein paar Zeilenende-Markierungen)
      // in die Datei schreiben:
      bruno.write("A Eine kurze Zeichenkette,");
      bruno.write("B Eine etwas laengere Zeichenkette,");
      bruno.newLine();
      bruno.write("C Eine noch ein bisschen laengere Zeichenkette!");
      bruno.newLine();
      bruno.write("D 12, E 345678! F 9!");
      bruno.newLine();
      bruno.write("G Noch eine Zeichenkette,");
      bruno.newLine();
      bruno.close();
      // ------------------------------------------------------------------
      // Token aus einer Datei lesen
      // ------------------------------------------------------------------
      // Drei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fiona  = new FileInputStream  (pfad);
      InputStreamReader ilse   = new InputStreamReader(fiona);
      StreamTokenizer   stella = new StreamTokenizer  (ilse);
      // Datenfluss: Datei --> fiona --> ilse --> stella --> Programm

      // Die Zeichen Komma ',' und Ausrufezeichen '!' als Trennzeichen
      // zwischen Token festlegen, alle anderen Unicode-Zeichen sollen
      // zu den Token gehoeren:
      stella.wordChars(0, Character.MAX_VALUE);
      stella.whitespaceChars(',', ',');
      stella.whitespaceChars('!', '!');

      // Die Datei Token fuer Token lesen (nebenbei werden die Zeilen
      // gezaehlt):
      while (true) {
         stella.nextToken();
         // Ist das Ende der Datei erreicht?
         if (stella.ttype == StreamTokenizer.TT_EOF) break;
         pln("Zeile " + stella.lineno() + ": " + stella.sval);
      } // while
      pln("-----------------------------");
      pln("Stroeme03: Das war's erstmal!");

      ilse.close();
      dat01.delete(); // Datei dat01 loeschen
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Stroeme03
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme03 zur Standardausgabe (Bildschirm):

Zeile 1: A Eine kurze Zeichenkette
Zeile 1: B Eine etwas laengere Zeichenkette
Zeile 2: C Eine noch ein bisschen laengere Zeichenkette
Zeile 3: D 12
Zeile 3: E 345678
Zeile 3: F 9
Zeile 4: G Noch eine Zeichenkette
-----------------------------
Stroeme03: Das war's erstmal!
------------------------------------------------------------------------ */