// Datei Stroeme04.java
/* ------------------------------------------------------------------------
Demonstriert byteorientierte Ein-/Ausgabe von/zu Dateien.

Daten, die mit einem DataOutputStream-Objekt in eine Datei geschrieben
wurden, enthalten keine Informationen ueber ihren Typ. Eine bestimmte Bit-
kombination in der Datei kann verschiedene Werte verschiedener Typen dar-
stellen (z.B. kann die Bitkombination 0x0000000A den int-Wert 10 oder den
float-Wert 1.4E-44F darstellen und die Bitkombination 0x41200000 kann den
int-Wert 1092616192 (ca. 1 Milliarde) oder den float-Wert 10.0F darstellen).
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.File;

class Stroeme04 {
   // ---------------------------------------------------------------------
   // Vier int- und vier float-Variablen. Ihre Werte wurden so gewaehlt,
   // dass untereinanderstehende Werte (z.B. 10 und 1.4E-44F) durch die
   // selbe Bitkombination dargestellt werden:
   static int    i1=10,       i2=20,       i3=1092616192, i4=1101004800;
   static float  f1=1.4E-44F, f2=2.8E-44F, f3=10.0F,      f4=20.0F;
   // ---------------------------------------------------------------------
   static void gibAlleAus(String text) {
      // Gibt den text und dann die acht Variablen (i1 bis i4, f1 bis f4)
      // zur Standardausgabe (d.h. zum Bildschirm) aus:
      final int LEN = 16;
      printf("-------------------------------");
      printf("-------------------------------\n");
      printf("%s\n", text);

      printf("i1: %-11s i2: %-11s i3: %-11s i4: %-11s\n",
              i1,       i2,       i3,       i4);
      printf("f1: %-11s f2: %-11s f3: %-11s f4: %-11s\n",
              f1,       f2,       f3,       f4);
   } // gibAlleAus
   // ---------------------------------------------------------------------
   static public void main(String[] salome) throws java.io.IOException {
      // In eine Datei werden vier int- und vier float-Werte geschrieben.
      // Dann wird die Datei zweimal gelesen. Die geschriebenen und die
      // die gelesenen Werte werden zur Standardausgabe ausgegeben.

      String pfad  = "Stroeme04.tmp";
      File   dat01 = new File(pfad);
      // ------------------------------------------------------------------
      // Eine Datei und zwei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream felix = new FileOutputStream(dat01);
      DataOutputStream david = new DataOutputStream(felix);
      // Datenfluss: Datei Stroeme04.tmp <-- felix <-- davic <-- Programm

      // Zuerst vier int-Werte schreiben und ...
      david.writeInt  (i1);
      david.writeInt  (i2);
      david.writeInt  (i3);
      david.writeInt  (i4);

      // ... dann vier float-Werte:
      david.writeFloat(f1);
      david.writeFloat(f2);
      david.writeFloat(f3);
      david.writeFloat(f4);

      david.close();

      // Auch nach dem Schliessen der Datei weiss der Strom david noch,
      // wieviele Bytes er insgesamt in die Datei geschrieben hat:
      printf("In die Datei " + pfad         + " wurden geschrieben: ");
      printf(                  david.size() + " Bytes\n");

      gibAlleAus("Erst i1 bis i4 geschrieben, dann f1 bis f4:");
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream fiona = new FileInputStream(pfad);
      DataInputStream dora  = new DataInputStream(fiona);
      // Datenfluss: Datei Stroeme04.tmp --> fiona --> dora --> Programm

      // Zuerst vier int-Werte lesen und ...
      i1 = dora.readInt  ();
      i2 = dora.readInt  ();
      i3 = dora.readInt  ();
      i4 = dora.readInt  ();

      // ... danach vier float-Werte:
      f1 = dora.readFloat();
      f2 = dora.readFloat();
      f3 = dora.readFloat();
      f4 = dora.readFloat();

      dora.close();

      gibAlleAus("Erst i1 bis i4 gelesen, dann f1 bis f4:");
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      fiona = new java.io.FileInputStream(pfad);
      dora  = new java.io.DataInputStream(fiona);
      // Datenfluss: Datei Stroeme04.tmp --> fiona --> dora --> Programm

      // Zuerst vier float-Werte lesen und ...
      f1 = dora.readFloat();
      f2 = dora.readFloat();
      f3 = dora.readFloat();
      f4 = dora.readFloat();

      // ... danach vier int-Werte:
      i1 = dora.readInt  ();
      i2 = dora.readInt  ();
      i3 = dora.readInt  ();
      i4 = dora.readInt  ();

      dora.close();
      dat01.delete(); // Datei dat01 loeschen

      gibAlleAus("Erst f1 bis f4 gelesen, dann i1 bis i4:");
      // ------------------------------------------------------------------
      printf("-----------------------------%n");
      printf("Stroeme04: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Stroeme04
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme04 zur Standardausgabe (zum Bildschirm):

In die Datei Stroeme04.tmp wurden geschrieben: 32 Bytes
--------------------------------------------------------------
Erst i1 bis i4 geschrieben, dann f1 bis f4:
i1: 10          i2: 20          i3: 1092616192  i4: 1101004800
f1: 1.4E-44     f2: 2.8E-44     f3: 10.0        f4: 20.0
--------------------------------------------------------------
Erst i1 bis i4 gelesen, dann f1 bis f4:
i1: 10          i2: 20          i3: 1092616192  i4: 1101004800
f1: 1.4E-44     f2: 2.8E-44     f3: 10.0        f4: 20.0
--------------------------------------------------------------
Erst f1 bis f4 gelesen, dann i1 bis i4:
i1: 10          i2: 20          i3: 1092616192  i4: 1101004800
f1: 1.4E-44     f2: 2.8E-44     f3: 10.0        f4: 20.0
-----------------------------
Stroeme04: Das war's erstmal!
===========================================================================
Inhalt der Datei Stroeme04.tmp (in hexadezimaler Darstellung):

000000  00 00 00 0A  00 00 00 14  41 20 00 00  41 A0 00 00  ........A ..A...
000010  00 00 00 0A  00 00 00 14  41 20 00 00  41 A0 00 00  ........A ..A...
------------------------------------------------------------------------ */