// Datei Stroeme05.java
/* ------------------------------------------------------------------------
Demonstriert byteorientierte Ein-/Ausgabe von/zu Dateien.

Schreibt man Strings mit der Methode writeUTF (eines DataOutputStream-
Objekts) in eine Datei, dann steht in der Datei auch die Laenge des
geschriebenen Strings. Solche Strings kann man dann mit der Methode
readUTF eines DataInputStreams wieder einlesen.

UCS  : Universal Character Set
UTF-8: UCS Tramsformation Code, 8-bit form.

Im UTF-8 Code belegen Grossbuchstaben, Kleinbuchstaben, Ziffern und ein
paar Sonderzeichen je 8 Bit, landesspezifische Zeichen wie , , ,  etc.
je 16 Bit und weitere Zeichen (z.B. chinesische Zeichen) 24 bzw. 32 Bit.
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.File;

class Stroeme05 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) throws java.io.IOException {
      // In eine Datei werden werden Strings unterschiedlicher Laenge
      // geschrieben (mit der Methode writeUTF) und dann von dort wieder
      // eingelesen  (mit der Methode readUTF).

      String pfad  = "Stroeme05.tmp";
      File   dat01 = new File(pfad);
      // ------------------------------------------------------------------
      // Eine Datei und zwei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream felix = new FileOutputStream(dat01);
      DataOutputStream david = new DataOutputStream(felix);
      // Datenfluss: Datei Stroeme05.tmp <-- felix <-- david <-- Programm

                                              // Laengen der Strings:
      david.writeUTF("ABCDEF");               //  6 Zeichen, hex: 00 06
      david.writeUTF("abcdef");               //  6 Zeichen, hex: 00 06
      david.writeUTF("12345678901234");       // 14 Zeichen, hex: 00 0E
      david.writeUTF("()[]{}<>.,:;!?");       // 14 Zeichen, hex: 00 0E
      david.writeUTF("");              // 14 Zeichen, hex: 00 0E
                                              // ----------------------
                                              // 54 Zeichen insgesamt
      david.close();

      // Auch nach dem Schliessen der Datei weiss der Strom david noch,
      // wieviele Bytes er insgesamt in die Datei geschrieben hat:
      printf("In die Datei %s wurden geschrieben: %d Bytes%n",
         pfad, david.size());
      printf("-------------------------------------------------------%n");
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream fiona = new FileInputStream(pfad);
      DataInputStream dora  = new DataInputStream(fiona);
      // Datenfluss: Datei Stroeme05.tmp --> fiona --> dora --> Programm

      // Alle Strings aus der Datei lesen und zur Standardausgabe
      // ausgeben:
      while(dora.available() > 0) { // Solange noch Zeichen vorhanden sind
         printf("%s%n", dora.readUTF());
      } // while

      dora.close();
      dat01.delete(); // Datei dat01 loeschen
      // ------------------------------------------------------------------
      printf("-----------------------------%n");
      printf("Stroeme05: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Stroeme05
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme05 zur Standardausgabe (zum Bildschirm):

In die Datei Stroeme05.tmp wurden geschrieben: 64 Bytes
-------------------------------------------------------
ABCDEF
abcdef
12345678901234
()[]{}<>.,:;!?

-----------------------------
Stroeme05: Das war's erstmal!
===========================================================================
Inhalt der Datei Stroeme05.tmp (in hexadezimaler Darstellung):

000000  00 06 41 42  43 44 45 46  00 06 61 62  63 64 65 66  ..ABCDEF..abcdef
000010  00 0E 31 32  33 34 35 36  37 38 39 30  31 32 33 34  ..12345678901234
000020  00 0E 28 29  5B 5D 7B 7D  3C 3E 2E 2C  3A 3B 21 3F  ..()[]{}<>.,:;!?
000030  00 0E C3 84  C3 A4 C3 96  C3 B6 C3 9C  C3 BC C3 9F  ..++++++++
------------------------------------------------------------------------ */