// Datei Stroeme06.java
/* ------------------------------------------------------------------------
Zwei Filterklassen (Erweiterungen der Klasse FilterWriter) namens
BlanksRausFilterWriter und AllesGrossFilterWriter) werden vereinbart.
Objekte fw1 und fw2 dieser Klassen werden mit einer Roehre (engl. pipe,
bestehend aus einem PipedWriter-Objekt pw und einem PipedReader-Objekt pr)
zu einer "vierteiligen" Roehre fw2 --> fw1 --> pw --> pr verbunden.
Ein paar Strings werden durch diese Roehre geschickt (und voher und
nachher zur Standardausgabe ausgegeben).

Anstelle der Klasse FilterWriter haette man hier auch die Klasse
FilterReader erweitern koennen, aber ein moeglichst fruehes Entfernen
von Blanks "schon am Schreibende der Roehre" ist ein bisschen effizienter
als ein spaeteres Entfernen "am Leseende der Roehre".
------------------------------------------------------------------------ */
import java.io.IOException;
import java.io.PipedWriter;
import java.io.PipedReader;
import java.io.Writer;
import java.io.FilterReader;
import java.io.FilterWriter;
// ------------------------------------------------------------------------
class BlanksRausFilterWriter extends FilterWriter {
   // Filtert alle Blanks raus
   BlanksRausFilterWriter(Writer out) {super(out);} // Konstruktor

   static final char BLANK = ' ';

   // Eine write-Methode, mit der man ein einzelnes Zeichen (als
   // int-Wert, nicht als char-Wert!) in diesen Filterstrom
   // schreiben kann:
   public void write(int n) throws IOException {
      if (n != BLANK) super.write(n);
   } // write

   // Eine write-Methode, mit der man einen String in diesen
   // Filterstrom schreiben kann:
   public void write(String s) throws IOException {
      StringBuffer sb = new StringBuffer(s.length());
      for (int i=0; i<s.length(); i++) {
         if (s.charAt(i) != BLANK) sb.append(s.charAt(i));
      }
      if (sb.length() > 0) super.write(sb.toString());
   } // write
} // BlanksRausFilterWriter
// ------------------------------------------------------------------------
class AllesGrossFilterWriter extends FilterWriter {
   // Wandelt alle Kleinbuchstaben in Grossbuchstaben um
   AllesGrossFilterWriter(Writer out) {super(out);} // Konstruktor

   public void write(int n) throws IOException {
      super.write(Character.toUpperCase((char)n));
   } // write

   public void write(String s) throws IOException {
      StringBuffer sb = new StringBuffer(s.length());
      for (int i=0, laenge=s.length(); i<laenge; i++) {
         write(s.charAt(i));
      }
   } // write
} // AllesGrossFilterWriter
// ------------------------------------------------------------------------
class Stroeme06 {
   // ---------------------------------------------------------------------
   static public void main(String[] susi) throws IOException{
      pln("Stroeme06: Jetzt geht es los!");
      // ------------------------------------------------------------------
      // Einen Ausgabepuffer pw und einen Eingabepuffer pr vereinbaren
      // und miteinander verbinden:
      PipedReader  pr  = new PipedReader();
      PipedWriter  pw  = new PipedWriter(pr);
      FilterWriter fw1 = new BlanksRausFilterWriter(pw);
      FilterWriter fw2 = new AllesGrossFilterWriter(fw1);
      // Datenfluss:     pw <-- fw1 <-- fw2 <-- Programm
      //                  |
      //                 pr ------------------> Programm
      // ------------------------------------------------------------------
      // 4 * 10 gleich 40 Zeichen in den Strom f2 schreiben und
      // zur Standardausgabe ausgeben:
      pln("-------------------------------------");
      pln("In den Strom fw2 geschrieben:");
      for (int i=1; i<=4; i++) {
         String s1 = "Hallo" + i + ",   ";
         fw2.write(s1);
         p(s1);
      }
      System.out.println();
      // ------------------------------------------------------------------
      // Alle Zeichen aus dem Strom pr lesen und
      // zur Standardausgabe ausgeben:
      pln("-------------------------------------");
      pln("Aus dem Strom pr gelesen:");
      while (pr.ready()) { // Solange noch Zeichen im Puffer pr sind
         char c = (char) pr.read();
         System.out.print(c);
      }
      pln();
      // ------------------------------------------------------------------
      // Die Stroeme schliessen:
      pw.close();
      pr.close();
      // ------------------------------------------------------------------
      pln("-------------------------------------");
      pln("Stroeme06: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
} // class Stroeme06
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme06:

Stroeme06: Jetzt geht es los!
-------------------------------------
In den Strom fw2 geschrieben:
Hallo1,   Hallo2,   Hallo3,   Hallo4,
-------------------------------------
Aus dem Strom pr gelesen:
HALLO1,HALLO2,HALLO3,HALLO4,
-------------------------------------
Stroeme06: Das war's erstmal!
------------------------------------------------------------------------ */