// Datei Stroeme07.java
/* ------------------------------------------------------------------------
Wenn man z.B. den int-Wert 1094861636 "direkt" (nur in eine Folge von
vier byte-Werten umgewandelt) zum Bildschirm ausgibt, erscheint dort
die Zeichenfolge ABCD. Wenn man andere int-Werte "direkt" zum Bildschirm
ausgibt, erscheinen dort nur "schwer lesbare Zeichenfolgen".

Um einen int-Wert lesbar auszugeben, muss man ihn vorher in eine geeignete
Folge von Zeichen umwandeln (z.B. in eine Folge von Dezimalziffern oder
in eine Folge von Hexadezimalziffern oder ... etc.).
------------------------------------------------------------------------ */
import java.io.IOException;
import java.io.DataOutputStream;
// ------------------------------------------------------------------------
class Stroeme07 {
   // Die Standardausgabe System.out (normalerweise: der Bildschirm) wird
   // mit einem DataOutputStream-Objekt dos verbunden. Ein Befehl wie
   // dos.writeInt(123) bewirkt dann, dass der int-Wert 123 nur in eine
   // Folge von vier byte-Werten umgewandelt und so zur Standardausgabe
   // ausgegeben wird.

   static DataOutputStream dos = new DataOutputStream(System.out);

   // Datenfluss:
   // Bildschirm <-- System.out <-+- dos <-- Programm (z.B. writeInt)
   //                             |
   //                             +------<-- Programm (z.B. print)

   // ---------------------------------------------------------------------
   static void gibIntAus(int n) {
      // Gibt den int-Wert n auf drei verschiedene Weisen zur
      // Standardausgabe (System.out bzw. dos) aus:
      // 1. Umgewandelt in eine Zeichenfolge, als Dezimalzahl.
      // 2. Umgewandelt in eine Zeichenfolge, als Hexadezimalzahl.
      // 3. Umgewandelt in eine Folge von vier byte-Werten.

      String hex = Integer.toHexString(n);

      try {
         printf("%14d",   n);  // dezimal
         printf("%14s", hex);  // hexadezimal
         printf("%14s", " ");
         dos.writeInt(n);      // vier byte-Werte
         printf("%n");
      } catch (IOException ioe) {
         printf("Ausnahme in Methode gibIntAus:%n");
         printf("%s%n", ioe);
      } // try-catch
   } // gibIntAus
   // ---------------------------------------------------------------------
   static void gibUeberschriftAus() {
      printf("Ein paar spezielle int-Werte werden auf drei%n");
      printf("verschiedene Weisen ausgegeben:%n");
      printf("%14s", "Dezimal:");
      printf("%14s", "Hexadezimal:");
      printf("%18s", "Als 4 byte-Werte:");
      printf("%n");
   } // gibUeberschriftAus
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws IOException{
      // ------------------------------------------------------------------
      // Ein paar spezielle int-Werte werden auf verschiedene Weisen
      // zum Bilschirm ausgegeben:

      gibUeberschriftAus();
      printf("----------------------------------------------%n");
      gibIntAus(1094861636);  // hex.: 0x41424344
      gibIntAus(1633837924);  // hex.: 0x61626364
      gibIntAus( 825373492);  // hex.: 0x31323334
      gibIntAus( 808464432);  // hex.: 0x30303030
      gibIntAus( 556411711);  // hex.: 0x212A2B3F
      gibIntAus( 543236128);  // hex.: 0x20612020
      gibIntAus(1234567890);  // hex.: 0x499602d2
      printf("----------------------------------------------%n");
      // ------------------------------------------------------------------
      printf("Stroeme07: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Stroeme07
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme07:

Ein paar spezielle int-Werte werden auf drei
verschiedene Weisen ausgegeben:
      Dezimal:  Hexadezimal: Als 4 byte-Werte:
----------------------------------------------
    1094861636      41424344              ABCD
    1633837924      61626364              abcd
     825373492      31323334              1234
     808464432      30303030              0000
     556411711      212a2b3f              !*+?
     543236128      20612020               a
    1234567890      499602d2              I
----------------------------------------------
Stroeme07: Das war's erstmal!
------------------------------------------------------------------------ */