// Datei Stroeme08.java
/* ------------------------------------------------------------------------
Vier PipedReader-Objekte und vier PipedWriter-Objekte werden vereinbart
und auf verschiedene Weisen paarweise miteinander verbunden.
Dann wird jedes Paar (bestehend aus einem Reader und einem Writer)
"ein bisschen ausprobiert".

Ein BufferedReader wird mit einem der        PipedReader verbunden und
ein BufferedWriter wird mit dem zugehoerigen PipedWriter verbunden.
Dann werden die beiden gepufferten Stroeme "ein bisschen ausprobiert".
------------------------------------------------------------------------ */
import java.io.PipedReader;
import java.io.PipedWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.IOException;
// ------------------------------------------------------------------------
class Stroeme08 {
   // ---------------------------------------------------------------------
   static void machWas(PipedWriter w, PipedReader r)  throws IOException {
      // Ein String und paar Zeichen werden nach w geschrieben.
      // Anschliessend werden alle Zeichen aus r gelesen und zur
      // Standardausgabe ausgegeben.

      // Einen String nach w schreiben:
      w.write("012345");

      // Zeichen nach w schreiben:
      for (char c='A'; c<='E'; c++) {
         w.write(c);
      }

      // Alle Zeichen aus r lesen und ausgeben:
      while (r.ready()) {
         p((char) r.read() + " ");
      }
      pln();

   } // machWas
   // ---------------------------------------------------------------------
   static void machWas(BufferedWriter w, BufferedReader r)  throws
      IOException
   {
      // Ein paar Strings mit und ohne newline-Zeichen darin werden
      // nach w geschrieben. Dann werden Zeilen (begrenzt durch die
      // newline-Zeichen) aus r gelesen und zur Standardausgabe
      // ausgegeben.

      // Die Strings mit und ohne \n-Zeichen darin:
      String[] sr = {"ABCDE\nab", "c\n1234\nA", "BC", "DE", "F\n"};

      // Alle Strings aus sr nach w schreiben:
      for (String s: sr) w.write(s);

      // Damit die letzten Zeichen aus w weitergereicht werden ist
      // einer der folgenden beiden Befehle notwendig:
//    w.flush(); // Entweder
      w.close(); // oder!

      // Alle Zeilen aus r lesen und ausgeben:
      while (r.ready()) {
         pln(r.readLine());
      }
      r.close();
   } // machWas
   // ---------------------------------------------------------------------
   static public void main(String[] sonja)  throws IOException {
      // ------------------------------------------------------------------
      PipedReader r1 = new PipedReader();
      PipedWriter w1 = new PipedWriter(r1);
      // Datenfluss:   w1 <-- Programm
      //               |
      //               r1 --> Programm

      PipedWriter w2 = new PipedWriter();
      PipedReader r2 = new PipedReader(w2);
      // Datenfluss:   w2 <-- Programm
      //               |
      //               r3 --> Programm

      PipedReader r3 = new PipedReader();
      PipedWriter w3 = new PipedWriter();
      r3.connect (w3);
      // Datenfluss:   w3 <-- Programm
      //               |
      //               r3 --> Programm

      PipedReader r4 = new PipedReader();
      PipedWriter w4 = new PipedWriter();
      w4.connect (r4);
      // Datenfluss:   w4 <-- Programm
      //               |
      //               r4 --> Programm

      BufferedReader br4 = new BufferedReader(r4);
      BufferedWriter bw4 = new BufferedWriter(w4);
      // Datenfluss:   r4 <-- br4 <-- Programm
      //               |
      //               w4 --> bw4 --> Programm

      pln("-----------------------------");
      machWas( w1,  r1);
      machWas( w2,  r2);
      machWas( w3,  r3);
      machWas( w4,  r4);
      pln("-----------------------------");
      machWas(bw4, br4);
      pln("-----------------------------");
      r1.close(); r2.close(); r3.close(); r4.close();
      w1.close(); w2.close(); w3.close(); w4.close();
      pln("Stroeme08: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Stroeme08
/* ------------------------------------------------------------------------
Ausgabe des Progrmms Stroeme08:

-----------------------------
0 1 2 3 4 5 A B C D E
0 1 2 3 4 5 A B C D E
0 1 2 3 4 5 A B C D E
0 1 2 3 4 5 A B C D E
-----------------------------
ABCDE
abc
1234
ABCDEF
-----------------------------
Stroeme08: Das war's erstmal!
------------------------------------------------------------------------ */