// Datei Stroeme09.java
/* ------------------------------------------------------------------------
In dieser Datei werden folgende Klassen vereinbart:

 BlanksRausFilterWriter (Entfernt alle Blanks)
ZiffernRausFilterWriter (Entfernt alle Ziffern)
  MinusReinFilterReader (Fuegt zwischen je 2 Zeichen ein Minuszeichen ein)
 AllesGrossFilterReader (Wandelt alle kleinen Buchstaben in grosse um)
 Stroeme09              (Hauptklasse eines Programms)

Im Programm Stroeme09 werden zwei FilterWriter (fw), ein PipedWriter
(pw), ein PipedReader (pr) und zwei FilterReader (fr) miteinander
verbunden. Damit ergibt sich folgender Datenfluss:

     pw3 <-- fw2 <-- fw1 <-- Programm
      |
     pr4 --> fr5 --> fr6 --> Programm

Ein paar Strings werden in den Filter fw1 geschrieben. Dann werden alle
Zeichen aus dem Filter fr6 gelesen. Die verschiedenen Filter bearbeiten
die Daten, die durch sie durchgehen, und die Roehren (pipes, pw3 und pr4)
speichern die schon geschriebenen aber noch nicht wieder gelesenen Daten.

FilterWriter eignen sich besonders zum Entfernen von Zeichen.
FilterReader eignen sich besonders zum Einfuegen von Zeichen.
Zum Umwandeln von Zeichen eignen sich beide Arten von Filtern.
------------------------------------------------------------------------ */
import java.io.IOException;
import java.io.PipedWriter;
import java.io.PipedReader;
import java.io.Writer;
import java.io.Reader;
import java.io.FilterReader;
import java.io.FilterWriter;
// ========================================================================
class BlanksRausFilterWriter extends FilterWriter {
   // Filtert alle Blanks raus

   // In das folgende Writer-Objekt schreibt dieser BlanksRausFilterWriter
   // seine Daten:
   Writer nachFolger;

   public BlanksRausFilterWriter(Writer nachFolger) {
      super(nachFolger);
      this.nachFolger = nachFolger;
   } // Konstruktor BlanksRausFilterWriter

   static final char BLANK = ' ';

   public void write(int n) throws IOException {
      if (n != BLANK) nachFolger.write(n);
   } // write

   public void write(String s) throws IOException {
      StringBuffer sb = new StringBuffer(s.length());
      for (int i=0; i<s.length(); i++) {
         char c = s.charAt(i);
         if (c != BLANK) sb.append(c);
      }
      if (sb.length() > 0) nachFolger.write(sb.toString());
   } // write

} // class BlanksRausFilterWriter
// ========================================================================
class ZiffernRausFilterWriter extends FilterWriter {
   // Filtert alle Dezimalziffern (0 bis 9) raus

   // In das folgende Writer-Objekt schreibt dieser ZiffernRausFilterWriter
   // seine Daten:
   Writer nachFolger;

   public ZiffernRausFilterWriter(Writer nachFolger) {
      super(nachFolger);
      this.nachFolger = nachFolger;
   } // Konstruktor ZiffernRausFilterWriter

   public void write(int n) throws IOException {
      if (n < '0' || '9' < n) nachFolger.write(n);
   } // write

   public void write(String s) throws IOException {
      StringBuffer sb = new StringBuffer(s.length());
      for (int i=0; i<s.length(); i++) {
         char c = s.charAt(i);
         if (c < '0' || '9' < c) sb.append(c);
      }
      if (sb.length() > 0) nachFolger.write(sb.toString());
   } // write

} // class ZiffernRausFilterWriter
// ========================================================================
class MinusReinFilterReader extends FilterReader {
   // Fuegt zwischen je zwei Zeichen ein Minuszeichen ein.

   // Aus dem folgenden Reader-Objekt bezieht dieser MinusReinFilterReader
   // seine Daten:
   Reader nachFolger;

   public MinusReinFilterReader(Reader nachFolger) {
      super(nachFolger);
      this.nachFolger = nachFolger;
   } // Konstruktor MinusReinFilterReader

   boolean geradeZeichenNr = true;

   public int read()  throws IOException {
      geradeZeichenNr = ! geradeZeichenNr;
      if (!nachFolger.ready())  return  -1;
      if (geradeZeichenNr)      return '-';
      return nachFolger.read();
   } // read

} // class MinusReinFilterReader
// ========================================================================
class AllesGrossFilterReader extends FilterReader {
   // Wandelt alle kleinen Buchstaben in grosse Buchstaben um

   // Aus dem folgenden Reader-Objekt liest dieses AllesGrossFilterReader-
   // Objekt seine Daten:
   Reader nachFolger;

   public AllesGrossFilterReader(Reader nachFolger) {
      super(nachFolger);
      this.nachFolger = nachFolger;
   } // Konstruktor AllesGrossFilterReader

   public int read() throws IOException {
      return (Character.toUpperCase((char) nachFolger.read()));
   } // read

} // class AllesGrossFilterReader
// ========================================================================
class Stroeme09 {
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) throws IOException{
      // ------------------------------------------------------------------
      // Insgesamt sechs Stroeme erzeugen und miteinander vebinden:
       PipedReader pr4 = new PipedReader            ();
      FilterReader fr5 = new AllesGrossFilterReader (pr4);
      FilterReader fr6 = new MinusReinFilterReader  (fr5);
       PipedWriter pw3 = new PipedWriter            (pr4);
      FilterWriter fw2 = new BlanksRausFilterWriter (pw3);
      FilterWriter fw1 = new ZiffernRausFilterWriter(fw2);
      // Datenfluss:
      // Programm-->fw1-->fw2-->pw3-->pr4-->fr5-->fr6-->Programm
      // ------------------------------------------------------------------
      // Drei Strings der Laenge 8 in den Ausgabefilter fw1 schreiben und
      // zur Standardausgabe ausgeben:
      pln("-------------------------------------------------------------");
      p  ("In  den Filter fw1 geschrieben: ");
      for (int i=1; i<=3; i++) {
         String s1 = "Hallo" + i + "  "; // 5 Buchstaben, 1 Ziffer, 2 Blanks
         fw1.write(s1);                  // In den Filter fw1
         p(s1);                          // Zur Standardausgabe
      }
      pln();
      // ------------------------------------------------------------------
      // Alle Zeichen aus dem Eingabefilter fr6 lesen und
      // zur Standardausgabe ausgeben:
      int  n;
      char c;
      p  ("Aus dem Filter fr6 gelesen:     ");
      while (fr6.ready()) { // Solange noch Zeichen im Filter fr6 sind
         n = fr6.read();    // Ein Zeichen (als int-Wert) lesen
         c = (char) n;      // In einen char-Wert umwandeln
         p(c + "");         // und zur Standardausgabe ausgeben.
      } // while
      pln();
      pln("-------------------------------------------------------------");
      // ------------------------------------------------------------------
      pln("Stroeme09: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Stroeme09
/* ------------------------------------------------------------------------
Ausgabe des Programms Stroeme09:

-------------------------------------------------------------
In  den Filter fw1 geschrieben: Hallo1  Hallo2  Hallo3
Aus dem Filter fr6 gelesen:     H-A-L-L-O-H-A-L-L-O-H-A-L-L-O
-------------------------------------------------------------
Stroeme09: Das war's erstmal!
------------------------------------------------------------------------ */