// Datei Switch05.java
/* ------------------------------------------------------------------------
In den Fallmarken einer switch-Anweisung darf man nur sogenannte
konstante Ausdruecke verwenden. Das sind Ausdruecke, deren Werte
der Ausfuehrer schon bei der Uebergabe des Programms ("zur Compile-
Zeit") berechnen kann, und nicht erst waehrend einer Ausfuehrung
des Programms ("zur Laufzeit"). Solche Ausdruecke duerfen im Wesent-
lichen nur aus Literalen, ein paar Operatoren (den einstelligen
Operatoren + und - und den zweistelligen Operatoren +, -, *, /, % und
ein paar weiteren) und und aus Konstanten bestehen, die mit konstanten
Ausdruecken initialisiert wurden.
Das Unterprogramm upro enthaelt ein paar Beispiele.
------------------------------------------------------------------------ */
class Switch05 {
   // ---------------------------------------------------------------------
   static void upro(int n) {
      // Eine Konstante N1, die mit einem nicht-konstanten Ausdruck
      // initialisiert wird (und deshabl selbst nicht in einem
      // konstanten Ausdruck vorkommen darf):
      final int N1 = n+3;

      // Eine Konstante K1, die mit einem konstanten Ausdruck
      // initialisiert wird (und deshalb selbst auch in einem
      // konstanten Ausdruck vorkommen darf):
      final int K1 = 3+2;


      // In den Fallmarken einer switch-Amweisung (nach dem
      // Schluesselwort case) darf man nur konstante Ausdruecke
      // angeben:

      switch (n) {
         case 17:   pln("Fall   17! (" + n + ")");  break;
         case 'A':  pln("Fall  'A'! (" + n + ")");  break;
         case K1:   pln("Fall   K1! (" + n + ")");  break;
         case K1+3: pln("Fall K1+3! (" + n + ")");  break;
         case -K1:  pln("Fall  -K1! (" + n + ")");  break;
         case 2*K1: pln("Fall 2*K1! (" + n + ")");  break;
//       case N1:   pln("Fall N1!");                break;
      } // switch
   } // upro
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Das Unterprogramm upro wird mit verschiedenen Parametern
      // aufgerufen.

      pln("----------------------------");
      p("A  "); upro(17);
      p("B  "); upro('A');
      p("C  "); upro(5);
      p("D  "); upro(8);
      p("E  "); upro(-5);
      p("F  "); upro(10);
      pln("----------------------------");
      pln("Switch05: Das war's erstmal!");

   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Switch05
/* ------------------------------------------------------------------------
Ausgabe des Programms Switch05:

----------------------------
A  Fall   17! (17)
B  Fall  'A'! (65)
C  Fall   K1! (5)
D  Fall K1+3! (8)
E  Fall  -K1! (-5)
F  Fall 2*K1! (10)
----------------------------
Switch05: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldung des Compilers (jikes von IBM) wenn die Zeile 39 kein
Kommentar ist:

    39.          case N1:   pln("Fall N1!");                break;
                      <>
*** Error: A constant expression is expected in this context.
------------------------------------------------------------------------ */