// Datei Switch07.java
/* ------------------------------------------------------------------------
Der Ausdruck einer switch-Anweisung muss zu einem der Typen byte, char,
short oder int gehoeren und darf insbesondere nicht zum Typ String ge-
hoeren. Wenn man trotzdem eine "Fallunterscheidung ueber Strings"
programmieren will, kann man so vorgehen:

1. Man vereinbart eine Reihung sr mit allen relevanten Strings darin.
2. Mit der Funktion Arrays.asList wandelt man die Reihung sr in ein
   List-Objekt sl um.
3. Mit der Funktion sl.indexOf kann man dann einen beliebigen String s
   in einen ensprechenden Index fuer dieses List-Objekt umwandeln. Der
   Ausdruck sl.indexOf(s) gehoert zum Typ int und ist somit als
   Ausdruck einer switch-Anweisung geeignet.
   Uebrigens: Falls der String s nicht in sl vorkommt, hat der Ausdruck
   sl.indexOf(s) den Wert -1.
4. Um die switch-Anweisung leichter lesbar zu machen, kann man fuer
   jeden String in der Reihung sr (bzw. in der Liste sl) eine
   int-Konstante mit ihrem Index vereinbaren.

Das Unterprogramm stringFaelle enthaelt eine Fallunterscheidung ueber
drei Strings ("Rotkehlchen", "Blaukehlchen" und "Buntspecht").
------------------------------------------------------------------------ */
import java.util.List;
import java.util.Arrays;

class Switch07 {
   // ---------------------------------------------------------------------
   static void stringFaelle(String vogelName) {

      // Eine Reihung sr mit allen relevanten Strings darin vereinbaren
      // und in ein List-Objekt sl umwandeln:
      final String[] sr  = {"Rotkehlchen", "Blaukehlchen", "Buntspecht"};
      final List     sl  = Arrays.asList(sr);

      // Fuer jeden relevanten String eine int-Konstante vereinbaren:
      final int ROTKEHLCHEN  = 0;
      final int BLAUKEHLCHEN = 1;
      final int BUNTSPECHT   = 2;

      // Jetzt ist eine Fallunterscheidung ueber den int-Wert des
      // Ausdrucks sl.indexOf(s) moeglich:
      switch (sl.indexOf(vogelName)) {
         case ROTKEHLCHEN:  pln("Fall Rotkehlchen!");   break;
         case BLAUKEHLCHEN: pln("Fall Blaukehlchen!");  break;
         case BUNTSPECHT:   pln("Fall Buntspecht!");    break;
         default:           pln("Unebekannter Vogel!"); break;
      } // switch
   } // stringFaelle
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die Funktion stringFaelle wird mit verschiedenen Parametern
      // aufgerufen:

      pln("----------------------------");
      stringFaelle("Rotkehlchen");
      stringFaelle("Blaukehlchen");
      stringFaelle("Buntspecht");
      stringFaelle("Forelle blau");
      pln("----------------------------");
      pln("Switch07: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Switch07
/* ------------------------------------------------------------------------
Ausgabe des Programms Switch07:

----------------------------
Fall Rotkehlchen!
Fall Blaukehlchen!
Fall Buntspecht!
Unebekannter Vogel!
----------------------------
Switch07: Das war's erstmal!
------------------------------------------------------------------------ */