// Datei TypUmwandlungen01.java
/* ------------------------------------------------------------------------
Wenn aus einem Wert w1 eines Typs T1 ein entsprechender Wert w2 eines
anderen Typs T2 berechnet wird, dann bezeichnet man das als eine
Typumwandlung.

Dieses Programm demonstriert Typumwandlungen zwischen numerischen Typen
(d.h. zwischen den Typen byte, char, short, int, long, float und double).
Es gibt keine Daten aus (d.h. es soll nur dem Ausfuehrer uebergeben,
aber nicht ausgefuehrt werden).

Zwischen diesen 7 Typen gibt es insgesamt 49 verschiedene Typumwandlungen.
Davon sind 7 identische Umwandlungen (z.B. von byte nach byte oder von int
nach int etc.). Von den restlichen 42 Typumwandlungen sind 19 erweiternde
Umwandlungen und 23 verengende Umwandlungen.

Eine Typumwandlung von T1 nach T2 ist erweiternd, wenn es zu jedem Wert
des Typs T1 einen gleichen oder zumindest ungefaehr gleichen Wert des
Typs T2 gibt.

Beispiel: Zum int-Wert 1234567890 gibt es keinen genau gleichen float-Wert,
aber der ungefaehr gleichen float-Wert 1.234568E9.
------------------------------------------------------------------------ */
class TypUmwandlungen01 {
   static public void main(String[] sally) {
      // ------------------------------------------------------------------
      // Ein paar numerische Variablen als "Rohmaterial" fuer die
      // nachfolgenden (erweiternden und verengenden) Typumwandlungen:

      byte     byte01 = 17;
      char     char01 = 17;
      short   short01 = 17;
      int       int01 = 17;
      long     long01 = 17;
      float   float01 = 17.3F; // F wie "float"  muss sein!
      double double01 = 17.3D; // D wie "double" kann sein!
      // ------------------------------------------------------------------
      // Beispiele fuer erweiternde Typumwandlungen, die der Ausfueher
      // automatisch und ohne expliziten Cast-Befehl durchfuehrt:

      short01  =  byte01;  // Umwandlung von byte nach short

      int01    =  byte01;  // Umwandlung von byte nach int
      int01    =  char01;  // Umwandlung von char nach int
      int01    = short01;  // ... etc.

      long01   =  byte01;
      long01   =  char01;
      long01   = short01;
      long01   =   int01;

      float01  =  byte01;
      float01  =  char01;
      float01  = short01;
      float01  =   int01;  // Verlust an Genauigkeit moeglich!
      float01  =  long01;  // Verlust an Genauigkeit moeglich!

      double01 =  byte01;
      double01 =  char01;
      double01 = short01;
      double01 =   int01;
      double01 =  long01;  // Verlust an Genauigkeit moeglich!
      double01 = float01;
      // ------------------------------------------------------------------
      // Ein paar numerische Konstanten und Variablen als "Rohmaterial"
      // fuer die nachfolgenden verengenden Typumwandlungen:

      final int   INT01  = 120;     // Eine Konstante
      final int   INT02  = 65000;   // Eine Konstante
      final int   INT03  = 32000;   // Eine Konstante
            int   int04  = 1;       // Eine Variable, keine Konstante!
      final long  LONG01 = 1;       // Eine Konstante, aber long!
      // ------------------------------------------------------------------
      // Beispiele fuer verengende Typumwandlungen, von denen der
      // Ausfuehrer einige automatisch durchfuehrt und einige ablehnt
      // (weil er feststellt, dass sie zu einem Verlust der Groessenordnung
      // des betreffenden Wertes fuehren wuerden, oder einfach so, weil er
      // keine long-Werte verengend umwandeln mag):

      byte01  = INT01;       // Erlaubt
      byte01  = INT01 + 7;   // Erlaubt
//    byte01  = INT01 + 8;   // Ohne Cast nicht erlaubt
//    byte01  = int04;       // Ohne Cast nicht erlaubt
//    byte01  = LONG01;      // Ohne Cast nicht erlaubt

      char01  = INT02;       // Erlaubt
      char01  = INT02 + 535; // Erlaubt
//    char01  = INT02 + 536; // Ohne Cast nicht erlaubt
//    char01  = int04;       // Ohne Cast nicht erlaubt
//    char01  = LONG01;      // Ohne Cast nicht erlaubt

      short01 = INT03;       // Erlaubt
      short01 = INT03 + 767; // Erlaubt
//    short01 = INT03 + 768; // Ohne Cast nicht erlaubt
//    short01 = int04;       // Ohne Cast nicht erlaubt
//    short01 = LONG01;      // Ohne Cast nicht erlaubt
      // ------------------------------------------------------------------
      // Beispiele fuer die 23 verengenden Typumwandlungen, die der
      // Ausfuehrer nur auf ausdruecklichen Befehl (Cast-Befehl) des
      // Programmierers durchfuehrt. Indem er einen Cast-Befehl
      // verwendet, uebernimmt der Programmierer die Verantwortung
      // fuer einen eventuell auftretenden Verlust an Genauigkeit
      // oder der Groessenordnung.
      // In den folgenden Beispielen tritt kein Verlust der Groessen-
      // ordnung auf, weil alle beteiligten Variablen den Wert 17 oder
      // den ungefaehr gleichen Wert 17.3 haben.

      byte01  = (byte)    char01;
      byte01  = (byte)   short01;
      byte01  = (byte)     int01;
      byte01  = (byte)    long01;
      byte01  = (byte)   float01;  // Verlust an Genauigkeit!
      byte01  = (byte)  double01;  // Verlust an Genauigkeit!

      char01  = (char)    byte01;
      char01  = (char)   short01;
      char01  = (char)     int01;
      char01  = (char)    long01;
      char01  = (char)   float01;  // Verlust an Genauigkeit!
      char01  = (char)  double01;  // Verlust an Genauigkeit!

      short01 = (short)   char01;
      short01 = (short)    int01;
      short01 = (short)   long01;
      short01 = (short)  float01;  // Verlust an Genauigkeit!
      short01 = (short) double01;  // Verlust an Genauigkeit!

      int01   = (int)     long01;
      int01   = (int)    float01;  // Verlust an Genauigkeit!
      int01   = (int)   double01;  // Verlust an Genauigkeit!

      long01  = (long)   float01;  // Verlust an Genauigkeit!
      long01  = (long)  double01;  // Verlust an Genauigkeit!

      float01 = (float) double01;  // Verlust an Genauigkeit!

      pln("TypUmwandlungen01: Keine Ausgaben, nur Fehlermeldungen");
      pln("                   als Kommentare im Programmtext!");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // TypUmwandlungen01
/* ------------------------------------------------------------------------
Fehlermeldungen eines Compilers (jikes, von IBM) wenn die entsprechenden
Zeilen nicht auskommentiert sind:

    82.       byte01  = INT01 + 8;   // Ohne Cast nicht erlaubt
                        ^-------^
*** Semantic Error: A byte value must be an integer value in
the range -128..127.


    83.       byte01  = int04;       // Ohne Cast nicht erlaubt
              ^-------------^
*** Semantic Error: The type of the right sub-expression, "int", is
not assignable to the variable, of type "byte".


    84.       byte01  = LONG01;      // Ohne Cast nicht erlaubt
              ^--------------^
*** Semantic Error: The type of the right sub-expression, "long", is
not assignable to the variable, of type "byte".


    88.       char01  = INT02 + 536; // Ohne Cast nicht erlaubt
                        ^---------^
*** Semantic Error: A character must be in
the range 0..65535 ('\u0000'..'\uffff').


    89.       char01  = int04;       // Ohne Cast nicht erlaubt
              ^-------------^
*** Semantic Error: The type of the right sub-expression, "int", is
not assignable to the variable, of type "char".


    90.       char01  = LONG01;      // Ohne Cast nicht erlaubt
              ^--------------^
*** Semantic Error: The type of the right sub-expression, "long", is
not assignable to the variable, of type "char".


    94.       short01 = INT03 + 768; // Ohne Cast nicht erlaubt
                        ^---------^
*** Semantic Error: A short value must be an integer value in
the range -32768..32767.


    95.       short01 = int04;       // Ohne Cast nicht erlaubt
              ^-------------^
*** Semantic Error: The type of the right sub-expression, "int", is
not assignable to the variable, of type "short".


    96.       short01 = LONG01;      // Ohne Cast nicht erlaubt
              ^--------------^
*** Semantic Error: The type of the right sub-expression, "long", is
not assignable to the variable, of type "short".
------------------------------------------------------------------------ */