// Datei Typen05.java
/* ------------------------------------------------------------------------
Die extremen Werte (MIN_VALUE und MAX_VALUE) der numerischen Typen,
d.h. der primitiven Typen byte, char, short, int, long, float und double.

Den char-Werten Character.MIN_VALUE und Character.MAX_VALUE entsprechen
keine "druckbaren Zeichen". Sie werden deshalb als int-Werte ausgegeben.

Bei allen 7 numerischen Typen bezeichnet MAX_VALUE den groessten Wert.

MIN_VALUE bezeichnet bei den 5 Ganzzahltypen byte, char, short, int und
long den kleinsten Wert. Bei char ist das der Wert 0, bei den uebrigen 4
Ganzzahltypen ist das eine negative Zahl mit dem groesstmoeglichen Betrag.

MIN_VALUE bezeichnet bei den 2 Gleitpunkttypen float und double den
kleinsten positiven Wert, d.h. die kleinste Zahl, die groesser als 0 ist.
------------------------------------------------------------------------ */
import java.math.BigDecimal;

class Typen05 {
   // ---------------------------------------------------------------------
   static public void main(String[] siegfried) {

      pln("Typen05: Jetzt geht es los!");
      pln("--------------------------------------------------");
      pln("A Byte     .MIN_VALUE: " +       Byte     .MIN_VALUE);
      pln("B Byte     .MAX_VALUE: " +       Byte     .MAX_VALUE);

      pln("C Character.MIN_VALUE: " + (int) Character.MIN_VALUE);
      pln("D Character.MAX_VALUE: " + (int) Character.MAX_VALUE);

      pln("E Short    .MIN_VALUE: " +       Short    .MIN_VALUE);
      pln("F Short    .MAX_VALUE: " +       Short    .MAX_VALUE);

      pln("G Integer  .MIN_VALUE: " +       Integer  .MIN_VALUE);
      pln("H Integer  .MAX_VALUE: " +       Integer  .MAX_VALUE);

      pln("I Long     .MIN_VALUE: " +       Long     .MIN_VALUE);
      pln("J Long     .MAX_VALUE: " +       Long     .MAX_VALUE);

      pln("K Float    .MIN_VALUE: " +       Float    .MIN_VALUE);
      pln("L Float    .MAX_VALUE: " +       Float    .MAX_VALUE);

      pln("M Double   .MIN_VALUE: " +       Double   .MIN_VALUE);
      pln("N Double   .MAX_VALUE: " +       Double   .MAX_VALUE);
      // ------------------------------------------------------------------
      // Um die Extremwerte (MIN_VALUE und MAX_VALUE) der Gleitpunkttypen
      // float und double exakt auszugeben, werden entsprechende Objekte
      // der Klasse BigDecimal erzeugt:
      pln("--------------------------------------------------");
      pln("O Float    .MIN_VALUE: " + new BigDecimal(Float.MIN_VALUE));
      pln("P Float    .MAX_VALUE: " + new BigDecimal(Float.MAX_VALUE));
      pln("--------------------------------------------------");
      pln("Q Double   .MIN_VALUE: " + new BigDecimal(Double.MIN_VALUE));
      pln("R Double   .MAX_VALUE: " + new BigDecimal(Double.MAX_VALUE));
      pln("--------------------------------------------------");
      // Der Versuch, die Extremwerte der Gleitpunkttypen mit
      // printf exakt auszugeben fuehrt zu entaeuschend ungenauen
      // Ausgauben (siehe Zeile S bis V):
      printf("S Float    .MIN_VALUE: %90.88f\n",   Float.MIN_VALUE);
      printf("T Float    .MAX_VALUE: %90.50f\n",   Float.MAX_VALUE);
      pln("--------------------------------------------------");
      printf("U Double   .MIN_VALUE: %330.328f\n", Double.MIN_VALUE);
      printf("V Double   .MAX_VALUE: %330.20f\n",  Double.MAX_VALUE);
      pln("--------------------------------------------------");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln   (Object ob)             {System.out.println(ob);}
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Typen05
/* ------------------------------------------------------------------------
Ausgabe des Programms Typen05:
Jede der Ausgaben O bis V stand urspruenglich auf *einer* Zeile und wurde
von Hand auf mehrere Zeilen (mit jeweils 50 Ziffern pro Zeile) verteilt.

Typen05: Jetzt geht es los!
--------------------------------------------------
A Byte     .MIN_VALUE: -128
B Byte     .MAX_VALUE: 127
C Character.MIN_VALUE: 0
D Character.MAX_VALUE: 65535
E Short    .MIN_VALUE: -32768
F Short    .MAX_VALUE: 32767
G Integer  .MIN_VALUE: -2147483648
H Integer  .MAX_VALUE: 2147483647
I Long     .MIN_VALUE: -9223372036854775808
J Long     .MAX_VALUE: 9223372036854775807
K Float    .MIN_VALUE: 1.4E-45
L Float    .MAX_VALUE: 3.4028235E38
M Double   .MIN_VALUE: 4.9E-324
N Double   .MAX_VALUE: 1.7976931348623157E308
--------------------------------------------------
O Float    .MIN_VALUE: 1.401298464324817070923729583289916131280261941876
                       51577175706828388979108268586060148663818836212158
                       203125E-45
P Float    .MAX_VALUE: 340282346638528859811704183484516925440
--------------------------------------------------
Q Double   .MIN_VALUE: 4.940656458412465441765687928682213723650598026143
                       24764425585682500675507270208751865299836361635992
                       37979656469544571773092665671035593979639877479601
                       07818781263007131903114045278458171678489821036887
                       18636056998730723050006387409153564984387312473397
                       27316961514003171538539807412623856559117102665855
                       66867681870395603106249319452715914924553293054565
                       44401127480129709999541931989409080416563324524757
                       14786901472678015935523861155013480352649347201937
                       90268107107491703332226844753335720832431936092382
                       89345836806010601150616980975307834227731832924790
                       49825247307763759272478746560847782037344696995336
                       47017972677717585125660551199131504891101451037862
                       73816725095583738973359899366480994116420570263709
                       02792427675445652290875386825064197182655334472656
                       25E-324
R Double   .MAX_VALUE: 17976931348623157081452742373170435679807056752584
                       49965989174768031572607800285387605895586327668781
                       71540458953514382464234321326889464182768467546703
                       53751698604991057655128207624549009038932894407586
                       85084551339423045832369032229481658085593321233482
                       74797826204144723168738177180919299881250404026184
                       124858368
--------------------------------------------------
S Float    .MIN_VALUE: 0,000000000000000000000000000000000000000000001401
                       2984643248170000000000000000000000000000
T Float    .MAX_VALUE: 340282346638528860000000000000000000000,0000000000
                       0000000000000000000000000000000000000000
--------------------------------------------------
U Double   .MIN_VALUE: 0,000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       000000000000000000000000049000
V Double   .MAX_VALUE: 17976931348623157000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       00000000000000000000000000000000000000000000000000
                       000000000,00000000000000000000
--------------------------------------------------
------------------------------------------------------------------------ */