// Datei Typen12.java
/* ------------------------------------------------------------------------
Welcher Typ (wie Object, Vector, Vector<K>, Vector<Object>,
Vector<Double> etc.) ist ein Untertyp von welchem anderen solchen Typ?

java.util.Vector ist eine generische Klasse.

Es gibt keine "normalen" Variablen des Typs Vector<K>, aber
eine Methode kann einen Parameter dieses Typs haben.

Vector          ist ein roher Typ
Vector<K>       ist ein parametrisierbarer Vector-Typ
Vector<Object>  ist ein parametrisierter   Vector-Typ
Vector<Double>  ist ein parametrisierter   Vector-Typ

VectorR         ist ein Untertyp des rohen Typs Vector (nicht generisch!)
VectorP<K>      ist ein Untertyp von Vector<K>
VectorObject    ist ein Untertyp von Vector<Object>
VectorDouble    ist ein Untertyp von Vector<Double>
------------------------------------------------------------------------ */
import java.util.Vector;

class VectorR      extends Vector         {};
class VectorP<K>   extends Vector<K>      {};
class VectorObject extends Vector<Object> {};
class VectorDouble extends Vector<Double> {};
// ------------------------------------------------------------------------
class Typen12 {
   // ---------------------------------------------------------------------
   // Der Typ Vector ist ein Obertyp des Typs Vector<K>,
   // Die Typen Vector<K> und Vector<?> sind gegenseitig Untertypen
   // von sich.
   static     void mA(Vector    p) {mB(p); mC(p);} // Warnung: Vector -> Vector<K>
   static <K> void mB(Vector<K> q) {mC(q); mA(q);}
   static     void mC(Vector<?> r) {mA(r); mB(r);}
   // ---------------------------------------------------------------------
// static     void mD(Vector<Object> s) {mE(s);}   // Fehler: Vector<Object> -> Vector<String>
// static     void mE(Vector<String> t) {mD(t);}   // Fehler: Vector<String> -> Vector<Object>
   // ---------------------------------------------------------------------
   // Methoden mit einem formalen Parameter eines "interessanten Typs".
   // Jede wird spaeter mit aktuellen Parametern verschiedener
   // "interessanter Typen"  aufgerufen:
   static     void mObject (Object         p) {}
   static     void mVector (Vector         p) {}
   static <K> void mVectorK(Vector<K>      p) {} // K wie Parameter K
   static     void mVectorF(Vector<?>      p) {} // F wie Fragezeichen
   static     void mVectorO(Vector<Object> p) {} // O wie Parameter Object
   static     void mVectorD(Vector<Double> p) {} // D wie Parameter Double
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Typen12: Jetzt geht es los!%n");

      mObject (new Vector         ());
      mObject (new Vector<Object> ());
      mObject (new Vector<Number> ());
      mObject (new VectorR        ());
//    mObject (new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mObject (new VectorR<Number>()); // Fehler  (VectorR ist nicht generisch)
      mObject (new VectorP        ());
      mObject (new VectorP<Object>());
      mObject (new VectorP<Number>());
      mObject (new VectorObject   ());
      mObject (new VectorDouble   ());

      mVector (new Vector         ());
      mVector (new Vector<Object> ());
      mVector (new Vector<Number> ());
      mVector (new VectorR        ());
//    mVector (new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mVector (new VectorR<Number>()); // Fehler  (VectorR ist nicht generisch)
      mVector (new VectorP        ());
      mVector (new VectorP<Object>());
      mVector (new VectorP<Number>());
      mVector (new VectorObject   ());
      mVector (new VectorDouble   ());

      mVectorK(new Vector         ()); // Warnung: Vector  -> Vector<K>
      mVectorK(new Vector<Object> ());
      mVectorK(new Vector<Number> ());
      mVectorK(new VectorR        ()); // Warnung: VectorR -> Vector<K>
//    mVectorK(new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mVectorK(new VectorR<Number>()); // Fehler  (VectorR ist nicht generisch)
      mVectorK(new VectorP        ()); // Warnung: VectorP -> Vector<K>
      mVectorK(new VectorP<Object>());
      mVectorK(new VectorP<Number>());
      mVectorK(new VectorObject   ());
      mVectorK(new VectorDouble   ());

      mVectorF(new Vector         ());
      mVectorF(new Vector<Object> ());
      mVectorF(new Vector<Number> ());
      mVectorF(new VectorR        ());
//    mVectorF(new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mVectorF(new VectorR<Number>()); // Fehler  (VectorR ist nicht generisch)
      mVectorF(new VectorP        ());
      mVectorF(new VectorP<Object>());
      mVectorF(new VectorP<Number>());
      mVectorF(new VectorObject   ());
      mVectorF(new VectorDouble   ());

      mVectorO(new Vector         ()); // Warnung: Vector  -> Vector<Object>
      mVectorO(new Vector<Object> ());
//    mVectorO(new Vector<Number> ()); // Fehler
      mVectorO(new VectorR        ()); // Warnung: VectorR -> Vector<Object>
//    mVectorO(new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mVectorO(new VectorR<Double>()); // Fehler  (VectorR ist nicht generisch)
      mVectorO(new VectorP        ()); // Warnung: VectorP -> Vector<Object>
      mVectorO(new VectorP<Object>());
//    mVectorO(new VectorP<Double>()); // Fehler: Vector<Double> -> Vector<Object>
      mVectorO(new VectorObject   ());
//    mVectorO(new VectorDouble   ()); // Fehler: VectorDouble   -> Vector<Object>

      mVectorD(new Vector         ()); // Warnung: Vector         -> Vector<Double>
//    mVectorD(new Vector<Object> ());
      mVectorD(new Vector<Double> ()); // Fehler:  Vector<Object> -> Vector<Double>
      mVectorD(new VectorR        ()); // Warnung: VectorR        -> Vector<Double>
//    mVectorD(new VectorR<Object>()); // Fehler  (VectorR ist nicht generisch)
//    mVectorD(new VectorR<Double>()); // Fehler  (VectorR ist nicht generisch)
      mVectorD(new VectorP        ()); // Warnung: VectorP        -> Vector<Double>
//    mVectorD(new VectorP<Object>()); // Fehler:  Vector<Object> -> Vector<Dobule>
      mVectorD(new VectorP<Double>());
//    mVectorD(new VectorObject   ()); // Fehler:  VectorObject   -> Vector<Double>
      mVectorD(new VectorDouble   ());

      printf("Typen12: Das war's erstmal!%n");
   } // main

   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
} // class Typen12