// Datei Unicode03.java
/* ------------------------------------------------------------------------
Dieses Programm schreibt 8*16 kyrillische ("russische")
Unicode-Zeichen im UTF-8-Format in eine Datei,
liest sie von dort wieder ein und gibt sie zur Standard-
ausgabe aus.
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.OutputStreamWriter;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.InputStreamReader;

import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
// ------------------------------------------------------------------------
class Unicode03 {
   static public void main(String [] sonja) throws java.io.IOException {
      // Name der Ausgabedatei:
      String   pfad = "Unicode03.tmp";
      // ------------------------------------------------------------------
      // Beim Ausgeben muessen char-Werte in byte-Folgen
      // umgewandelt werden. Beim Einlesen muessen diese
      // byte-Folgen wieder in char-Werte umgewandelt werden.
      Charset        charset = Charset.forName("UTF-8");
      CharsetDecoder byte2char = charset.newDecoder();
      CharsetEncoder char2byte = charset.newEncoder();
      // ------------------------------------------------------------------
      // Eine Datei und zwei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream   felix = new FileOutputStream(pfad);
      OutputStreamWriter david = new OutputStreamWriter(felix, char2byte);
      // Datenfluss: Datei Unicode03.tmp <-- felix <-- david <-- Programm

      // Ein paar char-Werte (durch die Stroeme) in die Datei schreiben:
      char zeichen = 'Ё'; // Das erste auszugebende kyrillische Zeichen
      for (int i=1; i<=8; i++) {
         for(int j=1; j<=16; j++) {
            david.write(zeichen++);
         } // for j
         david.write('\n');
      } // for i
      david.close();
      // ------------------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fiona = new FileInputStream(pfad);
      InputStreamReader dora  = new InputStreamReader(fiona, byte2char);
      // Datenfluss: Datei Unicode03.tmp --> fiona --> dora --> Programm

      // Alle Zeichen (durch die Stroeme) aus der Datei lesen
      // und zur Standardausgabe ausgeben:

      pln("Die Datei " + pfad + " enthaelt folgende Zeichen (in hex-Darstellung):");
      int nr = 1;
      while (dora.ready()) { // Falls Datei noch nicht zu Ende
         // Die Methode read liest ein Zeichen und liefert es als int-Wert
         // (nicht als char-Wert!):
         int ein =  dora.read();
         // Zeilenwechsel-Markierungen (CR- und LF-Zeichen) ignorieren:
         if (ein == '\n' || ein == 'r') continue;
         p(Integer.toHexString(ein) + " "); // Ein Zeichen als hex-Zahl ausgeben

         // Nach jeweils 16 Zeichen eine neue Zeile beginnen:
         if (nr == 16) {
            nr = 1;
            pln("");
         } else {
            nr++;
         } // if
      }
      dora.close();
      // ------------------------------------------------------------------
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void   p(Object ob) {System.out.print(ob);}
   // ---------------------------------------------------------------------
} // class Unicode03
/* ------------------------------------------------------------------------
Ausgabe des Programms Unicode03 zur Standardausgabe (Bildschirm):

Die Datei Unicode03.tmp enthaelt folgende Zeichen (in hex-Darstellung):
401 402 403 404 405 406 407 408 409 40a 40b 40c 40d 40e 40f 410
411 412 413 414 415 416 417 418 419 41a 41b 41c 41d 41e 41f 420
421 422 423 424 425 426 427 428 429 42a 42b 42c 42d 42e 42f 430
431 432 433 434 435 436 437 438 439 43a 43b 43c 43d 43e 43f 440
441 442 443 444 445 446 447 448 449 44a 44b 44c 44d 44e 44f 450
451 452 453 454 455 456 457 458 459 45a 45b 45c 45d 45e 45f 460
461 462 463 464 465 466 467 468 469 46a 46b 46c 46d 46e 46f 470
471 472 473 474 475 476 477 478 479 47a 47b 47c 47d 47e 47f 480
------------------------------------------------------------------------ */
