// Datei Unicode04.java
/* ------------------------------------------------------------------------
Dieses Programm schreibt CJK-Zeichen ("chinesische
Schriftzeichen") in eine Datei. Der Quelltext des Programms
enthaelt aber selbst keine CJK-Zeichen und kann mit
einem gewoehnlichen Editor betrachtet und bearbeitet
werden (die Datei Unicode05.java enthaelt ein sehr
aehnliches Programm, sollte aber mit einem speziellen
Unicode-Editor betrachtet und bearbeitet werden).

CJK-Zeichen werden in China, Japan, Korea, Taiwan
und Vietnam benutzt.
Der Unicode enthaelt unter anderem Codes fuer 27.484 CJK-Zeichen
(und zusaetzlich eine Reihe von Erweiterungszeichen).
Dieses Programm schreibt 8*16 CJK-Unicode-Zeichen-Codes
im UTF-8-Format in eine Datei, liest sie von dort wieder ein
und gibt sie zur Standardausgabe aus.
------------------------------------------------------------------------ */
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.OutputStreamWriter;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.InputStreamReader;

import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
// ------------------------------------------------------------------------
class Unicode04 {
   static public void main(String [] sonja) throws java.io.IOException {
      // Name der Ausgabedatei:
      String   pfad = "Unicode04.tmp";
      // --------------------------------------------------------
      // Beim Ausgeben muessen char-Werte in byte-Folgen
      // umgewandelt werden. Beim Einlesen muessen diese
      // byte-Folgen wieder in char-Werte umgewandelt werden.
      Charset        charset = Charset.forName("UTF-8");
      CharsetDecoder byte2char = charset.newDecoder();
      CharsetEncoder char2byte = charset.newEncoder();
      // --------------------------------------------------------
      // Eine Datei und zwei Ausgabestroeme erzeugen und miteinander
      // verbinden:
      FileOutputStream   felix = new FileOutputStream(pfad);
      OutputStreamWriter oskar = new OutputStreamWriter(felix, char2byte);
      // Datenfluss: Datei Unicode04.tmp <-- felix <-- oskar <-- Programm

      // Ein paar char-Werte (durch die Stroeme) in die Datei schreiben:

      char zeichen = '\u5000'; // Das CJK-Zeichen mit der Codezahl 5000
      for (int i=1; i<=8; i++) {
         for(int j=1; j<=16; j++) {
            oskar.write(zeichen++);
         } // for j
         oskar.write('\n');
      } // for i
      oskar.close();
      // --------------------------------------------------------
      // Zwei Eingabestroeme erzeugen und mit der Datei verbinden:
      FileInputStream   fiona = new FileInputStream(pfad);
      InputStreamReader ilse  = new InputStreamReader(fiona, byte2char);
      // Datenfluss: Datei Unicode04.tmp --> fiona --> ilse --> Programm

      // Alle Zeichen (durch die Stroeme) aus der Datei lesen
      // und zur Standardausgabe ausgeben:

      pln("Die Datei " + pfad + " enthaelt folgende " +
          "Zeichen (in hex-Darstellung):");
      int nr = 1;
      while (ilse.ready()) { // Falls Datei noch nicht zu Ende
         // Die Methode read liest ein Zeichen und liefert es als int-Wert
         // (nicht als char-Wert!):
         int ein =  ilse.read();
         // Zeilenwechsel-Markierungen (CR- und LF-Zeichen) ignorieren:
         if (ein == '\n' || ein == '\r') continue;
         // Ein Zeichen als hex-Zahl ausgeben
         p(Integer.toHexString(ein) + " ");

         // Nach jeweils 16 Zeichen eine neue Zeile beginnen:
         if (nr == 16) {
            nr = 1;
            pln();
         } else {
            nr++;
         } // if

      } // while
      pln();
      ilse.close();
      // --------------------------------------------------------
   } // main
      // --------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   static void pln()          {System.out.println();  }
   // ---------------------------------------------------------------------
} // class Unicode04
/* ------------------------------------------------------------------------
Ausgabe des Programms Unicode04 zur Standardausgabe (Bildschirm):

Die Datei Unicode04.tmp enthaelt folgende Zeichen (in hex-Darstellung):
5000 5001 5002 5003 5004 5005 5006 5007 5008 5009 500a 500b 500c 500d 500e 500f
5010 5011 5012 5013 5014 5015 5016 5017 5018 5019 501a 501b 501c 501d 501e 501f
5020 5021 5022 5023 5024 5025 5026 5027 5028 5029 502a 502b 502c 502d 502e 502f
5030 5031 5032 5033 5034 5035 5036 5037 5038 5039 503a 503b 503c 503d 503e 503f
5040 5041 5042 5043 5044 5045 5046 5047 5048 5049 504a 504b 504c 504d 504e 504f
5050 5051 5052 5053 5054 5055 5056 5057 5058 5059 505a 505b 505c 505d 505e 505f
5060 5061 5062 5063 5064 5065 5066 5067 5068 5069 506a 506b 506c 506d 506e 506f
5070 5071 5072 5073 5074 5075 5076 5077 5078 5079 507a 507b 507c 507d 507e 507f
------------------------------------------------------------------------ */
