// Datei VarPar01.java
/* ------------------------------------------------------------------------
"Normale" Methoden haben eine feste Anzahl von Parameter (z. B. 2 oder 5
oder 0). Seit Java 5.0 gibt es aber auch Methoden mit variable vielen
Parametern, bei denen nur eine Mindestanzahl von Parametern festgelegt
ist, aber kein Hoechstanzahl.

Die Methode sum kann man mit beliebig vielen aktuellen Parametern
aufrufen (man muss aber mindestens 2 Parameter angeben).

Die Methode max kann man mit beliebig vielen aktuellen Parametern
aufrufen (auch Aufrufe mit 0 oder einem Parameter(n) sind erlaubt).
------------------------------------------------------------------------ */
class VarPar01 {
   // ---------------------------------------------------------------------
   static public int sum(int n1, int n2, int... fir) {
      // Berechnet und liefert die Summe aller Parameter.
      // (fir: formaler Parameter, Integer-Reihung)
      int erg = n1 + n2;
      for (int n: fir) erg += n;
      return erg;
   } // sum
   // ---------------------------------------------------------------------
   static public int max(int... fir) {
      // Berechnet und liefert das Maximum aller Parameter.
      // Wenn diese Funktion mit 0 aktuellen Parametern aufgerufen
      // wird, lifert sie Integer.MIN_VALUE als Ergebnis.
      // (fir: formaler Parameter, Integer-Reihung)
      int erg = Integer.MIN_VALUE;
      for (int n: fir) if (n > erg) erg = n;
      return erg;
   } // max
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("VarPar01: Jetzt geht es los!%n");

      int[] air = {5, 7, 2}; // Aktueller Parameter, Integer-Reihung

      printf("-------------------------------%n");
//    printf("sum()             : %3d%n", sum());
//    printf("sum(3)            : %3d%n", sum(3));
      printf("sum(3, 1)         : %3d%n", sum(3, 1));
      printf("sum(3, 1, 5)      : %3d%n", sum(3, 1, 5));
      printf("sum(3, 1, 5, 7)   : %3d%n", sum(3, 1, 5, 7));
      printf("sum(3, 1, 5, 7, 2): %3d%n", sum(3, 1, 5, 7, 2));
      printf("sum(3, 1, air)    : %3d%n", sum(3, 1, air));
      printf("-------------------------------%n");
      printf("max()             : %3d%n", max());
      printf("max(3)            : %3d%n", max(3));
      printf("max(3, 1)         : %3d%n", max(3, 1));
      printf("max(3, 1, 5)      : %3d%n", max(3, 1, 5));
      printf("max(3, 1, 5, 7)   : %3d%n", max(3, 1, 5, 7));
      printf("max(3, 1, 5, 7, 2): %3d%n", max(3, 1, 5, 7, 2));
      printf("max(air)          : %3d%n", max(air));
      printf("-------------------------------%n");
      printf("VarPar01: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class VarPar01
/* ------------------------------------------------------------------------
Ausgabe des Programms VarPar01:

VarPar01: Jetzt geht es los!
-------------------------------
sum(3, 1)         :   4
sum(3, 1, 5)      :   9
sum(3, 1, 5, 7)   :  16
sum(3, 1, 5, 7, 2):  18
sum(3, 1, air)    :  18
-------------------------------
max()             : -2147483648
max(3)            :   3
max(3, 1)         :   3
max(3, 1, 5)      :   5
max(3, 1, 5, 7)   :   7
max(3, 1, 5, 7, 2):   7
max(air)          :   7
-------------------------------
VarPar01: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldungen eines Compilers, wenn die entsprechenden Zeilen keine
Kommentare sind:

D:\Java\VarPar01.java:40: sum(int,int,java.lang.Integer...) in VarPar01
cannot be applied to ()
    printf("sum()             : %3d%n", sum());
                                        ^
D:\Java\VarPar01.java:41: sum(int,int,java.lang.Integer...) in VarPar01
cannot be applied to (int)
    printf("sum(3)            : %3d%n", sum(3));
                                        ^
------------------------------------------------------------------------ */
