// Datei Vergleichen02.java
/* ------------------------------------------------------------------------
Objekte der Klassen MangoA, MangoB und MangoC enthalten nur ein
double-Attribut namens preis und sind nach diesem Attribut
"natuerlich geordnet" (d.h. ueber die Schnittstelle Comparable):
Je kleiner der Preis, desto groesser die Mango.

Die drei Klassen MangoA, MangoB und MangoC unterscheiden sich
eigentlich nur durch die compareTo-Methode, mit der sie die
Schnittstelle Comparable implementieren.
------------------------------------------------------------------------ */
import java.util.Formatter;
// ==============================================================
class MangoA implements Comparable<MangoA> {
   double  preis;

   public String toString() {
      Formatter f = new Formatter();
      f.format("MangoA, Preis: %4.2f", preis);
      return f.toString();
   } // toString

   // Eine naive compareTo-Methode. Sie stellt erst einen Unterschied
   // fest, wenn die Preise sich um 1.0 (Euro? Dollar?)
   // oder mehr unterscheiden.
   public int compareTo(MangoA that) {
      return (int) (that.preis - this.preis);
   } // compareTo

   MangoA(double preis) {
      this.preis     = preis;
   } // Konstruktor MangoA

} // class MangoA
// ==============================================================
class MangoB implements Comparable<MangoB> {
   double  preis;

   public String toString() {
      Formatter f = new Formatter();
      f.format("MangoB, Preis: %4.2f", preis);
      return f.toString();
   } // toString

   // Eine compareTo-Methode, die schon dann einen Unterschied
   // feststellt, wenn die Preise sich um 0.01 (Euro? Dollar?)
   // oder mehr unterscheiden ("absolute Differenz")
   public int compareTo(MangoB that) {
      return (int) (100.0 * that.preis - 100 * this.preis);
   } // compareTo

   MangoB(double preis) {
      this.preis     = preis;
   } // Konstruktor MangoB

} // class MangoB
// ==============================================================
class MangoC implements Comparable<MangoC> {
   double  preis;

   public String toString() {
      Formatter f = new Formatter();
      f.format("MangoC, Preis: %4.2f", preis);
      return f.toString();
   } // toString

   // Eine compareTo-Methode, die schon dann einen Unterschied
   // feststellt, wenn die Preise sich um ein Promille oder
   // mehr voneinander unterscheiden ("relative Differenz")
   public int compareTo(MangoC that) {
      double einProMille = this.preis / 1000.0;
      return (int) ((that.preis - this.preis) / einProMille);
   } // compareTo

   MangoC(double preis) {
      this.preis     = preis;
   } // Konstruktor MangoC

} // class MangoC
// ==============================================================
class Vergleichen02 {
   static public void main(String[] sonja) {
      printf("Vergleichen02: Jetzt geht es los!%n");

      // Von jeder Mango-Klasse werden 3 Objekte mit jeweils
      // gleichen Preisen vereinbart:

      MangoA   mA1 = new MangoA(4.00);
      MangoA   mA2 = new MangoA(4.05);
      MangoA   mA3 = new MangoA(4.01);

      MangoB   mB1 = new MangoB(4.00);
      MangoB   mB2 = new MangoB(4.05);
      MangoB   mB3 = new MangoB(4.01);

      MangoC   mC1 = new MangoC(4.00);
      MangoC   mC2 = new MangoC(4.05);
      MangoC   mC3 = new MangoC(4.01);

      printf("---------------------------------%n");
      printf("A mA1: %s%n", mA1);
      printf("B mA2: %s%n", mA2);
      printf("C mA3: %s%n", mA3);
      printf("--------------------------%n");
      printf("D mA1.compareTo(mA2): %4d%n",  mA1.compareTo(mA2));
      printf("E mA2.compareTo(mA3): %4d%n",  mA2.compareTo(mA3));
      printf("F mA3.compareTo(mA1): %4d%n",  mA3.compareTo(mA1));
      printf("---------------------------------%n");
      printf("G mB1: %s%n", mB1);
      printf("H mB2: %s%n", mB2);
      printf("I mB3: %s%n", mB3);
      printf("--------------------------%n");
      printf("J mB1.compareTo(mB2): %4d%n",  mB1.compareTo(mB2));
      printf("K mB2.compareTo(mB3): %4d%n",  mB2.compareTo(mB3));
      printf("L mB3.compareTo(mB1): %4d%n",  mB3.compareTo(mB1));
      printf("---------------------------------%n");
      printf("M mC1: %s%n", mC1);
      printf("N mC2: %s%n", mC2);
      printf("O mC3: %s%n", mC3);
      printf("--------------------------%n");
      printf("P mC1.compareTo(mC2): %4d%n",  mC1.compareTo(mC2));
      printf("Q mC2.compareTo(mC3): %4d%n",  mC2.compareTo(mC3));
      printf("R mC3.compareTo(mC1): %4d%n",  mC3.compareTo(mC1));
      printf("---------------------------------%n");

      printf("Vergleichen02: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Vergleichen02
/* ------------------------------------------------------------------------
Ausgabe des Programms Vergleichen02:

Vergleichen02: Jetzt geht es los!
---------------------------------
A mA1: MangoA, Preis: 4,00
B mA2: MangoA, Preis: 4,05
C mA3: MangoA, Preis: 4,01
--------------------------
D mA1.compareTo(mA2):    0
E mA2.compareTo(mA3):    0
F mA3.compareTo(mA1):    0
---------------------------------
G mB1: MangoB, Preis: 4,00
H mB2: MangoB, Preis: 4,05
I mB3: MangoB, Preis: 4,01
--------------------------
J mB1.compareTo(mB2):    5
K mB2.compareTo(mB3):   -4
L mB3.compareTo(mB1):   -1
---------------------------------
M mC1: MangoC, Preis: 4,00
N mC2: MangoC, Preis: 4,05
O mC3: MangoC, Preis: 4,01
--------------------------
P mC1.compareTo(mC2):   12
Q mC2.compareTo(mC3):   -9
R mC3.compareTo(mC1):   -2
---------------------------------
Vergleichen02: Das war's erstmal!
------------------------------------------------------------------------ */
