// Datei VergroesserbarAnwendung.java
/* ------------------------------------------------------------------------
Demonstriert eine Amwendung der Schnittstelle Vergroesserbar.

Def.: Eine Vergroesserbar-Klasse ist eine Klasse, die die Schnittstelle
      Vergroesserbar implementiert.

Def.: Ein Vergroesserbar-Objekt ist ein Objekt einer Vergroesserbar-Klasse.

Mit der Methode reihungVervierfachen kann man die Komponenten einer
Reihung vervierfachen ohne genau zu untersuchen, zu welchen Klassen
die Komponenten gehoeren (sie duerfen zu beliebigen Vergroesserbar-Klassen
wie GanzZahl01 oder SeitenFormat oder WalFisch etc. gehoeren).
------------------------------------------------------------------------ */
class VergroesserbarAnwendung {
   static private Vergroesserbar[] produziereReihung() {
      // Erzeugt und liefert eine Reihung, die eine "wilde Mischung" von
      // Vergroesserbar-Objekten enthaelt:
      Vergroesserbar[] vr = new Vergroesserbar[7];
      vr[0] = new GanzZahl03 (111);
      vr[1] = new GanzZahl01 (222);
      vr[2] = new BruchZahl01(33.3F);
      vr[3] = new BruchZahl01(44.4F);
      vr[4] = new GanzZahl01 (555);
      vr[5] = new SeitenFormat(210.0, 297.0); // DIN A4
      vr[6] = new WalFisch(10);               // Ein kleiner Blauwal

      return vr;
   } // produziereReihung
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Eine Reihung von Vergroesserbar-Objekten wird erzeugt, ausge-
      // gegeben, mit der Methode reihungVervierfachen bearbeitet und
      // erneut ausgegeben.

      Vergroesserbar[] vr01 = produziereReihung();
      pln("VergroesserbarAnwendung: Jetzt geht es los!");
      pln("-----------------------------------------");
      pln("Vor der Vervierfachung:");
      pln("-----------------------------------------");
      reihungAusgeben     (vr01, "vr01");
      reihungVervierfachen(vr01);
      pln("-----------------------------------------");
      pln("Nach der Vervierfachung:");
      pln("-----------------------------------------");
      reihungAusgeben     (vr01, "vr01");
      pln("-----------------------------------------");

   } // main
   // ---------------------------------------------------------------------
   static void reihungAusgeben(Vergroesserbar[] vr, String name) {
      // Gibt alle Komponenten von vr aus.
      pln("Die Komponenten der Reihung " + name + ":");
      int i=0;
      for (Vergroesserbar v: vr) {
         p  (name + "[" + i++ + "]: ");
         pln(v);
      }
   } // reihungAusgeben
   // ---------------------------------------------------------------------
   static void reihungVervierfachen(Vergroesserbar[] vr) {
      // Vervierfacht die Groesse von jeder Komponenten von vr:
      for (Vergroesserbar v: vr) {
         v.verdopple();
         v.verdopple();
      }
   } // reihungVervierfachen
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // VergroesserbarAnwendung
/* ------------------------------------------------------------------------
Ausgabe des Programms VergroesserbarAnwendung:

VergroesserbarAnwendung: Jetzt geht es los!
-----------------------------------------
Vor der Vervierfachung:
-----------------------------------------
Die Komponenten der Reihung vr01:
vr01[0]: GanzZahl03  mit Groesse 111
vr01[1]: GanzZahl01  mit Groesse 222
vr01[2]: BruchZahl01 mit Groesse 33.3
vr01[3]: BruchZahl01 mit Groesse 44.4
vr01[4]: GanzZahl01  mit Groesse 555
vr01[5]: SeitenFormat 210.0 mal 297.0 mm
vr01[6]: WalFisch, Gewicht 10 t
-----------------------------------------
Nach der Vervierfachung:
-----------------------------------------
Die Komponenten der Reihung vr01:
vr01[0]: GanzZahl03  mit Groesse 444
vr01[1]: GanzZahl01  mit Groesse 888
vr01[2]: BruchZahl01 mit Groesse 133.2
vr01[3]: BruchZahl01 mit Groesse 177.6
vr01[4]: GanzZahl01  mit Groesse 2220
vr01[5]: SeitenFormat 840.0 mal 1188.0 mm
vr01[6]: WalFisch, Gewicht 40 t
-----------------------------------------
------------------------------------------------------------------------ */