// Datei Wettlauf01.java
/* ------------------------------------------------------------------------
Grabo-Programme (Programme mit einer grafischen Benutzeroberflaeche)
werden von mindestens zwei Faeden (threads) ausgefuehrt, einem Haupt-
faden (main thread) und einem Ereignisfaden (event thread). Es ist
moeglich, diese beiden Faeden "um die Wette laufen zu lassen" und
damit sogenannte Nebenlaeufigkeitsfehler zu produzieren, die ohne
Faeden nicht moeglich sind.

Graphikelemente (ein buntes Rechteck und 2 Texte) werden in ein
JFrame-Fenster ausgegeben. In diesem Beispiel werden sie direkt
mit dem Graphics-Objekt des JFrame-Objekts ausgegeben. Das ist
NICHT EMPFEHLENSWERT, weil ein Wettlauf zwischem dem Haupt-Faden
und dem Ereignis-Faden stoerende Effekte verursachen kann (beide
schreiben nebenlaeufig zur selben Stelle auf dem Bildschirm). Mit
Hilfe einer Verzoegerung (s.u.) kann man diese Stoerungen provo-
zieren, falls sie nicht von allein auftreten.

Der Befehl Thread.sleep loest moeglicherweise eine Ausnahme des
Typs InterruptedException aus.
------------------------------------------------------------------------ */
import javax.swing.JFrame;
import java .awt  .Graphics;
import java .awt  .Color;

class Wettlauf01 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja)
      throws InterruptedException          // wegen Thread.sleep
   {
      // Die Prioritaet des (hier aktiven) Haupt-Fadens erhoehen
      // (damit er vor dem Ereignis-Faden "drankommt"):
      Thread aktiverFaden = Thread.currentThread();
      aktiverFaden.setPriority(aktiverFaden.getPriority() + 1);

      // Wieviele Millisec. soll der Haupt-Faden vor dem Zeichnen
      // zoegern?
      printf("Verzoegerung (-1 fuer keine Verz.)? ");
      int verzoegerung = EM.liesInt();

      // Einen Rahmen vom Haupt-Faden erzeugen und vom
      // Ereignis-Faden sichtbar machen (d.h. zeichnen) lassen:
      JFrame rahmen1 = new JFrame("Wettlauf01-Rahmen");
      rahmen1.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      rahmen1.setBounds(200, 100, 500, 250);
      rahmen1.setVisible(true); // Sonst wird gisela null!

      Graphics gisela = rahmen1.getGraphics();

      // Falls gewuenscht den Haupt-Faden (der eine hoehere
      // Prioritaet hat als der Ereignis-Faden) ein bisschen
      // verzoegern:
      if (verzoegerung >= 0) Thread.sleep(verzoegerung);

      // Vom Haupt-Faden ein buntes Rechteck zeichnen lassen
      // (evtl. noch bevor der rahmen1 vom Ereignis-Faden
      // gezeichnet wurde):
      gisela.setColor(Color.magenta);
      gisela.fillRect(0, 0, 300, 150);

      // Vom Haupt-Faden einen Text zeichnen lassen (evtl.
      // immer noch bevor der Ereignis-Faden den rahmen1
      // gezeichnet hat):
      gisela.setColor(Color.black);
      gisela.drawString("TEXT: Ist da was Buntes?", 10, 180);

      // Den Haupt-Faden schlafen lassen (bis der Ereignis-
      // Faden sehr wahrscheinlich den rahmen1 gezeichnet hat)
      // und dann noch einen zweiten Text zeichnen lassen:
      Thread.sleep(500);
      gisela.drawString("Ist der TEXT noch da?", 10, 200);

      printf("Wettlauf01: Das war's erstmal!\n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Wettlauf01