// Datei While01.java
/* ------------------------------------------------------------------------
Mit verschiedenen while-Schleifen werden Strings erzeugt, die Zufalls-
zahlen enthalten.

Hier werden 3 Formen von while-Schleifen unterschieden:
Allgemeine Form:           while (true)      RUMPF
Spezialform 1:             while (BEDINGUNG) RUMPF
Spezialform 2:    do RUMPF while (BEDINGUNG);

while-Schleifen der allgemeinen Form werden mit einer break-Anweisung
(oder einer return-Anweisung oder mit einer Ausnahme) beendet.
------------------------------------------------------------------------ */
import java.util.Random;

class While01 {
   // ---------------------------------------------------------------------
   static final char   BLANK  = ' ';
   // ---------------------------------------------------------------------
   // Wenn man beim Erzeugen eines Random-Objekts eine Zahl angibt
   // (z.B. 23), dann liefert das Objekt bei jedem Programmstart die
   // gleiche Folge von Zufallszahlen ("reproduzierbare Zufallszahlen").
   // Wenn man keine Zahl angibt, liefert das Objekt jedesmal eine andere
   // Folge von Zufallszahlen ("nicht-reproduzierbare Zufallszahlen").
   static Random quelle = new Random(23); // Reproduzierbare Zufallszahlen
// static Random quelle = new Random();   // Nicht-reproduzierbare Zufalls-
                                          // zahlen
   static int zufallsZahl() {
      // Liefert bei jedem Aufruf einen zufaellig gewaehlten Wert
      // zwischen 10 + 0 und 10 + 89.
      return 10 + quelle.nextInt(90);
   } // zufallsZiffer
   // ---------------------------------------------------------------------
   static String zufallsZahlenA() {
      // Ruft wiederholt die Funktion zufallsZahl auf bis diese eine
      // Zufallszahl liefert, die groesser oder gleich 85 ist.
      // Fasst alle vorher gelieferten Zufallszahlen zu einem
      // String zusammen und liefert diesen als Ergebnis.

      int    eineZahl;
      String ergebnis = "A ";

      // Eine while-Schleife der allgemeinen Form:
      while (true) {
         // Eine Zufallszahl berechnen lassen:
         eineZahl = zufallsZahl();

         // Aufhoeren wenn die Zufallszahl groesser als 85 ist:
         if (eineZahl > 85) break;

         // Sonst die Zufallszahl in einen String umwandeln und an
         // das ergebnis anhaengen:
         ergebnis = ergebnis + eineZahl + BLANK;
      } // while

      return ergebnis;
   } // zufallsZahlenA
   // ---------------------------------------------------------------------
   static String zufallsZahlenB(int anzahl) {
      // Ruft die Funktion zufallsZahl anzahl Mal auf, fasst die
      // gelieferten Zufallszahlen zu einem String zusammen und
      // lieferten diesen als Ergebnis. Falls anzahl kleiner oder
      // gleich 0 ist, wird die Funktion zufallsZahl 0 Mal (also
      // keinmal) aufgerufen.

      int    eineZahl;
      String ergebnis = "B ";

      // Eine while-Schleife der allgemeinen Form:
      while (true) {
         if (anzahl <= 0) break;
         eineZahl = zufallsZahl();
         ergebnis = ergebnis + eineZahl + BLANK;
         anzahl   = anzahl - 1;
      } // while

      return ergebnis;
   } // zufallsZahlenB
   // ---------------------------------------------------------------------
   static String zufallsZahlenC(int anzahl) {
      // Leistet das Gleiche wie die Funktion zufallsZahlenB.

      int    eineZahl;
      String ergebnis = "C ";

      // Eine while-Schleife der Spezialform 1:
      while (! (anzahl <= 0)) {
         eineZahl = zufallsZahl();
         ergebnis = ergebnis + eineZahl + BLANK;
         anzahl   = anzahl - 1;
      } // while

      return ergebnis;
   } // zufallsZahlenC
   // ---------------------------------------------------------------------
   static String zufallsZahlenD(int anzahl) {
      // Ruft die Funktion zufallsZahl anzahl Mal (mindestens aber
      // einmal) auf, fasst die gelieferten Zufallszahlen zu einem
      // String zusammen und  lieferten diesen als Ergebnis.
      int    eineZahl;
      String ergebnis = "D ";

      // Eine while-Schleife der allgemeinen Form:
      while (true) {
         eineZahl = zufallsZahl();
         ergebnis = ergebnis + eineZahl + BLANK;
         anzahl   = anzahl - 1;
         if (anzahl <= 0) break;
      } // while

      return ergebnis;
   } // zufallsZahlenD
   // ---------------------------------------------------------------------
   static String zufallsZahlenE(int anzahl) {
      // Leistet das Gleiche wie die Funktion zufallsZahlenD.

      int    eineZahl;
      String ergebnis = "E ";

      // Eine while-Schleife der Spezialform 2 (eine do-while-Schleife):
      do {
         eineZahl = zufallsZahl();
         ergebnis = ergebnis + eineZahl + BLANK;
         anzahl   = anzahl - 1;
      } while (! (anzahl <= 0));

      return ergebnis;
   } // zufallsZahlenE
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die Funktionen zufallsZahlenA bis E werden je einmal aufgerufen
      // und ihre Ergebnisse werden ausgegeben:
      pln("-------------------------------");
      pln(zufallsZahlenA());
      pln("-------------------------------");
      pln(zufallsZahlenB(7));
      pln(zufallsZahlenC(7));
      pln(zufallsZahlenD(5));
      pln(zufallsZahlenE(5));
      pln("-------------------------------");
      pln(zufallsZahlenB(0));
      pln(zufallsZahlenC(0));
      pln(zufallsZahlenD(0));
      pln(zufallsZahlenE(0));
      pln("-------------------------------");
      pln("While01: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class While01
/* ------------------------------------------------------------------------
Ausgabe des Programms While01:

-------------------------------
A 17 48 13 37 58 13 80 60 71 25
-------------------------------
B 17 99 30 71 76 74 80
C 52 37 82 27 94 85 54
D 64 51 50 43 39
E 42 52 10 57 99
-------------------------------
B
C
D 72
E 11
-------------------------------
While01: Das war's erstmal!
------------------------------------------------------------------------ */