// Datei While02.java
/* ------------------------------------------------------------------------
while-Schleifen, die mit einer return-Anweisung beendet werden.
------------------------------------------------------------------------ */
class While02 {
   // ---------------------------------------------------------------------
   static int groessterTeiler(int n) {
      // Liefert den groessten Teiler von n und liefert 1, falls (der
      // Betrag von) n eine Primzahl oder kleiner oder gleich 1 ist.

      // Fuer den Fall dass n negativ ist, wird n durch seinen Betrag
      // (absolute value) ersetzt:
      n = Math.abs(n);

      if (n <= 1) return 1; // 0 und 1 haben keine echten Teiler

      // Der groesste Teiler von n kann nicht groesser als n/2 sein.
      int teiler = n/2;

      // Das Prozentzeichen % bezeichnet die Rest-Operation. Beispiele:
      // 19 % 5 ist gleich 4 (denn 19 durch 5 ist 3 Rest 4)
      // 36 % 5 ist gleich 1 (denn 36 durch 5 ist 7 Rest 1)
      //
      // Der Ausdruck (n % teiler == 0) hat genau dann den Wert true,
      // wenn n durch teiler teilbar ist.

      while (true) {
         if (n % teiler == 0) return teiler;
         teiler--;
      } // while
    } // groessterTeiler
   // ---------------------------------------------------------------------
   static boolean istPrim(int n) {
      // Liefert true, wenn (der Betrag von) n eine Primzahl ist, und
      // sonst false.
      // Hinweis: Es gibt erheblich effizientere Funktionen, dieses
      // Problem loesen.

      n = Math.abs(n);
      if (n <= 1) return false;

      // Der kleinste Teiler von n kann nicht groesser als die
      // Wurzel aus n sein.
      int teiler = (int) Math.sqrt(n);
      while (true) {
         if (n % teiler == 0) return (teiler == 1);
         teiler--;
      } // while
    } // groessterTeiler
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      // Die Funktionen groessterTeiler und istPrim werden mehrmals
      // aufgerufen.
      pln("-------------------------------------------------------------");
      pln("A groessterTeiler(153): " + groessterTeiler(153));
      pln("B groessterTeiler(-51): " + groessterTeiler(-51));
      pln("C groessterTeiler(127): " + groessterTeiler(127));
      pln("-------------------------------------------------------------");
      pln("D istPrim(127)        : " + istPrim(127));
      pln("E istPrim(153)        : " + istPrim(153));
      pln("F istPrim(1234577)    : " + istPrim(1234577));
      pln("-------------------------------------------------------------");
      for (int n=-15; n<=+15; n++) {
         printf("G groessterTeiler(%+3d): %d%n", n, groessterTeiler(n));
         printf("H istPrim        (%+3d): %b%n", n, istPrim(n));
      }
      pln("-------------------------------------------------------------");
      pln("While02: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // class While02
/* ------------------------------------------------------------------------
Ausgabe des Programms While02:

-------------------------------------------------------------
A groessterTeiler(153): 51
B groessterTeiler(-51): 17
C groessterTeiler(127): 1
-------------------------------------------------------------
D istPrim(127)        : true
E istPrim(153)        : false
F istPrim(1234577)    : true
-------------------------------------------------------------
G groessterTeiler(-15): 5
H istPrim        (-15): false
G groessterTeiler(-14): 7
H istPrim        (-14): false
G groessterTeiler(-13): 1
H istPrim        (-13): true
G groessterTeiler(-12): 6
H istPrim        (-12): false
G groessterTeiler(-11): 1
H istPrim        (-11): true
G groessterTeiler(-10): 5
H istPrim        (-10): false
G groessterTeiler( -9): 3
H istPrim        ( -9): false
G groessterTeiler( -8): 4
H istPrim        ( -8): false
G groessterTeiler( -7): 1
H istPrim        ( -7): true
G groessterTeiler( -6): 3
H istPrim        ( -6): false
G groessterTeiler( -5): 1
H istPrim        ( -5): true
G groessterTeiler( -4): 2
H istPrim        ( -4): false
G groessterTeiler( -3): 1
H istPrim        ( -3): true
G groessterTeiler( -2): 1
H istPrim        ( -2): true
G groessterTeiler( -1): 1
H istPrim        ( -1): false
G groessterTeiler( +0): 1
H istPrim        ( +0): false
G groessterTeiler( +1): 1
H istPrim        ( +1): false
G groessterTeiler( +2): 1
H istPrim        ( +2): true
G groessterTeiler( +3): 1
H istPrim        ( +3): true
G groessterTeiler( +4): 2
H istPrim        ( +4): false
G groessterTeiler( +5): 1
H istPrim        ( +5): true
G groessterTeiler( +6): 3
H istPrim        ( +6): false
G groessterTeiler( +7): 1
H istPrim        ( +7): true
G groessterTeiler( +8): 4
H istPrim        ( +8): false
G groessterTeiler( +9): 3
H istPrim        ( +9): false
G groessterTeiler(+10): 5
H istPrim        (+10): false
G groessterTeiler(+11): 1
H istPrim        (+11): true
G groessterTeiler(+12): 6
H istPrim        (+12): false
G groessterTeiler(+13): 1
H istPrim        (+13): true
G groessterTeiler(+14): 7
H istPrim        (+14): false
G groessterTeiler(+15): 5
H istPrim        (+15): false
-------------------------------------------------------------
While02: Das war's erstmal!
------------------------------------------------------------------------ */