// Datei Zufall03.java
/* ------------------------------------------------------------------------
Die Methode reproduzierbareFolge hat einen long-Parameter und gibt
eine reproduzierbare Folge von Zufallszahlen (vom Typ double) aus.
"Reproduzierbar" heisst: Wenn man die Methode mehrmals mit gleichem
Parameter aufruft (z.B. mit dem Parameter 123) wird immer die selbe
Folge von Zahlen ausgegeben.

Die Methode nichtReproduzierbareFolge() hat keinen Parameter und
gibt eine nicht-reproduzierbare Folge von Zufallszahlen (vom Typ
double) aus.

Diese beiden Methoden erzeugen Zufallszahlen mit der Methode nextGaussian
(normal verteilte Zufallszahlen) und werden mehrmals aufgerufen (siehe die
mit "// <---" gekennzeichneten Zeilen).

Die Methode gibAusserhalbAus soll dabei helfen, ein Gefuehl fuer
"normal verteilte Zahlenmengen" zu entwickeln.
------------------------------------------------------------------------ */
import java.util.Random;

class Zufall03 {
   // ---------------------------------------------------------------------
   static void reproduzierbareFolge(long keim) {
      // Gibt 5 reproduzierbare Zufallszahlen aus.
      // Die Zahlen sind vom Typ double und normal verteilt mit dem
      // Mittelwert 0.0 und der Standardabweichung 1.0

      Random r = new Random(keim); // Ein Zufallsgenerator mit Keim

      printf("      reproduzierbar: ");
      for (int i=1; i<=5; i++) {
         double z = r.nextGaussian();                               // <---
         printf("%+4.2f ", z);
      }
      printf("%n");
   } // reproduzierbareFolge
   // ---------------------------------------------------------------------
   static void nichtReproduzierbareFolge() {
      // Gibt 5 nicht-reproduzierbare Zufallszahlen aus.
      // Die Zahlen sind vom Typ double und normal verteilt mit dem
      // Mittelwert 0.0 und der Standardabweichung 1.0

      Random r = new Random();      // Ein Zufallsgenerator ohne Keim

      printf("nicht-reproduzierbar: ");
      for (int i=1; i<=5; i++) {
         double z = r.nextGaussian();                               // <---
         printf("%+4.2f ", z);
      }
      printf("%n");
   } // nichtReproduzierbareFolge
   // ---------------------------------------------------------------------
   static void gibAusserhalbAus(double grenze, int anzahlVersuche) {
      // Die Methode nextGaussian liefert normal verteilte Zahlen
      // mit dem Mittelwert 0.0 und der Standardabweichung 1.0.
      // Das bedeutet etwa: Jede double-Zahl hat eine Chance, ge-
      // waehlt zu werden, aber je weiter eine Zahl von 0.0 weg ist,
      // desto kleiner ist ihre Chance (Zahlen, die kleiner als -5
      // oder groesser als +5 sind haben nur noch eine sehr sehr
      // kleine Chance).
      //
      // Diese Methode Erzeugt (mit der Methode nextGaussian) anzahl
      // Versuche viele Zufallszahlen und gibt alle aus, die kleiner
      // als -grenze oder groesser als +grenze sind.

      Random r = new Random();

      // grenze nicht-negativ machen:
      if (grenze < 0.0) grenze = -grenze;

      // Anzahl der Zufallszahlen, die ausserhabl des Intervalls
      // (-grenze, +grenze) liegen:
      int anzAusserhalb = 0;

      printf("Von %d gleichverteilten Zufallszahlen%n",
              anzahlVersuche);
      printf("lagen folgende ausserhalb von -%3.1f .. +%3.1f:%n%n",
              grenze, grenze);

      for (int i=0; i<anzahlVersuche; i++) {
         double z = r.nextGaussian();                               // <---
         if (-grenze < z && z < +grenze) continue;

         printf("%+4.2f ", z);
         anzAusserhalb++;

         // Nach jeweils 7 Zahlen eine neue Zeile beginnen:
         if (anzAusserhalb % 7 == 0) printf("%n");
      } // for
      printf("%n");

      printf("Das waren %d Zahlen!%n", anzAusserhalb);
   } // gibAusserhalbAus
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      printf("Zufall03: Jetzt geht es los!%n");
      printf("---------------------------------------------------%n");
      reproduzierbareFolge(123);
      reproduzierbareFolge(123);
      reproduzierbareFolge(123);
      printf("---------------------------------------------------%n");
      reproduzierbareFolge(756);
      reproduzierbareFolge(756);
      printf("---------------------------------------------------%n");
      nichtReproduzierbareFolge();
      nichtReproduzierbareFolge();
      nichtReproduzierbareFolge();
      printf("---------------------------------------------------%n");
      gibAusserhalbAus(3.0,    10000); // 10 Tausend
      printf("---------------------------------------------------%n");
      gibAusserhalbAus(5.0, 10000000); // 10 Millionen
      printf("---------------------------------------------------%n");
      printf("Zufall03: Das war's erstmal!%n");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Zufall03
/* ------------------------------------------------------------------------
Zwei verschiedene Ausgaben des Programms Zufall03:

Zufall03: Jetzt geht es los!
---------------------------------------------------
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
---------------------------------------------------
      reproduzierbar: +1,70 +0,78 +0,48 +2,07 +0,61
      reproduzierbar: +1,70 +0,78 +0,48 +2,07 +0,61
---------------------------------------------------
nicht-reproduzierbar: +1,36 +0,07 +1,35 +0,53 -0,47
nicht-reproduzierbar: -1,13 -0,53 +0,94 +0,63 -0,38
nicht-reproduzierbar: -1,88 -0,79 -1,09 +0,47 +0,72
---------------------------------------------------
Von 10000 gleichverteilten Zufallszahlen
lagen folgende ausserhalb von -3,0 .. +3,0:

+3,04 +3,15 -3,22 -3,14 -3,32 -3,17 +3,37
+3,12 +3,01 +3,01 +3,08 -3,05 +3,22 -3,06
-3,07 -3,00 -3,01 -3,71 +3,22 +3,52 +3,17
-3,40 +3,03 -3,14 -3,03 -3,06 -3,27 +3,18

Das waren 28 Zahlen!
---------------------------------------------------
Von 10000000 gleichverteilten Zufallszahlen
lagen folgende ausserhalb von -5,0 .. +5,0:

+5,17 -5,48 +5,29 -5,24 +5,17 -5,06
Das waren 6 Zahlen!
---------------------------------------------------
Zufall03: Das war's erstmal!

Zufall03: Jetzt geht es los!
---------------------------------------------------
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
      reproduzierbar: -1,44 +0,63 +0,23 +0,28 +0,18
---------------------------------------------------
      reproduzierbar: +1,70 +0,78 +0,48 +2,07 +0,61
      reproduzierbar: +1,70 +0,78 +0,48 +2,07 +0,61
---------------------------------------------------
nicht-reproduzierbar: +1,30 -0,65 -1,60 +1,78 +0,67
nicht-reproduzierbar: +0,80 +1,40 +0,07 +1,70 +0,79
nicht-reproduzierbar: -1,14 -0,39 +1,03 +0,99 -0,47
---------------------------------------------------
Von 10000 gleichverteilten Zufallszahlen
lagen folgende ausserhalb von -3,0 .. +3,0:

-3,10 -3,20 -3,17 -3,11 -3,05 +3,05 +3,22
-3,17 +3,31 +3,25 -3,46 -3,11 -3,70 -3,02
-3,37 +3,05 -3,52 -3,34 +3,04 -3,79 -3,85
+3,03 -3,17 -3,00 -3,55 -3,04 +3,28
Das waren 27 Zahlen!
---------------------------------------------------
Von 10000000 gleichverteilten Zufallszahlen
lagen folgende ausserhalb von -5,0 .. +5,0:

+5,15 -5,27 +5,21 -5,43
Das waren 4 Zahlen!
---------------------------------------------------
Zufall03: Das war's erstmal!
------------------------------------------------------------------------ */
