// Datei Zufall05.java
/* ------------------------------------------------------------------------
Jedes Random-Objekt r enthaelt 8 Funktionen, die verschiedene Zufalls-
werte (unterschiedlicher Typen, unterschiedlich verteilt) liefern.

In diesem Programm werden die folgenden beiden Schritte ein
Dutzend Mal wiederholt:

1. Eine der 8 Funktionen eines Random-Objekts r wird zufaellig
   gewaehlt.
2. Mit dieser Funktion wird ein Zufallswert erzeugt und
   ausgegeben.

Die Funktionen r.nextBytes und r.nextInt erwarten je einen Parameter
(vom Typ byte[] bzw. vom Typ int). Diese Parameter werden (natuerlich?)
ebenfalls zufaellig gewaehlt.

Alle Zufallswerte in diesem Programm sind nicht-reproduzierbar.
------------------------------------------------------------------------ */
import java.util.Random;
import java.util.Arrays;

class Zufall05 {
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {

      Random r = new Random();

      printf("Zufall05: Jetzt geht es los!%n");
      printf("-----------------------------------------%n");
      for (int i=1; i<=12; i++) {

         // Nr. einer Methode zufaellig waehlen:
         int welcheMethode = r.nextInt(8);

         switch (welcheMethode) {
            case 0:
               printf("nextBoolean() : %s%n",     r.nextBoolean());
               break;
            case 1:
               // Eine zufaelliger Laenge erzeugen:
               byte[] br = new byte[r.nextInt(10)];
                                                  r.nextBytes(br);
               String s  = Arrays.toString(br);
               printf("nextBytes(br) : %s%n", s);
               break;
            case 2:
               printf("nextDouble()  : %+4.3g%n", r.nextDouble());
               break;
            case 3:
               printf("nextFloat()   : %+4.3g%n", r.nextFloat());
               break;
            case 4:
               printf("nextGaussian(): %+4.3g%n", r.nextGaussian());
               break;
            case 5:
               printf("nextInt()     : %d%n",     r.nextInt());
               break;
            case 6:
               // Einen int-Wert (zwischen 0 und 100) als
               // Grenze zufaellig waehlen:
               int g = r.nextInt(100);
               printf("nextInt(%3d)  : %d%n", g,  r.nextInt(g));
               break;
            case 7:
               printf("nextLong()    : %d%n",     r.nextLong());
               break;
         } // switch
      } // for
      printf("-----------------------------------------%n");
      printf("Zufall05: Das war's erstmal!%n");

   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class Zufall05
/* ------------------------------------------------------------------------
Zwei verschiedene Ausgaben des Programms Zufall05:

Zufall05: Jetzt geht es los!
-----------------------------------------
nextInt( 10)  : 7
nextFloat()   : +0.770
nextInt( 25)  : 17
nextLong()    : -3090419015867064852
nextGaussian(): -0.257
nextLong()    : 5934907575219286226
nextLong()    : 7689369682030693907
nextInt()     : -423542294
nextBoolean() : false
nextLong()    : 1541143566678226113
nextInt( 10)  : 2
nextInt( 55)  : 37
-----------------------------------------
Zufall05: Das war's erstmal!

Zufall05: Jetzt geht es los!
-----------------------------------------
nextDouble()  : +0.0688
nextInt( 10)  : 8
nextBoolean() : false
nextBytes(br) : [44, 106, 124, -102, 125, -69]
nextBoolean() : true
nextDouble()  : +0.444
nextBytes(br) : [91, -92, -118, -78, 101, -65, -78, -23]
nextBytes(br) : [-21, 22, 3, 32, 96]
nextInt( 59)  : 5
nextBytes(br) : [-6, 29, 28, 61, 90]
nextLong()    : -4534839761150846419
nextFloat()   : +0.425
-----------------------------------------
Zufall05: Das war's erstmal!
------------------------------------------------------------------------ */
