// Datei Zufall06.java
/* ------------------------------------------------------------------------
Mit Hilfe von Zufallszahlen wird die Kreiszahl PI berechnet.

Hier sollte man sich vorstellen:
Ein Quadrat      mit der Kantenlaenge 1.0
Ein Viertelkreis mit dem Radius       1.0 um einen Eckpunkt des Quadrats
(so dass der Viertelkreis voll im Quadrat liegt).

Auf das Quadrat lassen wir ("an zufaellige Stellen") Regentropfen fallen.
Ihre Anzahl bezeichnen wir mit Tq ("Tropfen im Quadrat").
Einige dieser Tropfen fallen in den Viertelkreis innerhalb des Quadrats.
Ihre Anzahl bezeichnen wir mit Tv ("Tropfen im Viertelkreis").

Seien Fq und Fv die Flaechen des Quadrats bzw. des Viertelkreises.

Die Anzahl T der "zufaelligen Tropfen", die auf eine bestimmte Flaeche F
fallen, wird ungefaehr proportional zur Groesse von F sein. Also gilt:

     Fv  / Fq  = Tv / Tq
pi / 4.0 / 1.0 = Tv / Tq
pi             = Tv / Tq * 4.0

Nach dieser letzten Formel kann man aus Tv und Tq einen Naeherungswert
fuer PI berechnen.
------------------------------------------------------------------------ */
import java.util.Random;

class Zufall06 {
   static public void main(String [] sonja) {
      // Zum Vergleich erst einen *guten* Naeherungswert fuer pi ausgeben:
      pln("Math.PI ist gleich " + Math.PI);

      // Ein Zufalls-Objekt mit Keim (seed) erzeugen, damit die Zufalls-
      // zahlen reproduzierbar (d.h. bei jedem Programmstart wieder
      // gleich) sind
      Random r = new Random(1); // Keim (seed) 1
      double Tq; // Anzahl der Tropfen, die in das Quadrat  fallen
      double Tv; // Anzahl der Tropfen, die im Viertelkreis landen
      double x;  // x-Koord. eines Tropfens, aus Intervall [0.0 .. 1.0)
      double y;  // y-Koord. eines Tropfens, aus Intervall [0.0 .. 1.0)

      while (true) {

         // Den Benutzer fragen, wieviele Regentropfen fallen sollen:
         pln("----------------------------------------");
         p  ("Wie viele Regentropfen (0 zum Beenden)? ");
         Tq = EM.liesInt();     // Tropfen im Quadrat
         if (Tq == 0) break;    // Evtl. Schleife beenden
         Tq = Math.max(+1, Tq); // Mindestens +1 Tropfen

         // Jetzt lassen wir Tq viele Tropfen auf das Quadrat fallen
         // und zaehlen, wieviele davon auf dem Viertelkreis landen:
         Tv = 0;                // Anzahl Tropfen im Viertelkreis
         for (int i=1; i<=Tq; i++) {
            x = r.nextDouble(); // Zufallszahl aus Intervall [0.0 .. 1.0)
            y = r.nextDouble(); // Zufallszahl aus Intervall [0.0 .. 1.0)
            // Liegt der Punkt (x, y) m Viertelkreis?
            if (Math.sqrt(x*x + y*y) <= 1.0) Tv++;
         }

         // Aus Tv und Tq einen Naeherungswert fuer pi berechnen:
         double pi = Tv / Tq * 4.0;
         pln("     PI ist circa  " + pi);

      } // while (true)

      pln("----------------------------------------");
      pln("Zufall06: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
} // class Zufall06
/* ------------------------------------------------------------------------
Ein Dialog mit dem Programm Zufall06:

Math.PI ist gleich 3.141592653589793
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 100
     PI ist circa  3.12
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 100
     PI ist circa  3.16
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 100000
     PI ist circa  3.1394
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 100000
     PI ist circa  3.14932
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 1000000
     PI ist circa  3.142724
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 1000000
     PI ist circa  3.140724
----------------------------------------
Wie viele Regentropfen (0 zum Beenden)? 0
----------------------------------------
Zufall06: Das war's erstmal!
------------------------------------------------------------------------ */
