// Datei Zusicherung01.java
/* ------------------------------------------------------------------------
Die Methode istPrim enthaelt zwei Zusicherungen (assert-Anweisungen,
engl. assertion statements).
------------------------------------------------------------------------ */
class Zusicherung01 {
   // ---------------------------------------------------------------------
   static boolean istPrim(int n) {
      // Liefert true, wenn n eine Primzahl ist, und sonst false.
      // Negative Zahlen werden dabei wie ihr Betrag behandelt, z. B.
      // gilt -7 hier als Primzahl, aber -8 nicht.

      if (n<0) n = -n;

//    assert n>=0 : n + " ist immer noch negativ!"; // Mit zwei  Operanden
      assert n>=0;                                  // Mit einem Operanden

      if (n<=1)   return false; // 0 und 1 sind keine Primzahlen
      if (n==2)   return true;  // 2         ist eine Primzahl
      if (n%2==0) return false; // Andere gerade Zahlen sind keine Primzahlen

      assert n%2!=0 && n >= 3;  // n ist ungerade und groesser/gleich 3

      // Der Zeitbedarf dieses Programms haengt ganz wesentlich von der
      // Vereinbarung der Konstanten MAX ab. Fuer einem bestimmten Rechner
      // gilt:
//    final int MAX = n-1;                // ca. 45 sec
//    final int MAX = n/2;                // ca. 23 sec
      final int MAX = (int) Math.sqrt(n); // Weniger als 1 sec

      for (int teilerKandidat=3; teilerKandidat<MAX; teilerKandidat+=2) {
         if (n % teilerKandidat == 0) return false;
      }

      return true;
   } // istPrim
   // ---------------------------------------------------------------------
   static public void main(String [] sonja) {
      pln("--------------------------------");
      pln("Zusicherung01: Jetzt geht es los!");

      pln("istPrim(-17)              : " + istPrim(-17));
      pln("istPrim(+18)              : " + istPrim(+18));
      pln("istPrim(+27)              : " + istPrim(+27));
      pln("istPrim(-1234567891)      : " + istPrim(-1234567891));
      pln("istPrim(Integer.MAX_VALUE): " + istPrim(Integer.MAX_VALUE));
      pln("istPrim(Integer.MIN_VALUE): " + istPrim(Integer.MIN_VALUE));

      pln("Zusicherung01: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   // ---------------------------------------------------------------------
} // class Zusicherung01
/* ------------------------------------------------------------------------
Die Ausgaben des Programms Zusicherung01 von vier Ausführungen, bei denen
das Programm mit unterschiedlichen ea-Optionen (ea wie "enable assertions")
gestartet wurde:

> java -ea               Zusicherung01
--------------------------------
Zusicherung01: Jetzt geht es los!
istPrim(-17)              : true
istPrim(+18)              : false
istPrim(+27)              : false
istPrim(-1234567891)      : true
istPrim(Integer.MAX_VALUE): true
Exception in thread "main" java.lang.AssertionError
   at Zusicherung01.istPrim(Zusicherung01.java:16)
   at Zusicherung01.main(Zusicherung01.java:44)

> java -ea:java.lang...  Zusicherung01
--------------------------------
Zusicherung01: Jetzt geht es los!
istPrim(-17)              : true
istPrim(+18)              : false
istPrim(+27)              : false
istPrim(-1234567891)      : true
istPrim(Integer.MAX_VALUE): true
istPrim(Integer.MIN_VALUE): false
Zusicherung01: Das war's erstmal!

> java -ea:...           Zusicherung01
--------------------------------
Zusicherung01: Jetzt geht es los!
istPrim(-17)              : true
istPrim(+18)              : false
istPrim(+27)              : false
istPrim(-1234567891)      : true
istPrim(Integer.MAX_VALUE): true
Exception in thread "main" java.lang.AssertionError
   at Zusicherung01.istPrim(Zusicherung01.java:16)
   at Zusicherung01.main(Zusicherung01.java:44)

> java -ea:Neben01       Zusicherung01
--------------------------------
Zusicherung01: Jetzt geht es los!
istPrim(-17)              : true
istPrim(+18)              : false
istPrim(+27)              : false
istPrim(-1234567891)      : true
istPrim(Integer.MAX_VALUE): true
istPrim(Integer.MIN_VALUE): false
Zusicherung01: Das war's erstmal!
------------------------------------------------------------------------ */
