// Datei Zuweisungen02.java
/* ------------------------------------------------------------------------
Wenn man den Zuweisungsoperator = und den Gleichheitsoperator == ver-
wechselt, entsteht fast immer ein formaler Fehler, den der Ausfuehrer
entdeckt. Dieses Programm zeigt einen Ausnahm von dieser Regel (eine
Verwechslung, die der Ausfuehrer nicht als falsch erkennt).

Ausserdem wird die Benutzung von zusammengesetzten Zuweisungsoperatoren
wie += und *= vorgefuehrt.
------------------------------------------------------------------------ */
import java.util.Arrays; // fuer toString

class Zuweisungen02 {
   // ---------------------------------------------------------------------
   static void zuweisungStattVergleich() {
      // In einem boolean-Ausdruck kann man den einfachen Zuweisungs-
      // operator = mit dem Gleichheitsoperator == verwechseln ohne dass
      // der Ausfhrer einen formalen Fehler entdeckt.
      pln("-----------------------------------------------------");
      pln("zuweisungStattVergleich:");
      boolean a = false;
      boolean b = true;

      if (a = b) {             // gemeint ist vermutlich if (a == b) {
         pln("a und b sind gleich!");
      }
   } // zuweisungStattVergleich
   // ---------------------------------------------------------------------
   // Eine gloabale Variable, auf die alle Methoden dieser Klasse
   // zugreifen koennen:
   static int anna=9;

   static void pAR() {
      // print and reset anna
      // Gibt den Namen und den Wert der globalene Variablen anna (mit
      // einem kleinen Kommentar, aber ohne Zeilenwechsel) aus und setzt
      // anna dann auf ihren Anfangswert 9 zuerueck.
      p("anna nach Kurzform: " + anna);
      anna = 9;
   } // pAR

   static void plnAR() {
      // print line and reset anna
      // Gibt den Namen und den Wert der globalene Variablen anna (mit
      // einem kleinen Kommentar und einem Zeilenwechsel) aus und setzt
      // anna dann auf ihren Anfangswert 9 zuerueck.
      pln(", anna nach Langform: " + anna);
      anna = 9;
   } // plnAR
   // ---------------------------------------------------------------------
   static void zusammengesetzteOperatoren() {
      // Es wird gezeigt, wie man kurze Zuweisungen (mit einem zusammen-
      // gesetzten Zuweisungsoperator) ersetzen kann durch entsprechende
      // lange Zuweisungen (mit dem einfachen Zuweisungsoperator).
      // Dazu wird der Wert der globalen Variablen anna mehrmals
      // veraendert, jeweils mit einer "Kurzform" und dann mit einer
      // entsprechenden "Langform". Die Werte werden ausgegeben um zu
      // zeigen, dass sie immer gleich sind.
      pln("-----------------------------------------------------");
      pln("zusammengesetzteOperatoren:");

      // Kurzform:             Entsprechende Langform:
      anna += 1;     pAR();    anna =        anna + 1;       plnAR();
      anna *= 2;     pAR();    anna =        anna * 2;       plnAR();
      anna += anna;  pAR();    anna =        anna + anna;    plnAR();
      anna *= anna;  pAR();    anna =        anna * anna;    plnAR();
      anna *= 2 + 3; pAR();    anna =        anna * (2 + 3); plnAR();
      anna *= 1.5;   pAR();    anna = (int) (anna * 1.5);    plnAR();
//                             anna =       (anna * 1.5);    // Typfehler!
   } // zusammengesetzteOperatoren
   // ---------------------------------------------------------------------
   static void mitSeiteneffekt() {
      // Es wird gezeigt, dass kurze Zuweisungen (mit einem zusammen-
      // gesetzten Zuweisungsoperator) andere Ergebnisse produzieren
      // koennen als die entsprechende lange Zuweisung (mit dem einfachen
      // Zuweisungsoperator) wenn Seiteneffekte ins Spiel kommen.
      // Dazu wird der Wert der Reihungskomponenten annaR[0] zuerst mit
      // einer "Kurzform" und dann mit der entsprechenden "Langform"
      // veraendert. Die Werte werden ausgegeben um zu zeigen, dass
      // sie verschieden sind.
      pln("-----------------------------------------------------");
      pln("mitSeiteneffekt:");

      int[] annaR;    // Eine Reihung von int-Variablen (int array)
      int   indAnnaR; // Indexvariable fuer die Reihung annaR
      // ----------------------------------------
      annaR    = new int[] {1001, 1020, 1300};
      indAnnaR = 0; // Index fuer annaR
      pln("A annaR vor Zuweisung: " + Arrays.toString(annaR));

      // annaR[0] veraendern mit Kurzform (und ausgeben):
      annaR[indAnnaR++] += 4;
      pln("B annaR nach Kurzform: " + Arrays.toString(annaR));
      // ----------------------------------------
      annaR    = new int[] {1001, 1020, 1300};
      indAnnaR = 0;
      pln("C annaR vor Zuweisung: " + Arrays.toString(annaR));

      // annaR[0] veraendern mit Langform (und ausgeben):
      annaR[indAnnaR++] = annaR[indAnnaR++] + 4;
      pln("D annaR nach Langform: " + Arrays.toString(annaR));
      // ----------------------------------------
   } // mitSeiteneffekt
   // ---------------------------------------------------------------------
   // Mehrere Methoden mit kurzen Namen:
   static void pln(Object ob) {System.out.println(ob);}
   static void p  (Object ob) {System.out.print  (ob);}
   // ---------------------------------------------------------------------
   static public void main(String[] sonja) {
      zuweisungStattVergleich();
      zusammengesetzteOperatoren();
      mitSeiteneffekt();
      pln("-----------------------------------------------------");
      pln("Zuweisungen02: Das war's erstmal!");
   } // main
   // ---------------------------------------------------------------------
} // Zuweisungen02
/* ------------------------------------------------------------------------
Ausgabe des Programms Zuweisungen02:

-----------------------------------------------------
zuweisungStattVergleich:
a und b sind gleich!
-----------------------------------------------------
zusammengesetzteOperatoren:
anna nach Kurzform: 10, anna nach Langform: 10
anna nach Kurzform: 18, anna nach Langform: 18
anna nach Kurzform: 18, anna nach Langform: 18
anna nach Kurzform: 81, anna nach Langform: 81
anna nach Kurzform: 45, anna nach Langform: 45
anna nach Kurzform: 13, anna nach Langform: 13
-----------------------------------------------------
mitSeiteneffekt:
A annaR vor Zuweisung: [1001, 1020, 1300]
B annaR nach Kurzform: [1005, 1020, 1300]
C annaR vor Zuweisung: [1001, 1020, 1300]
D annaR nach Langform: [1024, 1020, 1300]
-----------------------------------------------------
Zuweisungen02: Das war's erstmal!
---------------------------------------------------------------------------
Fehlermeldung des jikes-Compilers, wenn Zeile 61 kein Kommentar ist:

Found 1 semantic error compiling "D:/JavaQuellen/Zuweisungen02.java":
    69.                         anna =       (anna * 1.5);    // Typfehler!
                                <----------------------->
*** Error: The type of the left-hand side in this assignment, "int", is not
compatible with the type of the right-hand side expression, "double".
------------------------------------------------------------------------ */