// Datei RodentListener.java
/* ------------------------------------------------------------------------
Es gibt acht Arten von Mausereignissen (mouseClicked bis mouseWheelMoved).
Die entsprechenden acht Behandlermethoden sind auf drei Schnittstellen
(MouseListener, MouseMotionListener und MouseWheelListener) verteilt und
Behandlerobjekte ("Objekte mit Behandlermethoden darin") muessen mit drei
unterschiedlichen Methoden bei dem betreffenden Component-Objekt an- bzw.
abgemeldet werden. Es folgt eine tabellarische Uebersicht:

+----------------+----------------------+------------------------+
|Ereignis-Art/   |Methode zum           |Methode zum             |
|Behandlermethode|Anmelden              |Abmelden                |
+----------------+----------------------+------------------------+
|mouseClicked    |addMouseListener      |removeMouseListener     |
|mouseEntered    |addMouseListener      |removeMouseListener     |
|mouseExited     |addMouseListener      |removeMouseListener     |
|mousePressed    |addMouseListener      |removeMouseListener     |
|mouseReleased   |addMouseListener      |removeMouseListener     |
|                |                      |                        |
|mouseDragged    |addMouseMotionListener|removeMousMotionListener|
|mouseMoved      |addMouseMotionListener|removeMousMotionListener|
|                |                      |                        |
|mouseWheelMoved |addMouseWheelListener |removeMouseWheelListener|
+----------------+----------------------+------------------------+

Problem 1: Wenn man z.B. 8 Behandlerobjekte mit jeweils nur einer
"richtigen" Behandlermethode darin z.B. bei einem JFrame-Objekt dyna-
misch anmelden will, muss man mit umstaendlichen Fallunterscheidungen
zwischen den drei Methoden addMouseListener, addMouseMotionListener
und addMouseWheelListener unterscheiden. Ganz Entsprechendes gilt
fuer das dynamische abmelden solcher Behandlerobjekte.

Problem 2: Behandlerobjekte der unterschiedlichen Type (MouseListener,
MouseMotionListener und MouseWheelListener) kann man zwar in einer
Reihung mit dem Komponententyp Event speichern, aber die Komponenten
dieser Reihung sind dann nicht an- und abmeldbar (sondern muessen
mit Cast-Befehlen umgedeutet werden, beovr man sie an- oder abmelden
kann).

Wuensche zur Beseitigung dieser Probleme:
1. Anstelle der drei unterschiedlichen Methodennamen addMouseListener,
addMouseMotionListener und addMouseWheelListener *einen* ueberladenen
Methodennamen, z.B. addThisListener. Fuer das Abmelden einen entsprech-
end ueberladenen Methodennamen, z.B. removeThisListener.

2. Einen gemeinsamen Obertyp von MouseListener, MouseMotionListener
und MouseWheelListener, dessen Objekte (ohne Cast-Befehle) an- und
abmeldbar sind (MouseInputListener ist leider nur ein Obertyp von
MouseListener und MouseMotionListener, aber nicht von MouseWheel-
Listener).

Diese Wuensche werden hier mit einer abstrakten Oberklasse (RodentListener)
und drei konkreten Unterklassen (RodentPlainListener, RodentMotionListener
und RodentWheelListener) erfuellt. Dabei soll "Rodent" (Nagetier) ein
bisschen abstrakter als "Mouse" klingen.

Ein RodentListener-Objekt enthaelt ein Behandlerobjekt fuer Mausereignisse
und zwei Methoden namens addThisListener und removeThisListener, mit denen
man dieses Behandlerobjekt bei einem beliebigen Component-Objekt (z.B.
bei einem JFrame-Objekt) an- bzw. abmelden kann. Die Objekte der Unter-
typen von RodentListener unterscheiden sich durch die Typen der
Behandlerobjekte, die sie enthalten:

+--------------------+-----------------------+
|Jedes Objekt der    |enthaelt ein Behandler-|
|Unterklasse         |objekt des Typs        |
+--------------------+-----------------------+
|RodentPlainListener |MouseListener          |
|RodentMotionListener|MouseMotionListener    |
|RodentWheelListner  |MouseWheelListener     |
+--------------------+-----------------------+

Die abstrakte Klasse RodentListener enthaelt drei gleichnamige
Klassenmethoden ("Fabrikmethoden") namens makeRL, mit denen man
Behandlerobjekte der Typen MouseListener, MouseMotionListener und
MouseWheelListener in RodentListener-Objekte "einwickeln" kann.

Sei
comp ein Component-Objekt (z.B. ein JFrame-Objekt),
ml   ein Behandlerobjekt des Typs MouseListener, MouseMotionListener
     oder MouseWheelListener
rl   ein entsprechendes RodentListener-Objekt (RodentListener.makeRL(ml)).

Um das Behandlerobjekt ml beim Component-Objekt comp anzumelden,
musste man bisher (abheangig vom genauen Typ von ml) eine der folgenden
Methodenaufrufe auswaehelen:

comp.addMouseListener      (ml); oder
comp.addMouseMotionListener(ml); oder
comp.addMouseWheelListener (ml);

Jetzt kann man statt dessen (unabhaengig vom genauen Typ von ml)
einfach folgenden Befehl verwenden:

rl.addThisListener(comp);

Eigentlich gehoeren die vier Klassen RodentListener, RodentPlainListener,
RodentMotionListener und RodentWheelListener in das Paket java.awt.event.
Da es aber verboten (oder schwierig?) ist, dieses Standardpaket zu
erweitern, werden die Klassen im Paket myja.awt.event abgelegt (myja
steht dabei fuer "my Java").
------------------------------------------------------------------------ */
package myja.awt.event;

import  java.awt      .Component;
import  java.awt.event.MouseListener;
import  java.awt.event.MouseMotionListener;
import  java.awt.event.MouseWheelListener;
// ========================================================================
public abstract class RodentListener {
   /* ---------------------------------------------------------------------
   Jedes Objekt dieser Klasse (d.h. einer konkreten Unterklasse
   dieser Klasse) enthaelt ein Behandlerobjekt des Typs
   MouseListener, MouseMotionListener oder MouseWheelListener.
   Mit den Methoden addThisListener und remvoveThisListener kann man
   dieses Behandlerobjekt bei einem Component-Objekt an- bzw.
   abmelden.

   Diese Klasse hat 3 konkrete Unterklassen:
   RodentPlainListener, RodentMotionListener und RodentWheelListener
   Weitere Unterklassen sind (zur Zeit) nicht sinnvoll.
   --------------------------------------------------------------------- */
   abstract public void    addThisListener(Component comp);
   abstract public void removeThisListener(Component comp);

   static public RodentListener makeRL(MouseListener ml) {
      return new RodentPlainListener(ml);
   } // makeRL

   static public RodentListener makeRL(MouseMotionListener mml) {
      return new RodentMotiontListener(mml);
   } // makeRL

   static public RodentListener makeRL(MouseWheelListener mwl) {
      return new RodentWheelListener(mwl);
   } // makeRL
   // ---------------------------------------------------------------------
   // Eine Methode mit einem kurzen Namen:
   static void printf(String f, Object... v) {System.out.printf(f, v);}
   // ---------------------------------------------------------------------
} // class RodentListener